@@ -1,10 +1,10 @@
-// -*- C++ -*-
 #ifndef _Empire_h_
 #define _Empire_h_
 
 #include "ResourcePool.h"
 #include "../util/Export.h"
 #include "../universe/Meter.h"
+#include "../universe/ShipDesign.h"
 
 #include <GG/Clr.h>
 
@@ -15,6 +15,7 @@ struct ItemSpec;
 class ShipDesign;
 class SitRepEntry;
 extern const int INVALID_GAME_TURN;
+extern const int INVALID_OBJECT_ID;
 
 class Alignment {
 public:
@@ -146,8 +147,10 @@ struct FO_COMMON_API ProductionQueue {
 
         bool operator<(const ProductionItem& rhs) const;
 
-        BuildType   build_type;
+        std::map<std::string, std::map<int, float> >    CompletionSpecialConsumption(int location_id) const;// for each special name, what object ids have those special capacities reduced by what amount
+        std::map<MeterType, std::map<int, float> >      CompletionMeterConsumption(int location_id) const;  // for each meter type, what object ids have those meters reduced by what amount
 
+        BuildType   build_type;
         // only one of these may be valid, depending on BuildType
         std::string name;
         int         design_id;
@@ -177,6 +180,7 @@ struct FO_COMMON_API ProductionQueue {
         int             blocksize_memory;           ///< used along with progress_memory
         int             turns_left_to_next_item;
         int             turns_left_to_completion;
+        int             rally_point_id;
 
     private:
         friend class boost::serialization::access;
@@ -277,7 +281,7 @@ public:
     typedef std::set<std::string>::const_iterator       TechItr;
     typedef std::set<std::string>::const_iterator       BuildingTypeItr;
     typedef std::set<int>::const_iterator               SystemIDItr;
-    typedef std::set<int>::const_iterator               ShipDesignItr;
+    typedef std::list<int>::const_iterator              ShipDesignItr;
     typedef std::vector<SitRepEntry>::const_iterator    SitRepItr;
     //@}
 
@@ -328,6 +332,7 @@ public:
     bool        ResearchableTech(const std::string& name) const;        ///< Returns true iff \a name is a tech that has not been researched, and has no unresearched prerequisites.
     float       ResearchProgress(const std::string& name) const;        ///< Returns the RPs spent towards tech \a name if it has partial research progress, or 0.0 if it is already researched.
     bool        TechResearched(const std::string& name) const;          ///< Returns true iff this tech has been completely researched.
+    bool        HasResearchedPrereqAndUnresearchedPrereq(const std::string& name) const;    ///< Returns true iff this tech has some but not all prerequisites researched
     TechStatus  GetTechStatus(const std::string& name) const;           ///< Returns the status (researchable, researched, unresearchable) for this tech for this
 
     bool        BuildingTypeAvailable(const std::string& name) const;   ///< Returns true if the given building type is known to this empire, false if it is not
@@ -349,38 +354,28 @@ public:
     bool                    ProducibleItem(const ProductionQueue::ProductionItem& item, int location) const;    ///< Returns true iff this empire can produce the specified item at the specified location.
 
     bool                    EnqueuableItem(BuildType build_type, const std::string& name, int location) const;  ///< Returns true iff this empire can enqueue the specified item at the specified location.
+    bool                    EnqueuableItem(const ProductionQueue::ProductionItem& item, int location) const;    ///< Returns true iff this empire can enqueue the specified item at the specified location.
 
     bool                    HasExploredSystem(int ID) const;                            ///< returns  true if the given item is in the appropriate list, false if it is not.
 
+    bool                    Eliminated() const;                                         ///< whether this empire has lost the game
+    bool                    Won() const;                                                ///< whether this empire has won the game
+
     int                     NumSitRepEntries(int turn = INVALID_GAME_TURN) const;       ///< number of entries in the SitRep.
 
     /** Returns distance in jumps away from each system that this empire can
-      * propegate supply. */
-    const std::map<int, int>&               SystemSupplyRanges() const;
+      * propagate supply. */
+    const std::map<int, float>&             SystemSupplyRanges() const;
+
     /** Returns set of system ids that are able to propagate supply from one
       * system to the next, or at which supply can be delivered to fleets if
       * supply can reach the system from elsewhere, or in which planets can
       * exchange supply between themselves (even if not leaving the system). */
     const std::set<int>&                    SupplyUnobstructedSystems() const;
-    /** Returns set of directed starlane traversals along which supply can flow.
-      * Results are pairs of system ids of start and end system of traversal. */
-    const std::set<std::pair<int, int> >&   SupplyStarlaneTraversals() const;
-    /** Returns set of directed starlane traversals along which supply could
-      * flow for this empire, but which can't due to some obstruction in one
-      * of the systems. */
-    const std::set<std::pair<int, int> >&   SupplyObstructedStarlaneTraversals() const;
-    /** Returns set of system ids where fleets can be supplied by this empire
-      * (as determined by object supply meters and rules of supply propagation
-      * and blockade). */
-    const std::set<int>&                    FleetSupplyableSystemIDs() const;
-    /** Returns true if system with id \a system_id is fleet supplyable or in
-      * one of the resource supply groups of this empire. */
-    bool                                    SystemHasFleetSupply(int system_id) const;
-    /** Returns true if the start system is known to this empire and travel to the dest system is not restricted by blockade */
+
+    /** Returns true if the start system is known to this empire and travel to
+        the dest system is not restricted by blockade.  */
     const bool                              UnrestrictedLaneTravel(int start_system_id, int dest_system_id) const;
-    /** Returns set of sets of systems that can share industry (systems in
-      * separate groups are blockaded or otherwise separated). */
-    const std::set<std::set<int> >&         ResourceSupplyGroups() const;
 
     const std::set<int>&                    ExploredSystems() const;    ///< returns set of ids of systems that this empire has explored
     const std::map<int, std::set<int> >     KnownStarlanes() const;     ///< returns map from system id (start) to set of system ids (endpoints) of all starlanes known to this empire
@@ -399,7 +394,7 @@ public:
 
     const boost::shared_ptr<ResourcePool>   GetResourcePool(ResourceType resource_type) const;  ///< Returns ResourcePool for \a resource_type or 0 if no such ResourcePool exists
     float                   ResourceStockpile(ResourceType type) const;         ///< returns current stockpiled amount of resource \a type
-    float                   ResourceProduction(ResourceType type) const;        ///< returns amount of resource \a type being produced by ResourceCenters
+    float                   ResourceOutput(ResourceType type) const;            ///< returns amount of resource \a type being generated by ResourceCenters
     float                   ResourceAvailable(ResourceType type) const;         ///< returns amount of resource \a type immediately available.  This = production + stockpile
 
     const PopulationPool&   GetPopulationPool() const;                          ///< Returns PopulationPool
@@ -414,7 +409,7 @@ public:
     /** Returns the alignment meter with the indicated \a name, if any, or 0 if
       * no such alignment meter exists. */
     Meter*      GetMeter(const std::string& name);
-    void        BackPropegateMeters();
+    void        BackPropagateMeters();
 
     /** Adds \a tech to the research queue, placing it before position \a pos.
       * If \a tech is already in the queue, it is moved to \a pos, then removed
@@ -440,7 +435,8 @@ public:
       * placed at the end of the queue. */
     void        PlaceBuildInQueue(const ProductionQueue::ProductionItem& item, int number, int location, int pos = -1);
     void        SetBuildQuantity(int index, int quantity);      ///< Changes the remaining number to build for queue item \a index to \a quantity
-    void        SetBuildQuantityAndBlocksize(int index, int quantity, int blocksize);  ///< Changes the remaining number and blocksize to build for queue item \a index to \a quantity and \a blocksize 
+    void        SetBuildQuantityAndBlocksize(int index, int quantity, int blocksize);   ///< Changes the remaining number and blocksize to build for queue item \a index to \a quantity and \a blocksize 
+    void        SetBuildRallyPoint(int index, int rally_point_id = INVALID_OBJECT_ID);  ///< Sets the rally point for ships produced by this build, to which they are automatically ordered to move after they are produced.
     void        MoveBuildWithinQueue(int index, int new_index); ///< Moves \a tech from the production queue, if it is in the production queue already.
     void        RemoveBuildFromQueue(int index);                ///< Removes the build at position \a index in the production queue, if such an index exists.
 
@@ -450,11 +446,14 @@ public:
     void        AddPartType(const std::string& name);       ///< Inserts the given ship PartType into the Empire's list of available BuldingTypes.
     void        AddHullType(const std::string& name);       ///< Inserts the given ship HullType into the Empire's list of available BuldingTypes.
     void        AddExploredSystem(int ID);                  ///< Inserts the given ID into the Empire's list of explored systems.
-    void        AddShipDesign(int ship_design_id);          ///< inserts given design id into the empire's set of designs
+
+    /** inserts given design id into the empire's set of designs in front of next design */
+    void        AddShipDesign(int ship_design_id, int next_design_id = ShipDesign::INVALID_DESIGN_ID);
     int         AddShipDesign(ShipDesign* ship_design);     ///< inserts given ShipDesign into the Universe, adds the design's id to the Empire's set of ids, and returns the new design's id, which is INVALID_OBJECT_ID on failure.  If successful, universe takes ownership of passed ShipDesign.
 
     std::string NewShipName();                              ///< generates a random ship name, appending II, III, etc., to it if it has been used before by this empire
-    void        EliminationCleanup();                       ///< Cleans up empire after it is eliminated.  Queues are cleared, capital is reset, and other state info not relevant to an eliminated empire is cleared
+    void        Eliminate();                                ///< Marks empire as eliminated and cleans up empire after it is eliminated.  Queues are cleared, capital is reset, and other state info not relevant to an eliminated empire is cleared
+    void        Win(const std::string& reason);             ///< Marks this empire as having won for this reason, and sends the appropriate sitreps
 
     /** Inserts the a pointer to given SitRep entry into the empire's sitrep list.
      *  \warning When you call this method, you are transferring ownership
@@ -475,14 +474,14 @@ public:
 
     /** Calculates ranges that systems can send fleet and resource supplies,
       * using the specified st of \a known_objects as the source for supply-
-      * producing objects and systems through which it can be propegated. */
+      * producing objects and systems through which it can be propagated. */
     void        UpdateSystemSupplyRanges(const std::set<int>& known_objects);
     /** Calculates ranges that systems can send fleet and resource supplies. */
     void        UpdateSystemSupplyRanges();
-    /** Calculates systems that can propegate supply (fleet or resource) using
+    /** Calculates systems that can propagate supply (fleet or resource) using
       * the specified set of \a known_systems */
     void        UpdateSupplyUnobstructedSystems(const std::set<int>& known_systems);
-    /** Calculates systems that can propegate supply using this empire's own /
+    /** Calculates systems that can propagate supply using this empire's own /
       * internal list of explored systems. */
     void        UpdateSupplyUnobstructedSystems();
     /** Updates fleet ArrivalStarlane to flag fleets of this empire that are not blockaded post-combat 
@@ -492,15 +491,6 @@ public:
     void        RecordPendingLaneUpdate(int start_system_id, int dest_system_id);
     /** Processes all the pending lane access updates.  This is managed as a two step process to avoid order-of-processing issues. */
     void        UpdateAvailableLanes();
-    /** Calculates systems at which fleets of this empire can be supplied, and
-      * groups of systems that can exchange resources, and the starlane
-      * traversals used to do so, using the indicated \a starlanes but subject
-      * to obstruction of supply by various factors.  Call
-      * UpdateSystemSupplyRanges and UpdateSupplyUnobstructedSystems before
-      * calling this. */
-    void        UpdateSupply(const std::map<int, std::set<int> >& starlanes);
-    /** Updates supply using this empire's set of known starlanes. */
-    void        UpdateSupply();
 
     /** Checks for production projects that have been completed, and places them
       * at their respective production sites.  Which projects have been
@@ -609,6 +599,9 @@ private:
     GG::Clr                         m_color;                    ///< Empire's color
     int                             m_capital_id;               ///< the ID of the empire's capital planet
 
+    bool                            m_eliminated;               ///< Whether the empire has lost
+    std::set<std::string>           m_victories;                ///< The ways that the empire has won, if any
+
     std::set<std::string>           m_techs;                    ///< list of acquired technologies.  These are string names referencing Tech objects
 
     std::map<std::string, Meter>    m_meters;                   ///< empire meters, including ratings scales used by species to judge empires
@@ -622,7 +615,8 @@ private:
     std::set<std::string>           m_available_part_types;     ///< list of acquired ship PartType.  These are string names referencing PartType objects
     std::set<std::string>           m_available_hull_types;     ///< list of acquired ship HullType.  These are string names referencing HullType objects
     std::set<int>                   m_explored_systems;         ///< systems you've explored
-    std::set<int>                   m_ship_designs;             ///< The Empire's ship designs, indexed by design id
+    std::set<int>                   m_ship_designs;             ///< The Empire's ship designs sorted by id
+    std::list<int>                  m_ship_designs_ordered;     ///<The Empire's ship designs in preferred order
 
     std::vector<SitRepEntry>        m_sitrep_entries;           ///< The Empire's sitrep entries
 
@@ -640,7 +634,7 @@ private:
     std::map<int, int>              m_empire_ships_destroyed;   ///< how many ships of each empire has this empire destroyed?
     std::map<int, int>              m_ship_designs_destroyed;   ///< how many ships of each design has this empire destroyed?
     std::map<std::string, int>      m_species_ships_destroyed;  ///< how many ships crewed by each species has this empire destroyed?
-    std::map<std::string, int>      m_species_planets_invaded;  ///< how many planets populated by each species has this empire destroyed?
+    std::map<std::string, int>      m_species_planets_invaded;  ///< how many planets populated by each species has this empire captured?
 
     std::map<std::string, int>      m_species_ships_produced;   ///< how many ships crewed by each species has this empire produced?
     std::map<int, int>              m_ship_designs_produced;    ///< how many ships of each design has this empire produced?
@@ -656,14 +650,10 @@ private:
     std::map<std::string, int>      m_building_types_scrapped;  ///< how many buildings of each type has this empire scrapped?
 
     // cached calculation results, returned by reference
-    std::map<int, int>              m_supply_system_ranges;                 ///< number of starlane jumps away from each system (by id) supply can be conveyed.  This is the number due to a system's contents conveying supply and is computed and set by UpdateSystemSupplyRanges
-    std::set<int>                   m_supply_unobstructed_systems;          ///< ids of system that don't block supply from flowing
-    std::set<std::pair<int, int> >  m_supply_starlane_traversals;           ///< ordered pairs of system ids between which a starlane runs that can be used to convey resources between systems
-    std::set<std::pair<int, int> >  m_supply_starlane_obstructed_traversals;///< ordered pairs of system ids between which a starlane could be used to convey resources between system, but is not because something is obstructing the resource flow.  That is, the resource flow isn't limited by range, but by something blocking its flow.
-    std::map<int, std::set<int> >   m_available_system_exit_lanes;          ///< for each system known to this empire, the set of available/non-blockaded exit lanes for fleet travel
-    std::map<int, std::set<int> >   m_pending_system_exit_lanes;            ///< pending updates to m_available_system_exit_lanes
-    std::set<int>                   m_fleet_supplyable_system_ids;          ///< ids of systems where fleets can remain for a turn to be resupplied.
-    std::set<std::set<int> >        m_resource_supply_groups;               ///< sets of system ids that are connected by supply lines and are able to share resources between systems or between objects in systems
+    std::map<int, float>            m_supply_system_ranges;         ///< number of starlane jumps away from each system (by id) supply can be conveyed.  This is the number due to a system's contents conveying supply and is computed and set by UpdateSystemSupplyRanges
+    std::set<int>                   m_supply_unobstructed_systems;  ///< ids of system that don't block supply from flowing
+    std::map<int, std::set<int> >   m_available_system_exit_lanes;  ///< for each system known to this empire, the set of available/non-blockaded exit lanes for fleet travel
+    std::map<int, std::set<int> >   m_pending_system_exit_lanes;    ///< pending updates to m_available_system_exit_lanes
 
     friend class boost::serialization::access;
     Empire();
