@@ -128,6 +128,91 @@ private:
 };
 
 
+/** \brief Visual representation of a state button.
+
+    The StateButtonRepresenter is a stub interface to implement a visual
+    representation of a StateButton instance.  A representer can be shared
+    amongst multiple instances of a StateButton, so no subclass should
+    store instance specific attributes of a state button.
+ */
+class GG_API StateButtonRepresenter
+{
+public:
+    /** \brief Render the given state button according to its state.
+
+        \param button The StateButton instance to render.
+     */
+    virtual void Render(const StateButton& button) const;
+
+    /** \brief Respond to a state button change.
+
+        This method is called whenever a StateButton changes its
+        state.
+
+        \param checked True if the state button was checked, False
+                       if not.
+     */
+    virtual void OnChecked(bool checked) const;
+
+    /** \brief Return the minimum size required for a usable representation
+
+        \param button The state button to calculate the minimum size of.
+        \return The minimum size to create a usable representation of the
+           given state button.
+     */
+    virtual Pt MinUsableSize(const StateButton& button) const;
+
+protected:
+    /** \brief Calculate the layout of text and button component.
+
+        \param button The StateButton instance to layout.
+        \param button_ul[out] The upper left corner of the visual
+               button.
+        \param button_lr[out] The lower right corner of the visual
+               button.
+        \param text_ul[out] The upper left corner of the button
+               label.
+     */
+    virtual void DoLayout(const GG::StateButton& button, Pt& button_ul, Pt& button_lr, Pt& text_ul) const;
+};
+
+
+/** \brief Builtin representation of a check box state button. */
+class GG_API BeveledCheckBoxRepresenter: public StateButtonRepresenter
+{
+public:
+    explicit BeveledCheckBoxRepresenter(Clr int_color = CLR_ZERO);
+
+    virtual void Render(const StateButton& button) const;
+
+private:
+    Clr m_int_color;
+};
+
+
+/** \brief Builtin representation of a radio state button. */
+class GG_API BeveledRadioRepresenter: public StateButtonRepresenter
+{
+public:
+    explicit BeveledRadioRepresenter(Clr int_color = CLR_ZERO);
+
+    virtual void Render(const StateButton& button) const;
+
+private:
+    Clr m_int_color;
+};
+
+
+/** \brief Builtin representation of a tab state button (part of the TabWnd). */
+class GG_API BeveledTabRepresenter: public StateButtonRepresenter
+{
+public:
+    virtual void Render(const StateButton& button) const;
+
+    virtual Pt MinUsableSize(const StateButton& button) const;
+};
+
+
 /** \brief This is a basic state button control.
 
     This class is for checkboxes and radio buttons, etc.  The button/checkbox
@@ -150,7 +235,7 @@ public:
 
     /** \name Structors */ ///@{
     StateButton(const std::string& str, const boost::shared_ptr<Font>& font, Flags<TextFormat> format,
-                Clr color, Clr text_color = CLR_BLACK, Clr interior = CLR_ZERO, StateButtonStyle style = SBSTYLE_3D_CHECKBOX); ///< Ctor
+                Clr color, boost::shared_ptr<StateButtonRepresenter> representer, Clr text_color = CLR_BLACK); ///< Ctor
     //@}
 
     /** \name Accessors */ ///@{
@@ -159,10 +244,9 @@ public:
     const std::string& Text() const;        ///< Returns the label to be used as the button label
 
     bool             Checked() const;       ///< Returns true if button is checked
-    Clr              InteriorColor() const; ///< Returns the interior color of the box, circle, or other enclosing shape
+    bool             IsMouseover() const;   ///< Returns true if mouse pointer is over this button
 
-    /** Returns the visual style of the button \see StateButtonStyle */
-    StateButtonStyle Style() const;
+    TextControl* GetLabel() const;
 
     mutable CheckedSignalType CheckedSignal; ///< The checked signal object for this StaticButton
     //@}
@@ -174,40 +258,23 @@ public:
 
     void             Reset();                 ///< Unchecks button
     void             SetCheck(bool b = true); ///< (Un)checks button
-    void             SetButtonPosition(const Pt& ul, const Pt& lr); ///< places the button within the control
-    void             SetDefaultButtonPosition(); ///< Places the button to its default positionwithin the control
     virtual void     SetColor(Clr c);         ///< Sets the color of the button; does not affect text color
-    void             SetInteriorColor(Clr c); ///< Sets the interior color of the box, circle, or other enclosing shape
     void             SetTextColor(Clr c); ///< Sets the color of the box label text
-
-    /** Sets the visual style of the button \see StateButtonStyle */
-    void             SetStyle(StateButtonStyle bs);
     //@}
 
 protected:
-    /** \name Accessors */ ///@{
-    TextControl* GetLabel() const;
-    Pt  ButtonUpperLeft() const;  ///< Returns the upper-left of the button part of the control
-    Pt  ButtonLowerRight() const; ///< Returns the lower-right of the button part of the control
-    Pt  TextUpperLeft() const;    ///< Returns the upper-left of the text part of the control
-    //@}
-
     /** \name Mutators */ ///@{
     virtual void LClick(const Pt& pt, Flags<ModKey> mod_keys);
-
-    void RepositionButton();      ///< Places the button at the appropriate position based on the style flags, without resizing it
+    virtual void MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    virtual void MouseLeave();
     //@}
 
 private:
+    boost::shared_ptr<StateButtonRepresenter> m_representer;
     TextControl*      m_label;       ///< Label used to display text
 
     bool              m_checked;     ///< true when this button in a checked, active state
-    Clr               m_int_color;   ///< color inside border
-    StateButtonStyle  m_style;       ///< style of appearance to use when rendering button
-
-    Pt                m_button_ul;
-    Pt                m_button_lr;
-    Pt                m_text_ul;
+    bool              m_mouseover;   ///< true if the pointer is currently over the control
 };
 
 
