@@ -115,7 +115,7 @@ struct GUIImpl;
 class GG_API GUI
 {
 private:
-    struct OrCombiner 
+    struct OrCombiner
     {
         typedef bool result_type; 
         template<class InIt> bool operator()(InIt first, InIt last) const;
@@ -161,6 +161,7 @@ public:
     /** \name Accessors */ ///@{
     const std::string&          AppName() const;                    ///< returns the user-defined name of the application
     Wnd*                        FocusWnd() const;                   ///< returns the GG::Wnd that currently has the input focus
+    bool                        FocusWndAcceptsTypingInput() const; ///< returns true iff the current focus GG::Wnd accepts typing input
     Wnd*                        PrevFocusInteractiveWnd() const;    ///< returns the previous Wnd to the current FocusWnd. Cycles through INTERACTIVE Wnds, in order determined by parent-child relationships
     Wnd*                        NextFocusInteractiveWnd() const;    ///< returns the next Wnd to the current FocusWnd.
     Wnd*                        GetWindowUnder(const Pt& pt) const; ///< returns the GG::Wnd under the point pt
@@ -196,7 +197,10 @@ public:
         this function to exist, but the notion of what a "word" is is so
         application-specific that it was placed here so that users can
         customize this behavior. */
-    virtual std::set<std::pair<CPSize, CPSize> > FindWords(const std::string& str) const;
+    virtual std::set<std::pair<CPSize, CPSize> >    FindWords(const std::string& str) const;
+    virtual std::set<std::pair<StrSize, StrSize> >  FindWordsStringIndices(const std::string& str) const;
+    /** Returns true if \a word is a word that appears in \a str */
+    virtual bool                                    ContainsWord(const std::string& str, const std::string& word) const;
 
     const boost::shared_ptr<StyleFactory>& GetStyleFactory() const; ///< returns the currently-installed style factory
 
@@ -212,6 +216,12 @@ public:
     /** Returns the signal that is emitted when the requested keyboard accelerator is invoked. */
     AcceleratorSignalType& AcceleratorSignal(Key key, Flags<ModKey> mod_keys = MOD_KEY_NONE) const;
 
+    /** Returns true iff keyboard accelerator signals fire while modal windows are open. */
+    bool ModalAcceleratorSignalsEnabled() const;
+
+    /** Returns true iff any modal Wnds are open. */
+    bool ModalWndsOpen() const;
+
     /** Saves \a wnd to file \a filename during the next render cycle.  If \a
         wnd is not rendered during the render cycle, or PNG support is not
         enabled, this is a no-op. */
@@ -280,6 +290,9 @@ public:
     /** Removes a keyboard accelerator. */
     void           RemoveAccelerator(accel_iterator it);
 
+    /** Sets whether to emit keyboard accelerator signals while modal windows are open. */
+    void           EnableModalAcceleratorSignals(bool allow = true);
+
     /** Sets whether to swap left and right mouse button events. */
     void           SetMouseLRSwapped(bool swapped = true);
 
@@ -313,7 +326,7 @@ public:
     boost::shared_ptr<Font>    GetFont(const boost::shared_ptr<Font>& font, unsigned int pts);
 
     /** Removes the desired font from the managed pool; since shared_ptr's are
-        used, the font may be deleted much later */
+        used, the font may be deleted much later. */
     void                       FreeFont(const std::string& font_filename, unsigned int pts);
 
     /** Adds an already-constructed texture to the managed pool \warning
@@ -321,9 +334,24 @@ public:
         do that. */
     boost::shared_ptr<Texture> StoreTexture(Texture* texture, const std::string& texture_name);
 
-    boost::shared_ptr<Texture> StoreTexture(const boost::shared_ptr<Texture> &texture, const std::string& texture_name); ///< adds an already-constructed texture to the managed pool
-    boost::shared_ptr<Texture> GetTexture(const std::string& name, bool mipmap = false); ///< loads the requested texture from file \a name; mipmap textures are generated if \a mipmap is true
-    void                       FreeTexture(const std::string& name); ///< removes the desired texture from the managed pool; since shared_ptr's are used, the texture may be deleted much later
+    /** Adds an already-constructed texture to the managed pool. */
+    boost::shared_ptr<Texture> StoreTexture(const boost::shared_ptr<Texture> &texture, const std::string& texture_name);
+
+    /** Loads the requested texture from file \a name; mipmap textures are
+      * generated if \a mipmap is true. */
+    boost::shared_ptr<Texture> GetTexture(const std::string& name, bool mipmap = false);
+
+    /** Loads the requested texture from file \a name; mipmap textures are
+      * generated if \a mipmap is true. */
+    boost::shared_ptr<Texture> GetTexture(const boost::filesystem::path& path, bool mipmap = false);
+
+    /** Removes the desired texture from the managed pool; since shared_ptr's
+      * are used, the texture may be deleted much later. */
+    void                       FreeTexture(const std::string& name);
+
+    /** Removes the desired texture from the managed pool; since shared_ptr's
+      * are used, the texture may be deleted much later. */
+    void                       FreeTexture(const boost::filesystem::path& path);
 
     void SetStyleFactory(const boost::shared_ptr<StyleFactory>& factory); ///< sets the currently-installed style factory
 
@@ -353,14 +381,14 @@ public:
 
 
     /** Emitted whenever the GUI's AppWidth() and/or AppHeight() change. */
-    boost::signals2::signal<void (X, Y)> WindowResizedSignal;
+    boost::signals2::signal<void (X, Y)>    WindowResizedSignal;
 
     /** Emitted whenever the GUI's window's left and top positions change. */
-    boost::signals2::signal<void (X, Y)> WindowMovedSignal;
+    boost::signals2::signal<void (X, Y)>    WindowMovedSignal;
 
-    /** Emitted when the Window in which the GUI is operating
-     gains or loses focus. */
-     boost::signals2::signal<void ()> FocusChangedSignal;
+    /** Emitted when the Window in which the GUI is operating gains or loses
+      * focus. bool parameter is true when gaining focus, and false otherwise.*/
+     boost::signals2::signal<void (bool)>   FocusChangedSignal;
 
     /** \name Exceptions */ ///@{
     /** The base class for GUI exceptions. */
