@@ -36,15 +36,15 @@ using namespace GG;
 
 namespace {
     void ClickedEcho()
-    { std::cerr << "GG SIGNAL : Button::LeftClickedSignal()\n"; }
+    { std::cerr << "GG SIGNAL : Button::LeftClickedSignal()" << std::endl; }
 
     void CheckedEcho(bool checked)
-    { std::cerr << "GG SIGNAL : StateButton::CheckedSignal(checked=" << checked << ")\n"; }
+    { std::cerr << "GG SIGNAL : StateButton::CheckedSignal(checked=" << checked << ")" << std::endl; }
 
     void ButtonChangedEcho(std::size_t index)
     {
         std::cerr << "GG SIGNAL : RadioButtonGroup::ButtonChangedSignal(index="
-                  << index << ")\n";
+                  << index << ")" << std::endl;
     }
 }
 
@@ -254,21 +254,197 @@ void Button::RenderDefault()
 }
 
 
+////////////////////////////////////////////////
+// GG::BeveledCheckBoxRepresenter
+////////////////////////////////////////////////
+void StateButtonRepresenter::Render(const GG::StateButton& button) const
+{}
+
+void StateButtonRepresenter::DoLayout(const GG::StateButton& button, Pt& button_ul, Pt& button_lr, Pt& text_ul) const
+{
+    X bn_w = X(button.GetLabel()->GetFont()->PointSize()); // set button width and height to text he
+    Y bn_h = Y(button.GetLabel()->GetFont()->PointSize());
+
+    button_ul = Pt();
+    button_lr = Pt(bn_w, bn_h);
+
+    X w = button.Width();
+    Y h = button.Height();
+    const X BN_W = button_lr.x - button_ul.x;
+    const Y BN_H = button_lr.y - button_ul.y;
+    X bn_x = button_ul.x;
+    Y bn_y = button_ul.y;
+    Flags<TextFormat> format = button.GetLabel()->GetTextFormat();
+    Flags<TextFormat> original_format = format;
+    const double SPACING = 0.5; // the space to leave between the button and text, as a factor of the button's size (width or height)
+    if (format & FORMAT_VCENTER)       // center button vertically
+        bn_y = (h - BN_H) / 2.0 + 0.5;
+    if (format & FORMAT_TOP) {         // put button at top, text just below
+        bn_y = Y0;
+        text_ul.y = BN_H;
+    }
+    if (format & FORMAT_BOTTOM) {      // put button at bottom, text just above
+        bn_y = (h - BN_H);
+        text_ul.y = h - (BN_H * (1 + SPACING)) - (static_cast<int>(button.GetLabel()->GetLineData().size() - 1) * button.GetLabel()->GetFont()->Lineskip() + button.GetLabel()->GetFont()->Height()) + 0.5;
+    }
+
+    if (format & FORMAT_CENTER) {      // center button horizontally
+        if (format & FORMAT_VCENTER) { // if both the button and the text are to be centered, bad things happen
+            format |= FORMAT_LEFT;     // so go to the default (FORMAT_CENTER|FORMAT_LEFT)
+            format &= ~FORMAT_CENTER;
+        } else {
+            bn_x = (w - bn_x) / 2.0 - BN_W / 2.0 + 0.5;
+        }
+    }
+    if (format & FORMAT_LEFT) {        // put button at left, text just to the right
+        bn_x = X0;
+        if (format & FORMAT_VCENTER)
+            text_ul.x = BN_W * (1 + SPACING) + 0.5;
+    }
+    if (format & FORMAT_RIGHT) {       // put button at right, text just to the left
+        bn_x = (w - BN_W);
+        if (format & FORMAT_VCENTER)
+            text_ul.x = -BN_W * (1 + SPACING) + 0.5;
+    }
+    if (format != original_format)
+        button.GetLabel()->SetTextFormat(format);
+    button_ul = Pt(bn_x, bn_y);
+    button_lr = button_ul + Pt(BN_W, BN_H);
+}
+
+void StateButtonRepresenter::OnChecked(bool checked) const
+{}
+
+Pt StateButtonRepresenter::MinUsableSize(const StateButton& button) const
+{
+    Pt bn_ul, bn_lr, tx_ul;
+
+    DoLayout(button, bn_ul, bn_lr, tx_ul);
+
+    Pt text_lr = tx_ul + button.GetLabel()->MinUsableSize();
+    return Pt(std::max(bn_lr.x, text_lr.x) - std::min(bn_ul.x, tx_ul.x),
+              std::max(bn_lr.y, text_lr.y) - std::min(bn_ul.y, tx_ul.y));
+}
+
+
+////////////////////////////////////////////////
+// GG::BeveledCheckBoxRepresenter
+////////////////////////////////////////////////
+BeveledCheckBoxRepresenter::BeveledCheckBoxRepresenter(Clr interior/* = CLR_ZERO*/):
+    m_int_color(interior)
+{}
+
+void BeveledCheckBoxRepresenter::Render(const GG::StateButton& button) const
+{
+    const int BEVEL = 2;
+
+    // draw button
+    Pt cl_ul = button.ClientUpperLeft();
+    Pt bn_ul, bn_lr, tx_ul;
+
+    DoLayout(button, bn_ul, bn_lr, tx_ul);
+
+    bn_ul += cl_ul;
+    bn_lr += cl_ul;
+
+    const Pt DOUBLE_BEVEL(X(2 * BEVEL), Y(2 * BEVEL));
+
+    BeveledRectangle(bn_ul, bn_lr,
+                     button.Disabled() ? DisabledColor(m_int_color) : m_int_color,
+                     button.Disabled() ? DisabledColor(button.Color()) : button.Color(),
+                     false, BEVEL);
+    if (button.Checked())
+        BeveledCheck(bn_ul + DOUBLE_BEVEL, bn_lr - DOUBLE_BEVEL,
+                     button.Disabled() ? DisabledColor(button.Color()) : button.Color());
+
+    button.GetLabel()->OffsetMove(tx_ul);
+    button.GetLabel()->Render();
+    button.GetLabel()->OffsetMove(-tx_ul);
+}
+
+
+////////////////////////////////////////////////
+// GG::BeveledRadioRepresenter
+////////////////////////////////////////////////
+BeveledRadioRepresenter::BeveledRadioRepresenter(Clr interior/* = CLR_ZERO*/):
+    m_int_color(interior)
+{}
+
+void BeveledRadioRepresenter::Render(const GG::StateButton& button) const
+{
+    const int BEVEL = 2;
+
+    // draw button
+    Pt cl_ul = button.ClientUpperLeft();
+    Pt bn_ul, bn_lr, tx_ul;
+
+    DoLayout(button, bn_ul, bn_lr, tx_ul);
+
+    bn_ul += cl_ul;
+    bn_lr += cl_ul;
+
+    const Pt DOUBLE_BEVEL(X(2 * BEVEL), Y(2 * BEVEL));
+
+    BeveledCircle(bn_ul, bn_lr,
+                  button.Disabled() ? DisabledColor(m_int_color) : m_int_color,
+                  button.Disabled() ? DisabledColor(button.Color()) : button.Color(),
+                  false, BEVEL);
+    if (button.Checked())
+        Bubble(bn_ul + DOUBLE_BEVEL, bn_lr - DOUBLE_BEVEL,
+               button.Disabled() ? DisabledColor(button.Color()) : button.Color());
+
+    button.GetLabel()->OffsetMove(tx_ul);
+    button.GetLabel()->Render();
+    button.GetLabel()->OffsetMove(-(tx_ul));
+}
+
+Pt BeveledTabRepresenter::MinUsableSize(const StateButton& button) const
+{ return button.GetLabel()->MinUsableSize(); }
+
+
+////////////////////////////////////////////////
+// GG::BeveledTabRepresenter
+////////////////////////////////////////////////
+void BeveledTabRepresenter::Render(const StateButton& button) const
+{
+    const int BEVEL = 2;
+
+    // draw button
+    Pt cl_ul = button.ClientUpperLeft();
+    Pt cl_lr = button.ClientLowerRight();
+    Pt tx_ul = Pt();
+
+    Clr color_to_use = button.Checked() ? button.Color() : DarkColor(button.Color());
+    color_to_use = button.Disabled() ? DisabledColor(color_to_use) : color_to_use;
+    if (!button.Checked()) {
+        cl_ul.y += BEVEL;
+        tx_ul.y = Y(BEVEL / 2);
+    }
+    BeveledRectangle(cl_ul, cl_lr,
+                     color_to_use, color_to_use,
+                     true, BEVEL,
+                     true, true, true, !button.Checked());
+
+    button.GetLabel()->OffsetMove(tx_ul);
+    button.GetLabel()->Render();
+    button.GetLabel()->OffsetMove(-(tx_ul));
+}
+
+
 ////////////////////////////////////////////////
 // GG::StateButton
 ////////////////////////////////////////////////
 StateButton::StateButton(const std::string& str, const boost::shared_ptr<Font>& font, Flags<TextFormat> format,
-                         Clr color, Clr text_color/* = CLR_BLACK*/, Clr interior/* = CLR_ZERO*/, StateButtonStyle style/* = SBSTYLE_3D_CHECKBOX*/) :
+                         Clr color, boost::shared_ptr<StateButtonRepresenter> representer, Clr text_color/* = CLR_BLACK*/) :
     Control(X0, Y0, X1, Y1, INTERACTIVE),
+    m_representer(representer),
     m_label(new TextControl(X0, Y0, X1, Y1, str, font, text_color, format, NO_WND_FLAGS)),
     m_checked(false),
-    m_int_color(interior),
-    m_style(style)
+    m_mouseover(false)
 {
     m_color = color;
     AttachChild(m_label);
     m_label->Hide();
-    SetDefaultButtonPosition();
 
     if (INSTRUMENT_ALL_SIGNALS)
         Connect(CheckedSignal, &CheckedEcho);
@@ -276,9 +452,10 @@ StateButton::StateButton(const std::string& str, const boost::shared_ptr<Font>&
 
 Pt StateButton::MinUsableSize() const
 {
-    Pt text_lr = m_text_ul + m_label->MinUsableSize();
-    return Pt(std::max(m_button_lr.x, text_lr.x) - std::min(m_button_ul.x, m_text_ul.x),
-              std::max(m_button_lr.y, text_lr.y) - std::min(m_button_ul.y, m_text_ul.y)) + GG::Pt(GG::X(25), GG::Y0);
+    if (m_representer)
+        return m_representer->MinUsableSize(*this);
+
+    return Pt();
 }
 
 const std::string& StateButton::Text() const
@@ -287,75 +464,13 @@ const std::string& StateButton::Text() const
 bool StateButton::Checked() const
 { return m_checked; }
 
-Clr StateButton::InteriorColor() const
-{ return m_int_color; }
-
-StateButtonStyle StateButton::Style() const
-{ return m_style; }
+bool StateButton::IsMouseover() const
+{ return m_mouseover; }
 
 void StateButton::Render()
 {
-    const int BEVEL = 2;
-
-    // draw button
-    Pt cl_ul = ClientUpperLeft();
-    Pt cl_lr = ClientLowerRight();
-    Pt bn_ul = cl_ul + m_button_ul;
-    Pt bn_lr = cl_ul + m_button_lr;
-
-    Pt additional_text_offset;
-
-    const Pt DOUBLE_BEVEL(X(2 * BEVEL), Y(2 * BEVEL));
-
-    switch (m_style) {
-    case SBSTYLE_3D_CHECKBOX:
-        BeveledRectangle(bn_ul, bn_lr,
-                         Disabled() ? DisabledColor(m_int_color) : m_int_color,
-                         Disabled() ? DisabledColor(m_color) : m_color,
-                         false, BEVEL);
-        if (m_checked)
-            BeveledCheck(bn_ul + DOUBLE_BEVEL, bn_lr - DOUBLE_BEVEL,
-                         Disabled() ? DisabledColor(m_color) : m_color);
-        break;
-    case SBSTYLE_3D_RADIO:
-        BeveledCircle(bn_ul, bn_lr,
-                      Disabled() ? DisabledColor(m_int_color) : m_int_color,
-                      Disabled() ? DisabledColor(m_color) : m_color,
-                      false, BEVEL);
-        if (m_checked)
-            Bubble(bn_ul + DOUBLE_BEVEL, bn_lr - DOUBLE_BEVEL,
-                   Disabled() ? DisabledColor(m_color) : m_color);
-        break;
-    case SBSTYLE_3D_TOP_ATTACHED_TAB: {
-        Clr color_to_use = m_checked ? m_color : DarkColor(m_color);
-        color_to_use = Disabled() ? DisabledColor(color_to_use) : color_to_use;
-        if (!m_checked) {
-            cl_ul.y += BEVEL;
-            additional_text_offset.y = Y(BEVEL / 2);
-        }
-        BeveledRectangle(cl_ul, cl_lr,
-                         color_to_use, color_to_use,
-                         true, BEVEL,
-                         true, true, true, false);
-        break;
-    }
-    case SBSTYLE_3D_TOP_DETACHED_TAB: {
-        Clr color_to_use = m_checked ? m_color : DarkColor(m_color);
-        color_to_use = Disabled() ? DisabledColor(color_to_use) : color_to_use;
-        if (!m_checked) {
-            cl_ul.y += BEVEL;
-            additional_text_offset.y = Y(BEVEL / 2);
-        }
-        BeveledRectangle(cl_ul, cl_lr,
-                         color_to_use, color_to_use,
-                         true, BEVEL);
-        break;
-    }
-    }
-
-    m_label->OffsetMove(m_text_ul + additional_text_offset);
-    m_label->Render();
-    m_label->OffsetMove(-(m_text_ul + additional_text_offset));
+    if (m_representer)
+        m_representer->Render(*this);
 }
 
 void StateButton::Show(bool children/* = true*/)
@@ -365,15 +480,22 @@ void StateButton::LClick(const Pt& pt, Flags<ModKey> mod_keys)
 {
     if (!Disabled()) {
         SetCheck(!m_checked);
+        if (m_representer)
+            m_representer->OnChecked(m_checked);
         CheckedSignal(m_checked);
     }
 }
 
+void StateButton::MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
+{ m_mouseover = true; }
+
+void StateButton::MouseLeave()
+{ m_mouseover = false; }
+
 void StateButton::SizeMove(const Pt& ul, const Pt& lr)
 {
     Control::SizeMove(ul, lr);
     m_label->Resize(Size());
-    RepositionButton();
 }
 
 void StateButton::Reset()
@@ -382,108 +504,15 @@ void StateButton::Reset()
 void StateButton::SetCheck(bool b/* = true*/)
 { m_checked = b; }
 
-void StateButton::RepositionButton()
-{
-    if (m_style == SBSTYLE_3D_TOP_ATTACHED_TAB ||
-        m_style == SBSTYLE_3D_TOP_DETACHED_TAB) {
-        m_button_ul = Pt();
-        m_button_lr = Pt();
-        m_text_ul = Pt();
-    } else {
-        X w = Width();
-        Y h = Height();
-        const X BN_W = m_button_lr.x - m_button_ul.x;
-        const Y BN_H = m_button_lr.y - m_button_ul.y;
-        X bn_x = m_button_ul.x;
-        Y bn_y = m_button_ul.y;
-        Flags<TextFormat> format = m_label->GetTextFormat();
-        Flags<TextFormat> original_format = format;
-        const double SPACING = 0.5; // the space to leave between the button and text, as a factor of the button's size (width or height)
-        if (format & FORMAT_VCENTER)       // center button vertically
-            bn_y = (h - BN_H) / 2.0 + 0.5;
-        if (format & FORMAT_TOP) {         // put button at top, text just below
-            bn_y = Y0;
-            m_text_ul.y = BN_H;
-        }
-        if (format & FORMAT_BOTTOM) {      // put button at bottom, text just above
-            bn_y = (h - BN_H);
-            m_text_ul.y = h - (BN_H * (1 + SPACING)) - (static_cast<int>(m_label->GetLineData().size() - 1) * m_label->GetFont()->Lineskip() + m_label->GetFont()->Height()) + 0.5;
-        }
-
-        if (format & FORMAT_CENTER) {      // center button horizontally
-            if (format & FORMAT_VCENTER) { // if both the button and the text are to be centered, bad things happen
-                format |= FORMAT_LEFT;     // so go to the default (FORMAT_CENTER|FORMAT_LEFT)
-                format &= ~FORMAT_CENTER;
-            } else {
-                bn_x = (w - bn_x) / 2.0 - BN_W / 2.0 + 0.5;
-            }
-        }
-        if (format & FORMAT_LEFT) {        // put button at left, text just to the right
-            bn_x = X0;
-            if (format & FORMAT_VCENTER)
-                m_text_ul.x = BN_W * (1 + SPACING) + 0.5;
-        }
-        if (format & FORMAT_RIGHT) {       // put button at right, text just to the left
-            bn_x = (w - BN_W);
-            if (format & FORMAT_VCENTER)
-                m_text_ul.x = -BN_W * (1 + SPACING) + 0.5;
-        }
-        if (format != original_format)
-            m_label->SetTextFormat(format);
-        m_button_ul = Pt(bn_x, bn_y);
-        m_button_lr = m_button_ul + Pt(BN_W, BN_H);
-    }
-}
-
-void StateButton::SetButtonPosition(const Pt& ul, const Pt& lr)
-{
-    X bn_x = ul.x;
-    Y bn_y = ul.y;
-    X bn_w = lr.x - ul.x;
-    Y bn_h = lr.y - ul.y;
-
-    if (bn_w <= 0 || bn_h <= 0) {              // if one of these is invalid,
-        bn_w = X(m_label->GetFont()->PointSize()); // set button width and height to text height
-        bn_h = Y(m_label->GetFont()->PointSize());
-    }
-
-    if (bn_x == -1 || bn_y == -1) {
-        m_button_ul = Pt(X0, Y0);
-        m_button_lr = Pt(bn_w, bn_h);
-        RepositionButton();
-    } else {
-        m_button_ul = Pt(bn_x, bn_y);
-        m_button_lr = m_button_ul + Pt(bn_w, bn_h);
-    }
-}
-
-void StateButton::SetDefaultButtonPosition()
-{ SetButtonPosition(Pt(-X1, -Y1), Pt(-X1, -Y1)); }
-
 void StateButton::SetColor(Clr c)
 { Control::SetColor(c); }
 
-void StateButton::SetInteriorColor(Clr c)
-{ m_int_color = c; }
-
 void StateButton::SetTextColor(Clr c)
 { m_label->SetTextColor(c); }
 
-void StateButton::SetStyle(StateButtonStyle bs)
-{ m_style = bs; }
-
 TextControl* StateButton::GetLabel() const
 { return m_label; }
 
-Pt StateButton::ButtonUpperLeft() const
-{ return m_button_ul; }
-
-Pt StateButton::ButtonLowerRight() const
-{ return m_button_lr; }
-
-Pt StateButton::TextUpperLeft() const
-{ return m_text_ul; }
-
 
 ////////////////////////////////////////////////
 // GG::RadioButtonGroup
