@@ -37,7 +37,7 @@ namespace {
     {
         EditedEcho(const std::string& name) : m_name(name) {}
         void operator()(const std::string& str)
-        { std::cerr << "GG SIGNAL : " << m_name << "(str=" << str << ")\n"; }
+        { std::cerr << "GG SIGNAL : " << m_name << "(str=" << str << ")" << std::endl; }
         std::string m_name;
     };
 
@@ -83,6 +83,9 @@ Edit::Edit(const std::string& str, const boost::shared_ptr<Font>& font, Clr colo
 Pt Edit::MinUsableSize() const
 { return Pt(X(4 * PIXEL_MARGIN), HeightFromFont(GetFont(), PIXEL_MARGIN)); }
 
+Pt Edit::MinUsableSize(X width) const
+{ return MinUsableSize(); }
+
 Pt Edit::ClientUpperLeft() const
 { return UpperLeft() + Pt(X(PIXEL_MARGIN), Y(PIXEL_MARGIN)); }
 
@@ -119,19 +122,25 @@ void Edit::Render()
 
     BeginScissorClipping(Pt(client_ul.x - 1, client_ul.y), client_lr);
 
+    if (GetLineData().empty())
+        return;
     const std::vector<Font::LineData::CharData>& char_data = GetLineData()[0].char_data;
+
     X first_char_offset = FirstCharOffset();
     Y text_y_pos(ul.y + ((lr.y - ul.y) - GetFont()->Height()) / 2.0 + 0.5);
     CPSize last_visible_char = LastVisibleChar();
     const StrSize INDEX_0 = StringIndexOf(0, m_first_char_shown, GetLineData());
     const StrSize INDEX_END = StringIndexOf(0, last_visible_char, GetLineData());
-    if (MultiSelected())   { // if one or more chars are selected, hilite, then draw the range in the selected-text color
-        CPSize low_cursor_pos  = std::min(m_cursor_pos.first, m_cursor_pos.second);
-        CPSize high_cursor_pos = std::max(m_cursor_pos.first, m_cursor_pos.second);
+
+    if (MultiSelected()) {
+        // if one or more chars are selected, hilite, then draw the range in
+        // the selected-text color
+        CPSize low_cursor_pos  = std::min(CPSize(char_data.size()), std::max(CP0, std::min(m_cursor_pos.first, m_cursor_pos.second)));
+        CPSize high_cursor_pos = std::min(CPSize(char_data.size()), std::max(CP0, std::max(m_cursor_pos.first, m_cursor_pos.second)));
 
         // draw hiliting
         Pt hilite_ul(client_ul.x + (low_cursor_pos < 1 ? X0 : char_data[Value(low_cursor_pos - 1)].extent) - first_char_offset, client_ul.y);
-        Pt hilite_lr(client_ul.x + char_data[Value(high_cursor_pos - 1)].extent - first_char_offset, client_lr.y);
+        Pt hilite_lr(client_ul.x + (high_cursor_pos < 1 ? X0 : char_data[Value(high_cursor_pos - 1)].extent) - first_char_offset, client_lr.y);
         FlatRectangle(hilite_ul, hilite_lr, hilite_color_to_use, CLR_ZERO, 0);
 
         // INDEX_0 to INDEX_1 is unhilited, INDEX_1 to
@@ -141,7 +150,7 @@ void Edit::Render()
         const StrSize INDEX_2 = StringIndexOf(0, std::min(high_cursor_pos, last_visible_char), GetLineData());
 
         // draw text
-        X text_x_pos = ul.x + PIXEL_MARGIN;
+        X text_x_pos = client_ul.x;
         glColor(text_color_to_use);
 
         // TODO: Use subrange RenderTex()
@@ -157,17 +166,13 @@ void Edit::Render()
         text_x_pos +=
             GetFont()->RenderText(Pt(text_x_pos, text_y_pos),
                                   Text().substr(Value(INDEX_2), Value(INDEX_END - INDEX_2)));
+
     } else { // no selected text
         glColor(text_color_to_use);
         GetFont()->RenderText(Pt(client_ul.x, text_y_pos), Text().substr(Value(INDEX_0), Value(INDEX_END - INDEX_0)));
         if (GUI::GetGUI()->FocusWnd() == this) { // if we have focus, draw the caret as a simple vertical line
             X caret_x = ScreenPosOfChar(m_cursor_pos.second);
-            glDisable(GL_TEXTURE_2D);
-            glBegin(GL_LINES);
-            glVertex(caret_x, client_ul.y);
-            glVertex(caret_x, client_lr.y);
-            glEnd();
-            glEnable(GL_TEXTURE_2D);
+            Line(caret_x, client_ul.y, caret_x, client_lr.y);
         }
     }
 
@@ -202,6 +207,7 @@ void Edit::DeselectAll()
 
 void Edit::SelectRange(CPSize from, CPSize to)
 {
+    //std::cout << "Selecting range from: " << from << "  to: " << to << std::endl << std::flush;
     if (from < to) {
         m_cursor_pos.first = std::max(CP0, from);
         m_cursor_pos.second = std::min(to, Length());
@@ -292,16 +298,33 @@ X Edit::FirstCharOffset() const
 X Edit::ScreenPosOfChar(CPSize idx) const
 {
     X first_char_offset = FirstCharOffset();
-    return Left() + PIXEL_MARGIN + ((idx ? GetLineData()[0].char_data[Value(idx - 1)].extent : X0) - first_char_offset);
+    return ClientUpperLeft().x + ((idx ? GetLineData()[0].char_data[Value(idx - 1)].extent : X0) - first_char_offset);
 }
 
 CPSize Edit::LastVisibleChar() const
 {
     X first_char_offset = FirstCharOffset();
     CPSize retval = m_first_char_shown;
-    for ( ; retval < Length(); ++retval) {
-        if (Size().x - 2 * PIXEL_MARGIN <= (retval ? GetLineData()[0].char_data[Value(retval - 1)].extent : X0) - first_char_offset)
-            break;
+
+    const std::vector<Font::LineData>& line_data = GetLineData();
+    if (line_data.empty())
+        return CP0;
+    const Font::LineData& first_line_data = line_data.at(0);
+    const std::vector<Font::LineData::CharData>& char_data = first_line_data.char_data;
+
+    CPSize line_limit = std::min(Length(), CPSize(char_data.size()));
+    X client_size_x = ClientSize().x;
+
+    for ( ; retval < line_limit; ++retval) {
+        if (!retval) {
+            if (client_size_x <= X0 - first_char_offset)
+                break;
+        } else {
+            size_t retval_minus_1 = Value(retval - 1);
+            Font::LineData::CharData retval_minus_1_char_data = char_data.at(retval_minus_1);
+            if (client_size_x <= retval_minus_1_char_data.extent - first_char_offset)
+                break;
+        }
     }
     return retval;
 }
@@ -320,7 +343,7 @@ void Edit::LButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
     if (Disabled())
         return;
     //std::cout << "Edit::LButtonDown start" << std::endl;
-    X click_xpos = ScreenToWindow(pt).x - PIXEL_MARGIN; // x coord of click within text space
+    X click_xpos = ScreenToClient(pt).x; // x coord of click within text space
     CPSize idx = CharIndexOf(click_xpos);
     //std::cout << "Edit::LButtonDown got idx: " << idx << std::endl;
     m_cursor_pos.first = m_cursor_pos.second = idx;
@@ -335,11 +358,13 @@ void Edit::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
     if (Disabled())
         return;
 
-    X xpos = ScreenToWindow(pt).x - PIXEL_MARGIN; // x coord for mouse position within text space
+    X xpos = ScreenToClient(pt).x; // x coord for mouse position within text space
     CPSize idx = CharIndexOf(xpos);
+    //std::cout << "CharIndexOf mouse x-pos: " << xpos << std::endl << std::flush;
+
     if (m_in_double_click_mode) {
-        std::pair<CPSize, CPSize> word_indices =
-            GetDoubleButtonDownDragWordIndices(idx);
+        std::pair<CPSize, CPSize> word_indices = GetDoubleButtonDownDragWordIndices(idx);
+
         if (word_indices.first == word_indices.second) {
             if (idx < m_double_click_cursor_pos.first) {
                 m_cursor_pos.second = idx;
@@ -362,9 +387,11 @@ void Edit::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
     } else {
         // when a single-click drag occurs, move m_cursor_pos.second to where the mouse is, which selects a range of characters
         m_cursor_pos.second = idx;
-        if (xpos < 0 || Size().x - 2 * PIXEL_MARGIN < xpos) // if we're dragging past the currently visible text
+        if (xpos < 0 || ClientSize().x < xpos) // if we're dragging past the currently visible text
             AdjustView();
     }
+
+    //std::cout << "LDrag selected from: " << m_cursor_pos.first << "  to: " << m_cursor_pos.second << std::endl << std::flush;
 }
 
 void Edit::LButtonUp(const Pt& pt, Flags<ModKey> mod_keys)
@@ -381,9 +408,11 @@ void Edit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_k
     }
 
     bool shift_down = mod_keys & (MOD_KEY_LSHIFT | MOD_KEY_RSHIFT);
+    bool ctrl_down = mod_keys & (MOD_KEY_CTRL | MOD_KEY_RCTRL);
     bool emit_signal = false;
     bool numlock_on = mod_keys & MOD_KEY_NUM;
 
+
     if (!numlock_on) {
         // convert keypad keys into corresponding non-number keys
         switch (key) {
@@ -412,12 +441,20 @@ void Edit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_k
         break;
     case GGK_LEFT:
         if (MultiSelected() && !shift_down) {
-            m_cursor_pos.second = m_cursor_pos.first = std::min(m_cursor_pos.first, m_cursor_pos.second);
+            if (!ctrl_down)
+                m_cursor_pos.second = m_cursor_pos.first = std::min(m_cursor_pos.first, m_cursor_pos.second);
+            else
+                m_cursor_pos.second = m_cursor_pos.first = NextWordEdgeFrom(Text(), m_cursor_pos.second, false);
+
         } else if (0 < m_cursor_pos.second) {
-            --m_cursor_pos.second;
-            X extent = GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent;
-            while (0 < m_cursor_pos.second && extent == GetLineData()[0].char_data[Value(m_cursor_pos.second - 1)].extent)
+            if (!ctrl_down) {
                 --m_cursor_pos.second;
+                X extent = GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent;
+                while (0 < m_cursor_pos.second && extent == GetLineData()[0].char_data[Value(m_cursor_pos.second - 1)].extent)
+                    --m_cursor_pos.second;
+            } else {
+                m_cursor_pos.second = NextWordEdgeFrom(Text(), m_cursor_pos.second, false);
+            }
             if (!shift_down)
                 m_cursor_pos.first = m_cursor_pos.second;
         }
@@ -425,11 +462,19 @@ void Edit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_k
         break;
     case GGK_RIGHT:
         if (MultiSelected() && !shift_down) {
-            m_cursor_pos.second = m_cursor_pos.first = std::max(m_cursor_pos.first, m_cursor_pos.second);
+            if (!ctrl_down)
+                m_cursor_pos.second = m_cursor_pos.first = std::max(m_cursor_pos.first, m_cursor_pos.second);
+            else
+                m_cursor_pos.second = m_cursor_pos.first = NextWordEdgeFrom(Text(), m_cursor_pos.second, true);
+
         } else if (m_cursor_pos.second < Length()) {
-            X extent = GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent;
-            while (m_cursor_pos.second < Length() && extent == GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent)
-                ++m_cursor_pos.second;
+            if (!ctrl_down) {
+                X extent = GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent;
+                while (m_cursor_pos.second < Length() && extent == GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent)
+                    ++m_cursor_pos.second;
+            } else {
+                m_cursor_pos.second = NextWordEdgeFrom(Text(), m_cursor_pos.second, true);
+            }
             if (!shift_down)
                 m_cursor_pos.first = m_cursor_pos.second;
         }
@@ -543,7 +588,7 @@ void Edit::ClearSelected()
     Erase(0, low, high - low);
 
     // make sure deletion has not left m_first_char_shown in an out-of-bounds position
-    if (GetLineData()[0].char_data.empty())
+    if (GetLineData().empty() || GetLineData()[0].char_data.empty())
         m_first_char_shown = CP0;
     else if (GetLineData()[0].char_data.size() <= m_first_char_shown)
         m_first_char_shown = CodePointIndexOf(0, INVALID_CP_SIZE, GetLineData());
@@ -551,7 +596,7 @@ void Edit::ClearSelected()
 
 void Edit::AdjustView()
 {
-    X text_space = Size().x - 2 * PIXEL_MARGIN;
+    X text_space = ClientSize().x;
     X first_char_offset = FirstCharOffset();
     if (m_cursor_pos.second < m_first_char_shown) { // if the caret is at a place left of the current visible area
         if (m_first_char_shown - m_cursor_pos.second < 5) // if the caret is less than five characters before m_first_char_shown
@@ -579,9 +624,7 @@ void Edit::AdjustView()
 ////////////////////////////////////////////////////////////
 // Free Functions
 ////////////////////////////////////////////////////////////
-void GG::GetTranslatedCodePoint(Key key,
-                                boost::uint32_t key_code_point,
-                                Flags<ModKey> mod_keys,
+void GG::GetTranslatedCodePoint(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys,
                                 std::string& translated_code_point)
 {
     // only process it if it's a valid code point or a known printable
@@ -601,3 +644,93 @@ void GG::GetTranslatedCodePoint(Key key,
             translated_code_point = key;
     }
 }
+
+CPSize GG::NextWordEdgeFrom(const std::string& text, CPSize from_position, bool search_right) {
+    std::set<std::pair<CPSize, CPSize> > words = GUI::GetGUI()->FindWords(text);
+    CPSize retval = CP0;
+
+    if (!search_right) {
+        // searching ON THE LEFT from the reference position
+
+        // start with the leftmost word, traverse words to the right
+        // until past the reference point
+        for (std::set<std::pair<CPSize, CPSize> >::const_iterator it = words.begin();
+            it != words.end(); ++it)
+        {
+            if (it->first > from_position) {
+                // found word is after of the position. can stop
+                // searching and use whatever the last found word's position was
+                break;
+
+            } else if (it->first < from_position && it->second >= from_position) {
+                // found word starting before and ending at/after the position. can
+                // stop searching and use the start of the found word.
+                retval = it->first;
+                break;
+
+            } else if (it->second < from_position) {
+                // found word ending before the position. can use the start
+                // or end of the found word...
+                if (it->second < from_position - 1) {
+                    // there is a gap between the end of the word and the search
+                    // reference position. use one past the end of the word
+                    retval = it->second + 1;
+                    // don't break, as there might be later words that are closer to
+                    // the search reference position
+                } else {
+                    // the end of the word is immediately before the search
+                    // reference position. use the start of the word.
+                    retval = it->first;
+                    // can stop searching since the word is right next to the
+                    // search reference position
+                    break;
+                }
+            }
+        }
+        return retval;
+
+    } else {
+        // searching ON THE RIGHT from the reference position
+        if (!words.empty())
+            retval = std::max(from_position, words.rbegin()->second);
+
+        // start and the rightmost end, traverse the words leftwards
+        // until past the reference point
+        for (std::set<std::pair<CPSize, CPSize> >::const_reverse_iterator rit = words.rbegin();
+            rit != words.rend(); ++rit)
+        {
+            if (rit->second < from_position) {
+                // found word is before the position. can stop
+                // searching and use whatever the last found word's position was
+                break;
+
+            } else if (rit->first <= from_position && rit->second > from_position) {
+                // found word starting before/at and ending after the position. can
+                // stop searching and use the end of the found word.
+                retval = rit->second;
+                break;
+
+            } else if (rit->first > from_position) {
+                // found word starting after the position. can use the start
+                // or end of the found word...
+                if (rit->first > from_position + 1) {
+                    // there is a gap between the end of the word and the search
+                    // reference position. use one before the start of the word
+                    retval = rit->first - 1;
+                } else {
+                    // the start of the word is immediately after the search
+                    // reference position. use the end of the word.
+                    retval = rit->second;
+                    // can stop searching since the word is right next to the
+                    // search reference position
+                    break;
+                }
+                // don't break, as there might be later words that are closer to
+                // the search reference position
+            }
+        }
+
+        return retval;
+    }
+}
+
