@@ -633,10 +633,10 @@ bool Font::RenderState::ColorsEmpty() const
 
 // Must be here for scoped_ptr deleter to work
 Font::RenderCache::RenderCache() :
-    vertices(new GG::GLPtBuffer()),
+    vertices(new GG::GL2DVertexBuffer()),
     coordinates(new GG::GLTexCoordBuffer()),
     colors(new GG::GLRGBAColorBuffer()),
-    underline_vertices(new GG::GLPtBuffer()),
+    underline_vertices(new GG::GL2DVertexBuffer()),
     underline_colors(new GG::GLRGBAColorBuffer())
 {}
 
@@ -684,12 +684,46 @@ Font::Glyph::Glyph(const boost::shared_ptr<Texture>& texture, const Pt& ul, cons
     width(ul.x - lr.x)
 {}
 
+namespace {
+    // Global set of tags requiring action fron Font. Wrapped in a function for deterministic static initialization order.
+    std::set<std::string>& ActionTags()
+    {
+        static std::set<std::string> action_tags;
+        return action_tags;
+    }
+
+    // Global set of known tags. Wrapped in a function for deterministic static initialization order.
+    std::set<std::string>& KnownTags()
+    {
+        static std::set<std::string> known_tags;
+        return known_tags;
+    }
+
+    // Registers the default action and known tags.
+    int RegisterDefaultTags() {
+        ActionTags().insert("i");
+        ActionTags().insert("s");
+        ActionTags().insert("u");
+        ActionTags().insert("rgba");
+        ActionTags().insert(ALIGN_LEFT_TAG);
+        ActionTags().insert(ALIGN_CENTER_TAG);
+        ActionTags().insert(ALIGN_RIGHT_TAG);
+        ActionTags().insert(PRE_TAG);
+
+        // Always know the action tags.
+        KnownTags().insert(ActionTags().begin(), ActionTags().end());
+
+        // Must have return value to call at static initialization time.
+        return 0;
+    }
+
+    // Register the default tags at static initialization time.
+    int register_tags_dummy = RegisterDefaultTags();
+}
 
 ///////////////////////////////////////
 // class GG::Font
 ///////////////////////////////////////
-std::set<std::string> Font::s_action_tags;
-std::set<std::string> Font::s_known_tags;
 
 Font::Font() :
     m_pt_sz(0),
@@ -846,7 +880,7 @@ void Font::PreRenderText(const Pt& ul, const Pt& lr, const std::string& text, Fl
     }
 
     PreRenderText(ul, lr, text, format, *line_data, *render_state,
-                  0, CP0, line_data->size(), CPSize(line_data->back().char_data.size()), cache);
+                  0, CP0, line_data->size(), line_data->empty() ? CP0 : CPSize(line_data->back().char_data.size()), cache);
 }
 
 void Font::PreRenderText(const Pt& ul, const Pt& lr, const std::string& text, Flags<TextFormat>& format,
@@ -908,12 +942,9 @@ void Font::PreRenderText(const Pt& ul, const Pt& lr, const std::string& text, Fl
 
 void Font::RenderCachedText(RenderCache& cache) const
 {
-    double orig_color[4];
-    glGetDoublev(GL_CURRENT_COLOR, orig_color);
-
-
-    //RenderStoredGlyph(it->second, running_index, *cache.vertices, *cache.coordinates);
     glBindTexture(GL_TEXTURE_2D, m_texture->OpenGLId());
+
+    glPushClientAttrib(GL_CLIENT_ALL_ATTRIB_BITS);
     glEnableClientState(GL_VERTEX_ARRAY);
     glEnableClientState(GL_TEXTURE_COORD_ARRAY);
     glEnableClientState(GL_COLOR_ARRAY);
@@ -930,9 +961,7 @@ void Font::RenderCachedText(RenderCache& cache) const
     cache.underline_colors->activate();
     glDrawArrays(GL_QUADS, 0, cache.underline_vertices->size());
 
-    glDisableClientState(GL_VERTEX_ARRAY);
-    glDisableClientState(GL_COLOR_ARRAY);
-    glColor4dv(orig_color);
+    glPopClientAttrib();
 }
 
 void Font::ProcessTagsBefore(const std::vector<LineData>& line_data, RenderState& render_state,
@@ -977,6 +1006,56 @@ Pt Font::DetermineLines(const std::string& text, Flags<TextFormat>& format, X bo
                               const_cast<std::vector<boost::shared_ptr<TextElement> >*>(&text_elements));
 }
 
+std::string Font::StripTags(const std::string& text, bool strip_unpaired_tags)
+{
+    using namespace boost::xpressive;
+
+    bool temp_bool = false;
+    std::stack<Substring> tag_stack;
+    MatchesKnownTag matches_known_tag(KnownTags(), temp_bool);
+    MatchesTopOfStack matches_tag_stack(tag_stack, temp_bool);
+
+    mark_tag tag_name_tag(1);
+    mark_tag open_bracket_tag(2);
+    mark_tag close_bracket_tag(3);
+    mark_tag printable_text_tag(4);
+
+    const sregex TAG_PARAM =
+    -+~set[_s | '<'];
+    const sregex OPEN_TAG_NAME =
+        (+_w)[check(matches_known_tag)];
+    const sregex CLOSE_TAG_NAME =
+        (+_w)[check(matches_tag_stack)];
+    const sregex WHITESPACE =
+        (*blank >> (_ln | (set = '\n', '\r', '\f'))) | +blank;
+    const sregex TEXT =
+        ('<' >> *~set[_s | '<']) | (+~set[_s | '<']);
+    const sregex PRINTABLE_TEXT = WHITESPACE | TEXT;
+    sregex EVERYTHING;
+    if (!strip_unpaired_tags)
+        EVERYTHING =    // push open tag matches to the tag stack, and make sure close tags match the top open tag on the stack
+            ('<' >> (tag_name_tag = OPEN_TAG_NAME) >> repeat<0, 9>(+blank >> TAG_PARAM) >> (open_bracket_tag.proto_base() = '>'))
+            [Push(boost::xpressive::ref(text), boost::xpressive::ref(tag_stack), ref(temp_bool), tag_name_tag)] |
+            ("</" >> (tag_name_tag = CLOSE_TAG_NAME) >> (close_bracket_tag.proto_base() = '>')) |
+            (printable_text_tag = PRINTABLE_TEXT);
+    else
+        EVERYTHING =    // don't care about matching with tag stack when matching close tags, or updating the stack when matching open tags
+            ('<' >> OPEN_TAG_NAME >> repeat<0, 9>(+blank >> TAG_PARAM) >> '>') |
+            ("</" >> OPEN_TAG_NAME >> '>') |
+            (printable_text_tag = PRINTABLE_TEXT);
+
+    std::string retval;
+
+    // scan through matched markup and text, saving only the non-tag-text
+    sregex_iterator it(text.begin(), text.end(), EVERYTHING);
+    sregex_iterator end_it;
+    for (; it != end_it; ++it) {
+        retval += Substring(text, (*it)[printable_text_tag]);
+    }
+
+    return retval;
+}
+
 Pt Font::TextExtent(const std::string& text, Flags<TextFormat> format/* = FORMAT_NONE*/, X box_width/* = X0*/) const
 {
     std::vector<LineData> lines;
@@ -995,26 +1074,21 @@ Pt Font::TextExtent(const std::string& text, const std::vector<LineData>& line_d
 }
 
 void Font::RegisterKnownTag(const std::string& tag)
-{ s_known_tags.insert(tag); }
+{ KnownTags().insert(tag); }
 
 void Font::RemoveKnownTag(const std::string& tag)
 {
-    if (s_action_tags.find(tag) == s_action_tags.end())
-        s_known_tags.erase(tag);
+    if (KnownTags().find(tag) == KnownTags().end())
+        KnownTags().erase(tag);
 }
 
 void Font::ClearKnownTags()
 {
-    s_action_tags.clear();
-    s_action_tags.insert("i");
-    s_action_tags.insert("s");
-    s_action_tags.insert("u");
-    s_action_tags.insert("rgba");
-    s_action_tags.insert(ALIGN_LEFT_TAG);
-    s_action_tags.insert(ALIGN_CENTER_TAG);
-    s_action_tags.insert(ALIGN_RIGHT_TAG);
-    s_action_tags.insert(PRE_TAG);
-    s_known_tags = s_action_tags;
+    // Clear known tags.
+    KnownTags().clear();
+
+    // Always know the default tags.
+    RegisterDefaultTags();
 }
 
 void Font::ThrowBadGlyph(const std::string& format_str, boost::uint32_t c)
@@ -1047,7 +1121,7 @@ Pt Font::DetermineLinesImpl(const std::string& text,
 
         std::stack<Substring> tag_stack;
         bool ignore_tags = format & FORMAT_IGNORETAGS;
-        MatchesKnownTag matches_known_tag(s_known_tags, ignore_tags);
+        MatchesKnownTag matches_known_tag(KnownTags(), ignore_tags);
         MatchesTopOfStack matches_tag_stack(tag_stack, ignore_tags);
 
         mark_tag tag_name_tag(1);
@@ -1437,9 +1511,6 @@ void Font::CheckFace(FT_Face face, FT_Error error)
 
 void Font::Init(FT_Face& face)
 {
-    if (s_action_tags.empty()) // if this is the first Font to get initialized, it needs to initialize some static members
-        ClearKnownTags();
-
     FT_Fixed scale;
 
     if (!m_pt_sz) {
@@ -1487,8 +1558,9 @@ void Font::Init(FT_Face& face)
     }
 
     //Get maximum texture size
-    GLint TEX_MAX_SIZE;
-    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &TEX_MAX_SIZE);
+    GLint GL_TEX_MAX_SIZE;
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &GL_TEX_MAX_SIZE);
+    const std::size_t TEX_MAX_SIZE = GL_TEX_MAX_SIZE;
 
     std::map<boost::uint32_t, TempGlyphData> temp_glyph_data;
 
@@ -1524,7 +1596,7 @@ void Font::Init(FT_Face& face)
                     // We cannot make the texture any larger. The font does not fit.
                     ThrowBadGlyph("GG::Font::Init : Face too large for buffer. First glyph to no longer fit: '%1%'", c);
                 }
-                if (Value(y) + glyph_bitmap.rows > Value(max_y)) {
+                if (y + Y(glyph_bitmap.rows) > max_y) {
                     max_y = y + Y(glyph_bitmap.rows + 1); //Leave a one pixel gap between glyphs
                 }
 
@@ -1532,11 +1604,11 @@ void Font::Init(FT_Face& face)
                 // Resize buffer to fit new data
                 buffer.at(x + X(glyph_bitmap.width), y + Y(glyph_bitmap.rows)) = 0;
 
-                for (int row = 0; row < glyph_bitmap.rows; ++row) {
+                for (unsigned int row = 0; row < glyph_bitmap.rows; ++row) {
                     boost::uint8_t*  src = src_start + row * glyph_bitmap.pitch;
-                    boost::uint16_t* dst = &buffer.get(x, Y(y + row));
+                    boost::uint16_t* dst = &buffer.get(x, y + Y(row));
                     // Rows are always contiguous, so we can copy along a row using simple incrementation
-                    for (int col = 0; col < glyph_bitmap.width; ++col) {
+                    for (unsigned int col = 0; col < glyph_bitmap.width; ++col) {
 #ifdef __BIG_ENDIAN__
                         *dst++ = *src++ | (255 << 8); // big-endian uses different byte ordering
 #else
@@ -1586,8 +1658,13 @@ bool Font::GenerateGlyph(FT_Face face, boost::uint32_t ch)
     using boost::lexical_cast;
     FT_UInt index = FT_Get_Char_Index(face, ch);
     if (index) {
-        if (FT_Load_Glyph(face, index, FT_LOAD_DEFAULT))
-            ThrowBadGlyph("GG::Font::GetGlyphBitmap : Freetype could not load the glyph for character '%1%'", ch);
+        if (FT_Load_Glyph(face, index, FT_LOAD_DEFAULT)) {
+            // loading of a glpyh failed so we replace it with
+            // the 'Replacement Character' at codepoint 0xFFFD
+            FT_UInt tmp_index = FT_Get_Char_Index(face, 0xFFFD);
+            if (FT_Load_Glyph(face, tmp_index, FT_LOAD_DEFAULT))
+                ThrowBadGlyph("GG::Font::GetGlyphBitmap : Freetype could not load the glyph for character '%1%'", ch);
+        }
 
         FT_GlyphSlot glyph = face->glyph;
 
@@ -1741,7 +1818,8 @@ void Font::HandleTag(const boost::shared_ptr<FormattingTag>& tag, double* orig_c
                     if (0 <= temp_color[0] && temp_color[0] <= 255 &&
                         0 <= temp_color[1] && temp_color[1] <= 255 &&
                         0 <= temp_color[2] && temp_color[2] <= 255 &&
-                        0 <= temp_color[3] && temp_color[3] <= 255) {
+                        0 <= temp_color[3] && temp_color[3] <= 255)
+                    {
                         color[0] = temp_color[0];
                         color[1] = temp_color[1];
                         color[2] = temp_color[2];
@@ -1761,7 +1839,8 @@ void Font::HandleTag(const boost::shared_ptr<FormattingTag>& tag, double* orig_c
                         if (0.0 <= color[0] && color[0] <= 1.0 &&
                             0.0 <= color[1] && color[1] <= 1.0 &&
                             0.0 <= color[2] && color[2] <= 1.0 &&
-                            0.0 <= color[3] && color[3] <= 1.0) {
+                            0.0 <= color[3] && color[3] <= 1.0)
+                        {
                             glColor4dv(color);
                             render_state.PushColor(color[0], color[1], color[2], color[3]);
                         } else {
