@@ -49,6 +49,7 @@
 
 #include <boost/format.hpp>
 #include <boost/thread.hpp>
+#include <boost/algorithm/string.hpp>
 #include <boost/xpressive/xpressive.hpp>
 
 #include <cassert>
@@ -69,13 +70,13 @@ namespace {
                   boost::lexical_cast<std::string>(key) +
                   " mod_keys=" +
                   boost::lexical_cast<std::string>(mod_keys) +
-                  ")\n")
-            {}
+                  ")")
+        {}
         bool operator()()
-            {
-                std::cerr << m_str;
-                return false;
-            }
+        {
+            std::cerr << m_str << std::endl;
+            return false;
+        }
         std::string m_str;
     };
 
@@ -165,6 +166,7 @@ struct GG::GUIImpl
     Wnd*         m_focus_wnd;             // GUI window that currently has the input focus (this is the base level focus window, used when no modal windows are active)
     std::list<std::pair<Wnd*, Wnd*> >
                  m_modal_wnds;            // modal GUI windows, and the window with focus for that modality (only the one in back is active, simulating a stack but allowing traversal of the list)
+    bool         m_allow_modal_accelerator_signals; // iff true: keyboard accelerator signals will be output while modal window(s) is open
 
     bool         m_mouse_button_state[3]; // the up/down states of the three buttons on the mouse are kept here
     Pt           m_mouse_pos;             // absolute position of mouse, based on last MOUSEMOVE event
@@ -194,8 +196,8 @@ struct GG::GUIImpl
     Wnd*         m_drag_wnds[3];          // GUI window currently being clicked or dragged by each mouse button
     Pt           m_prev_wnd_drag_position;// the upper-left corner of the dragged window when the last *Drag message was generated
     Pt           m_wnd_drag_offset;       // the offset from the upper left corner of the dragged window to the cursor for the current drag
-    bool         m_curr_drag_wnd_dragged; // true iff the currently-pressed window (m_drag_wnds[N]) has actually been dragged some distance (in which case releasing the mouse button is not a click)
-    Wnd*         m_curr_drag_wnd;         // nonzero iff m_curr_drag_wnd_dragged is true (that is, we have actually started dragging the Wnd, not just pressed the mouse button); will always be one of m_drag_wnds
+    bool         m_curr_drag_wnd_dragged; // true iff the currently-pressed window (m_drag_wnds[N]) has actually been dragged some distance (in which case releasing the mouse button is not a click). note that a dragged wnd is one being continuously repositioned by the dragging, and not a wnd being drag-dropped.
+    Wnd*         m_curr_drag_wnd;         // nonzero iff m_curr_drag_wnd_dragged is true (that is, we have actually started dragging the Wnd, not just pressed the mouse button); will always be one of m_drag_wnds.
     Wnd*         m_curr_drag_drop_here_wnd;// the Wnd that most recently received a DragDropEnter or DragDropHere message (0 if DragDropLeave was sent as well, or if none)
     Pt           m_wnd_resize_offset;     // offset from the cursor of either the upper-left or lower-right corner of the GUI window currently being resized
     WndRegion    m_wnd_region;            // window region currently being dragged or clicked; for non-frame windows, this will always be WR_NONE
@@ -246,6 +248,7 @@ struct GG::GUIImpl
 
 GUIImpl::GUIImpl() :
     m_focus_wnd(0),
+    m_allow_modal_accelerator_signals(false),
     m_mouse_pos(X(-1000), Y(-1000)),
     m_mouse_rel(X(0), Y(0)),
     m_mod_keys(),
@@ -269,6 +272,7 @@ GUIImpl::GUIImpl() :
     m_curr_drag_wnd(0),
     m_curr_drag_drop_here_wnd(0),
     m_wnd_region(WR_NONE),
+    m_browse_info_mode(0),
     m_browse_target(0),
     m_drag_drop_originating_wnd(0),
     m_mouse_lr_swap(false),
@@ -278,6 +282,7 @@ GUIImpl::GUIImpl() :
     m_calc_FPS(false),
     m_max_FPS(0.0),
     m_double_click_wnd(0),
+    m_double_click_button(0),
     m_double_click_start_time(-1),
     m_double_click_time(-1),
     m_style_factory(new StyleFactory()),
@@ -370,8 +375,7 @@ void GUIImpl::HandleMouseDrag(unsigned int mouse_button, const Pt& pos, int curr
                 // instead signal to dragged-over Wnd that a drag has occurred.
                 Pt start_pos = dragged_wnd->UpperLeft();
                 Pt move = (pos - m_wnd_drag_offset) - m_prev_wnd_drag_position;
-                dragged_wnd->HandleEvent(WndEvent(ButtonEvent(WndEvent::LDrag, mouse_button),
-                                         pos, move, m_mod_keys));
+                dragged_wnd->HandleEvent(WndEvent(ButtonEvent(WndEvent::LDrag, mouse_button), pos, move, m_mod_keys));
 
                 // update "latest" drag position, so future drags will have
                 // proper position from which to judge distance dragged since
@@ -400,19 +404,22 @@ void GUIImpl::HandleMouseDrag(unsigned int mouse_button, const Pt& pos, int curr
             drag_drop_wnds[dragged_wnd] = m_wnd_drag_offset;
 
             if (m_curr_wnd_under_cursor && m_prev_wnd_under_cursor == m_curr_wnd_under_cursor) {
-                if (m_curr_drag_drop_here_wnd) {
-                    assert(m_curr_wnd_under_cursor == m_curr_drag_drop_here_wnd);
-                    m_curr_wnd_under_cursor->HandleEvent(
-                        WndEvent(WndEvent::DragDropHere, pos, m_drag_drop_wnds, m_mod_keys));
-                    m_curr_wnd_under_cursor->DropsAcceptable(m_drag_drop_wnds_acceptable.begin(),
-                                                             m_drag_drop_wnds_acceptable.end(),
-                                                             pos);
+                // Wnd under cursor has remained the same for the last two updates
+                if (m_curr_drag_drop_here_wnd && m_curr_drag_drop_here_wnd == m_curr_wnd_under_cursor) {
+                    // Wnd being dragged over is still being dragged over...
+                    WndEvent event(WndEvent::DragDropHere, pos, m_drag_drop_wnds, m_mod_keys);
+                    m_curr_wnd_under_cursor->HandleEvent(event);
+                    m_drag_drop_wnds_acceptable = event.GetAcceptableDropWnds();
+
                 } else {
-                    m_curr_wnd_under_cursor->HandleEvent(
-                        WndEvent(WndEvent::DragDropEnter, pos, m_drag_drop_wnds, m_mod_keys));
-                    m_curr_wnd_under_cursor->DropsAcceptable(m_drag_drop_wnds_acceptable.begin(),
-                                                             m_drag_drop_wnds_acceptable.end(),
-                                                             pos);
+                    // pass drag-drop event to check if the various dragged Wnds are acceptable to drop
+                    WndEvent event(WndEvent::CheckDrops, pos, m_drag_drop_wnds, m_mod_keys);
+                    m_curr_wnd_under_cursor->HandleEvent(event);
+                    m_drag_drop_wnds_acceptable = event.GetAcceptableDropWnds();
+
+                    // Wnd being dragged over is new; give it an Enter message
+                    WndEvent enter_event(WndEvent::DragDropEnter, pos, m_drag_drop_wnds, m_mod_keys);
+                    m_curr_wnd_under_cursor->HandleEvent(enter_event);
                     m_curr_drag_drop_here_wnd = m_curr_wnd_under_cursor;
                 }
             }
@@ -481,17 +488,23 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
     m_mouse_button_state[mouse_button] = false;
     m_drag_wnds[mouse_button] = 0; // if the mouse button is released, stop the tracking the drag window
     m_wnd_region = WR_NONE;        // and clear this, just in case
-    // if the release is over the Wnd where the button-down event occurred, and that Wnd has not been dragged
+
     if (!in_drag_drop && click_wnd && m_curr_wnd_under_cursor == click_wnd) {
-        // if this is second click over a window that just received an click within
-        // the time limit -- it's a double-click, not a click
+        // the release is over the Wnd where the button-down event occurred
+        // and that Wnd has not been dragged
+
         if (m_double_click_time > 0 && m_double_click_wnd == click_wnd &&
-            m_double_click_button == mouse_button) {
+            // this is second click over a window that just received an click
+            // within the time limit, so it's a double-click, not a click
+            m_double_click_button == mouse_button)
+        {
             m_double_click_wnd = 0;
             m_double_click_start_time = -1;
             m_double_click_time = -1;
             click_wnd->HandleEvent(WndEvent(ButtonEvent(WndEvent::LDoubleClick, mouse_button), pos, m_mod_keys));
+
         } else {
+            // just a single click
             if (m_double_click_time > 0) {
                 m_double_click_wnd = 0;
                 m_double_click_start_time = -1;
@@ -504,56 +517,82 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
             }
             click_wnd->HandleEvent(WndEvent(ButtonEvent(WndEvent::LClick, mouse_button), pos, m_mod_keys));
         }
+
     } else {
+        // drag-dropping
         m_double_click_wnd = 0;
         m_double_click_time = -1;
         if (click_wnd)
             click_wnd->HandleEvent(WndEvent(ButtonEvent(WndEvent::LButtonUp, mouse_button), pos, m_mod_keys));
+
         if (m_curr_wnd_under_cursor) {
+            // dropped onto a Wnd, which can react to the drop
+
             if (m_drag_drop_wnds.empty()) {
+                // dropped a dragged Wnd without having dragged it anywhere yet
                 if (click_wnd && click_wnd->DragDropDataType() != "" && mouse_button == 0) {
+                    // pass drag-drop-here event to check if the single dragged Wnd is acceptable to drop
+                    WndEvent event(WndEvent::CheckDrops, pos, click_wnd, m_mod_keys);
+                    m_curr_wnd_under_cursor->HandleEvent(event);
+                    m_drag_drop_wnds_acceptable = event.GetAcceptableDropWnds();
+
+                    // prep / handle end of drag-drop
                     m_drag_drop_originating_wnd = click_wnd->Parent();
                     m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
                     m_curr_drag_drop_here_wnd = 0;
-                    m_drag_drop_wnds_acceptable[click_wnd] = false;
-                    m_curr_wnd_under_cursor->DropsAcceptable(m_drag_drop_wnds_acceptable.begin(),
-                                                             m_drag_drop_wnds_acceptable.end(),
-                                                             pos);
+
+                    // put dragged Wnd into container depending on whether it is accepted by the drop target
                     std::vector<Wnd*> accepted_wnds;
                     std::vector<const Wnd*> unaccepted_wnds;
                     if (m_drag_drop_wnds_acceptable[click_wnd])
                         accepted_wnds.push_back(click_wnd);
                     else
                         unaccepted_wnds.push_back(click_wnd);
+
+                    // if dragged Wnd came from somehwere, inform originating
+                    // Wnd its child is or is not being dragged away
                     if (m_drag_drop_originating_wnd) {
                         m_drag_drop_originating_wnd->CancellingChildDragDrop(unaccepted_wnds);
                         m_drag_drop_originating_wnd->ChildrenDraggedAway(accepted_wnds, m_curr_wnd_under_cursor);
                     }
+                    // implement drop onto target if the dragged Wnd was accepted
                     if (!accepted_wnds.empty())
-                        m_curr_wnd_under_cursor->AcceptDrops(accepted_wnds, pos);
+                        m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDroppedOn, pos, accepted_wnds, m_mod_keys));
                 }
+
             } else {
+                // dragged one or more Wnds to another location and then dropped them
+                // pass checkdrops event to check if the dropped Wnds are acceptable to drop here
+                WndEvent event(WndEvent::CheckDrops, pos, m_drag_drop_wnds, m_mod_keys);
+                m_curr_wnd_under_cursor->HandleEvent(event);
+                m_drag_drop_wnds_acceptable = event.GetAcceptableDropWnds();
+
+                // prep / handle end of drag-drop
+                m_drag_drop_originating_wnd = click_wnd->Parent();
                 m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
                 m_curr_drag_drop_here_wnd = 0;
-                m_curr_wnd_under_cursor->DropsAcceptable(m_drag_drop_wnds_acceptable.begin(),
-                                                         m_drag_drop_wnds_acceptable.end(),
-                                                         pos);
+
+                // put dragged Wnds into containers depending on whether they were accepted by the drop target
                 std::vector<Wnd*> accepted_wnds;
                 std::vector<const Wnd*> unaccepted_wnds;
                 for (std::map<const Wnd*, bool>::iterator it = m_drag_drop_wnds_acceptable.begin();
-                     it != m_drag_drop_wnds_acceptable.end();
-                     ++it) {
+                     it != m_drag_drop_wnds_acceptable.end(); ++it)
+                {
                     if (it->second)
                         accepted_wnds.push_back(const_cast<Wnd*>(it->first));
                     else
                         unaccepted_wnds.push_back(it->first);
                 }
+                // if dragged Wnds came from somehwere, inform originating
+                 // Wnd its children are or are not being dragged away
                 if (m_drag_drop_originating_wnd) {
                     m_drag_drop_originating_wnd->CancellingChildDragDrop(unaccepted_wnds);
                     m_drag_drop_originating_wnd->ChildrenDraggedAway(accepted_wnds, m_curr_wnd_under_cursor);
                 }
+                // implement drop onto target if any of the dragged Wnds were accepted
                 if (!accepted_wnds.empty())
-                    m_curr_wnd_under_cursor->AcceptDrops(accepted_wnds, pos);
+                    m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDroppedOn, pos,
+                                                                  accepted_wnds, m_mod_keys));
             }
         }
     }
@@ -620,7 +659,7 @@ void GUIImpl::HandleKeyPress(Key key, boost::uint32_t key_code_point, Flags<ModK
     bool processed = false;
     // only process accelerators when there are no modal windows active;
     // otherwise, accelerators would be an end-run around modality
-    if (m_modal_wnds.empty()) {
+    if (m_modal_wnds.empty() || m_allow_modal_accelerator_signals) {
         // the focus_wnd may care about the state of the numlock and
         // capslock, or which side of the keyboard's CTRL, SHIFT, etc.
         // was pressed, but the accelerators don't
@@ -649,13 +688,12 @@ void GUIImpl::HandleKeyRelease(Key key, boost::uint32_t key_code_point, Flags<Mo
             WndEvent::KeyRelease, key, key_code_point, mod_keys));
 }
 
-void GUIImpl::HandleTextInput (const std::string* text) {
+void GUIImpl::HandleTextInput(const std::string* text) {
     m_browse_info_wnd.reset();
     m_browse_info_mode = -1;
     m_browse_target = 0;
     if (GUI::s_gui->FocusWnd())
-        GUI::s_gui->FocusWnd()->HandleEvent(WndEvent(
-            WndEvent::TextInput, text));
+        GUI::s_gui->FocusWnd()->HandleEvent(WndEvent(WndEvent::TextInput, text));
 }
 
 void GUIImpl::HandleMouseMove(Flags<ModKey> mod_keys, const GG::Pt& pos, const Pt& rel, int curr_ticks)
@@ -767,6 +805,14 @@ const std::string& GUI::AppName() const
 Wnd* GUI::FocusWnd() const
 { return s_impl->m_modal_wnds.empty() ? s_impl->m_focus_wnd : s_impl->m_modal_wnds.back().second; }
 
+bool GUI::FocusWndAcceptsTypingInput() const
+{
+    const Wnd* focus_wnd = FocusWnd();
+    if (!focus_wnd)
+        return false;
+    return dynamic_cast<const Edit*>(focus_wnd);    // currently only Edit controls accept text input, so far as I'm aware. Could add a ->AcceptsTypingInput() function to Wnd if needed
+}
+
 Wnd* GUI::PrevFocusInteractiveWnd() const
 {
     Wnd* focus_wnd = FocusWnd();
@@ -789,7 +835,7 @@ Wnd* GUI::PrevFocusInteractiveWnd() const
     // loop around until finding an interactive enabled control sibling or
     // returning to the focus wnd
     std::list<Wnd*>::const_reverse_iterator loop_it = focus_it;
-    loop_it++;
+    ++loop_it;
     while (loop_it != focus_it) {
         if (loop_it == siblings.rend()) {
             loop_it = siblings.rbegin();
@@ -805,7 +851,7 @@ Wnd* GUI::PrevFocusInteractiveWnd() const
             }
         }
 
-        loop_it++;
+        ++loop_it;
     }
     return focus_wnd;
 }
@@ -832,7 +878,7 @@ Wnd* GUI::NextFocusInteractiveWnd() const
     // loop around until finding an interactive enabled control sibling or
     // returning to the focus wnd
     std::list<Wnd*>::const_iterator loop_it = focus_it;
-    loop_it++;
+    ++loop_it;
     while (loop_it != focus_it) {
         if (loop_it == siblings.end()) {
             loop_it = siblings.begin();
@@ -849,7 +895,7 @@ Wnd* GUI::NextFocusInteractiveWnd() const
         }
 
 
-        loop_it++;
+        ++loop_it;
     }
     return focus_wnd;
 }
@@ -947,6 +993,59 @@ std::set<std::pair<CPSize, CPSize> > GUI::FindWords(const std::string& str) cons
     return retval;
 }
 
+std::set<std::pair<StrSize, StrSize> > GUI::FindWordsStringIndices(const std::string& str) const
+{
+    std::set<std::pair<StrSize, StrSize> > retval;
+
+    utf8_wchar_iterator first(str.begin(), str.begin(), str.end());
+    utf8_wchar_iterator last(str.end(), str.begin(), str.end());
+    word_regex_iterator it(first, last, DEFAULT_WORD_REGEX);
+    word_regex_iterator end_it;
+
+    typedef word_regex_iterator::value_type match_result_type;
+
+    for ( ; it != end_it; ++it) {
+        match_result_type match_result = *it;
+        utf8_wchar_iterator word_pos_it = first;
+
+        std::advance(word_pos_it, match_result.position());
+        StrSize start_idx(std::distance(str.begin(), word_pos_it.base()));
+        std::advance(word_pos_it, match_result.length());
+        StrSize end_idx(std::distance(str.begin(), word_pos_it.base()));
+
+        retval.insert(std::make_pair(start_idx, end_idx));
+    }
+    return retval;
+}
+
+bool GUI::ContainsWord(const std::string& str, const std::string& word) const
+{
+    if (word.empty())
+        return false;
+
+    utf8_wchar_iterator first(str.begin(), str.begin(), str.end());
+    utf8_wchar_iterator last(str.end(), str.begin(), str.end());
+    word_regex_iterator it(first, last, DEFAULT_WORD_REGEX);
+    word_regex_iterator end_it;
+
+    typedef word_regex_iterator::value_type match_result_type;
+
+    for ( ; it != end_it; ++it) {
+        match_result_type match_result = *it;
+        utf8_wchar_iterator word_pos_it = first;
+
+        std::advance(word_pos_it, match_result.position());
+        std::string::const_iterator start_it = word_pos_it.base();
+        std::advance(word_pos_it, match_result.length());
+        std::string word_in_str(start_it, word_pos_it.base());
+
+        if (boost::iequals(word_in_str, word))
+            return true;
+    }
+
+    return false;
+}
+
 const boost::shared_ptr<StyleFactory>& GUI::GetStyleFactory() const
 { return s_impl->m_style_factory; }
 
@@ -972,6 +1071,12 @@ GUI::AcceleratorSignalType& GUI::AcceleratorSignal(Key key, Flags<ModKey> mod_ke
     return *sig_ptr;
 }
 
+bool GUI::ModalAcceleratorSignalsEnabled() const
+{ return s_impl->m_allow_modal_accelerator_signals; }
+
+bool GUI::ModalWndsOpen() const
+{ return !s_impl->m_modal_wnds.empty(); }
+
 void GUI::SaveWndAsPNG(const Wnd* wnd, const std::string& filename) const
 {
     s_impl->m_save_as_png_wnd = wnd;
@@ -1090,7 +1195,7 @@ bool GUI::SetNextFocusWndInCycle()
 }
 
 void GUI::Wait(unsigned int ms)
-{ boost::this_thread::sleep(boost::posix_time::milliseconds(ms)); }
+{ boost::this_thread::sleep_for(boost::chrono::milliseconds(ms)); }
 
 void GUI::Register(Wnd* wnd)
 { if (wnd) s_impl->m_zlist.Add(wnd); }
@@ -1163,7 +1268,7 @@ void GUI::WndDying(Wnd* wnd)
 void GUI::EnableFPS(bool b/* = true*/)
 {
     s_impl->m_calc_FPS = b;
-    if (!b) 
+    if (!b)
         s_impl->m_FPS = -1.0f;
 }
 
@@ -1267,6 +1372,9 @@ void GUI::RemoveAccelerator(Key key, Flags<ModKey> mod_keys/* = MOD_KEY_NONE*/)
 void GUI::RemoveAccelerator(accel_iterator it)
 { s_impl->m_accelerators.erase(it); }
 
+void GUI::EnableModalAcceleratorSignals(bool allow)
+{ s_impl->m_allow_modal_accelerator_signals = allow; }
+
 void GUI::SetMouseLRSwapped(bool swapped/* = true*/)
 { s_impl->m_mouse_lr_swap = swapped; }
 
@@ -1302,11 +1410,11 @@ boost::shared_ptr<Texture> GUI::StoreTexture(Texture* texture, const std::string
 boost::shared_ptr<Texture> GUI::StoreTexture(const boost::shared_ptr<Texture>& texture, const std::string& texture_name)
 { return GetTextureManager().StoreTexture(texture, texture_name); }
 
-boost::shared_ptr<Texture> GUI::GetTexture(const std::string& name, bool mipmap/* = false*/)
-{ return GetTextureManager().GetTexture(name, mipmap); }
+boost::shared_ptr<Texture> GUI::GetTexture(const boost::filesystem::path& path, bool mipmap/* = false*/)
+{ return GetTextureManager().GetTexture(path, mipmap); }
 
-void GUI::FreeTexture(const std::string& name)
-{ GetTextureManager().FreeTexture(name); }
+void GUI::FreeTexture(const boost::filesystem::path& path)
+{ GetTextureManager().FreeTexture(path); }
 
 void GUI::SetStyleFactory(const boost::shared_ptr<StyleFactory>& factory)
 {
@@ -1349,7 +1457,7 @@ bool GUI::CopyWndText(const Wnd* wnd)
             // is selected, revert to copying the full text of the TextControl.
             std::string selected_text = edit_control->SelectedText();
             if (!selected_text.empty()) {
-                SetClipboardText(selected_text);
+                SetClipboardText(GG::Font::StripTags(selected_text));
                 return true;
             }
         }
@@ -1457,7 +1565,7 @@ void GUI::RenderWindow(Wnd* wnd)
             if (clip)
                 wnd->BeginClipping();
             for (std::list<Wnd*>::iterator it = wnd->m_children.begin(); it != wnd->m_children.end(); ++it) {
-                if ((*it)->Visible())
+                if ((*it) && (*it)->Visible())
                     RenderWindow(*it);
             }
             if (clip)
@@ -1471,7 +1579,7 @@ void GUI::RenderWindow(Wnd* wnd)
             if (children_copy.begin() != client_child_begin) {
                 wnd->BeginNonclientClipping();
                 for (std::vector<Wnd*>::iterator it = children_copy.begin(); it != client_child_begin; ++it) {
-                    if ((*it)->Visible())
+                    if ((*it) && (*it)->Visible())
                         RenderWindow(*it);
                 }
                 wnd->EndNonclientClipping();
@@ -1480,7 +1588,7 @@ void GUI::RenderWindow(Wnd* wnd)
             if (client_child_begin != children_copy.end()) {
                 wnd->BeginClipping();
                 for (std::vector<Wnd*>::iterator it = client_child_begin; it != children_copy.end(); ++it) {
-                    if ((*it)->Visible())
+                    if ((*it) && (*it)->Visible())
                         RenderWindow(*it);
                 }
                 wnd->EndClipping();
@@ -1518,17 +1626,19 @@ void GUI::ProcessBrowseInfo()
 {
     assert(s_impl->m_curr_wnd_under_cursor);
     if (!s_impl->m_mouse_button_state[0] && !s_impl->m_mouse_button_state[1] && !s_impl->m_mouse_button_state[2] &&
-        (s_impl->m_modal_wnds.empty() || s_impl->m_curr_wnd_under_cursor->RootParent() == s_impl->m_modal_wnds.back().first)) {
+        (s_impl->m_modal_wnds.empty() || s_impl->m_curr_wnd_under_cursor->RootParent() == s_impl->m_modal_wnds.back().first))
+    {
         Wnd* wnd = s_impl->m_curr_wnd_under_cursor;
-        while (!ProcessBrowseInfoImpl(wnd) && wnd->Parent() && (dynamic_cast<Control*>(wnd) || dynamic_cast<Layout*>(wnd))) {
-            wnd = wnd->Parent();
-        }
+        while (!ProcessBrowseInfoImpl(wnd) &&
+               wnd->Parent() &&
+               (dynamic_cast<Control*>(wnd) || dynamic_cast<Layout*>(wnd)))
+        { wnd = wnd->Parent(); }
     }
 }
 
 void GUI::Render()
 {
-    // handle timers
+    // update timers
     int ticks = Ticks();
     for (std::set<Timer*>::iterator it = s_impl->m_timers.begin(); it != s_impl->m_timers.end(); ++it) {
         (*it)->Update(ticks);
@@ -1540,11 +1650,15 @@ void GUI::Render()
         if (*it)
             RenderWindow(*it);
     }
-    // render modal windows back-to-front
-    for (std::list<std::pair<Wnd*, Wnd*> >::iterator it = s_impl->m_modal_wnds.begin(); it != s_impl->m_modal_wnds.end(); ++it) {
+
+    // render modal windows back-to-front (on top of non-modal Wnds rendered above)
+    for (std::list<std::pair<Wnd*, Wnd*> >::iterator it = s_impl->m_modal_wnds.begin();
+         it != s_impl->m_modal_wnds.end(); ++it)
+    {
         if (it->first)
             RenderWindow(it->first);
     }
+
     // render the active browse info window, if any
     if (s_impl->m_browse_info_wnd) {
         if (!s_impl->m_curr_wnd_under_cursor) {
@@ -1604,56 +1718,75 @@ Wnd* GUI::ModalWindow() const
 Wnd* GUI::CheckedGetWindowUnder(const Pt& pt, Flags<ModKey> mod_keys)
 {
     Wnd* w = GetWindowUnder(pt);
-    Wnd* dragged_wnd = s_impl->m_curr_drag_wnd;
+    Wnd* dragged_wnd = s_impl->m_curr_drag_wnd; // wnd being continuously repositioned / dragged around, not a drag-drop
 
-    bool unregistered_drag_drop =
-        dragged_wnd && !dragged_wnd->DragDropDataType().empty();
+    //std::cout << "GUI::CheckedGetWindowUnder w: " << w << "  dragged_wnd: " << dragged_wnd << std::endl << std::flush;
+
+    bool unregistered_drag_drop = dragged_wnd && !dragged_wnd->DragDropDataType().empty();
     bool registered_drag_drop = !s_impl->m_drag_drop_wnds.empty();
-    std::map<Wnd*, Pt> drag_drop_wnds;
-
-    drag_drop_wnds[dragged_wnd] = s_impl->m_wnd_drag_offset;
 
     if (s_impl->m_curr_drag_drop_here_wnd && !unregistered_drag_drop && !registered_drag_drop) {
         s_impl->m_curr_drag_drop_here_wnd->HandleEvent(WndEvent(WndEvent::DragDropLeave));
         s_impl->m_curr_drag_drop_here_wnd = 0;
     }
-    if (w != s_impl->m_curr_wnd_under_cursor) {
-        if (s_impl->m_curr_wnd_under_cursor) {
-            if (unregistered_drag_drop) {
-                s_impl->m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
-                s_impl->m_drag_drop_wnds_acceptable[dragged_wnd] = false;
-                s_impl->m_curr_drag_drop_here_wnd = 0;
-            } else if (registered_drag_drop) {
-                s_impl->m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
-                for (std::map<const Wnd*, bool>::iterator it = s_impl->m_drag_drop_wnds_acceptable.begin();
-                     it != s_impl->m_drag_drop_wnds_acceptable.end();
-                     ++it) {
-                    it->second = false;
-                }
-                s_impl->m_curr_drag_drop_here_wnd = 0;
-            } else {
-                s_impl->m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::MouseLeave));
-            }
-        }
-        if (w) {
-            if (unregistered_drag_drop) {
-                w->HandleEvent(WndEvent(WndEvent::DragDropEnter, pt, drag_drop_wnds, mod_keys));
-                s_impl->m_drag_drop_wnds_acceptable[dragged_wnd] = false;
-                w->DropsAcceptable(s_impl->m_drag_drop_wnds_acceptable.begin(),
-                                   s_impl->m_drag_drop_wnds_acceptable.end(),
-                                   pt);
-                s_impl->m_curr_drag_drop_here_wnd = w;
-            } else if (registered_drag_drop) {
-                w->HandleEvent(WndEvent(WndEvent::DragDropEnter, pt, s_impl->m_drag_drop_wnds, mod_keys));
-                w->DropsAcceptable(s_impl->m_drag_drop_wnds_acceptable.begin(),
-                                   s_impl->m_drag_drop_wnds_acceptable.end(),
-                                   pt);
-                s_impl->m_curr_drag_drop_here_wnd = w;
-            } else {
-                s_impl->HandleMouseEnter(mod_keys, pt, w);
-            }
+
+    if (w == s_impl->m_curr_wnd_under_cursor)
+        return w;   // same Wnd is under cursor as before; nothing to do
+
+    if (s_impl->m_curr_wnd_under_cursor) {
+        // inform previous Wnd under the cursor that the cursor has been dragged away
+        if (unregistered_drag_drop) {
+            s_impl->m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
+            s_impl->m_drag_drop_wnds_acceptable[dragged_wnd] = false;
+            s_impl->m_curr_drag_drop_here_wnd = 0;
+
+        } else if (registered_drag_drop) {
+            s_impl->m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
+            for (std::map<const Wnd*, bool>::iterator it = s_impl->m_drag_drop_wnds_acceptable.begin();
+                 it != s_impl->m_drag_drop_wnds_acceptable.end(); ++it)
+            { it->second = false; }
+            s_impl->m_curr_drag_drop_here_wnd = 0;
+
+        } else {
+            s_impl->m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::MouseLeave));
         }
     }
+
+    if (!w) {
+        //std::cout << "CheckedGetWindowUnder returning " << w << std::endl << std::flush;
+        return w;
+    }
+
+
+    // inform new Wnd under cursor that something was dragged over it
+    if (unregistered_drag_drop) {
+        // pass drag-drop event to check if the single dragged Wnd is acceptable to drop
+        WndEvent event(WndEvent::CheckDrops, pt, dragged_wnd, mod_keys);
+        w->HandleEvent(event);
+        s_impl->m_drag_drop_wnds_acceptable[dragged_wnd] = false;
+        s_impl->m_drag_drop_wnds_acceptable = event.GetAcceptableDropWnds();
+
+        // Wnd being dragged over is new; give it an Enter message
+        WndEvent enter_event(WndEvent::DragDropEnter, pt, dragged_wnd, mod_keys);
+        w->HandleEvent(enter_event);
+        s_impl->m_curr_drag_drop_here_wnd = w;
+
+    } else if (registered_drag_drop) {
+        // pass drag-drop event to check if the various dragged Wnds are acceptable to drop
+        WndEvent event(WndEvent::CheckDrops, pt, s_impl->m_drag_drop_wnds, mod_keys);
+        w->HandleEvent(event);
+        s_impl->m_drag_drop_wnds_acceptable = event.GetAcceptableDropWnds();
+
+        // Wnd being dragged over is new; give it an Enter message
+        WndEvent enter_event(WndEvent::DragDropEnter, pt, s_impl->m_drag_drop_wnds, mod_keys);
+        w->HandleEvent(enter_event);
+        s_impl->m_curr_drag_drop_here_wnd = w;
+
+    } else {
+        s_impl->HandleMouseEnter(mod_keys, pt, w);
+    }
+
+    //std::cout << "CheckedGetWindowUnder returning " << w << std::endl << std::flush;
     return w;
 }
 
