@@ -35,6 +35,7 @@
 #include <boost/multi_index/member.hpp>
 #include <boost/multi_index/ordered_index.hpp>
 
+
 using namespace GG;
 
 namespace {
@@ -268,13 +269,12 @@ const std::string& Wnd::Name() const
 const std::string& Wnd::DragDropDataType() const
 { return m_drag_drop_data_type; }
 
-void Wnd::DropsAcceptable(DropsAcceptableIter first,
-                          DropsAcceptableIter last,
-                          const Pt& pt) const
+void Wnd::DropsAcceptable(DropsAcceptableIter first, DropsAcceptableIter last,
+                          const Pt& pt, Flags<ModKey> mod_keys) const
 {
-    for (std::map<const Wnd*, bool>::iterator it = first; it != last; ++it) {
+    // default reject all drops. derived classes can override to accept drops
+    for (std::map<const Wnd*, bool>::iterator it = first; it != last; ++it)
         it->second = false;
-    }
 }
 
 Pt Wnd::UpperLeft() const
@@ -330,7 +330,7 @@ Pt Wnd::MaxSize() const
 { return m_max_size; }
 
 Pt Wnd::MinUsableSize() const
-{ return Size(); }
+{ return m_layout ? m_layout->MinUsableSize() : Size(); }
 
 Pt Wnd::ClientUpperLeft() const
 { return UpperLeft(); }
@@ -422,10 +422,17 @@ void Wnd::SetDragDropDataType(const std::string& data_type)
 void Wnd::StartingChildDragDrop(const Wnd* wnd, const Pt& offset)
 {}
 
-void Wnd::AcceptDrops(const std::vector<Wnd*>& wnds, const Pt& pt)
-{}
+void Wnd::AcceptDrops(const Pt& pt, const std::vector<Wnd*>& wnds, Flags<ModKey> mod_keys)
+{
+    if (!Interactive() && Parent())
+        ForwardEventToParent();
+    // if dropped Wnds were accepted, but no handler or parent exists to handle them, delete them
+    for (std::vector<Wnd*>::const_iterator it = wnds.begin(); it != wnds.end(); ++it)
+        delete *it;
+}
 
-void Wnd::CancellingChildDragDrop(const std::vector<const Wnd*>& wnds) {}
+void Wnd::CancellingChildDragDrop(const std::vector<const Wnd*>& wnds)
+{}
 
 void Wnd::ChildrenDraggedAway(const std::vector<Wnd*>& wnds, const Wnd* destination)
 {
@@ -541,50 +548,53 @@ void Wnd::SetMaxSize(const Pt& sz)
 
 void Wnd::AttachChild(Wnd* wnd)
 {
-    if (wnd) {
-        // remove from previous parent, if any
-        if (wnd->Parent())
-            wnd->Parent()->DetachChild(wnd);
-        GUI::GetGUI()->Remove(wnd);
-        m_children.push_back(wnd);
-        wnd->m_parent = this;
-        if (Layout* this_as_layout = dynamic_cast<Layout*>(this))
-            wnd->m_containing_layout = this_as_layout;
-    }
+    if (!wnd)
+        return;
+
+    // remove from previous parent, if any
+    if (wnd->Parent())
+        wnd->Parent()->DetachChild(wnd);
+    GUI::GetGUI()->Remove(wnd);
+    m_children.push_back(wnd);
+    wnd->SetParent(this);
+    if (Layout* this_as_layout = dynamic_cast<Layout*>(this))
+        wnd->m_containing_layout = this_as_layout;
 }
 
 void Wnd::MoveChildUp(Wnd* wnd)
 {
-    if (wnd) {
-        if (std::find(m_children.begin(), m_children.end(), wnd) != m_children.end()) {
-            m_children.remove(wnd);
-            m_children.push_back(wnd);
-        }
+    if (!wnd)
+        return;
+    if (std::find(m_children.begin(), m_children.end(), wnd) != m_children.end()) {
+        m_children.remove(wnd);
+        m_children.push_back(wnd);
     }
 }
  
 void Wnd::MoveChildDown(Wnd* wnd)
 {
-    if (wnd) {
-        if (std::find(m_children.begin(), m_children.end(), wnd) != m_children.end()) {
-            m_children.remove(wnd);
-            m_children.push_front(wnd);
-        }
+    if (!wnd)
+        return;
+    if (std::find(m_children.begin(), m_children.end(), wnd) != m_children.end()) {
+        m_children.remove(wnd);
+        m_children.push_front(wnd);
     }
 }
 
 void Wnd::DetachChild(Wnd* wnd)
 {
-    if (wnd) {
-        std::list<Wnd*>::iterator it = std::find(m_children.begin(), m_children.end(), wnd);
-        if (it != m_children.end()) {
-            m_children.erase(it);
-            wnd->m_parent = 0;
-            if (Layout* this_as_layout = dynamic_cast<Layout*>(this)) {
-                this_as_layout->Remove(wnd);
-                wnd->m_containing_layout = 0;
-            }
-        }
+    if (!wnd)
+        return;
+    std::list<Wnd*>::iterator it = std::find(m_children.begin(), m_children.end(), wnd);
+    if (it == m_children.end())
+        return;
+    m_children.erase(it);
+    wnd->SetParent(0);
+    if (wnd == m_layout)
+        m_layout = 0;
+    if (Layout* this_as_layout = dynamic_cast<Layout*>(this)) {
+        this_as_layout->Remove(wnd);
+        wnd->m_containing_layout = 0;
     }
 }
 
@@ -599,36 +609,45 @@ void Wnd::DetachChildren()
 
 void Wnd::DeleteChild(Wnd* wnd)
 {
-    if (wnd && std::find(m_children.begin(), m_children.end(), wnd) != m_children.end()) {
+    if (!wnd)
+        return;
+
+    if (wnd == m_layout)
+        RemoveLayout();
+    std::list<Wnd*>::iterator found = std::find(m_children.begin(), m_children.end(), wnd);
+    if (found != m_children.end()) {
+        m_children.erase(found);
         delete wnd;
     }
 }
 
 void Wnd::DeleteChildren()
 {
+    m_layout = 0;
     for (std::list<Wnd*>::iterator it = m_children.begin(); it != m_children.end();) {
         Wnd* wnd = *it++;
         delete wnd;
     }
+    m_children.clear();
 }
 
 void Wnd::InstallEventFilter(Wnd* wnd)
 {
-    if (wnd) {
-        RemoveEventFilter(wnd);
-        m_filters.push_back(wnd);
-        wnd->m_filtering.insert(this);
-    }
+    if (!wnd)
+        return;
+    RemoveEventFilter(wnd);
+    m_filters.push_back(wnd);
+    wnd->m_filtering.insert(this);
 }
 
 void Wnd::RemoveEventFilter(Wnd* wnd)
 {
-    if (wnd) {
-        std::vector<Wnd*>::iterator it = std::find(m_filters.begin(), m_filters.end(), wnd);
-        if (it != m_filters.end())
-            m_filters.erase(it);
-        wnd->m_filtering.erase(this);
-    }
+    if (!wnd)
+        return;
+    std::vector<Wnd*>::iterator it = std::find(m_filters.begin(), m_filters.end(), wnd);
+    if (it != m_filters.end())
+        m_filters.erase(it);
+    wnd->m_filtering.erase(this);
 }
 
 void Wnd::HorizontalLayout()
@@ -827,8 +846,10 @@ void Wnd::RemoveLayout()
         for (std::list<Wnd*>::iterator it = layout_children.begin(); it != layout_children.end(); ++it) {
             AttachChild(*it);
         }
-        DeleteChild(m_layout);
+        // prevent DeleteChild from recursively calling RemoveLayout
+        GG::Layout* temp_layout = m_layout;
         m_layout = 0;
+        DeleteChild(temp_layout);
     }
 }
 
@@ -836,7 +857,6 @@ Layout* Wnd::DetachLayout()
 {
     Layout* retval = m_layout;
     DetachChild(m_layout);
-    m_layout = 0;
     return retval;
 }
 
@@ -995,11 +1015,23 @@ void Wnd::MouseLeave()
 void Wnd::MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::DragDropEnter(const Pt& pt, const std::map<Wnd*, Pt>& drag_drop_wnds, Flags<ModKey> mod_keys)
+void Wnd::DragDropEnter(const Pt& pt, std::map<const Wnd*, bool>& drop_wnds_acceptable, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::DragDropHere(const Pt& pt, const std::map<Wnd*, Pt>& drag_drop_wnds, Flags<ModKey> mod_keys)
-{ if (!Interactive()) ForwardEventToParent(); }
+void Wnd::DragDropHere(const Pt& pt, std::map<const Wnd*, bool>& drop_wnds_acceptable, Flags<ModKey> mod_keys)
+{
+    if (!Interactive())
+        ForwardEventToParent();
+    this->DropsAcceptable(drop_wnds_acceptable.begin(), drop_wnds_acceptable.end(), pt, mod_keys);
+}
+
+void Wnd::CheckDrops(const Pt& pt, std::map<const Wnd*, bool>& drop_wnds_acceptable,
+                     Flags<ModKey> mod_keys)
+{
+    if (!Interactive())
+        ForwardEventToParent();
+    this->DropsAcceptable(drop_wnds_acceptable.begin(), drop_wnds_acceptable.end(), pt, mod_keys);
+}
 
 void Wnd::DragDropLeave()
 { if (!Interactive()) ForwardEventToParent(); }
@@ -1089,14 +1121,20 @@ void Wnd::HandleEvent(const WndEvent& event)
             MouseLeave();
             break;
         case WndEvent::DragDropEnter:
-            DragDropEnter(event.Point(), event.DragDropWnds(), event.ModKeys());
+            DragDropEnter(event.Point(), event.GetAcceptableDropWnds(), event.ModKeys());
             break;
         case WndEvent::DragDropHere:
-            DragDropHere(event.Point(), event.DragDropWnds(), event.ModKeys());
+            DragDropHere(event.Point(), event.GetAcceptableDropWnds(), event.ModKeys());
+            break;
+        case WndEvent::CheckDrops:
+            CheckDrops(event.Point(), event.GetAcceptableDropWnds(), event.ModKeys());
             break;
         case WndEvent::DragDropLeave:
             DragDropLeave();
             break;
+        case WndEvent::DragDroppedOn:
+            AcceptDrops(event.Point(), event.GetDragDropWnds(), event.ModKeys());
+            break;
         case WndEvent::MouseWheel:
             MouseWheel(event.Point(), event.WheelMove(), event.ModKeys());
             break;
@@ -1189,3 +1227,6 @@ void Wnd::BeginNonclientClippingImpl()
 
 void Wnd::EndNonclientClippingImpl()
 { EndStencilClipping(); }
+
+void Wnd::SetParent(Wnd* wnd)
+{ m_parent = wnd; }
