@@ -31,6 +31,7 @@
 #include <GG/Slider.h>
 #include <GG/StyleFactory.h>
 #include <GG/WndEvent.h>
+#include <GG/GLClientAndServerBuffer.h>
 
 
 using namespace GG;
@@ -178,28 +179,66 @@ void HueSaturationPicker::Render()
     Pt ul = UpperLeft(), lr = LowerRight();
     Pt size = Size();
     glDisable(GL_TEXTURE_2D);
+
+    glPushClientAttrib(GL_CLIENT_ALL_ATTRIB_BITS);
+    glEnableClientState(GL_VERTEX_ARRAY);
+    glEnableClientState(GL_COLOR_ARRAY);
+    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+    // grid of quads of varying hue and saturation
     glPushMatrix();
     glTranslated(Value(ul.x), Value(ul.y), 0.0);
     glScaled(Value(size.x), Value(size.y), 1.0);
-    glEnableClientState(GL_VERTEX_ARRAY);
-    glEnableClientState(GL_COLOR_ARRAY);
     for (std::size_t i = 0; i < m_vertices.size(); ++i) {
         glVertexPointer(2, GL_DOUBLE, 0, &m_vertices[i][0]);
         glColorPointer(4, GL_UNSIGNED_BYTE, 0, &m_colors[i][0]);
         glDrawArrays(GL_QUAD_STRIP, 0, m_vertices[i].size());
     }
-    glDisableClientState(GL_VERTEX_ARRAY);
-    glDisableClientState(GL_COLOR_ARRAY);
     glPopMatrix();
+
+    glDisableClientState(GL_COLOR_ARRAY);
+
+    // lines to indicate currently selected colour
+    glLineWidth(1.5f);
     Pt color_position(static_cast<X>(ul.x + size.x * m_hue),
                       static_cast<Y>(ul.y + size.y * (1.0 - m_saturation)));
     glColor(CLR_SHADOW);
-    glBegin(GL_LINES);
-    glVertex(color_position.x, ul.y);
-    glVertex(color_position.x, lr.y);
-    glVertex(ul.x, color_position.y);
-    glVertex(lr.x, color_position.y);
-    glEnd();
+
+    const float GAP(3.0f);
+
+    GL2DVertexBuffer lines_verts;
+    lines_verts.reserve(16);
+    lines_verts.store(Value(color_position.x),      Value(ul.y));
+    lines_verts.store(Value(color_position.x),      Value(color_position.y) - GAP);
+
+    lines_verts.store(Value(color_position.x),      Value(lr.y));
+    lines_verts.store(Value(color_position.x),      Value(color_position.y) + GAP);
+
+    lines_verts.store(Value(ul.x),                  Value(color_position.y));
+    lines_verts.store(Value(color_position.x) - GAP,Value(color_position.y));
+
+    lines_verts.store(Value(lr.x),                  Value(color_position.y));
+    lines_verts.store(Value(color_position.x) + GAP,Value(color_position.y));
+
+
+    lines_verts.store(Value(color_position.x),      Value(color_position.y) - GAP);
+    lines_verts.store(Value(color_position.x) - GAP,Value(color_position.y));
+
+    lines_verts.store(Value(color_position.x) - GAP,Value(color_position.y));
+    lines_verts.store(Value(color_position.x),      Value(color_position.y) + GAP);
+
+    lines_verts.store(Value(color_position.x),      Value(color_position.y) + GAP);
+    lines_verts.store(Value(color_position.x) + GAP,Value(color_position.y));
+
+    lines_verts.store(Value(color_position.x) + GAP,Value(color_position.y));
+    lines_verts.store(Value(color_position.x),      Value(color_position.y) - GAP);
+
+    lines_verts.activate();
+
+    glDrawArrays(GL_LINES, 0, lines_verts.size());
+    glLineWidth(1.0f);
+
+    glPopClientAttrib();
     glEnable(GL_TEXTURE_2D);
 }
 
@@ -246,29 +285,63 @@ ValuePicker::ValuePicker(X x, Y y, X w, Y h, Clr arrow_color) :
 
 void ValuePicker::Render()
 {
-    Pt eff_ul = UpperLeft(), eff_lr = LowerRight() - Pt(X(4), Y0);
+    Pt eff_ul = UpperLeft() + Pt(X(3), Y0), eff_lr = LowerRight() - Pt(X(4), Y0);
     Y h = Height();
     glDisable(GL_TEXTURE_2D);
-    glBegin(GL_QUADS);
-    glColor(Convert(HSVClr(m_hue, m_saturation, 1.0)));
-    glVertex(eff_lr.x, eff_ul.y);
-    glVertex(eff_ul.x, eff_ul.y);
-    glColor(Convert(HSVClr(m_hue, m_saturation, 0.0)));
-    glVertex(eff_ul.x, eff_lr.y);
-    glVertex(eff_lr.x, eff_lr.y);
-    glEnd();
+
+    GL2DVertexBuffer vert_buf;
+    vert_buf.reserve(12);
+    GLRGBAColorBuffer colour_buf;   // need to give each vertex in lightness bar its own colour so can't just use a glColor call
+    colour_buf.reserve(12);
+
+    // bar for picking lightness
+    vert_buf.store(Value(eff_lr.x),    Value(eff_ul.y));
+    colour_buf.store(Convert(HSVClr(m_hue, m_saturation, 1.0)));
+    vert_buf.store(Value(eff_ul.x),    Value(eff_ul.y));
+    colour_buf.store(Convert(HSVClr(m_hue, m_saturation, 1.0)));
+
+    vert_buf.store(Value(eff_ul.x),    Value(eff_lr.y));
+    colour_buf.store(Convert(HSVClr(m_hue, m_saturation, 0.0)));
+    vert_buf.store(Value(eff_lr.x),    Value(eff_lr.y));
+    colour_buf.store(Convert(HSVClr(m_hue, m_saturation, 0.0)));
+
+    // line indicating currently-selected lightness
     Y color_position(eff_ul.y + h * (1.0 - m_value));
-    glColor(CLR_SHADOW);
-    glBegin(GL_LINES);
-    glVertex(eff_ul.x, color_position);
-    glVertex(eff_lr.x, color_position);
-    glEnd();
-    glColor(m_arrow_color);
-    glBegin(GL_TRIANGLES);
-    glVertex(eff_lr.x + 4, color_position - 3);
-    glVertex(eff_lr.x + 1, color_position);
-    glVertex(eff_lr.x + 4, color_position + 3);
-    glEnd();
+    vert_buf.store(Value(eff_ul.x),    Value(color_position));
+    colour_buf.store(CLR_SHADOW);
+    vert_buf.store(Value(eff_lr.x),    Value(color_position));
+    colour_buf.store(CLR_SHADOW);
+
+    // arrows marking lightness position
+    vert_buf.store(Value(eff_lr.x + 5),Value(color_position - 4));
+    colour_buf.store(m_arrow_color);
+    vert_buf.store(Value(eff_lr.x + 1),Value(color_position));
+    colour_buf.store(m_arrow_color);
+    vert_buf.store(Value(eff_lr.x + 5),Value(color_position + 4));
+    colour_buf.store(m_arrow_color);
+    vert_buf.store(Value(eff_ul.x - 5),Value(color_position - 4));
+    colour_buf.store(m_arrow_color);
+    vert_buf.store(Value(eff_ul.x),    Value(color_position));
+    colour_buf.store(m_arrow_color);
+    vert_buf.store(Value(eff_ul.x - 5),Value(color_position + 4));
+    colour_buf.store(m_arrow_color);
+
+
+    glDisable(GL_TEXTURE_2D);
+    glPushClientAttrib(GL_CLIENT_ALL_ATTRIB_BITS);
+    glEnableClientState(GL_VERTEX_ARRAY);
+    glEnableClientState(GL_COLOR_ARRAY);
+    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+    vert_buf.activate();
+    colour_buf.activate();
+    glDrawArrays(GL_QUADS, 0, 4);
+    glLineWidth(1.5f);
+    glDrawArrays(GL_LINES, 4, 2);
+    glLineWidth(1.0f);
+    glDrawArrays(GL_TRIANGLES, 6, 6);
+
+    glPopClientAttrib();
     glEnable(GL_TEXTURE_2D);
 }
 
@@ -347,8 +420,11 @@ void ColorDlg::ColorDisplay::Render()
 {
     Pt ul = UpperLeft(), lr = LowerRight();
     const int SQUARE_SIZE = 7;
-    glDisable(GL_TEXTURE_2D);
-    glBegin(GL_QUADS);
+
+    GL2DVertexBuffer    vert_buf;
+    GLRGBAColorBuffer   colour_buf;
+
+    // background checkerboard for curent colour display (to see through transparent areas)
     int i = 0, j = 0;
     for (Y y = lr.y; y > ul.y; y -= SQUARE_SIZE, ++j) {
         Y y0 = y - SQUARE_SIZE;
@@ -359,26 +435,53 @@ void ColorDlg::ColorDisplay::Render()
             X x0 = x - SQUARE_SIZE;
             if (x0 < ul.x)
                 x0 = ul.x;
-            glColor(((i + j) % 2) ? CLR_WHITE : CLR_BLACK);
-            glVertex(x, y0);
-            glVertex(x0, y0);
-            glVertex(x0, y);
-            glVertex(x, y);
+            Clr vert_clr = ((i + j) % 2) ? CLR_WHITE : CLR_BLACK;
+            colour_buf.store(vert_clr);
+            vert_buf.store(Value(x), Value(y0));
+            colour_buf.store(vert_clr);
+            vert_buf.store(Value(x0), Value(y0));
+            colour_buf.store(vert_clr);
+            vert_buf.store(Value(x0), Value(y));
+            colour_buf.store(vert_clr);
+            vert_buf.store(Value(x), Value(y));
         }
     }
-    glEnd();
+
+
     Clr full_alpha_color = Color();
     full_alpha_color.a = 255;
-    glBegin(GL_TRIANGLES);
+
+
+    GLfloat verts[12];
+    // upper left: full alpha colour
+    verts[0] = Value(lr.x); verts[1] = Value(ul.y);
+    verts[2] = Value(ul.x); verts[3] = Value(ul.y);
+    verts[4] = Value(ul.x); verts[5] = Value(lr.y);
+    // bottom right: actual alpha colour
+    verts[6] = Value(ul.x); verts[7] = Value(lr.y);
+    verts[8] = Value(lr.x); verts[9] = Value(lr.y);
+    verts[10]= Value(lr.x); verts[11]= Value(ul.y);
+
+
+    glDisable(GL_TEXTURE_2D);
+    glPushClientAttrib(GL_CLIENT_ALL_ATTRIB_BITS);
+    glEnableClientState(GL_VERTEX_ARRAY);
+    glEnableClientState(GL_COLOR_ARRAY);
+    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+    vert_buf.activate();
+    colour_buf.activate();
+    glDrawArrays(GL_QUADS, 0, vert_buf.size());
+
+    glDisableClientState(GL_COLOR_ARRAY);
+
+    glVertexPointer(2, GL_FLOAT, 0, verts);
     glColor(full_alpha_color);
-    glVertex(lr.x, ul.y);
-    glVertex(ul.x, ul.y);
-    glVertex(ul.x, lr.y);
+    glDrawArrays(GL_TRIANGLES, 0, 3);
     glColor(Color());
-    glVertex(ul.x, lr.y);
-    glVertex(lr.x, lr.y);
-    glVertex(lr.x, ul.y);
-    glEnd();
+    glDrawArrays(GL_TRIANGLES, 3, 3);
+
+    glPopClientAttrib();
     glEnable(GL_TEXTURE_2D);
 }
 
@@ -455,11 +558,31 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
 
     boost::shared_ptr<StyleFactory> style = GetStyleFactory();
 
-    const int COLOR_BUTTON_ROWS = 3;
+    const int COLOR_BUTTON_ROWS = 4;
     const int COLOR_BUTTON_COLS = 5;
     if (s_custom_colors.empty()) {
-        for (int i = 0; i < COLOR_BUTTON_ROWS * COLOR_BUTTON_COLS; ++i) {
-            s_custom_colors.push_back(CLR_BLACK);
+        s_custom_colors.push_back(GG::CLR_WHITE);
+        s_custom_colors.push_back(GG::CLR_LIGHT_GRAY);
+        s_custom_colors.push_back(GG::CLR_GRAY);
+        s_custom_colors.push_back(GG::CLR_DARK_GRAY);
+        s_custom_colors.push_back(GG::CLR_BLACK);
+        s_custom_colors.push_back(GG::CLR_PINK);
+        s_custom_colors.push_back(GG::CLR_RED);
+        s_custom_colors.push_back(GG::CLR_DARK_RED);
+        s_custom_colors.push_back(GG::CLR_MAGENTA);
+        s_custom_colors.push_back(GG::CLR_PURPLE);
+        s_custom_colors.push_back(GG::CLR_BLUE);
+        s_custom_colors.push_back(GG::CLR_DARK_BLUE);
+        s_custom_colors.push_back(GG::CLR_TEAL);
+        s_custom_colors.push_back(GG::CLR_CYAN);
+        s_custom_colors.push_back(GG::CLR_GREEN);
+        s_custom_colors.push_back(GG::CLR_DARK_GREEN);
+        s_custom_colors.push_back(GG::CLR_OLIVE);
+        s_custom_colors.push_back(GG::CLR_YELLOW);
+        s_custom_colors.push_back(GG::CLR_ORANGE);
+
+        for (unsigned int i = s_custom_colors.size(); i < COLOR_BUTTON_ROWS * COLOR_BUTTON_COLS; ++i) {
+            s_custom_colors.push_back(CLR_GRAY);
         }
     }
 
@@ -472,7 +595,7 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
     m_pickers_layout = new Layout(X0, Y0, X(HUE_SATURATION_PICKER_SIZE + 30), Y(HUE_SATURATION_PICKER_SIZE),
                                   1, 2, 0, 5);
     m_pickers_layout->SetColumnStretch(0, 1);
-    m_pickers_layout->SetMinimumColumnWidth(1, X(20));
+    m_pickers_layout->SetMinimumColumnWidth(1, X(24));
     m_pickers_layout->Add(m_hue_saturation_picker, 0, 0);
     m_pickers_layout->Add(m_value_picker, 0, 1);
 
