@@ -30,6 +30,10 @@ namespace {
     const std::string PROD_SELECTOR_WND_NAME = "production.selector";
     const std::string PROD_SIDEPANEL_WND_NAME = "production.sidepanel";
 
+    void    AddOptions(OptionsDB& db)
+    { db.Add("UI.windows." + PROD_PEDIA_WND_NAME + ".persistently-hidden", UserStringNop("OPTIONS_DB_PRODUCTION_PEDIA_HIDDEN"), false, Validator<bool>()); }
+    bool temp_bool = RegisterOptions(&AddOptions);
+
     //////////////////////////////////
     // ProductionItemPanel
     //////////////////////////////////
@@ -190,20 +194,8 @@ namespace {
         return source;
     }
 
-    std::string LocationConditionDescription(const std::string& building_name, int candidate_object_id,
-                                             int empire_id)
-    {
-        std::vector<Condition::ConditionBase*> location_conditions;
-        Condition::OwnerHasBuildingTypeAvailable bld_avail_cond(building_name);
-        location_conditions.push_back(&bld_avail_cond);
-        if (const BuildingType* building_type = GetBuildingType(building_name))
-            location_conditions.push_back(const_cast<Condition::ConditionBase*>(building_type->Location()));
-        TemporaryPtr<const UniverseObject> source = GetSourceObjectForEmpire(empire_id);
-        return ConditionDescription(location_conditions, GetUniverseObject(candidate_object_id), source);
-    }
-
     std::string EnqueueAndLocationConditionDescription(const std::string& building_name, int candidate_object_id,
-                                                       int empire_id)
+                                                       int empire_id, bool only_failed_conditions)
     {
         std::vector<Condition::ConditionBase*> enqueue_conditions;
         Condition::OwnerHasBuildingTypeAvailable bld_avail_cond(building_name);
@@ -213,11 +205,14 @@ namespace {
             enqueue_conditions.push_back(const_cast<Condition::ConditionBase*>(building_type->Location()));
         }
         TemporaryPtr<const UniverseObject> source = GetSourceObjectForEmpire(empire_id);
-        return ConditionDescription(enqueue_conditions, GetUniverseObject(candidate_object_id), source);
+        if (only_failed_conditions)
+            return ConditionFailedDescription(enqueue_conditions, GetUniverseObject(candidate_object_id), source);
+        else
+            return ConditionDescription(enqueue_conditions, GetUniverseObject(candidate_object_id), source);
     }
 
     std::string LocationConditionDescription(int ship_design_id, int candidate_object_id,
-                                             int empire_id)
+                                             int empire_id, bool only_failed_conditions)
     {
         std::vector<Condition::ConditionBase*> location_conditions;
         boost::shared_ptr<Condition::ConditionBase> can_prod_ship_cond(new Condition::CanProduceShips());
@@ -241,25 +236,103 @@ namespace {
             }
         }
         TemporaryPtr<const UniverseObject> source = GetSourceObjectForEmpire(empire_id);
-        return ConditionDescription(location_conditions, GetUniverseObject(candidate_object_id), source);
+
+        if (only_failed_conditions)
+            return ConditionFailedDescription(location_conditions, GetUniverseObject(candidate_object_id), source);
+        else
+            return ConditionDescription(location_conditions, GetUniverseObject(candidate_object_id), source);
     }
 
     boost::shared_ptr<GG::BrowseInfoWnd> ProductionItemRowBrowseWnd(const ProductionQueue::ProductionItem& item,
                                                                     int candidate_object_id, int empire_id)
     {
+        // production item is a building
         if (item.build_type == BT_BUILDING) {
+            const BuildingType* building_type = GetBuildingType(item.name);
+            if (!building_type) {
+                boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
+                return browse_wnd;
+            }
+
+            // create title, description, production time and cost
+            const std::string& title = UserString(item.name);
+            std::string main_text = UserString(building_type->Description());
+            float total_cost = building_type->ProductionCost(empire_id, candidate_object_id);
+            int production_time = building_type->ProductionTime(empire_id, candidate_object_id);
+
+            main_text += "\n\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_COST") + ": " + DoubleToString(total_cost, 3, false);
+            main_text += "\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_TIME") + ": " + boost::lexical_cast<std::string>(production_time);
+
+            // show build conditions
+            const std::string& enqueue_and_location_condition_failed_text = EnqueueAndLocationConditionDescription(item.name, candidate_object_id, empire_id, true);
+            if (!enqueue_and_location_condition_failed_text.empty())
+                if (TemporaryPtr<UniverseObject> location = GetUniverseObject(candidate_object_id)) {
+                    std::string failed_cond_loc = boost::io::str(FlexibleFormat(UserString("PRODUCTION_WND_TOOLTIP_FAILED_COND")) % location->Name());
+                    main_text += "\n\n" + failed_cond_loc + ":\n" + enqueue_and_location_condition_failed_text;
+            }
+
+            // create tooltip
             boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd(new IconTextBrowseWnd(
-                ClientUI::BuildingIcon(item.name), UserString(item.name),
-                EnqueueAndLocationConditionDescription(item.name, candidate_object_id, empire_id)));
+                ClientUI::BuildingIcon(item.name), title, main_text));
             return browse_wnd;
-        } else if (item.build_type == BT_SHIP) {
-            const ShipDesign* ship_design = GetShipDesign(item.design_id);
-            const std::string& name = (ship_design ? ship_design->Name() : UserString("SHIP_DESIGN"));
+        }
+
+        // production item is a ship
+        if (item.build_type == BT_SHIP) {
+            const ShipDesign* design = GetShipDesign(item.design_id);
+            if (!design) {
+                boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
+                return browse_wnd;
+            }
+
+            // create title, description, production time and cost, hull type
+            const std::string& title = design->Name(true);
+            std::string main_text = design->Description(true);
+            float total_cost = design->ProductionCost(empire_id, candidate_object_id);
+            int production_time = design->ProductionTime(empire_id, candidate_object_id);
+
+            main_text += "\n\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_COST") + ": " + DoubleToString(total_cost, 3, false);
+            main_text += "\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_TIME") + ": " + boost::lexical_cast<std::string>(production_time);
+            main_text += "\n\n" + UserString("ENC_SHIP_HULL") + ": " + UserString(design->Hull());
+
+            // load ship parts, stack ship parts that are used multiple times
+            std::string ship_parts_formatted;
+            std::map<std::string, int> ship_part_names;
+            const std::vector<std::string>& parts = design->Parts();
+
+            for (std::vector<std::string>::const_iterator it = parts.begin(); it != parts.end(); ++it) {
+                if (ship_part_names.find(*it) != ship_part_names.end())
+                    ship_part_names[*it]++;
+                else
+                    ship_part_names.insert(std::pair<std::string, int>(*it, 1));
+            }
+
+            for (std::map<std::string, int>::const_iterator it = ship_part_names.begin(); it != ship_part_names.end(); ++it) {
+                if (!UserStringExists(it->first)) continue;
+                if (ship_part_names[it->first] == 1)
+                    ship_parts_formatted += (UserString(it->first) + ", ");
+                else
+                    ship_parts_formatted += (UserString(it->first) + " x" + boost::lexical_cast<std::string>(it->second) + ", ");
+            }
+
+            main_text += "\n" + UserString("PRODUCTION_WND_TOOLTIP_PARTS") + ": " + ship_parts_formatted.substr(0, ship_parts_formatted.length() - 2);
+
+            // show build conditions
+            const std::string& location_condition_failed_text = LocationConditionDescription(item.design_id, candidate_object_id, empire_id, true);
+            if (!location_condition_failed_text.empty())
+                if (TemporaryPtr<UniverseObject> location = GetUniverseObject(candidate_object_id)) {
+                    std::string failed_cond_loc = boost::io::str(FlexibleFormat(UserString("PRODUCTION_WND_TOOLTIP_FAILED_COND")) % location->Name());
+                    main_text += ("\n\n" + failed_cond_loc + ":\n" + location_condition_failed_text);
+                }
+
+            // create tooltip
             boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd(new IconTextBrowseWnd(
-                ClientUI::ShipDesignIcon(item.design_id), name,
-                LocationConditionDescription(item.design_id, candidate_object_id, empire_id)));
+                ClientUI::ShipDesignIcon(item.design_id), title, main_text));
             return browse_wnd;
-        } else {
+        }
+
+        // other production item (?)
+        else {
             return boost::shared_ptr<GG::BrowseInfoWnd>();
         }
     }
@@ -421,9 +494,9 @@ public:
     //@}
 
     mutable boost::signals2::signal<void (const BuildingType*)>                DisplayBuildingTypeSignal;
-    mutable boost::signals2::signal<void (BuildType, const std::string&, int)> RequestNamedBuildItemSignal;
     mutable boost::signals2::signal<void (const ShipDesign*)>                  DisplayShipDesignSignal;
-    mutable boost::signals2::signal<void (BuildType, int, int)>                RequestIDedBuildItemSignal;
+    mutable boost::signals2::signal<void (const ProductionQueue::ProductionItem&, int, int)> RequestBuildItemSignal;
+    mutable boost::signals2::signal<void ()>                                   ShowPediaSignal;
 
 private:
     static const GG::X TEXT_MARGIN_X;
@@ -438,14 +511,16 @@ private:
       * filter settings. */
     void    PopulateList();
 
+    void    AddBuildItemToQueue(GG::ListBox::iterator it, bool top);
+
     /** respond to the user single-clicking a producible item in the build selector */
-    void    BuildItemLeftClicked(GG::ListBox::iterator it, const GG::Pt& pt);
+    void    BuildItemLeftClicked(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys);
 
     /** respond to the user double-clicking a producible item in the build selector */
-    void    BuildItemDoubleClicked(GG::ListBox::iterator it);
+    void    BuildItemDoubleClicked(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys);
 
     /** respond to the user right-clicking a producible item in the build selector */
-    void    BuildItemRightClicked(GG::ListBox::iterator it, const GG::Pt& pt);
+    void    BuildItemRightClicked(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys);
 
     std::map<BuildType, CUIButton*>         m_build_type_buttons;
     std::vector<CUIButton*>                 m_availability_buttons;
@@ -819,7 +894,7 @@ void BuildDesignatorWnd::BuildSelector::PopulateList() {
 }
 
 void BuildDesignatorWnd::BuildSelector::BuildItemLeftClicked(GG::ListBox::iterator it,
-                                                             const GG::Pt& pt)
+                                                             const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys)
 {
     ProductionItemRow* item_row = dynamic_cast<ProductionItemRow*>(*it);
     if (!item_row)
@@ -846,8 +921,7 @@ void BuildDesignatorWnd::BuildSelector::BuildItemLeftClicked(GG::ListBox::iterat
     }
 }
 
-void BuildDesignatorWnd::BuildSelector::BuildItemDoubleClicked(GG::ListBox::iterator it) {
-    //std::cout << "BuildDesignatorWnd::BuildSelector::BuildItemDoubleClicked" << std::endl;
+void BuildDesignatorWnd::BuildSelector::AddBuildItemToQueue(GG::ListBox::iterator it, bool top) {
     if ((*it)->Disabled())
         return;
     ProductionItemRow* item_row = dynamic_cast<ProductionItemRow*>(*it);
@@ -855,28 +929,63 @@ void BuildDesignatorWnd::BuildSelector::BuildItemDoubleClicked(GG::ListBox::iter
         return;
     const ProductionQueue::ProductionItem& item = item_row->Item();
 
-    if (item.build_type == BT_BUILDING)
-        RequestNamedBuildItemSignal(BT_BUILDING, item.name, 1);
-    else if (item.build_type == BT_SHIP)
-        RequestIDedBuildItemSignal(BT_SHIP, item.design_id, 1);
+    RequestBuildItemSignal(item, 1, top ? 0 : -1);
+}
+
+void BuildDesignatorWnd::BuildSelector::BuildItemDoubleClicked(GG::ListBox::iterator it,
+                                                               const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys)
+{
+    //std::cout << "BuildDesignatorWnd::BuildSelector::BuildItemDoubleClicked" << std::endl;
+    AddBuildItemToQueue(it, modkeys & GG::MOD_KEY_CTRL);
 }
 
 void BuildDesignatorWnd::BuildSelector::BuildItemRightClicked(GG::ListBox::iterator it,
-                                                              const GG::Pt& pt)
+                                                              const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys)
 {
-    if ((*it)->Disabled())
+    ProductionItemRow* item_row = dynamic_cast<ProductionItemRow*>(*it);
+    if (!item_row)
         return;
+    const ProductionQueue::ProductionItem& item = item_row->Item();
+
+    std::string item_name = "";
+    if (item.build_type == BT_BUILDING) {
+        item_name = item.name;
+    } else if (item.build_type == BT_SHIP) {
+        item_name = GetShipDesign(item.design_id)->Name(false);
+    } else {
+        ErrorLogger() << "Invalid build type (" << item.build_type << ") for item";
+        return;
+    }
 
     GG::MenuItem menu_contents;
-    menu_contents.next_level.push_back(GG::MenuItem(UserString("PRODUCTION_DETAIL_ADD_TO_QUEUE"),   1, false, false));
+
+    if (!((*it)->Disabled())) {
+        menu_contents.next_level.push_back(GG::MenuItem(UserString("PRODUCTION_DETAIL_ADD_TO_QUEUE"),   1, false, false));
+        menu_contents.next_level.push_back(GG::MenuItem(UserString("PRODUCTION_DETAIL_ADD_TO_TOP_OF_QUEUE"),   2, false, false));
+    }
+
+    if (UserStringExists(item_name))
+        item_name = UserString(item_name);
+
+    std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % item_name);
+    menu_contents.next_level.push_back(GG::MenuItem(popup_label, 3, false, false));
 
     GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
                         ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
 
     if (popup.Run()) {
         switch (popup.MenuID()) {
-        case 1: { // move item to top
-            BuildItemDoubleClicked(it);
+        case 1: { // add item to bottom of queue
+            AddBuildItemToQueue(it, false);
+            break;
+        }
+        case 2: { // add item to top of queue
+            AddBuildItemToQueue(it, true);
+            break;
+        }
+        case 3: { // pedia lookup
+            ShowPediaSignal();
+            BuildItemLeftClicked(it, pt, modkeys);
             break;
         }
         default:
@@ -888,13 +997,17 @@ void BuildDesignatorWnd::BuildSelector::BuildItemRightClicked(GG::ListBox::itera
 //////////////////////////////////////////////////
 // BuildDesignatorWnd
 //////////////////////////////////////////////////
+const std::string BuildDesignatorWnd::PRODUCTION_ITEM_DROP_TYPE = "Production Item";
+
 BuildDesignatorWnd::BuildDesignatorWnd(GG::X w, GG::Y h) :
     Wnd(GG::X0, GG::Y0, w, h, GG::INTERACTIVE | GG::ONTOP),
     m_enc_detail_panel(0),
     m_build_selector(0),
     m_side_panel(0)
 {
-    m_enc_detail_panel = new EncyclopediaDetailPanel(GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | PINABLE, PROD_PEDIA_WND_NAME);
+    m_enc_detail_panel = new EncyclopediaDetailPanel(GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | CLOSABLE | PINABLE, PROD_PEDIA_WND_NAME);
+    GG::Connect(m_enc_detail_panel->ClosingSignal, boost::bind(&BuildDesignatorWnd::HidePedia, this));     // Wnd is manually closed by user
+
     m_side_panel = new SidePanel(PROD_SIDEPANEL_WND_NAME);
     m_build_selector = new BuildSelector(PROD_SELECTOR_WND_NAME);
     InitializeWindows();
@@ -902,15 +1015,15 @@ BuildDesignatorWnd::BuildDesignatorWnd(GG::X w, GG::Y h) :
 
     m_side_panel->EnableSelection();
 
-    GG::Connect(m_build_selector->DisplayBuildingTypeSignal,    static_cast<void (EncyclopediaDetailPanel::*)(const BuildingType*)>(&EncyclopediaDetailPanel::SetItem),                 m_enc_detail_panel);
-    GG::Connect(m_build_selector->DisplayShipDesignSignal,      static_cast<void (EncyclopediaDetailPanel::*)(const ShipDesign*)>(&EncyclopediaDetailPanel::SetItem),                   m_enc_detail_panel);
-    GG::Connect(m_build_selector->RequestNamedBuildItemSignal,  static_cast<void (BuildDesignatorWnd::*)(BuildType, const std::string&, int)>(&BuildDesignatorWnd::BuildItemRequested), this);
-    GG::Connect(m_build_selector->RequestIDedBuildItemSignal,   static_cast<void (BuildDesignatorWnd::*)(BuildType, int, int)>(&BuildDesignatorWnd::BuildItemRequested),                this);
+    GG::Connect(m_build_selector->DisplayBuildingTypeSignal,    static_cast<void (EncyclopediaDetailPanel::*)(const BuildingType*)>(&EncyclopediaDetailPanel::SetItem),  m_enc_detail_panel);
+    GG::Connect(m_build_selector->DisplayShipDesignSignal,      static_cast<void (EncyclopediaDetailPanel::*)(const ShipDesign*)>(&EncyclopediaDetailPanel::SetItem),    m_enc_detail_panel);
+
+    GG::Connect(m_build_selector->ShowPediaSignal,        &BuildDesignatorWnd::ShowPedia,          this);
+    GG::Connect(m_build_selector->RequestBuildItemSignal, &BuildDesignatorWnd::BuildItemRequested, this);
 
     GG::Connect(m_side_panel->PlanetSelectedSignal, PlanetSelectedSignal);
     GG::Connect(m_side_panel->SystemSelectedSignal, SystemSelectedSignal);
 
-
     // connect build type button clicks to update display
     GG::Connect(m_build_selector->m_build_type_buttons[BT_BUILDING]->LeftClickedSignal, ToggleBuildTypeFunctor(this, BT_BUILDING));
     GG::Connect(m_build_selector->m_build_type_buttons[BT_SHIP]->LeftClickedSignal,     ToggleBuildTypeFunctor(this, BT_SHIP));
@@ -936,7 +1049,7 @@ const std::pair<bool, bool>& BuildDesignatorWnd::GetAvailabilitiesShown() const
 { return m_build_selector->GetAvailabilitiesShown(); }
 
 bool BuildDesignatorWnd::InWindow(const GG::Pt& pt) const
-{ return m_enc_detail_panel->InWindow(pt) || m_build_selector->InWindow(pt) || m_side_panel->InWindow(pt); }
+{ return (m_enc_detail_panel->InWindow(pt) && m_enc_detail_panel->Visible()) || m_build_selector->InWindow(pt) || m_side_panel->InWindow(pt); }
 
 bool BuildDesignatorWnd::InClient(const GG::Pt& pt) const
 { return m_enc_detail_panel->InClient(pt) || m_build_selector->InClient(pt) || m_side_panel->InClient(pt); }
@@ -954,7 +1067,7 @@ void BuildDesignatorWnd::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
     }
 }
 
-void BuildDesignatorWnd::CenterOnBuild(int queue_idx) {
+void BuildDesignatorWnd::CenterOnBuild(int queue_idx, bool open) {
     SetBuild(queue_idx);
 
     const ObjectMap& objects = GetUniverse().Objects();
@@ -974,6 +1087,10 @@ void BuildDesignatorWnd::CenterOnBuild(int queue_idx) {
             int system_id = build_location->SystemID();
             MapWnd* map = ClientUI::GetClientUI()->GetMapWnd();
             map->CenterOnObject(system_id);
+            if (open) {
+                SelectSystem(system_id);
+                SelectPlanet(location_id);
+            }
         }
     }
 }
@@ -1039,17 +1156,19 @@ void BuildDesignatorWnd::Update() {
 }
 
 void BuildDesignatorWnd::InitializeWindows() {
-    const GG::X SIDEPANEL_WIDTH = GG::X(384); // Formerly "UI.sidepanel-width" default
+    GG::X queue_width(GetOptionsDB().Get<int>("UI.queue-width"));
+
+    const GG::X SIDEPANEL_WIDTH(GetOptionsDB().Get<int>("UI.sidepanel-width"));
     const GG::Y PANEL_HEIGHT    = GG::Y(240);
 
-    const GG::Pt pedia_ul(GG::X0, GG::Y0);
-    const GG::Pt pedia_wh(Width() - SIDEPANEL_WIDTH, PANEL_HEIGHT);
+    const GG::Pt pedia_ul(queue_width, GG::Y0);
+    const GG::Pt pedia_wh(Width() - SIDEPANEL_WIDTH - queue_width, PANEL_HEIGHT);
 
     const GG::Pt sidepanel_ul(Width() - SIDEPANEL_WIDTH, GG::Y0);
     const GG::Pt sidepanel_wh(SIDEPANEL_WIDTH, Height());
 
-    const GG::Pt selector_ul(GG::X0, Height() - PANEL_HEIGHT);
-    const GG::Pt selector_wh(Width() - SIDEPANEL_WIDTH, PANEL_HEIGHT);
+    const GG::Pt selector_ul(queue_width, Height() - PANEL_HEIGHT);
+    const GG::Pt selector_wh(Width() - SIDEPANEL_WIDTH - queue_width, PANEL_HEIGHT);
 
     m_enc_detail_panel->InitSizeMove(pedia_ul,      pedia_ul + pedia_wh);
     m_side_panel->      InitSizeMove(sidepanel_ul,  sidepanel_ul + sidepanel_wh);
@@ -1162,25 +1281,61 @@ void BuildDesignatorWnd::ShowBuildingTypeInEncyclopedia(const std::string& build
 void BuildDesignatorWnd::ShowShipDesignInEncyclopedia(int design_id)
 { m_enc_detail_panel->SetDesign(design_id); }
 
+void BuildDesignatorWnd::ShowPlanetInEncyclopedia(int planet_id)
+{ m_enc_detail_panel->SetPlanet(planet_id); }
+
+void BuildDesignatorWnd::ShowTechInEncyclopedia(const std::string& tech_name)
+{ m_enc_detail_panel->SetTech(tech_name); }
+
+void BuildDesignatorWnd::ShowPartTypeInEncyclopedia(const std::string& part_type_name)
+{ m_enc_detail_panel->SetPartType(part_type_name); }
+
+void BuildDesignatorWnd::ShowSpeciesInEncyclopedia(const std::string& species_name)
+{ m_enc_detail_panel->SetSpecies(species_name); }
+
+void BuildDesignatorWnd::ShowEmpireInEncyclopedia(int empire_id)
+{ m_enc_detail_panel->SetEmpire(empire_id); }
+
+void BuildDesignatorWnd::ShowSpecialInEncyclopedia(const std::string& special_name)
+{ m_enc_detail_panel->SetSpecial(special_name); }
+
+void BuildDesignatorWnd::ShowFieldTypeInEncyclopedia(const std::string& field_type_name)
+{ m_enc_detail_panel->SetFieldType(field_type_name); }
+
+void BuildDesignatorWnd::ShowPedia() {
+    m_enc_detail_panel->Refresh();
+    m_enc_detail_panel->Show();
+
+    OptionsDB& db = GetOptionsDB();
+    db.Set("UI.windows." + PROD_PEDIA_WND_NAME + ".persistently-hidden", false);
+}
+
+void BuildDesignatorWnd::HidePedia() {
+    m_enc_detail_panel->Hide();
+
+    OptionsDB& db = GetOptionsDB();
+    db.Set("UI.windows." + PROD_PEDIA_WND_NAME + ".persistently-hidden", true);
+}
+
+void BuildDesignatorWnd::TogglePedia() {
+    if (!m_enc_detail_panel->Visible())
+        ShowPedia();
+    else
+        HidePedia();
+}
+
+bool BuildDesignatorWnd::PediaVisible()
+{ return m_enc_detail_panel->Visible(); }
+
 int BuildDesignatorWnd::BuildLocation() const
 { return m_side_panel->SelectedPlanetID(); }
 
-void BuildDesignatorWnd::BuildItemRequested(BuildType build_type, const std::string& item,
-                                            int num_to_build)
+void BuildDesignatorWnd::BuildItemRequested(const ProductionQueue::ProductionItem& item,
+                                            int num_to_build, int pos)
 {
-    //std::cout << "BuildDesignatorWnd::BuildItemRequested item name: " << item << std::endl;
     const Empire* empire = GetEmpire(HumanClientApp::GetApp()->EmpireID());
-    if (empire && empire->EnqueuableItem(build_type, item, BuildLocation()))
-        AddNamedBuildToQueueSignal(build_type, item, num_to_build, BuildLocation());
-}
-
-void BuildDesignatorWnd::BuildItemRequested(BuildType build_type, int design_id,
-                                            int num_to_build)
-{
-    //std::cout << "BuildDesignatorWnd::BuildItemRequested design id: " << design_id << std::endl;
-    const Empire* empire = GetEmpire(HumanClientApp::GetApp()->EmpireID());
-    if (empire && empire->ProducibleItem(build_type, design_id, BuildLocation()))
-        AddIDedBuildToQueueSignal(build_type, design_id, num_to_build, BuildLocation());
+    if (empire && empire->EnqueuableItem(item, BuildLocation()))
+        AddBuildToQueueSignal(item, num_to_build, BuildLocation(), pos);
 }
 
 void BuildDesignatorWnd::BuildQuantityChanged(int queue_idx, int quantity)
