@@ -4,7 +4,6 @@
 #include <GG/DrawUtil.h>
 #include <GG/StaticGraphic.h>
 
-#include "../util/Directories.h"
 #include "../util/i18n.h"
 #include "../util/Logger.h"
 #include "../util/OptionsDB.h"
@@ -19,7 +18,6 @@
 #include "MapWnd.h"
 #include "MultiIconValueIndicator.h"
 #include "MultiMeterStatusBar.h"
-#include "ShaderProgram.h"
 
 namespace {
     /** How big we want meter icons with respect to the current UI font size.
@@ -50,7 +48,7 @@ namespace {
 }
 
 BuildingsPanel::BuildingsPanel(GG::X w, int columns, int planet_id) :
-    AccordionPanel(w),
+    AccordionPanel(w, GG::Y(ClientUI::Pts()*2)),
     m_planet_id(planet_id),
     m_columns(columns),
     m_building_indicators()
@@ -155,12 +153,14 @@ void BuildingsPanel::Update() {
 
         double total_cost;
         int total_turns;
+        double turn_spending = elem.allocated_pp;
         boost::tie(total_cost, total_turns) = empire->ProductionCostAndTime(elem);
 
         double progress = std::max(0.0f, empire->ProductionStatus(queue_index));
         double turns_completed = total_turns * std::min<double>(1.0, progress / total_cost);
         BuildingIndicator* ind = new BuildingIndicator(GG::X(indicator_size), elem.item.name,
-                                                       turns_completed, total_turns);
+                                                       turns_completed, total_turns, total_cost, turn_spending);
+
         m_building_indicators.push_back(ind);
     }
 }
@@ -256,7 +256,7 @@ std::map<int, bool> BuildingsPanel::s_expanded_map;
 /////////////////////////////////////
 //       BuildingIndicator         //
 /////////////////////////////////////
-boost::shared_ptr<ShaderProgram> BuildingIndicator::s_scanline_shader;
+ScanlineRenderer BuildingIndicator::s_scanline_shader;
 
 BuildingIndicator::BuildingIndicator(GG::X w, int building_id) :
     GG::Wnd(GG::X0, GG::Y0, w, GG::Y(Value(w)), GG::INTERACTIVE),
@@ -272,7 +272,7 @@ BuildingIndicator::BuildingIndicator(GG::X w, int building_id) :
 }
 
 BuildingIndicator::BuildingIndicator(GG::X w, const std::string& building_type,
-                                     double turns_completed, double total_turns) :
+                                     double turns_completed, double total_turns, double total_cost, double turn_spending) :
     GG::Wnd(GG::X0, GG::Y0, w, GG::Y(Value(w)), GG::INTERACTIVE),
     m_graphic(0),
     m_scrap_indicator(0),
@@ -291,7 +291,7 @@ BuildingIndicator::BuildingIndicator(GG::X w, const std::string& building_type,
     m_graphic = new GG::StaticGraphic(texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
     AttachChild(m_graphic);
 
-    m_progress_bar = new MultiTurnProgressBar(total_turns, turns_completed, GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
+    m_progress_bar = new MultiTurnProgressBar(total_turns, turns_completed, total_cost, turn_spending, GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
                                               ClientUI::TechWndProgressBarColor(), GG::LightColor(ClientUI::ResearchableTechFillColor()));
     AttachChild(m_progress_bar);
 
@@ -308,36 +308,37 @@ void BuildingIndicator::Render() {
 
     // Scanlines for not currently-visible objects?
     int empire_id = HumanClientApp::GetApp()->EmpireID();
-    if (!s_scanline_shader || empire_id == ALL_EMPIRES || !GetOptionsDB().Get<bool>("UI.system-fog-of-war"))
+    if (empire_id == ALL_EMPIRES || !GetOptionsDB().Get<bool>("UI.system-fog-of-war"))
         return;
     if (m_building_id == INVALID_OBJECT_ID)
         return;
     if (GetUniverse().GetObjectVisibilityByEmpire(m_building_id, empire_id) >= VIS_BASIC_VISIBILITY)
         return;
 
-    float fog_scanline_spacing = static_cast<float>(GetOptionsDB().Get<double>("UI.system-fog-of-war-spacing"));
-    s_scanline_shader->Use();
-    s_scanline_shader->Bind("scanline_spacing", fog_scanline_spacing);
-    glBegin(GL_POLYGON);
-        glColor(GG::CLR_WHITE);
-        glVertex(ul.x, ul.y);
-        glVertex(lr.x, ul.y);
-        glVertex(lr.x, lr.y);
-        glVertex(ul.x, lr.y);
-        glVertex(ul.x, ul.y);
-    glEnd();
-    s_scanline_shader->stopUse();
+    s_scanline_shader.StartUsing();
+
+    GLfloat verts[8];
+    verts[0] = Value(ul.x); verts[1] = Value(ul.y);
+    verts[2] = Value(lr.x); verts[3] = Value(ul.y);
+    verts[4] = Value(lr.x); verts[5] = Value(lr.y);
+    verts[6] = Value(ul.x); verts[7] = Value(lr.y);
+
+    glDisable(GL_TEXTURE_2D);
+    glPushClientAttrib(GL_CLIENT_ALL_ATTRIB_BITS);
+    glEnableClientState(GL_VERTEX_ARRAY);
+    glDisableClientState(GL_COLOR_ARRAY);
+    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+    glVertexPointer(2, GL_FLOAT, 0, verts);
+    glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
+
+    glPopClientAttrib();
+    glEnable(GL_TEXTURE_2D);
+
+    s_scanline_shader.StopUsing();
 }
 
 void BuildingIndicator::Refresh() {
-    if (!s_scanline_shader && GetOptionsDB().Get<bool>("UI.system-fog-of-war")) {
-        boost::filesystem::path shader_path = GetRootDataDir() / "default" / "shaders" / "scanlines.frag";
-        std::string shader_text;
-        ReadFile(shader_path, shader_text);
-        s_scanline_shader = boost::shared_ptr<ShaderProgram>(
-            ShaderProgram::shaderProgramFactory("", shader_text));
-    }
-
     SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
 
     TemporaryPtr<const Building> building = GetBuilding(m_building_id);
@@ -363,8 +364,8 @@ void BuildingIndicator::Refresh() {
         std::string desc = UserString(type->Description());
         if (building->GetMeter(METER_STEALTH))
             desc = UserString("METER_STEALTH") + boost::io::str(boost::format(": %3.1f\n\n") % building->GetMeter(METER_STEALTH)->Current()) + desc;
-        if (GetOptionsDB().Get<bool>("UI.autogenerated-effects-descriptions") && !type->Effects().empty())
-            desc += boost::io::str(FlexibleFormat(UserString("ENC_EFFECTS_STR")) % AutoGeneratedDescription(type->Effects()));
+        if (GetOptionsDB().Get<bool>("UI.dump-effects-descriptions") && !type->Effects().empty())
+            desc += "\n" + Dump(type->Effects());
 
         SetBrowseInfoWnd(boost::shared_ptr<GG::BrowseInfoWnd>(
             new IconTextBrowseWnd(texture, UserString(type->Name()), desc)));
@@ -443,7 +444,7 @@ void BuildingIndicator::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
             std::map<int, int> pending_scrap_orders = PendingScrapOrders();
             std::map<int, int>::const_iterator it = pending_scrap_orders.find(building->ID());
             if (it != pending_scrap_orders.end()) {
-                HumanClientApp::GetApp()->Orders().RecindOrder(it->second);
+                HumanClientApp::GetApp()->Orders().RescindOrder(it->second);
             break;
             }
         }
