@@ -1,11 +1,10 @@
-// -*- C++ -*-
-//CUIControls.h
 #ifndef _CUIControls_h_
 #define _CUIControls_h_
 
 #include "ClientUI.h"
 #include "CUIDrawUtil.h"
 #include "CUIWnd.h"
+#include "TextBrowseWnd.h"
 #include <GG/Button.h>
 #include <GG/DropDownList.h>
 #include <GG/Edit.h>
@@ -22,11 +21,13 @@
 
 #include "LinkText.h"
 
-
-//! \file All CUI* classes are FreeOrion-style controls incorporating 
-//! the visual theme the project requires.  Implementation may
-//! depend on graphics and design team specifications.  They extend
-//! GG controls.
+/** \file
+ *
+ * All CUI* classes are FreeOrion-style controls incorporating
+ * the visual theme the project requires.  Implementation may
+ * depend on graphics and design team specifications.  They extend
+ * GG controls.
+ */
 
 /** a FreeOrion Label control */
 class CUILabel : public GG::TextControl {
@@ -36,6 +37,9 @@ public:
              GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE,
              GG::Flags<GG::WndFlag> flags = GG::NO_WND_FLAGS);
     //@}
+    /** \name Mutators */ //@{
+    virtual void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    //@}
 };
 
 /** a FreeOrion Button control */
@@ -115,26 +119,38 @@ private:
     bool             m_fill_background_with_wnd_color;
 };
 
+/** \brief A FreeOrion styled check box state button. */
+class CUICheckBoxRepresenter : public GG::StateButtonRepresenter {
+public:
+    virtual void Render(const GG::StateButton& button) const;
+
+    virtual void OnChecked(bool checked) const;
+};
+
+/** \brief A FreeOrion styled radio state button. */
+class CUIRadioRepresenter : public GG::StateButtonRepresenter {
+public:
+    virtual void Render(const GG::StateButton& button) const;
+
+    virtual void OnChecked(bool checked) const;
+};
+
+/** \brief A FreeOrion styled TabBar tab. */
+class CUITabRepresenter : public GG::StateButtonRepresenter {
+public:
+    virtual void Render(const GG::StateButton& button) const;
+
+    virtual void OnChecked(bool checked) const;
+
+    virtual GG::Pt MinUsableSize(const GG::StateButton& button) const;
+};
+
 /** a FreeOrion StateButton control */
 class CUIStateButton : public GG::StateButton {
 public:
     /** \name Structors */ //@{
-    CUIStateButton(const std::string& str, GG::Flags<GG::TextFormat> format, GG::StateButtonStyle style = GG::SBSTYLE_3D_CHECKBOX); ///< ctor
+    CUIStateButton(const std::string& str, GG::Flags<GG::TextFormat> format, boost::shared_ptr<GG::StateButtonRepresenter> representer); ///< ctor
     //@}
-
-    /** \name Accessors */ //@{
-    virtual GG::Pt MinUsableSize() const;
-    //@}
-
-    /** \name Mutators */ //@{
-    virtual void   Render();
-    virtual void   MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void   MouseLeave();
-    //@}
-
-private:
-    GG::Clr m_border_color;
-    bool    m_mouse_here;
 };
 
 /** Tab bar with buttons for selecting tabbed windows. */
@@ -143,7 +159,7 @@ public:
     /** \name Structors */ ///@{
     /** Basic ctor. */
     CUITabBar(const boost::shared_ptr<GG::Font>& font, GG::Clr color,
-              GG::Clr text_color, GG::TabBarStyle style);
+              GG::Clr text_color);
     //@}
 
 private:
@@ -200,22 +216,24 @@ public:
 class CUIDropDownList : public GG::DropDownList {
 public:
     /** \name Structors */ //@{
-    CUIDropDownList(size_t num_shown_elements); ///< basic ctor
+    explicit CUIDropDownList(size_t num_shown_elements); ///< basic ctor
     //@}
 
     /** \name Mutators */ //@{
-    virtual void Render();
-    virtual void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void MouseLeave();
+    virtual void    Render();
+    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    virtual void    MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    virtual void    MouseLeave();
 
-    void         DisableDropArrow();  ///< disables rendering of the small downward-facing arrow on the right of the control
-    void         EnableDropArrow();   ///< enables rendering of the small downward-facing arrow on the right of the control
+    void            DisableDropArrow();  ///< disables rendering of the small downward-facing arrow on the right of the control
+    void            EnableDropArrow();   ///< enables rendering of the small downward-facing arrow on the right of the control
     //@}
 
 private:
-    bool m_render_drop_arrow;
-    bool m_mouse_here;
+    virtual void    InitBuffer();
+
+    bool    m_render_drop_arrow;
+    bool    m_mouse_here;
 };
 
 /** a FreeOrion Edit control */
@@ -226,6 +244,8 @@ public:
     //@}
 
     /** \name Mutators */ //@{
+    virtual void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    virtual void KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
     virtual void GainingFocus();
     virtual void LosingFocus();
     virtual void Render();
@@ -243,7 +263,8 @@ public:
     //@}
 
     /** \name Mutators */ //@{
-    virtual void Render();
+    virtual void    Render();
+    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     //@}
 };
 
@@ -264,7 +285,6 @@ public:
     /** \name Mutators */ //@{
     virtual void    Render();
     virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    LDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    MouseLeave();
@@ -442,8 +462,8 @@ public:
 class ProductionInfoPanel : public CUIWnd {
 public:
     /** \name Structors */ //@{
-    ProductionInfoPanel(const std::string& title, const std::string& point_units_str,
-                        GG::X w = GG::X1, GG::Y h = GG::Y1);
+    ProductionInfoPanel(const std::string& title, const std::string& point_units_str, const GG::X x, const GG::Y y,
+                        const GG::X w, const GG::Y h, const std::string& config_name);
     //@}
 
     /** \name Accessors */ //@{
@@ -455,7 +475,9 @@ public:
 
     void            SetTotalPointsCost(float total_points, float total_cost);
     void            SetLocalPointsCost(float local_points, float local_cost, const std::string& location_name);
+    void            SetEmpireID(int empire_id);
     void            ClearLocalInfo();
+    void            Clear();
     //@}
 
 private:
@@ -463,6 +485,7 @@ private:
 
     std::string m_units_str;
     std::string m_title_str;
+    int         m_empire_id;
 
     GG::Label*  m_total_points_label;
     GG::Label*  m_total_points;
@@ -484,7 +507,7 @@ class MultiTurnProgressBar : public GG::Control {
 public:
     /** \name Structors */ //@{
     /** ctor */
-    MultiTurnProgressBar(int total_turns, double turns_completed,
+    MultiTurnProgressBar(int total_turns, double turns_completed, double total_cost, double turn_spending,
                          const GG::Clr& bar_color, const GG::Clr& background, const GG::Clr& outline_color);
     //@}
 
@@ -495,6 +518,8 @@ public:
 private:
     int     m_total_turns;
     double  m_turns_completed;
+    double  m_total_cost;
+    double  m_turn_spending;
     GG::Clr m_bar_color;
     GG::Clr m_background;
     GG::Clr m_outline_color;
@@ -510,8 +535,8 @@ private:
     bool m_enabled;
 };
 
-/** Functions like a StaticGraphic, except can have multiple textures that are rendered bottom to top
-  * in the order, rather than a single texture. */
+/** Functions like a StaticGraphic, except can have multiple textures rendered
+  * on top of eachother, rather than just a single texture. */
 class MultiTextureStaticGraphic : public GG::Control {
 public:
     /** \name Structors */ ///@{
@@ -551,4 +576,60 @@ private:
     std::vector<GG::Flags<GG::GraphicStyle> >   m_styles;   ///< position of texture wrt the window area
 };
 
+/** Functions like a StaticGraphic, except can be rotated with a fixed phase
+  * and/or at a continuous angular rate. */
+class RotatingGraphic : public GG::StaticGraphic {
+public:
+    /** \name Structors */ ///@{
+    RotatingGraphic(const boost::shared_ptr<GG::Texture>& texture, GG::Flags<GG::GraphicStyle> style = GG::GRAPHIC_NONE,
+                    GG::Flags<GG::WndFlag> flags = GG::NO_WND_FLAGS);
+    //@}
+
+    /** \name Mutators */ ///@{
+    void            SetRPM(float rpm)               { m_rpm = std::max(-3600.0f, std::min(3600.0f, rpm)); }
+    void            SetPhaseOffset(float degrees)   { m_phase_offset = degrees; }
+    virtual void    Render();
+    //@}
+
+private:
+    float   m_rpm;
+    float   m_phase_offset;
+};
+
+/** \brief Represents a CUIToggleButton
+ * 
+ * Renders a SubTexture depending on the checked state of the CUIToggleButton.
+ * When the \a button has single texture and is unchecked, it renders the same icon recolored towards grey.
+ * If a second SubTexture is assigned to the \a button, it will render the appropriate SubTexture.
+ * When a mouse pointer hovers over this button, the opposing SubTexture and color are used and highlighted
+ */
+class CUIToggleRepresenter : public GG::StateButtonRepresenter {
+public:
+    /** \name Structors */ //@{
+    CUIToggleRepresenter(boost::shared_ptr<GG::SubTexture> icon, const GG::Clr& highlight_clr); ///< ctor
+    CUIToggleRepresenter(boost::shared_ptr<GG::SubTexture> unchecked_icon, const GG::Clr& unchecked_clr,
+                         boost::shared_ptr<GG::SubTexture> checked_icon, const GG::Clr& checked_clr); ///< ctor
+    //@}
+
+    /** \name Mutators */ //@{
+    virtual void                        OnChecked(bool checked) const;
+    virtual void                        Render(const GG::StateButton& button) const;
+    //@}
+
+private:
+    boost::shared_ptr<GG::SubTexture>   m_unchecked_icon;
+    boost::shared_ptr<GG::SubTexture>   m_checked_icon;
+    GG::Clr                             m_unchecked_color;
+    GG::Clr                             m_checked_color;
+};
+
+/** Renders scanlines over its area. */
+class ScanlineControl : public GG::Control {
+public:
+    ScanlineControl(GG::X x, GG::Y y, GG::X w, GG::Y h, bool square = false);
+    virtual void Render();
+private:
+    bool m_square;
+};
+
 #endif // _CUIControls_h_
