@@ -55,13 +55,8 @@ void CUI_MinRestoreButton::Render() {
     if (m_mode == MIN_BUTTON) {
         // draw a dash to signify the minimize command
         GG::Y middle_y = (lr.y + ul.y) / 2;
-        glDisable(GL_TEXTURE_2D);
         glColor(color_to_use);
-        glBegin(GL_LINES);
-        glVertex(ul.x, middle_y);
-        glVertex(lr.x, middle_y);
-        glEnd();
-        glEnable(GL_TEXTURE_2D);
+        GG::Line(ul.x, middle_y, lr.x, middle_y);
     } else {
         // draw a square to signify the restore command
         GG::FlatRectangle(ul, lr, GG::CLR_ZERO, ClientUI::WndInnerBorderColor(), 1);
@@ -79,18 +74,6 @@ void CUI_MinRestoreButton::Toggle() {
 }
 
 
-////////////////////////////////////////////////
-// CUI_CloseButton
-////////////////////////////////////////////////
-CUI_CloseButton::CUI_CloseButton() :
-    GG::Button("", boost::shared_ptr<GG::Font>(), ClientUI::WndInnerBorderColor())
-{
-    GG::Connect(LeftClickedSignal, &PlayCloseSound, -1);
-    SetUnpressedGraphic(GG::SubTexture(ClientUI::GetTexture( ClientUI::ArtDir() / "icons" / "buttons" / "close.png"   )));
-    SetPressedGraphic  (GG::SubTexture(ClientUI::GetTexture( ClientUI::ArtDir() / "icons" / "buttons" / "close_clicked.png"  )));
-    SetRolloverGraphic (GG::SubTexture(ClientUI::GetTexture( ClientUI::ArtDir() / "icons" / "buttons" / "close_mouseover.png")));
-}
-
 ////////////////////////////////////////////////
 // CUI_PinButton
 ////////////////////////////////////////////////
@@ -133,10 +116,8 @@ namespace {
 }
 
 const GG::Y CUIWnd::BUTTON_TOP_OFFSET(3);
-const GG::X CUIWnd::BUTTON_RIGHT_OFFSET(15);
-const GG::X CUIWnd::MINIMIZED_WND_WIDTH(150);
+const GG::X CUIWnd::MINIMIZED_WND_WIDTH(50);
 const GG::X CUIWnd::BORDER_LEFT(5);
-const GG::Y CUIWnd::BORDER_TOP(21);
 const GG::X CUIWnd::BORDER_RIGHT(5);
 const GG::Y CUIWnd::BORDER_BOTTOM(5);
 const int CUIWnd::OUTER_EDGE_ANGLE_OFFSET = 8;
@@ -145,7 +126,7 @@ const int CUIWnd::TITLE_OFFSET = 3;
 const int CUIWnd::RESIZE_HASHMARK1_OFFSET = 7;
 const int CUIWnd::RESIZE_HASHMARK2_OFFSET = 3;
 
-CUIWnd::CUIWnd(const std::string& t,
+CUIWnd::CUIWnd(const std::string& wnd_name,
                GG::X x, GG::Y y,
                GG::X w, GG::Y h,
                GG::Flags<GG::WndFlag> flags,
@@ -164,21 +145,19 @@ CUIWnd::CUIWnd(const std::string& t,
     m_config_name(AddWindowOptions(config_name, x, y, w, h, visible, false, false)),
     m_close_button(0),
     m_minimize_button(0),
-    m_pin_button(0)
+    m_pin_button(0),
+    m_vertex_buffer(),
+    m_buffer_indices()
 {
-    Init(t);
+    Init(wnd_name);
     if (!m_config_name.empty()) {
         // Default position was already supplied
-        GetOptionsDB().Set<bool>("UI.windows."+m_config_name+".initialized", true);
+        GetOptionsDB().Set<bool>("UI.windows." + m_config_name + ".initialized", true);
     }
     ValidatePosition();
-    InitBuffers();
 }
 
-CUIWnd::CUIWnd(const std::string& t,
-               GG::Flags<GG::WndFlag> flags,
-               const std::string& config_name,
-               bool visible) :
+CUIWnd::CUIWnd(const std::string& wnd_name, GG::Flags<GG::WndFlag> flags, const std::string& config_name, bool visible) :
     GG::Wnd(INVALID_X, INVALID_Y, GG::X1, GG::Y1, flags & ~GG::RESIZABLE),
     m_resizable(flags & GG::RESIZABLE),
     m_closable(flags & CLOSABLE),
@@ -192,11 +171,13 @@ CUIWnd::CUIWnd(const std::string& t,
     m_config_name(AddWindowOptions(config_name, INVALID_POS, INVALID_POS, 1, 1, visible, false, false)),
     m_close_button(0),
     m_minimize_button(0),
-    m_pin_button(0)
-{ Init(t); }
+    m_pin_button(0),
+    m_vertex_buffer(),
+    m_buffer_indices()
+{ Init(wnd_name); }
 
-void CUIWnd::Init(const std::string& t) {
-    SetName(t);
+void CUIWnd::Init(const std::string& wnd_name) {
+    SetName(wnd_name);
     InitButtons();
     SetChildClippingMode(ClipToClientAndWindowSeparately);
 
@@ -215,31 +196,27 @@ void CUIWnd::Init(const std::string& t) {
         GG::Connect(HumanClientApp::GetApp()->WindowResizedSignal, boost::bind(&CUIWnd::ResetDefaultPosition, this));
 
     // call to CUIWnd::MinimizedWidth() because MinimizedWidth is virtual
-    SetMinSize(GG::Pt(CUIWnd::MinimizedSize().x, BORDER_TOP + INNER_BORDER_ANGLE_OFFSET + BORDER_BOTTOM + 50));
+    SetMinSize(GG::Pt(CUIWnd::MinimizedSize().x, TopBorder() + INNER_BORDER_ANGLE_OFFSET + BORDER_BOTTOM + 50));
 }
 
 void CUIWnd::InitSizeMove(const GG::Pt& ul, const GG::Pt& lr) {
     OptionsDB& db = GetOptionsDB();
 
     if (!m_config_name.empty()) {
-        if (db.OptionExists("UI.windows."+m_config_name+".initialized")) {
+        if (db.OptionExists("UI.windows." + m_config_name + ".initialized")) {
             std::string windowed = ""; // empty string in fullscreen mode, appends -windowed in windowed mode
             if (!db.Get<bool>("fullscreen"))
                 windowed = "-windowed";
             // If the window has already had its default position specified
             // (either in the ctor or a previous call to this function), apply
             // this position to the window.
-            if (db.Get<bool>("UI.windows."+m_config_name+".initialized") ||
-                db.Get<int>("UI.windows."+m_config_name+".left"+windowed) == INVALID_X)
-            {
-                SizeMove(ul, lr);
-            }
+            if (db.Get<bool>("UI.windows." + m_config_name + ".initialized") ||
+                db.Get<int>("UI.windows." + m_config_name + ".left" + windowed) == INVALID_X)
+            { SizeMove(ul, lr); }
             db.Set<bool>("UI.windows."+m_config_name+".initialized", true);
         } else {
-            ErrorLogger() << "CUIWnd::InitSizeMove() : attempted to check if "
-                             "window using name \"" << m_config_name << "\" "
-                             "was initialized but the options do not appear "
-                             "to be registered in the OptionsDB.";
+            ErrorLogger() << "CUIWnd::InitSizeMove() : attempted to check if window using name \"" << m_config_name
+                          << "\" was initialized but the options do not appear to be registered in the OptionsDB.";
         }
     } else {
         SizeMove(ul, lr);
@@ -248,15 +225,13 @@ void CUIWnd::InitSizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 
 CUIWnd::~CUIWnd() {
     try {
-        if (!m_config_name.empty() && GetOptionsDB().OptionExists("UI.windows."+m_config_name+".initialized"))
+        if (!m_config_name.empty() && GetOptionsDB().OptionExists("UI.windows." + m_config_name + ".initialized"))
             GetOptionsDB().Remove("UI.windows."+m_config_name+".initialized");
     } catch (std::exception& e) { // catch std::runtime_error, boost::bad_any_cast
-        ErrorLogger() << "CUIWnd::~CUIWnd() : caught exception while removing \"UI.windows."<<m_config_name<<".initialized\": " << e.what();
+        ErrorLogger() << "CUIWnd::~CUIWnd() : caught exception while removing \"UI.windows." << m_config_name
+                      << ".initialized\": " << e.what();
     }
-    m_minimized_buffer.clear();
-    m_outer_border_buffer.clear();
-    m_inner_border_buffer.clear();
-    m_resize_corner_lines_buffer.clear();
+    m_vertex_buffer.clear();
 }
 
 void CUIWnd::ValidatePosition()
@@ -264,9 +239,9 @@ void CUIWnd::ValidatePosition()
 
 void CUIWnd::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
     GG::Pt old_sz = Size();
-    if (m_config_save) {
-        GG::Pt available_size;
+    if (m_config_save) {    // can write position/size to OptionsDB
 
+        GG::Pt available_size;
         if (const GG::Wnd* parent = Parent()) {
             // Keep this CUIWnd entirely inside its parent.
             available_size = parent->ClientSize();
@@ -274,8 +249,9 @@ void CUIWnd::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
             // Keep this CUIWnd entirely inside the application window.
             available_size = GG::Pt(app->AppWidth(), app->AppHeight());
         } else {
+            available_size = GG::Pt(GG::X(HumanClientApp::MaximumPossibleWidth()),
+                                    GG::Y(HumanClientApp::MaximumPossibleHeight()));
             ErrorLogger() << "CUIWnd::SizeMove() could not get app instance!";
-            return;
         }
 
         // Limit window size to be no larger than the containing window.
@@ -284,64 +260,70 @@ void CUIWnd::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 
         // Clamp position of this window to keep its entire area visible in the
         // containing window.
-        GG::Pt new_ul(std::min(available_size.x - new_size.x,
-                               std::max(GG::X0, ul.x)),
-                      std::min(available_size.y - new_size.y,
-                               std::max(GG::Y0, ul.y)));
+        GG::Pt new_ul(std::min(available_size.x - new_size.x, std::max(GG::X0, ul.x)),
+                      std::min(available_size.y - new_size.y, std::max(GG::Y0, ul.y)));
 
         Wnd::SizeMove(new_ul, new_ul + new_size);
-    } else {
+
+    } else {    // don't write position/size to OptionsDB
         Wnd::SizeMove(ul, lr);
     }
-    if (Size() != old_sz) {
+
+    if (Size() != old_sz)
         PositionButtons();
-        InitBuffers();
-    }
 
     SaveOptions();
+    m_vertex_buffer.clear();    // force buffer re-init on next Render call
 }
 
 void CUIWnd::Render() {
-    GG::Pt ul = UpperLeft();
-    GG::Pt lr = LowerRight();
+    if (m_vertex_buffer.empty())
+        InitBuffers();
 
-    glPushMatrix();
-    glLoadIdentity();
-    glTranslatef(static_cast<GLfloat>(Value(ul.x)), static_cast<GLfloat>(Value(ul.y)), 0.0f);
     glDisable(GL_TEXTURE_2D);
     glLineWidth(1.0f);
+
+    glPushClientAttrib(GL_CLIENT_ALL_ATTRIB_BITS);
+
     glEnableClientState(GL_VERTEX_ARRAY);
+    glDisableClientState(GL_COLOR_ARRAY);
+    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+    m_vertex_buffer.activate();
+
+    // within m_vertex_buffer:
+    // [0] is the start and range for minimized background triangle fan and minimized border line loop
+    // [1] is ... the background fan / outer border line loop
+    // [2] is ... the inner border line loop
+    // [3] is ... the resize tab line list
 
     if (m_minimized) {
-        m_minimized_buffer.activate();
         glColor(ClientUI::WndColor());
-        glDrawArrays(GL_TRIANGLE_FAN,   0, m_minimized_buffer.size() - 1);
+        glDrawArrays(GL_TRIANGLE_FAN,   m_buffer_indices[0].first, m_buffer_indices[0].second);
         glColor(ClientUI::WndOuterBorderColor());
-        glDrawArrays(GL_LINE_STRIP,     0, m_minimized_buffer.size());
+        glDrawArrays(GL_LINE_LOOP,      m_buffer_indices[0].first, m_buffer_indices[0].second);
 
     } else {
-        m_outer_border_buffer.activate();
         glColor(ClientUI::WndColor());
-        glDrawArrays(GL_TRIANGLE_FAN,   0, m_outer_border_buffer.size() - 1);
+        glDrawArrays(GL_TRIANGLE_FAN,   m_buffer_indices[1].first, m_buffer_indices[1].second);
         glColor(ClientUI::WndOuterBorderColor());
-        glDrawArrays(GL_LINE_STRIP,     0, m_outer_border_buffer.size());
-
-        m_inner_border_buffer.activate();
+        glDrawArrays(GL_LINE_LOOP,      m_buffer_indices[1].first, m_buffer_indices[1].second);
         glColor(ClientUI::WndInnerBorderColor());
-        glDrawArrays(GL_LINE_STRIP,     0, m_inner_border_buffer.size());
+        glDrawArrays(GL_LINE_LOOP,      m_buffer_indices[2].first, m_buffer_indices[2].second);
 
         if (m_resizable) {
-            m_resize_corner_lines_buffer.activate();
             GG::Clr tab_lines_colour = m_mouse_in_resize_tab ? ClientUI::WndInnerBorderColor() : ClientUI::WndOuterBorderColor();
             glColor(tab_lines_colour);
-            glDrawArrays(GL_LINES,          0, m_resize_corner_lines_buffer.size());
+            glDrawArrays(GL_LINES,      m_buffer_indices[3].first, m_buffer_indices[3].second);
         }
     }
 
     glEnable(GL_TEXTURE_2D);
-    glPopMatrix();
-    glDisableClientState(GL_VERTEX_ARRAY);
 
+    glPopClientAttrib();
+
+    GG::Pt ul = UpperLeft();
+    GG::Pt lr = LowerRight();
     GG::BeginScissorClipping(ul, lr);
     glColor(ClientUI::TextColor());
     boost::shared_ptr<GG::Font> font = ClientUI::GetTitleFont();
@@ -416,7 +398,7 @@ void CUIWnd::MouseLeave() {
 }
 
 GG::Pt CUIWnd::ClientUpperLeft() const
-{ return m_minimized ? UpperLeft() : UpperLeft() + GG::Pt(BORDER_LEFT, BORDER_TOP); }
+{ return m_minimized ? UpperLeft() : UpperLeft() + GG::Pt(BORDER_LEFT, TopBorder()); }
 
 GG::Pt CUIWnd::ClientLowerRight() const
 { return m_minimized ? LowerRight() : LowerRight() - GG::Pt(BORDER_RIGHT, BORDER_BOTTOM); }
@@ -434,16 +416,16 @@ bool CUIWnd::InWindow(const GG::Pt& pt) const {
 
 void CUIWnd::PositionButtons() {
     // The buttons are to be positioned based on the presence of other buttons
-    GG::Pt button_ul = GG::Pt(Width() - BUTTON_RIGHT_OFFSET, BUTTON_TOP_OFFSET) + UpperLeft() - ClientUpperLeft();
+    GG::Pt button_ul = GG::Pt(Width() - ClientUI::Pts() - BORDER_RIGHT, BUTTON_TOP_OFFSET) + UpperLeft() - ClientUpperLeft();
 
     if (m_close_button) {
         m_close_button->MoveTo(GG::Pt(button_ul.x, button_ul.y));
-        button_ul -= GG::Pt(m_close_button->Width(), GG::Y0) + GG::Pt(GG::X(4), GG::Y0);
+        button_ul -= GG::Pt(m_close_button->Width(), GG::Y0) + GG::Pt(GG::X(ClientUI::TitlePts()/3), GG::Y0);
     }
 
     if (m_minimize_button) {
         m_minimize_button->MoveTo(GG::Pt(button_ul.x, button_ul.y));
-        button_ul -= GG::Pt(m_minimize_button->Width(), GG::Y0) + GG::Pt(GG::X(4), GG::Y0);
+        button_ul -= GG::Pt(m_minimize_button->Width(), GG::Y0) + GG::Pt(GG::X(ClientUI::TitlePts()/3), GG::Y0);
     }
 
     if (m_pin_button)
@@ -451,10 +433,17 @@ void CUIWnd::PositionButtons() {
 }
 
 void CUIWnd::InitButtons() {
+    boost::filesystem::path button_texture_dir = ClientUI::ArtDir() / "icons" / "buttons";
+
     // create the close button
     if (m_closable) {
-        m_close_button = new CUI_CloseButton();
-        m_close_button->Resize(GG::Pt(GG::X(12), GG::Y(12)));
+        m_close_button = new CUIButton(
+            GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "close.png")),
+            GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "close_clicked.png")),
+            GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "close_mouseover.png")));
+        m_close_button->SetColor(ClientUI::WndInnerBorderColor());
+        GG::Connect(m_close_button->LeftClickedSignal, &PlayCloseSound, -1);
+        m_close_button->Resize(GG::Pt(GG::X(ClientUI::TitlePts()), GG::Y(ClientUI::TitlePts())));
         GG::Connect(m_close_button->LeftClickedSignal, &CUIWnd::CloseClicked, this);
         AttachChild(m_close_button);
         m_close_button->NonClientChild(true);
@@ -463,7 +452,7 @@ void CUIWnd::InitButtons() {
     // create the minimize button
     if (m_minimizable) {
         m_minimize_button = new CUI_MinRestoreButton();
-        m_minimize_button->Resize(GG::Pt(GG::X(12), GG::Y(12)));
+        m_minimize_button->Resize(GG::Pt(GG::X(ClientUI::TitlePts()), GG::Y(ClientUI::TitlePts())));
         GG::Connect(m_minimize_button->LeftClickedSignal, &CUIWnd::MinimizeClicked, this);
         AttachChild(m_minimize_button);
         m_minimize_button->NonClientChild(true);
@@ -472,7 +461,7 @@ void CUIWnd::InitButtons() {
     // create the pin button
     if (m_pinable) {
         m_pin_button = new CUI_PinButton();
-        m_pin_button->Resize(GG::Pt(GG::X(12), GG::Y(12)));
+        m_pin_button->Resize(GG::Pt(GG::X(ClientUI::TitlePts()), GG::Y(ClientUI::TitlePts())));
         GG::Connect(m_pin_button->LeftClickedSignal, &CUIWnd::PinClicked, this);
         AttachChild(m_pin_button);
         m_pin_button->NonClientChild(true);
@@ -483,13 +472,13 @@ void CUIWnd::InitButtons() {
 }
 
 GG::Pt CUIWnd::MinimizedSize() const
-{ return GG::Pt(MINIMIZED_WND_WIDTH, BORDER_TOP); }
+{ return GG::Pt(MINIMIZED_WND_WIDTH, TopBorder()); }
 
 GG::X CUIWnd::LeftBorder() const
 { return BORDER_LEFT; }
 
 GG::Y CUIWnd::TopBorder() const
-{ return BORDER_TOP; }
+{ return GG::Y(ClientUI::TitlePts()*3/2); }
 
 GG::X CUIWnd::RightBorder() const
 { return BORDER_RIGHT; }
@@ -517,7 +506,7 @@ void CUIWnd::PinClicked() {
     m_pinned = !m_pinned;
     m_resizable = !m_pinned;
     m_pin_button->Toggle(m_pinned); // Change the icon on the pin button
-    InitBuffers();
+    m_vertex_buffer.clear();        // force buffer re-init on next Render call
     SaveOptions();
 }
 
@@ -540,7 +529,7 @@ void CUIWnd::MinimizeClicked() {
 
     } else {
         m_minimized = false;
-        SetMinSize(GG::Pt(MinimizedSize().x, BORDER_TOP + INNER_BORDER_ANGLE_OFFSET + BORDER_BOTTOM + 10));
+        SetMinSize(GG::Pt(MinimizedSize().x, TopBorder() + INNER_BORDER_ANGLE_OFFSET + BORDER_BOTTOM + 10));
         Resize(GG::Pt(m_original_size));
         Show();
     }
@@ -548,53 +537,69 @@ void CUIWnd::MinimizeClicked() {
 }
 
 void CUIWnd::InitBuffers() {
-    // for when minimized... m_minimized_buffer
-    GG::Pt m_sz = MinimizedSize();
+    m_vertex_buffer.clear();
+    m_vertex_buffer.reserve(17);
+    m_buffer_indices.resize(4);
+    std::size_t previous_buffer_size = m_vertex_buffer.size();
 
-    m_minimized_buffer.clear();
-    m_minimized_buffer.store(0.0f,          0.0f);
-    m_minimized_buffer.store(Value(m_sz.x), 0.0f);
-    m_minimized_buffer.store(Value(m_sz.x), Value(m_sz.y));
-    m_minimized_buffer.store(0.0f,          Value(m_sz.y));
-    m_minimized_buffer.store(0.0f,          0.0f);
-    m_minimized_buffer.createServerBuffer();
+    GG::Pt ul = UpperLeft(), lr = LowerRight();
+    GG::Pt cl_ul = ClientUpperLeft(), cl_lr = ClientLowerRight();
 
-    GG::Pt sz = Size();
-    GG::Pt cl_ul = ClientUpperLeft() - UpperLeft();
-    GG::Pt cl_lr = ClientLowerRight() - UpperLeft();
+    // within m_vertex_buffer:
+    // [0] is the start and range for minimized background triangle fan and minimized border line loop
+    // [1] is ... the background fan / outer border line loop
+    // [2] is ... the inner border line loop
+    // [3] is ... the resize tab line list
+
+    // minimized background fan and border line loop
+    m_vertex_buffer.store(Value(ul.x),  Value(ul.y));
+    m_vertex_buffer.store(Value(lr.x),  Value(ul.y));
+    m_vertex_buffer.store(Value(lr.x),  Value(lr.y));
+    m_vertex_buffer.store(Value(ul.x),  Value(lr.y));
+    m_buffer_indices[0].first = previous_buffer_size;
+    m_buffer_indices[0].second = m_vertex_buffer.size() - previous_buffer_size;
+    previous_buffer_size = m_vertex_buffer.size();
 
     // outer border, with optional corner cutout
-    m_outer_border_buffer.clear();
-    m_outer_border_buffer.store(0.0f,           0.0f);
-    m_outer_border_buffer.store(Value(sz.x),    0.0f);
+    m_vertex_buffer.store(Value(ul.x),  Value(ul.y));
+    m_vertex_buffer.store(Value(lr.x),  Value(ul.y));
     if (!m_resizable) {
-        m_outer_border_buffer.store(Value(sz.x),                            Value(sz.y) - OUTER_EDGE_ANGLE_OFFSET);
-        m_outer_border_buffer.store(Value(sz.x) - OUTER_EDGE_ANGLE_OFFSET,  Value(sz.y));
+        m_vertex_buffer.store(Value(lr.x),                            Value(lr.y) - OUTER_EDGE_ANGLE_OFFSET);
+        m_vertex_buffer.store(Value(lr.x) - OUTER_EDGE_ANGLE_OFFSET,  Value(lr.y));
     } else {
-        m_outer_border_buffer.store(Value(sz.x),Value(sz.y));
+        m_vertex_buffer.store(Value(lr.x),  Value(lr.y));
     }
-    m_outer_border_buffer.store(0.0f,           Value(sz.y));
-    m_outer_border_buffer.store(0.0f,           0.0f);
+    m_vertex_buffer.store(Value(ul.x),      Value(lr.y));
+    m_buffer_indices[1].first = previous_buffer_size;
+    m_buffer_indices[1].second = m_vertex_buffer.size() - previous_buffer_size;
+    previous_buffer_size = m_vertex_buffer.size();
 
     // inner border, with optional corner cutout
-    m_inner_border_buffer.clear();
-    m_inner_border_buffer.store(Value(cl_ul.x), Value(cl_ul.y));
-    m_inner_border_buffer.store(Value(cl_lr.x), Value(cl_ul.y));
+    m_vertex_buffer.store(Value(cl_ul.x),       Value(cl_ul.y));
+    m_vertex_buffer.store(Value(cl_lr.x),       Value(cl_ul.y));
     if (m_resizable) {
-        m_inner_border_buffer.store(Value(cl_lr.x),                             Value(cl_lr.y) - INNER_BORDER_ANGLE_OFFSET);
-        m_inner_border_buffer.store(Value(cl_lr.x) - INNER_BORDER_ANGLE_OFFSET, Value(cl_lr.y));
+        m_vertex_buffer.store(Value(cl_lr.x),                             Value(cl_lr.y) - INNER_BORDER_ANGLE_OFFSET);
+        m_vertex_buffer.store(Value(cl_lr.x) - INNER_BORDER_ANGLE_OFFSET, Value(cl_lr.y));
     } else {
-        m_inner_border_buffer.store(Value(cl_lr.x),Value(cl_lr.y));
+        m_vertex_buffer.store(Value(cl_lr.x),   Value(cl_lr.y));
     }
-    m_inner_border_buffer.store(Value(cl_ul.x), Value(cl_lr.y));
-    m_inner_border_buffer.store(Value(cl_ul.x), Value(cl_ul.y));
+    m_vertex_buffer.store(Value(cl_ul.x),       Value(cl_lr.y));
+    m_buffer_indices[2].first = previous_buffer_size;
+    m_buffer_indices[2].second = m_vertex_buffer.size() - previous_buffer_size;
+    previous_buffer_size = m_vertex_buffer.size();
 
     // resize hash marks
-    m_resize_corner_lines_buffer.clear();
-    m_resize_corner_lines_buffer.store(Value(cl_lr.x),                          Value(cl_lr.y) - RESIZE_HASHMARK1_OFFSET);
-    m_resize_corner_lines_buffer.store(Value(cl_lr.x) - RESIZE_HASHMARK1_OFFSET,Value(cl_lr.y));
-    m_resize_corner_lines_buffer.store(Value(cl_lr.x),                          Value(cl_lr.y) - RESIZE_HASHMARK2_OFFSET);
-    m_resize_corner_lines_buffer.store(Value(cl_lr.x) - RESIZE_HASHMARK2_OFFSET,Value(cl_lr.y));
+    m_vertex_buffer.store(Value(cl_lr.x),                           Value(cl_lr.y) - RESIZE_HASHMARK1_OFFSET);
+    m_vertex_buffer.store(Value(cl_lr.x) - RESIZE_HASHMARK1_OFFSET, Value(cl_lr.y));
+    m_vertex_buffer.store(Value(cl_lr.x),                           Value(cl_lr.y) - RESIZE_HASHMARK2_OFFSET);
+    m_vertex_buffer.store(Value(cl_lr.x) - RESIZE_HASHMARK2_OFFSET, Value(cl_lr.y));
+    m_buffer_indices[3].first = previous_buffer_size;
+    m_buffer_indices[3].second = m_vertex_buffer.size() - previous_buffer_size;
+    //previous_buffer_size = m_vertex_buffer.size();
+
+    m_vertex_buffer.createServerBuffer();
+
+    //std::cout << "CUIWnd vertex buffer final size: " << previous_buffer_size << std::endl;
 }
 
 void CUIWnd::Hide(bool children) {
@@ -748,17 +753,20 @@ const std::string CUIWnd::AddWindowOptions(const std::string& config_name,
             new_name = config_name;
         }
     } else if (!config_name.empty()) {
+        const int max_width_plus_one = HumanClientApp::MaximumPossibleWidth() + 1;
+        const int max_height_plus_one = HumanClientApp::MaximumPossibleHeight() + 1;
+
         db.Add<bool>("UI.windows."+config_name+".initialized",      UserStringNop("OPTIONS_DB_UI_WINDOWS_EXISTS"),          false,      Validator<bool>(),              false);
 
-        db.Add<int> ("UI.windows."+config_name+".left",             UserStringNop("OPTIONS_DB_UI_WINDOWS_LEFT"),            left,       OrValidator<int>(RangedValidator<int>(0, 2560), DiscreteValidator<int>(INVALID_POS)));
-        db.Add<int> ("UI.windows."+config_name+".top",              UserStringNop("OPTIONS_DB_UI_WINDOWS_TOP"),             top,        OrValidator<int>(RangedValidator<int>(0, 1600), DiscreteValidator<int>(INVALID_POS)));
-        db.Add<int> ("UI.windows."+config_name+".left-windowed",    UserStringNop("OPTIONS_DB_UI_WINDOWS_LEFT_WINDOWED"),   left,       OrValidator<int>(RangedValidator<int>(0, 2560), DiscreteValidator<int>(INVALID_POS)));
-        db.Add<int> ("UI.windows."+config_name+".top-windowed",     UserStringNop("OPTIONS_DB_UI_WINDOWS_TOP_WINDOWED"),    top,        OrValidator<int>(RangedValidator<int>(0, 1600), DiscreteValidator<int>(INVALID_POS)));
+        db.Add<int> ("UI.windows."+config_name+".left",             UserStringNop("OPTIONS_DB_UI_WINDOWS_LEFT"),            left,       OrValidator<int>(RangedValidator<int>(0, max_width_plus_one), DiscreteValidator<int>(INVALID_POS)));
+        db.Add<int> ("UI.windows."+config_name+".top",              UserStringNop("OPTIONS_DB_UI_WINDOWS_TOP"),             top,        OrValidator<int>(RangedValidator<int>(0, max_height_plus_one), DiscreteValidator<int>(INVALID_POS)));
+        db.Add<int> ("UI.windows."+config_name+".left-windowed",    UserStringNop("OPTIONS_DB_UI_WINDOWS_LEFT_WINDOWED"),   left,       OrValidator<int>(RangedValidator<int>(0, max_width_plus_one), DiscreteValidator<int>(INVALID_POS)));
+        db.Add<int> ("UI.windows."+config_name+".top-windowed",     UserStringNop("OPTIONS_DB_UI_WINDOWS_TOP_WINDOWED"),    top,        OrValidator<int>(RangedValidator<int>(0, max_height_plus_one), DiscreteValidator<int>(INVALID_POS)));
 
-        db.Add<int> ("UI.windows."+config_name+".width",            UserStringNop("OPTIONS_DB_UI_WINDOWS_WIDTH"),           width,      RangedValidator<int>(0, 2560));
-        db.Add<int> ("UI.windows."+config_name+".height",           UserStringNop("OPTIONS_DB_UI_WINDOWS_HEIGHT"),          height,     RangedValidator<int>(0, 1600));
-        db.Add<int> ("UI.windows."+config_name+".width-windowed",   UserStringNop("OPTIONS_DB_UI_WINDOWS_WIDTH_WINDOWED"),  width,      RangedValidator<int>(0, 2560));
-        db.Add<int> ("UI.windows."+config_name+".height-windowed",  UserStringNop("OPTIONS_DB_UI_WINDOWS_HEIGHT_WINDOWED"), height,     RangedValidator<int>(0, 1600));
+        db.Add<int> ("UI.windows."+config_name+".width",            UserStringNop("OPTIONS_DB_UI_WINDOWS_WIDTH"),           width,      RangedValidator<int>(0, max_width_plus_one));
+        db.Add<int> ("UI.windows."+config_name+".height",           UserStringNop("OPTIONS_DB_UI_WINDOWS_HEIGHT"),          height,     RangedValidator<int>(0, max_height_plus_one));
+        db.Add<int> ("UI.windows."+config_name+".width-windowed",   UserStringNop("OPTIONS_DB_UI_WINDOWS_WIDTH_WINDOWED"),  width,      RangedValidator<int>(0, max_width_plus_one));
+        db.Add<int> ("UI.windows."+config_name+".height-windowed",  UserStringNop("OPTIONS_DB_UI_WINDOWS_HEIGHT_WINDOWED"), height,     RangedValidator<int>(0, max_height_plus_one));
 
         db.Add<bool>("UI.windows."+config_name+".visible",          UserStringNop("OPTIONS_DB_UI_WINDOWS_VISIBLE"),         visible,    Validator<bool>());
         db.Add<bool>("UI.windows."+config_name+".pinned",           UserStringNop("OPTIONS_DB_UI_WINDOWS_PINNED"),          pinned,     Validator<bool>());
@@ -835,6 +843,11 @@ void CUIWnd::InvalidateUnusedOptions() {
     db.Commit();
 }
 
+void CUIWnd::SetParent(GG::Wnd* wnd) {
+    GG::Wnd::SetParent(wnd);
+    m_vertex_buffer.clear();    // force buffer re-init on next Render call, so background is properly positioned for new parent-relative position
+}
+
 ///////////////////////////////////////
 // class CUIEditWnd
 ///////////////////////////////////////
