@@ -1,4 +1,3 @@
-// -*- C++ -*-
 #ifndef _CensusBrowseWnd_h_
 #define _CensusBrowseWnd_h_
 
@@ -9,10 +8,10 @@
 /** A popup tooltop for display when mousing over in-game icons.  A title and some detail text.*/
 class CensusBrowseWnd : public GG::BrowseInfoWnd {
 public:
-    CensusBrowseWnd(const std::string& title_text, const std::map<std::string, float>& population_counts, const std::map<std::string, float>& tag_counts);
+    CensusBrowseWnd(const std::string& title_text, const std::map<std::string, float>& population_counts,
+                    const std::map<std::string, float>& tag_counts);
     virtual bool    WndHasBrowseInfo(const Wnd* wnd, std::size_t mode) const;
     virtual void    Render();
-    void            DoLayout();
 
 private:
     virtual void  InitBuffer();
@@ -25,7 +24,6 @@ private:
     GG::Label*      m_tags_text;
     GG::ListBox*    m_tags_list;
     GG::Pt          m_offset;
-    GG::Y           m_row_height;
 
     void InitRowSizes();
 };
