@@ -34,6 +34,7 @@
 #include <GG/DrawUtil.h>
 #include <GG/UnicodeCharsets.h>
 #include <GG/dialogs/ThreeButtonDlg.h>
+#include <GG/RichText/ImageBlock.h>
 
 #include <boost/spirit/include/classic.hpp>
 #include <boost/algorithm/string/predicate.hpp>
@@ -47,7 +48,7 @@
 const Tech* GetTech(const std::string& name);
 
 bool TextureFileNameCompare(const boost::shared_ptr<GG::Texture> t1, const boost::shared_ptr<GG::Texture> t2)
-{ return t1 && t2 && t1->Filename() < t2->Filename(); }
+{ return t1 && t2 && t1->Path() < t2->Path(); }
 
 namespace fs = boost::filesystem;
 
@@ -97,7 +98,7 @@ int         ClientUI::SystemIconSize()                  { return GetOptionsDB().
 int         ClientUI::SystemTinyIconSizeThreshold()     { return GetOptionsDB().Get<int>("UI.system-tiny-icon-size-threshold"); }
 int         ClientUI::SystemCircleSize()                { return static_cast<int>(SystemIconSize() * GetOptionsDB().Get<double>("UI.system-circle-size")); }
 int         ClientUI::SystemSelectionIndicatorSize()    { return static_cast<int>(SystemIconSize() * GetOptionsDB().Get<double>("UI.system-selection-indicator-size")); }
-int         ClientUI::SystemSelectionIndicatorFPS()     { return GetOptionsDB().Get<int>("UI.system-selection-indicator-fps"); }
+int         ClientUI::SystemSelectionIndicatorRPM()     { return GetOptionsDB().Get<int>("UI.system-selection-indicator-rpm"); }
 
 GG::Clr     ClientUI::SystemNameTextColor()             { return GetOptionsDB().Get<StreamableColor>("UI.system-name-unowned-color").ToClr(); }
 
@@ -182,6 +183,12 @@ boost::shared_ptr<GG::Texture> ClientUI::MeterIcon(MeterType meter_type) {
     case METER_HAPPINESS:
     case METER_TARGET_HAPPINESS:
         icon_filename = "happiness.png";    break;
+    case METER_CAPACITY:
+    case METER_MAX_CAPACITY:
+        icon_filename = "capacity.png";   break;
+    case METER_SECONDARY_STAT:
+    case METER_MAX_SECONDARY_STAT:
+        icon_filename = "secondary.png";   break;
     case METER_STRUCTURE:
     case METER_MAX_STRUCTURE:
         icon_filename = "structure.png";    break;
@@ -404,31 +411,61 @@ namespace {
             const std::string CREDITS_STR = "AöŁ";
             std::set<GG::UnicodeCharset> credits_charsets = GG::UnicodeCharsetsToRender(CREDITS_STR);
 
-            std::string file_name = GetOptionsDB().Get<std::string>("stringtable-filename");
-            std::string stringtable_str;
+            std::set<GG::UnicodeCharset> stringtable_charsets;
+            {
+                std::string file_name = GetOptionsDB().Get<std::string>("stringtable-filename");
+                std::string stringtable_str;
+                boost::filesystem::ifstream ifs(file_name);
+                while (ifs) {
+                    std::string line;
+                    std::getline(ifs, line);
+                    stringtable_str += line;
+                    stringtable_str += '\n';
+                }
+                stringtable_charsets = GG::UnicodeCharsetsToRender(stringtable_str);
+                DebugLogger() << "loading " << stringtable_charsets.size() << " charsets for current stringtable characters";
+            }
 
-            boost::filesystem::ifstream ifs(file_name);
-            while (ifs) {
-                std::string line;
-                std::getline(ifs, line);
-                stringtable_str += line;
-                stringtable_str += '\n';
+            if (!GetOptionsDB().IsDefaultValue("stringtable-filename")) {
+                DebugLogger() << "Non-default stringtable!";
+                std::string file_name = GetOptionsDB().GetDefault<std::string>("stringtable-filename");
+                std::string stringtable_str;
+                boost::filesystem::ifstream ifs(file_name);
+                while (ifs) {
+                    std::string line;
+                    std::getline(ifs, line);
+                    stringtable_str += line;
+                    stringtable_str += '\n';
+                }
+                std::set<GG::UnicodeCharset> default_stringtable_charsets = GG::UnicodeCharsetsToRender(stringtable_str);
+                DebugLogger() << "loading " << default_stringtable_charsets.size() << " charsets for default stringtable characters";
+
+                stringtable_charsets.insert(default_stringtable_charsets.begin(), default_stringtable_charsets.end());
+                DebugLogger() << "combined stringtable charsets have " << stringtable_charsets.size() << " charsets";
             }
-            std::set<GG::UnicodeCharset> stringtable_charsets = GG::UnicodeCharsetsToRender(stringtable_str);
 
             std::set_union(credits_charsets.begin(), credits_charsets.end(),
                            stringtable_charsets.begin(), stringtable_charsets.end(),
                            std::back_inserter(retval));
+
+            std::string message_text = "Loading " + boost::lexical_cast<std::string>(retval.size()) + " Unicode charsets: ";
+            for (std::vector<GG::UnicodeCharset>::const_iterator it = retval.begin();
+                 it != retval.end(); ++it)
+            { message_text += it->m_script_name + ", "; }
+
+            DebugLogger() << message_text;
         }
         return retval;
     }
 
     // command-line options
     void AddOptions(OptionsDB& db) {
-        db.Add('c', "color-depth",      UserStringNop("OPTIONS_DB_COLOR_DEPTH"),           24,     RangedStepValidator<int>(8, 16, 32));
+        db.Add('c', "color-depth",      UserStringNop("OPTIONS_DB_COLOR_DEPTH"),           24,      RangedStepValidator<int>(8, 16, 32));
         db.Add("show-fps",              UserStringNop("OPTIONS_DB_SHOW_FPS"),              false);
         db.Add("limit-fps",             UserStringNop("OPTIONS_DB_LIMIT_FPS"),             true);
-        db.Add("max-fps",               UserStringNop("OPTIONS_DB_MAX_FPS"),               60.0,   RangedStepValidator<double>(1.0, 10.0, 200.0));
+        db.Add("max-fps",               UserStringNop("OPTIONS_DB_MAX_FPS"),               60.0,    RangedStepValidator<double>(1.0, 10.0, 240.0));
+        db.Add("limit-fps-no-focus",    UserStringNop("OPTIONS_DB_LIMIT_FPS_NO_FOCUS"),    true);
+        db.Add("max-fps-no_focus",      UserStringNop("OPTIONS_DB_MAX_FPS_NO_FOCUS"),      15.0,    RangedStepValidator<double>(1.0, 1.0, 30.0));
 
         // sound and music
         db.Add<std::string>("UI.sound.bg-music",                UserStringNop("OPTIONS_DB_BG_MUSIC"),                          (GetRootDataDir() / "default" / "data" / "sound" / "artificial_intelligence_v3.ogg").string());
@@ -452,11 +489,11 @@ namespace {
         db.Add<std::string>("UI.sound.sidepanel-open",          UserStringNop("OPTIONS_DB_UI_SOUND_SIDEPANEL_OPEN"),           (GetRootDataDir() / "default" / "data" / "sound" / "sidepanel_open.ogg").string());
 
         // fonts
-        db.Add<std::string>("UI.font",          UserStringNop("OPTIONS_DB_UI_FONT"),                       (GetRootDataDir() / "default" / "DejaVuSans.ttf").string());
-        db.Add<std::string>("UI.font-bold",     UserStringNop("OPTIONS_DB_UI_FONT_BOLD"),                  (GetRootDataDir() / "default" / "DejaVuSans-Bold.ttf").string());
-        db.Add("UI.font-size",                  UserStringNop("OPTIONS_DB_UI_FONT_SIZE"),                  12,                     RangedValidator<int>(4, 40));
-        db.Add<std::string>("UI.title-font",    UserStringNop("OPTIONS_DB_UI_TITLE_FONT"),                 (GetRootDataDir() / "default" / "DejaVuSans.ttf").string());
-        db.Add("UI.title-font-size",            UserStringNop("OPTIONS_DB_UI_TITLE_FONT_SIZE"),            12,                     RangedValidator<int>(4, 40));
+        db.Add<std::string>("UI.font",          UserStringNop("OPTIONS_DB_UI_FONT"),                       (GetRootDataDir() / "default" / "data" / "fonts" / "Roboto-Regular.ttf").string());
+        db.Add<std::string>("UI.font-bold",     UserStringNop("OPTIONS_DB_UI_FONT_BOLD"),                  (GetRootDataDir() / "default" / "data" / "fonts" / "Roboto-Bold.ttf").string());
+        db.Add("UI.font-size",                  UserStringNop("OPTIONS_DB_UI_FONT_SIZE"),                  15,                     RangedValidator<int>(4, 40));
+        db.Add<std::string>("UI.title-font",    UserStringNop("OPTIONS_DB_UI_TITLE_FONT"),                 (GetRootDataDir() / "default" / "data" / "fonts" / "Roboto-Regular.ttf").string());
+        db.Add("UI.title-font-size",            UserStringNop("OPTIONS_DB_UI_TITLE_FONT_SIZE"),            16,                     RangedValidator<int>(4, 40));
 
         // colors
         db.Add("UI.wnd-color",                  UserStringNop("OPTIONS_DB_UI_WND_COLOR"),                  StreamableColor(GG::Clr(35, 35, 35, 240)),      Validator<StreamableColor>());
@@ -510,7 +547,7 @@ namespace {
 
     const GG::Y PANEL_HEIGHT(160); // Also formerly "UI.chat-panel-height" default
     const GG::X MESSAGE_PANEL_WIDTH(345); // Formerly "UI.chat-panel-width" default
-    const GG::X PLAYER_LIST_PANEL_WIDTH(424);
+    const GG::X PLAYER_LIST_PANEL_WIDTH(445);
 
     const std::string MESSAGE_WND_NAME = "map.messages";
     const std::string PLAYER_LIST_WND_NAME = "map.player-list";
@@ -586,6 +623,9 @@ ClientUI::ClientUI() :
     ConditionalConnectOption("UI.auto-reposition-windows",
                              HumanClientApp::GetApp()->RepositionWindowsSignal,
                              true, std::equal_to<bool>());
+
+    // Set the root path for image tags in rich text.
+    GG::ImageBlock::SetDefaultImagePath(ArtDir().string());
 }
 
 ClientUI::~ClientUI() {
@@ -629,9 +669,12 @@ bool ClientUI::ZoomToObject(int id) {
 }
 
 bool ClientUI::ZoomToPlanet(int id) {
-    // this just zooms to the appropriate system, until we create a planet window of some kind
-    if (TemporaryPtr<const Planet> planet = GetPlanet(id))
-        return ZoomToSystem(planet->SystemID());
+    if (TemporaryPtr<const Planet> planet = GetPlanet(id)) {
+        m_map_wnd->CenterOnObject(planet->SystemID());
+        m_map_wnd->SelectSystem(planet->SystemID());
+        m_map_wnd->SelectPlanet(id);
+        return true;
+    }        
     return false;
 }
 
@@ -911,9 +954,9 @@ void ClientUI::MessageBox(const std::string& message, bool play_alert_sound/* =
 boost::shared_ptr<GG::Texture> ClientUI::GetTexture(const boost::filesystem::path& path, bool mipmap/* = false*/) {
     boost::shared_ptr<GG::Texture> retval;
     try {
-        retval = HumanClientApp::GetApp()->GetTexture(path.string(), mipmap);
+        retval = HumanClientApp::GetApp()->GetTexture(path, mipmap);
     } catch(...) {
-        retval = HumanClientApp::GetApp()->GetTexture((ClientUI::ArtDir() / "misc" / "missing.png").string(), mipmap);
+        retval = HumanClientApp::GetApp()->GetTexture(ClientUI::ArtDir() / "misc" / "missing.png", mipmap);
     }
 #ifdef FREEORION_MACOSX
     if (!mipmap)
@@ -989,6 +1032,15 @@ ClientUI::TexturesAndDist ClientUI::PrefixedTexturesAndDist(const boost::filesys
     return prefixed_textures_it->second;
 }
 
+int FontBasedUpscale(int x) {
+    int retval(x);
+    int font_pts = ClientUI::Pts();
+    if (font_pts > 12) {
+        retval *= static_cast<float>(font_pts) / 12.0f;
+    }
+    return retval;
+}
+
 StreamableColor::StreamableColor() :
     r(0),
     g(0),
