@@ -4,8 +4,7 @@
 #include "MapWnd.h"
 #include "Sound.h"
 #include "CUIDrawUtil.h"
-#include "ShaderProgram.h"
-#include "../util/Directories.h"
+#include "CUIControls.h"
 #include "../util/i18n.h"
 #include "../util/Logger.h"
 #include "../util/OptionsDB.h"
@@ -16,6 +15,7 @@
 #include "../Empire/Empire.h"
 
 #include <GG/DrawUtil.h>
+#include <GG/GLClientAndServerBuffer.h>
 
 #include <algorithm>
 
@@ -55,76 +55,12 @@ namespace {
             return "";
     }
 
-    /* returns corner vertex x and y components of a quad centred at (0, 0) of side length 2 that is rotated
-       so the "top" or "up" face is perpendicular to and facing in the direction of \a direction_vector */
-    std::vector<double> VectorAlignedQuadVertices(const GG::Pt& direction_vector, int texture_height, int texture_width) {
-        // get unit vectors parallel and perpendicular to direction vector
-        int x = Value(direction_vector.x), y = Value(direction_vector.y);
-        int mag2 = x*x + y*y;
-
-        // first unit vector parallel to direction_vector
-        double U1X = static_cast<double>(x), U1Y = static_cast<double>(y);
-        if (mag2 > 1) {
-            double mag = std::sqrt(static_cast<double>(mag2));
-            U1X /= mag;
-            U1Y /= mag;
-        } else if (mag2 == 0) {
-            // default to straight up for zero vector
-            U1X =  0.0;
-            U1Y = -1.0;
-        } // else don't need to rescale if vector length already is 1
-
-        // second unit vector perpendicular to first
-        double U2X = -U1Y, U2Y = U1X;
-
-        // multiply unit vectors by (half) texture size to get properly-scaled side vectors
-        double V1X = U1X * texture_height / 2.0;
-        double V1Y = U1Y * texture_height / 2.0;
-        double V2X = U2X * texture_width / 2.0;
-        double V2Y = U2Y * texture_width / 2.0;
-
-        // get components of corner points by adding unit vectors
-        std::vector<double> retval;
-        retval.push_back( V1X - V2X);   retval.push_back( V1Y - V2Y);
-        retval.push_back( V1X + V2X);   retval.push_back( V1Y + V2Y);
-        retval.push_back(-V1X - V2X);   retval.push_back(-V1Y - V2Y);
-        retval.push_back(-V1X + V2X);   retval.push_back(-V1Y + V2Y);
-
-        return retval;
-    }
-
-    /* renders quad with passed vertices and texture */
-    void RenderTexturedQuad(const std::vector<double>& vertsXY, const boost::shared_ptr<GG::Texture>& texture) {
-        if (!texture || vertsXY.size() < 8)
-            return;
-
-        glBindTexture(GL_TEXTURE_2D, texture->OpenGLId());
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-
-        float tex_coord_x = static_cast<float>(Value(1.0 * texture->DefaultWidth() / texture->Width()));
-        float tex_coord_y = static_cast<float>(Value(1.0 * texture->DefaultHeight() / texture->Height()));
-
-        // render texture
-        glBegin(GL_TRIANGLE_STRIP);
-            glTexCoord2f(0.0f, 0.0f);
-            glVertex2d(vertsXY[0], vertsXY[1]);
-            glTexCoord2f(tex_coord_x, 0.0f);
-            glVertex2d(vertsXY[2], vertsXY[3]);
-            glTexCoord2f(0.0f, tex_coord_y);
-            glVertex2d(vertsXY[4], vertsXY[5]);
-            glTexCoord2f(tex_coord_x, tex_coord_y);
-            glVertex2d(vertsXY[6], vertsXY[7]);
-        glEnd();
-    }
-
     void AddOptions(OptionsDB& db) {
         db.Add("UI.fleet-selection-indicator-size", UserStringNop("OPTIONS_DB_UI_FLEET_SELECTION_INDICATOR_SIZE"), 1.625, RangedStepValidator<double>(0.125, 0.5, 5));
     }
     bool temp_bool = RegisterOptions(&AddOptions);
 
-    const double TWO_PI = 2.0*3.14159;
-    boost::shared_ptr<ShaderProgram> scanline_shader;
+    const float TWO_PI = 2.0*3.14159f;
 }
 
 ///////////////////////////
@@ -133,22 +69,18 @@ namespace {
 FleetButton::FleetButton(const std::vector<int>& fleet_IDs, SizeType size_type) :
     GG::Button("", boost::shared_ptr<GG::Font>(), GG::CLR_ZERO),
     m_fleets(),
-    m_head_icons(),
-    m_size_icon(),
-    m_selection_texture(),
-    m_vertex_components(),
+    m_icons(),
+    m_selection_indicator(0),
+    m_scanline_control(0),
     m_selected(false)
-{
-    Init(fleet_IDs, size_type);
-}
+{ Init(fleet_IDs, size_type); }
 
 FleetButton::FleetButton(int fleet_id, SizeType size_type) :
     GG::Button("", boost::shared_ptr<GG::Font>(), GG::CLR_ZERO),
     m_fleets(),
-    m_head_icons(),
-    m_size_icon(),
-    m_selection_texture(),
-    m_vertex_components(),
+    m_icons(),
+    m_selection_indicator(0),
+    m_scanline_control(0),
     m_selected(false)
 {
     std::vector<int> fleet_IDs;
@@ -156,14 +88,18 @@ FleetButton::FleetButton(int fleet_id, SizeType size_type) :
     Init(fleet_IDs, size_type);
 }
 
-void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
-    if (!scanline_shader && GetOptionsDB().Get<bool>("UI.system-fog-of-war")) {
-        boost::filesystem::path shader_path = GetRootDataDir() / "default" / "shaders" / "scanlines.frag";
-        std::string shader_text;
-        ReadFile(shader_path, shader_text);
-        scanline_shader = boost::shared_ptr<ShaderProgram>(
-            ShaderProgram::shaderProgramFactory("", shader_text));
+FleetButton::~FleetButton() {
+    DetachChild(m_selection_indicator);
+    delete m_selection_indicator;
+
+    if (m_scanline_control) {
+        DetachChild(m_scanline_control);
+        delete m_scanline_control;
     }
+}
+
+void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
+    //std::cout << "FleetButton::Init" << std::endl;
 
     // get fleets
     std::vector<TemporaryPtr<const Fleet> > fleets;
@@ -224,7 +160,6 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
             SetColor(GG::CLR_RED);
         else
             SetColor(GG::CLR_WHITE);
-
     } else {
         // single empire owner
         if (const Empire* empire = GetEmpire(owner_id))
@@ -234,6 +169,34 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
     }
 
 
+    // determine direction button should be rotated to orient along a starlane
+    GLfloat pointing_angle = 0.0f;
+
+    TemporaryPtr<const Fleet> first_fleet;
+    if (!m_fleets.empty())
+        first_fleet = *fleets.begin();
+    if (first_fleet && first_fleet->SystemID() == INVALID_OBJECT_ID && first_fleet->NextSystemID() != INVALID_OBJECT_ID) {
+        int next_sys_id = first_fleet->NextSystemID();
+        if (TemporaryPtr<const UniverseObject> obj = GetUniverseObject(next_sys_id)) {
+            // fleet is not in a system and has a valid next destination, so can orient it in that direction
+            // fleet icons might not appear on the screen in the exact place corresponding to their
+            // actual universe position, but if they're moving along a starlane, this code will assume
+            // their apparent position will only be different from their true position in a direction
+            // parallel with the starlane, so the direction from their true position to their destination
+            // position can be used to get a direction vector to orient the icon
+            float dest_x = obj->X(), dest_y = obj->Y();
+            float cur_x = first_fleet->X(), cur_y = first_fleet->Y();
+            const MapWnd* map_wnd = ClientUI::GetClientUI()->GetMapWnd();
+            GG::Pt dest = map_wnd->ScreenCoordsFromUniversePosition(dest_x, dest_y);
+            GG::Pt cur = map_wnd->ScreenCoordsFromUniversePosition(cur_x, cur_y);
+            GG::Pt direction_vector = dest - cur;
+
+            if (direction_vector.x != GG::X0 || direction_vector.y != GG::Y0)
+                pointing_angle = 360.0f / TWO_PI * std::atan2(static_cast<float>(Value(direction_vector.y)), static_cast<float>(Value(direction_vector.x))) + 90;
+        }
+    }
+
+
     // select icon(s) for fleet(s)
     int num_ships = 0;
     for (std::vector<TemporaryPtr<const Fleet> >::const_iterator it = fleets.begin(); it != fleets.end(); ++it) {
@@ -241,81 +204,72 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
         if (fleet)
             num_ships += fleet->NumShips();
     }
-    m_size_icon = FleetSizeIcon(num_ships, size_type);
-    m_head_icons = FleetHeadIcons(fleets, size_type);
+    boost::shared_ptr<GG::Texture> size_texture = FleetSizeIcon(num_ships, size_type);
+    std::vector<boost::shared_ptr<GG::Texture> > head_textures = FleetHeadIcons(fleets, size_type);
 
-    // resize to fit icon by first determining necessary size, and then resizing
-    GG::X texture_width(0);
-    GG::Y texture_height(0);
 
-    if (!m_head_icons.empty()) {
-        texture_width = m_head_icons.front()->DefaultWidth();
-        texture_height = m_head_icons.front()->DefaultHeight();
-    }
-    if (m_size_icon) {
-        texture_width = std::max(texture_width, m_size_icon->DefaultWidth());
-        texture_height = std::max(texture_height, m_size_icon->DefaultHeight());
+    // add RotatingGraphics for all icons needed
+    if (size_texture) {
+        RotatingGraphic* icon = new RotatingGraphic(size_texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
+        icon->SetPhaseOffset(pointing_angle);
+        icon->SetRPM(0.0f);
+        icon->SetColor(this->Color());
+        m_icons.push_back(icon);
+        Resize(GG::Pt(size_texture->DefaultWidth(), size_texture->DefaultHeight()));
+        AttachChild(icon);
     }
 
+    for (std::vector<boost::shared_ptr<GG::Texture> >::const_iterator it = head_textures.begin();
+         it != head_textures.end(); ++it)
+    {
+        RotatingGraphic* icon = new RotatingGraphic(*it, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
+        icon->SetPhaseOffset(pointing_angle);
+        icon->SetRPM(0.0f);
+        icon->SetColor(this->Color());
+        m_icons.push_back(icon);
+        if (Width() < (*it)->DefaultWidth())
+            Resize(GG::Pt((*it)->DefaultWidth(), (*it)->DefaultHeight()));
+        AttachChild(icon);
+    }
 
-    // determine if fleet icon should be rotated.  this should be done if the fleet is moving along
-    // a starlane, which is the case if the fleet is not in a system and has a valid next system
-    GG::Pt direction_vector(GG::X(0), GG::Y(1));    // default, unrotated button orientation
+    // set up selection indicator
+    m_selection_indicator = new RotatingGraphic(FleetSelectionIndicatorIcon(), GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
+    m_selection_indicator->SetRPM(ClientUI::SystemSelectionIndicatorRPM());
 
-    TemporaryPtr<const Fleet> first_fleet;
-    if (!m_fleets.empty())
-        first_fleet = *fleets.begin();
-    
-    if (first_fleet && first_fleet->SystemID() == INVALID_OBJECT_ID) {
-        int next_sys_id = first_fleet->NextSystemID();
-        if (TemporaryPtr<const UniverseObject> obj = GetUniverseObject(next_sys_id)) {
-            // fleet is not in a system and has a valid next destination, so can orient it in that direction
-            // fleet icons might not appear on the screen in the exact place corresponding to their 
-            // actual universe position, but if they're moving along a starlane, this code will assume
-            // their apparent position will only be different from their true position in a direction
-            // parallel with the starlane, so the direction from their true position to their destination
-            // position can be used to get a direction vector to orient the icon
-            double dest_x = obj->X(), dest_y = obj->Y();
-            double cur_x = first_fleet->X(), cur_y = first_fleet->Y();
-            const MapWnd* map_wnd = ClientUI::GetClientUI()->GetMapWnd();
-            GG::Pt dest = map_wnd->ScreenCoordsFromUniversePosition(dest_x, dest_y);
-            GG::Pt cur = map_wnd->ScreenCoordsFromUniversePosition(cur_x, cur_y);
-            direction_vector = dest - cur;
+    LayoutIcons();
+
+    // Scanlines for not currently-visible objects?
+    int empire_id = HumanClientApp::GetApp()->EmpireID();
+    if (empire_id == ALL_EMPIRES || !GetOptionsDB().Get<bool>("UI.system-fog-of-war"))
+        return;
+
+    bool at_least_one_fleet_visible = false;
+    for (std::vector<int>::const_iterator it = m_fleets.begin(); it != m_fleets.end(); ++it) {
+        if (GetUniverse().GetObjectVisibilityByEmpire(*it, empire_id) >= VIS_BASIC_VISIBILITY) {
+            at_least_one_fleet_visible = true;
+            break;
         }
     }
 
-    // check for unrotated texture
-    if (Value(direction_vector.x) == 0) {
-        // not rotated.  can do simple texture blits
-        m_vertex_components.clear();
-    } else {
-        // texture is rotated, so need some extra math
-        // get rotated corner vetex x and y components (x1, y1, x2, y2, x3, y3, x4, y4) for texture of appropriate size
-        m_vertex_components = VectorAlignedQuadVertices(direction_vector, Value(texture_height), Value(texture_width));
-    }
+    // Create scanline renderer control
+    m_scanline_control = new ScanlineControl(GG::X0, GG::Y0, Width(), Height());
 
-    // size icon according to texture size (average two dimensions)
-    int diameter = static_cast<int>((Value(texture_width) + Value(texture_height)) / 2.0);
-    Resize(GG::Pt(GG::X(diameter), GG::Y(diameter)));
-
-    // get selection indicator texture
-    m_selection_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "fleet" / "fleet_selection.png", true);
+    if (!at_least_one_fleet_visible)
+        AttachChild(m_scanline_control);
 }
 
 bool FleetButton::InWindow(const GG::Pt& pt) const {
     // find if cursor is within required distance of centre of icon
-    const int RADIUS = Value(Width()) / 2;
-    const int RADIUS2 = RADIUS*RADIUS;
-
     GG::Pt ul = UpperLeft(), lr = LowerRight();
-    GG::Pt size = lr - ul;
-    GG::Pt half_size = GG::Pt(size.x / 2, size.y / 2);
-    GG::Pt middle = ul + half_size;
+    const float midX = Value(ul.x + lr.x)/2.0f;
+    const float midY = Value(ul.y + lr.y)/2.0f;
 
-    GG::Pt delta = pt - middle;
+    const float RADIUS2 = Value(Width())*Value(Width())/4.0f;
 
-    const int distx = Value(delta.x);
-    const int disty = Value(delta.y);
+    const float ptX = Value(pt.x);
+    const float ptY = Value(pt.y);
+
+    const float distx = ptX - midX, disty = ptY - midY;
 
     return distx*distx + disty*disty <= RADIUS2;
 }
@@ -329,72 +283,64 @@ void FleetButton::MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
     }
 }
 
-void FleetButton::SetSelected(bool selected)
-{ m_selected = selected; }
+void FleetButton::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+    GG::Pt sz = Size();
+
+    Button::SizeMove(ul, lr);
+
+    if (sz == Size())
+        return;
+
+    LayoutIcons();
+}
+
+void FleetButton::LayoutIcons() {
+    GG::Pt middle = GG::Pt(Width() / 2, Height() / 2);
+    for (std::vector<RotatingGraphic*>::iterator it = m_icons.begin(); it != m_icons.end(); ++it) {
+        RotatingGraphic* graphic = *it;
+        GG::SubTexture subtexture = graphic->GetTexture();
+        GG::Pt subtexture_sz = GG::Pt(subtexture.Width(), subtexture.Height());
+        //std::cout << "FleetButton::LayoutIcons repositioning icon: sz: " << subtexture_sz << "  tex: " << subtexture.GetTexture()->Filename() << std::endl;
+        GG::Pt graphic_ul = middle - GG::Pt(subtexture_sz.x / 2, subtexture_sz.y / 2);
+        graphic->SizeMove(graphic_ul, graphic_ul + subtexture_sz);
+    }
+
+    if (m_selection_indicator) {
+        //GG::SubTexture subtexture = m_selection_indicator->GetTexture();
+        //GG::Pt subtexture_sz = GG::Pt(subtexture.Width(), subtexture.Height());
+        double sel_ind_scale = GetOptionsDB().Get<double>("UI.fleet-selection-indicator-size");
+        GG::Pt subtexture_sz = Size() * sel_ind_scale;
+        GG::Pt graphic_ul = middle - subtexture_sz / 2;
+        m_selection_indicator->SizeMove(graphic_ul, graphic_ul + subtexture_sz);
+    }
+}
+
+void FleetButton::SetSelected(bool selected) {
+    m_selected = selected;
+
+    if (!m_selected) {
+        DetachChild(m_selection_indicator);
+        m_selection_indicator->Hide();
+        return;
+    }
+
+    AttachChild(m_selection_indicator);
+    m_selection_indicator->Show();
+    MoveChildDown(m_selection_indicator);
+    LayoutIcons();
+}
 
 void FleetButton::RenderUnpressed() {
-    GG::Pt ul = UpperLeft(), lr = LowerRight();
-    const double midX = static_cast<double>(Value(ul.x + lr.x))/2.0;
-    const double midY = static_cast<double>(Value(ul.y + lr.y))/2.0;
+    // GG::Pt ul = UpperLeft(), lr = LowerRight();
+    // const float midX = Value(ul.x + lr.x)/2.0f;
+    // const float midY = Value(ul.y + lr.y)/2.0f;
 
-    if (m_selected && m_selection_texture) {
-        double sel_ind_scale = GetOptionsDB().Get<double>("UI.fleet-selection-indicator-size");
-        double sel_ind_half_size = Value(Width()) * sel_ind_scale / 2.0;
-
-        GG::Pt sel_ul = GG::Pt(GG::X(static_cast<int>(midX - sel_ind_half_size)), GG::Y(static_cast<int>(midY - sel_ind_half_size)));
-        GG::Pt sel_lr = GG::Pt(GG::X(static_cast<int>(midX + sel_ind_half_size)), GG::Y(static_cast<int>(midY + sel_ind_half_size)));
-
-        glColor(GG::CLR_WHITE);
-        m_selection_texture->OrthoBlit(sel_ul, sel_lr);
-    }
-
-    glColor(Color());
-    if (m_vertex_components.empty()) {
-        if (m_size_icon)
-            m_size_icon->OrthoBlit(ul);
-        for (std::vector<boost::shared_ptr<GG::Texture> >::iterator it = m_head_icons.begin(); it != m_head_icons.end(); ++it)
-            (*it)->OrthoBlit(ul);
-    } else {
-        std::vector<double> vertsXY;
-        vertsXY.push_back(midX + m_vertex_components[0]);
-        vertsXY.push_back(midY + m_vertex_components[1]);
-        vertsXY.push_back(midX + m_vertex_components[2]);
-        vertsXY.push_back(midY + m_vertex_components[3]);
-        vertsXY.push_back(midX + m_vertex_components[4]);
-        vertsXY.push_back(midY + m_vertex_components[5]);
-        vertsXY.push_back(midX + m_vertex_components[6]);
-        vertsXY.push_back(midY + m_vertex_components[7]);
-
-        for (std::vector<boost::shared_ptr<GG::Texture> >::iterator it = m_head_icons.begin(); it != m_head_icons.end(); ++it)
-            RenderTexturedQuad(vertsXY, *it);
-        RenderTexturedQuad(vertsXY, m_size_icon);
-    }
-
-
-    // Scanlines for not currently-visible objects?
-    int empire_id = HumanClientApp::GetApp()->EmpireID();
-    if (!scanline_shader || empire_id == ALL_EMPIRES || !GetOptionsDB().Get<bool>("UI.system-fog-of-war"))
-        return;
-
-    bool at_least_one_fleet_visible = false;
-    for (std::vector<int>::const_iterator it = m_fleets.begin(); it != m_fleets.end(); ++it) {
-        if (GetUniverse().GetObjectVisibilityByEmpire(*it, empire_id) >= VIS_BASIC_VISIBILITY) {
-            at_least_one_fleet_visible = true;
-            break;
-        }
-    }
-    if (at_least_one_fleet_visible)
-        return;
-
-    float fog_scanline_spacing = static_cast<float>(GetOptionsDB().Get<double>("UI.system-fog-of-war-spacing"));
-    scanline_shader->Use();
-    scanline_shader->Bind("scanline_spacing", fog_scanline_spacing);
-    CircleArc(ul, lr, 0.0, TWO_PI, true);
-    scanline_shader->stopUse();
+    //// debug
+    //GG::FlatRectangle(ul, lr, GG::CLR_ZERO, GG::CLR_RED, 2);
+    //// end debug
 }
 
 void FleetButton::RenderPressed() {
-    const double TWO_PI = 2.0*3.14159;
     glDisable(GL_TEXTURE_2D);
     glColor(Color());
     CircleArc(UpperLeft(), LowerRight(), 0.0, TWO_PI, true);
@@ -404,7 +350,6 @@ void FleetButton::RenderPressed() {
 }
 
 void FleetButton::RenderRollover() {
-    const double TWO_PI = 2.0*3.14159;
     glDisable(GL_TEXTURE_2D);
     glColor(GG::CLR_WHITE);
     CircleArc(UpperLeft(), LowerRight(), 0.0, TWO_PI, true);
@@ -423,7 +368,7 @@ void FleetButton::PlayFleetButtonOpenSound()
 // Free Functions
 /////////////////////
 std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(TemporaryPtr<const Fleet> fleet, FleetButton::SizeType size_type) {
-    std::vector< TemporaryPtr<const Fleet> > fleets(1U, fleet);
+    std::vector<TemporaryPtr<const Fleet> > fleets(1U, fleet);
     return FleetHeadIcons(fleets, size_type);
 }
 
@@ -440,16 +385,16 @@ std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(const std::vector< T
     bool hasColonyShips = false; bool hasOutpostShips = false; bool hasTroopShips = false; bool hasMonsters = false; bool hasArmedShips = false;
     for (std::vector< TemporaryPtr<const Fleet> >::const_iterator fleet_it = fleets.begin(); fleet_it != fleets.end(); ++fleet_it) {
         const TemporaryPtr<const Fleet> fleet = *fleet_it;
-        if (!fleet) 
+        if (!fleet)
             continue;
-        
+
         hasColonyShips  = hasColonyShips  || fleet->HasColonyShips();
         hasOutpostShips = hasOutpostShips || fleet->HasOutpostShips();
         hasTroopShips   = hasTroopShips   || fleet->HasTroopShips();
         hasMonsters     = hasMonsters     || fleet->HasMonsters();
         hasArmedShips   = hasArmedShips   || fleet->HasArmedShips();
     }
-    
+
     // get file name main part depending on type of fleet
     // symbol type prioritized by the ship type arbitrarily deemed "most important"
     std::vector<std::string> main_filenames;
@@ -463,12 +408,19 @@ std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(const std::vector< T
         if (hasTroopShips)   { main_filenames.push_back("head-lander.png");  }
     }
     if (main_filenames.empty()) { main_filenames.push_back("head-scout.png"); }
-    
+
     std::vector<boost::shared_ptr<GG::Texture> > result;
     for (std::vector<std::string>::const_iterator it = main_filenames.begin(); it != main_filenames.end(); ++it) {
-        result.push_back(ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "fleet" / (size_prefix + *it), false));
+        boost::shared_ptr<GG::Texture> texture_temp = ClientUI::GetTexture(
+            ClientUI::ArtDir() / "icons" / "fleet" / (size_prefix + *it), false);
+        glBindTexture(GL_TEXTURE_2D, texture_temp->OpenGLId());
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+        glBindTexture(GL_TEXTURE_2D, 0);
+
+        result.push_back(texture_temp);
     }
-    
+
     return result;
 }
 
@@ -497,5 +449,24 @@ boost::shared_ptr<GG::Texture> FleetSizeIcon(unsigned int fleet_size, FleetButto
     if (size_prefix.empty())
         return boost::shared_ptr<GG::Texture>();
 
-    return ClientUI::GetClientUI()->GetModuloTexture(ClientUI::ArtDir() / "icons" / "fleet", (size_prefix + "tail-"), FleetSizeIconNumber(fleet_size), false);
+    boost::shared_ptr<GG::Texture> texture_temp = ClientUI::GetClientUI()->GetModuloTexture(
+        ClientUI::ArtDir() / "icons" / "fleet", (size_prefix + "tail-"), FleetSizeIconNumber(fleet_size), false);
+    glBindTexture(GL_TEXTURE_2D, texture_temp->OpenGLId());
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+    glBindTexture(GL_TEXTURE_2D, 0);
+
+    return texture_temp;
+}
+
+boost::shared_ptr<GG::Texture> FleetSelectionIndicatorIcon() {
+    static boost::shared_ptr<GG::Texture> retval;
+    if (!retval) {
+        retval = ClientUI::GetClientUI()->GetTexture(ClientUI::ArtDir() / "icons" / "fleet" / "fleet_selection.png", false);
+        glBindTexture(GL_TEXTURE_2D, retval->OpenGLId());
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+        glBindTexture(GL_TEXTURE_2D, 0);
+    }
+    return retval;
 }
