@@ -10,6 +10,8 @@
 
 #include <GG/DrawUtil.h>
 #include <GG/WndEvent.h>
+#include <GG/Menu.h>
+#include <GG/GUI.h>
 
 namespace {
     static const bool RENDER_DEBUGGING_LINK_RECTS = false;
@@ -107,11 +109,34 @@ const std::string& LinkText::RawText() const
 void LinkText::LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
 { TextLinker::LClick_(pt, mod_keys); }
 
-void LinkText::LDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
-{ TextLinker::LDoubleClick_(pt, mod_keys); }
+void LinkText::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
+    // create popup menu
+    GG::MenuItem menu_contents;
+    if (GetLinkUnderPt(pt) != -1) {
+        menu_contents.next_level.push_back(GG::MenuItem(UserString("OPEN"),     9, false, false));
+        menu_contents.next_level.push_back(GG::MenuItem(true)); // separator
+    }
+    menu_contents.next_level.push_back(GG::MenuItem(UserString("HOTKEY_COPY"),  2, false, false));
 
-void LinkText::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
-{ TextLinker::RClick_(pt, mod_keys); }
+    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
+                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+    if (popup.Run()) {
+        switch (popup.MenuID()) {
+        case 9: {
+            TextLinker::RClick_(pt, mod_keys);
+            break;
+        }
+
+        case 2: { // copy
+            GG::GUI::GetGUI()->CopyWndText(this);
+            break;
+        }
+
+        default:
+            break;
+        }
+    }
+}
 
 void LinkText::MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
 { TextLinker::MouseHere_(pt, mod_keys); }
@@ -184,6 +209,7 @@ const std::string TextLinker::ENCYCLOPEDIA_TAG("encyclopedia");
 const std::string TextLinker::GRAPH_TAG("graph");
 
 TextLinker::TextLinker() :
+    m_links(),
     m_rollover_link(-1)
 {
     RegisterLinkTags();
@@ -192,32 +218,29 @@ TextLinker::TextLinker() :
 TextLinker::~TextLinker()
 {}
 
-void TextLinker::SetDecorator ( const std::string& link_type, LinkDecorator* decorator ) {
+void TextLinker::SetDecorator(const std::string& link_type, LinkDecorator* decorator) {
     m_decorators[link_type] = boost::shared_ptr<LinkDecorator>(decorator);
     MarkLinks();
 }
 
-std::string TextLinker::LinkDefaultFormatTag(const Link& link, const std::string& content) const{
-    
+std::string TextLinker::LinkDefaultFormatTag(const Link& link, const std::string& content) const {
     const LinkDecorator* decorator = &DEFAULT_DECORATOR;
-    
+
     std::map<std::string, LinkDecoratorPtr>::const_iterator it = m_decorators.find(link.type);
     if (it != m_decorators.end()){
         decorator = it->second.get();
     }
-    
+
     return decorator->Decorate(link.data, content);
 }
 
-std::string TextLinker::LinkRolloverFormatTag(const Link& link, const std::string& content) const{
-    
+std::string TextLinker::LinkRolloverFormatTag(const Link& link, const std::string& content) const {
     const LinkDecorator* decorator = &DEFAULT_DECORATOR;
-    
+
     std::map<std::string, LinkDecoratorPtr>::const_iterator it = m_decorators.find(link.type);
-    if (it != m_decorators.end()){
+    if (it != m_decorators.end())
         decorator = it->second.get();
-    }
-    
+
     return decorator->DecorateRollover(link.data, content);
 }
 
@@ -247,8 +270,8 @@ void TextLinker::LClick_(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
         return;
     }
 
-    const std::string& LINK_TYPE = m_links[sel_link].type;
-    const std::string& DATA = m_links[sel_link].data;
+    const std::string LINK_TYPE = m_links[sel_link].type;
+    const std::string DATA = m_links[sel_link].data;
 
     LinkClickedSignal(LINK_TYPE, DATA);
 }
@@ -296,6 +319,32 @@ void TextLinker::MouseLeave_() {
     MarkLinks();
 }
 
+const std::vector<GG::Font::LineData>& TextLinker::GetLineData() const {
+    static std::vector<GG::Font::LineData> retval;
+    return retval;
+}
+
+const boost::shared_ptr<GG::Font>& TextLinker::GetFont() const {
+    static boost::shared_ptr<GG::Font> retval;
+    if (!retval)
+        retval = ClientUI::GetFont();
+    return retval;
+}
+
+GG::Pt TextLinker::TextUpperLeft() const
+{ return GG::Pt(); }
+
+GG::Pt TextLinker::TextLowerRight() const
+{ return GG::Pt(); }
+
+void TextLinker::SetLinkedText(const std::string& str)
+{}
+
+const std::string& TextLinker::RawText() const {
+    static std::string retval;
+    return retval;
+}
+
 void TextLinker::FindLinks() {
     m_links.clear();
 
@@ -358,7 +407,7 @@ void TextLinker::LocateLinks() {
     const std::vector<GG::Font::LineData>& line_data = GetLineData();
     const boost::shared_ptr<GG::Font>& font = GetFont();
 
-    if(m_links.size() == 0)
+    if (m_links.empty())
         return;
 
     GG::Y y_posn(0); // y-coordinate of the top of the current line
@@ -381,7 +430,8 @@ void TextLinker::LocateLinks() {
             // The link text_posn is at the beginning of the tag, whereas
             // char_data jumps over tags. That is why we cannot test for precise equality
             if (!inside_link && curr_line.char_data[i].string_index >= current_link->real_text_posn.first &&
-                curr_line.char_data[i].string_index < current_link->real_text_posn.second){
+                curr_line.char_data[i].string_index < current_link->real_text_posn.second)
+            {
                 inside_link = true;
                 // Clear out the old rectangles
                 current_link->rects.clear();
@@ -403,9 +453,22 @@ void TextLinker::LocateLinks() {
 }
 
 int TextLinker::GetLinkUnderPt(const GG::Pt& pt) {
-    for (unsigned int i = 0; i < m_links.size(); ++i) {
-        for (unsigned int j = 0; j < m_links[i].rects.size(); ++j) {
-            GG::Rect r = TextUpperLeft() + m_links[i].rects[j];
+    std::vector<Link> links;
+    try {
+        links = m_links;
+    } catch (...) {
+        ErrorLogger() << "exception caught copying links in GetLinkUnderPt";
+        return -1;
+    }
+
+    GG::Pt tex_ul = TextUpperLeft();
+
+    for (unsigned int i = 0; i < links.size(); ++i) {
+        const Link& link = links[i];
+
+        for (unsigned int j = 0; j < link.rects.size(); ++j) {
+            GG::Rect link_rect = link.rects[j];
+            GG::Rect r = tex_ul + link_rect;
             if (r.Contains(pt))
                 return i;
         }
@@ -465,8 +528,6 @@ void TextLinker::MarkLinks() {
 
 const LinkDecorator TextLinker::DEFAULT_DECORATOR;
 
-
-
 std::string LinkTaggedText(const std::string& tag, const std::string& stringtable_entry)
 { return "<" + tag + " " + stringtable_entry + ">" + UserString(stringtable_entry) + "</" + tag + ">"; }
 
