@@ -1,4 +1,3 @@
-// -*- C++ -*-
 #ifndef _MapWnd_h_
 #define _MapWnd_h_
 
@@ -12,6 +11,9 @@
 #include "../universe/Fleet.h"
 #include "FleetButton.h"
 
+#include <boost/unordered_set.hpp>
+#include <boost/unordered_map.hpp>
+
 class FleetWnd;
 class MapWndPopup;
 class DesignWnd;
@@ -66,11 +68,24 @@ public:
       * save-and-load cycle */
     void                        GetSaveGameUIData(SaveGameUIData& data) const;
 
-    /** returns true iff this MapWnd is visible and usable for interaction, but
-      * the allowed interactions are restricted to those appropriate to the
-      * production screen */
+    /** returns true if MapWnd is visible and usable behind a production window.
+     * MapWnd interactions are restricted to those appropriate to the production window */
     bool                        InProductionViewMode() const;
 
+    /** returns true if MapWnd is visible and usable behind a research window.
+     * MapWnd interactions are restricted to those appropriate to the research window.
+     * Currently, there are no interactions with the MapWnd while the research window
+     * is visible because although the MapWnd is visible the research window is opaque
+     * and on top.*/
+    bool                        InResearchViewMode() const;
+
+    /** returns true if MapWnd is visible and usable behind a design window.
+     * MapWnd interactions are restricted to those appropriate to the design window
+     * Currently, there are no interactions with the MapWnd while the design window
+     * is visible because although the MapWnd is visible the design window is opaque
+     * and on top.*/
+    bool                        InDesignViewMode() const;
+
     /** returns the currently set moderator action in this MapWnd's
       * ModeratorActionsWnd. */
     ModeratorActionSetting      GetModeratorActionSetting() const;
@@ -86,7 +101,7 @@ public:
 
     /** Returns the id of the currently-selected planet, or
       * INVALID_OBJECT_ID if no planet is selected */
-    int                 SelectedPlanetID() const;
+    int                         SelectedPlanetID() const;
     //!@}
 
     //! \name Mutators //!@{
@@ -143,7 +158,7 @@ public:
     void            SelectFleet(TemporaryPtr<Fleet> fleet);                              //!< programatically selects fleets
     void            ReselectLastFleet();                                    //!< re-selects the most recent selected fleet, if a valid one exists
 
-    void            SetFleetMovementLine(const FleetButton* fleet_button);  //!< creates fleet movement lines for all fleets in the given FleetButton to indicate where (and whether) they are moving.  Move lines originate from the FleetButton.
+    void            RemoveFleet(int fleet_id); //!< removes specified fleet.
     void            SetFleetMovementLine(int fleet_id);                     //!< creates fleet movement line for a single fleet.  Move lines originate from the fleet's button location.
 
     /* creates specially-coloured projected fleet movement line for specified
@@ -154,12 +169,13 @@ public:
      * fleets following the specified route.  Move lines originates from the
      * fleets' button locations. */
     void            SetProjectedFleetMovementLines(const std::vector<int>& fleet_ids, const std::list<int>& travel_route);
-    void            RemoveProjectedFleetMovementLine(int fleet_id); //!< removes projected fleet movement line for specified fleet.
+
     void            ClearProjectedFleetMovementLines();             //!< removes all projected fleet movement lines
 
+    void            ResetEmpireShown();                             //!< auto-resets the shown empire in any contained Wnds, to the current client's empire (if any)
+
     void            RegisterPopup(MapWndPopup* popup);              //!< registers a MapWndPopup, which can be cleaned up with a call to DeleteAllPopups( )
     void            RemovePopup(MapWndPopup* popup);                //!< removes a MapWndPopup from the list cleaned up on a call to DeleteAllPopups( )
-    void            Cleanup();                                      //!< cleans up the MapWnd at the end of a turn (ie, closes all windows and disables all keyboard accelerators)
     void            Sanitize();                                     //!< sanitizes the MapWnd after a game
     //!@}
 
@@ -206,9 +222,37 @@ private:
     void            SetZoom(double steps_in, bool update_slide);//!< sets zoom level of the main map to zoom step size to the power of \a steps_in and updates zoom slider position if \a update_slide is true
     void            SetZoom(double steps_in, bool update_slide, const GG::Pt& position);//!< sets zoom level of the main map to zoom step size to the power of \a steps_in and updates zoom slider position if \a update_slide is true. Keeps the screen position \a position in the same place after zooming
 
-    void            RefreshFleetButtons();                      //!< removes old / existing and creates new fleet buttons
+    void            Pan(const GG::Pt& delta);                   //!< pans map
+    bool            PanX(GG::X x = GG::X(50));
+    bool            PanY(GG::Y y = GG::Y(50));
+
+    /** Mark all fleet buttons for a refresh. */
+    void            RefreshFleetButtons();
+    /**  Removes old / existing and create new fleet buttons. Only called once per render interval.*/
+    void            DeferredRefreshFleetButtons();
+
+    /** Use the vectors of fleet ids from \p fleets_map to create fleet buttons in \p
+        type_fleet_buttons and record the fleet buttons in \p m_fleet_buttons.*/
+    template <typename K>
+    void            CreateFleetButtonsOfType(
+        boost::unordered_map<K, boost::unordered_set<FleetButton*> >& type_fleet_buttons,
+        const boost::unordered_map<std::pair<K, int>, std::vector<int> > &fleets_map,
+        const FleetButton::SizeType& fleet_button_size);
+
+    /** Delete all fleet buttons.*/
+    void            DeleteFleetButtons();
+
     void            RefreshFleetButtonSelectionIndicators();    //!< marks (only) selected fleets' buttons as selected
-    void            FleetsAddedOrRemoved(const std::vector<TemporaryPtr<Fleet> >& fleets);
+
+    /** Connect all \p fleets StateChangedSignal to RefreshFleetButtons. */
+    void            AddFleetsStateChangedSignal(const std::vector<TemporaryPtr<Fleet> >& fleets);
+    /** Disconnect all \p fleets StateChangedSignal from RefreshFleetButtons. */
+    void            RemoveFleetsStateChangedSignal(const std::vector<TemporaryPtr<Fleet> >& fleets);
+    /** Handle FleetsInsertedSignal by connecting signals and refreshing fleet buttons. */
+    void            FleetsInsertedSignalHandler(const std::vector<TemporaryPtr<Fleet> >& fleets);
+    /** Handle FleetsRemovedSignal by disconnecting signals and refreshing fleet buttons. */
+    void            FleetsRemovedSignalHandler(const std::vector<TemporaryPtr<Fleet> >& fleets);
+
 
     void            DoFleetButtonsLayout();                     //!< does layout of fleet buttons
     std::pair<double, double>   MovingFleetMapPositionOnLane(TemporaryPtr<const Fleet> fleet) const; //!< returns position on map where a moving fleet should be displayed.  This is different from the fleet's actual universe position due to the squishing of fleets moving along a lane into the space between the system circles at the ends of the lane
@@ -225,6 +269,13 @@ private:
     void            ClearSystemRenderingBuffers();
     void            InitStarlaneRenderingBuffers();             //!< initializes or refreshes buffers for rendering of starlanes
     void            ClearStarlaneRenderingBuffers();
+    void            InitFieldRenderingBuffers();
+    void            ClearFieldRenderingBuffers();
+    void            InitVisibilityRadiiRenderingBuffers();
+    void            ClearVisibilityRadiiRenderingBuffers();
+    void            InitScaleCircleRenderingBuffer();
+    void            ClearScaleCircleRenderingBuffer();
+    void            ClearStarfieldRenderingBuffers();
 
     /* Takes X and Y coordinates of a pair of systems and moves these points inwards along the vector
      * between them by the radius of a system on screen (at zoom 1.0) and return result */ 
@@ -239,27 +290,28 @@ private:
                                     double thickness, bool coloured, bool doBase);
     void            RenderFields();                             //!< renders field objects
 
-    /* Gets systemID path (reversed) along supplylanes via BFS, is used to
-     * build ResourceGroupCores ideally, instead, a supplylane filter would be
-     * added the the universe graph impl. */
-    std::vector<int> GetLeastJumps(int startSys, int endSys, const std::set<int>& resGroup,
-                                   const std::set<std::pair<int, int> >& supplylanes, const ObjectMap& objMap);
-
     /* renders the dashed lines indicating where each fleet is going */
     void            RenderFleetMovementLines();
 
-    /* renders a single fleet movement line.  if \a clr is GG::CLR_ZERO, the lane
-     * is rendered with the .colour attribute of \a move_line */
-    void            RenderMovementLine(const MapWnd::MovementLineData& move_line, GG::Clr clr = GG::CLR_ZERO);
+    /* renders a single fleet movement line. if \a clr is GG::CLR_ZERO, the lane
+     * is rendered with the .colour attribute of \a move_line. assumes that the
+     * move dot texture has already been bound. */
+    void            RenderMovementLine(const MapWnd::MovementLineData& move_line, float dot_size, float dot_spacing, float dot_shift,
+                                       GG::Clr clr = GG::CLR_ZERO);
 
     /* renders ETA indicators at end-of-turn positions for a single fleet movement
      * line.  if \a clr is GG::CLR_ZERO, the indicators are filled with the .colour
      * attribute of \a move_line */
     void            RenderMovementLineETAIndicators(const MapWnd::MovementLineData& move_line, GG::Clr clr = GG::CLR_ZERO);
 
-    void            RenderVisibilityRadii();                    //!< renders circles around objects' locations indicating distance they have visibility
+    /* renders circles around objects' locations indicating distance they have
+     * visibility */
+    void            RenderVisibilityRadii();
 
-    void            CorrectMapPosition(GG::Pt &move_to_pt);     //!< ensures that the map data are positioned sensibly
+    /* renders scale circle around selected system. */
+    void            RenderScaleCircle();
+
+    void            CorrectMapPosition(GG::Pt& move_to_pt);     //!< constrains \a move_to_pt so that if the map is repositioned to that location, it will not be problematically positioned, so that galaxy contents remain visible
 
     void            FieldRightClicked(int field_id);
 
@@ -316,7 +368,7 @@ private:
     void            ShowPedia();
     void            HidePedia();
 
-    void            ShowGraphs();
+    bool            ShowGraphs();
 
     void            HideSidePanel();
     void            RestoreSidePanel();
@@ -360,17 +412,12 @@ private:
     void            SelectedFleetsChanged();
     void            SelectedShipsChanged();
 
-    void            HandleEmpireElimination(int empire_id);             //!< cleans up internal storage of now-invalidated empire ID
-
-    std::vector<boost::shared_ptr<GG::Texture> >    m_backgrounds;      //!< starfield backgrounds
-    std::vector<double>                             m_bg_scroll_rate;   //!< array, the rates at which each background scrolls
-
     std::set<int>               m_selected_fleet_ids;
     std::set<int>               m_selected_ship_ids;
 
     double                      m_zoom_steps_in;    //!< number of zoom steps in.  each 1.0 step increases display scaling by the same zoom step factor
     SidePanel*                  m_side_panel;       //!< planet view panel on the side of the main map
-    std::map<int, SystemIcon*>  m_system_icons;     //!< system icons in the main map, indexed by system id
+    boost::unordered_map<int, SystemIcon*>  m_system_icons;     //!< system icons in the main map, indexed by system id
     std::map<int, FieldIcon*>   m_field_icons;      //!< field icons in the main map, indexed by field id
     SitRepPanel*                m_sitrep_panel;     //!< sitrep panel
     ResearchWnd*                m_research_wnd;     //!< research screen
@@ -383,13 +430,13 @@ private:
 
     std::map<std::pair<int, int>, LaneEndpoints>    m_starlane_endpoints;                   //!< map from starlane start and end system IDs (stored in pair in increasing order) to the universe coordiates at which to draw the starlane ends
 
-    std::map<int, std::set<FleetButton*> >          m_stationary_fleet_buttons;             //!< icons representing fleets at a system that are not departing, indexed by system
-    std::map<int, std::set<FleetButton*> >          m_departing_fleet_buttons;              //!< icons representing fleets at a system that are departing, indexed by system
-    std::set<FleetButton*>                          m_moving_fleet_buttons;                 //!< icons representing fleets not at a system
-    std::map<int, FleetButton*>                     m_fleet_buttons;                        //!< fleet icons, index by fleet
+    boost::unordered_map<int, boost::unordered_set<FleetButton*> >          m_stationary_fleet_buttons;             //!< icons representing fleets at a system that are not departing, indexed by system
+    boost::unordered_map<int, boost::unordered_set<FleetButton*> >          m_departing_fleet_buttons;              //!< icons representing fleets at a system that are departing, indexed by system
+    boost::unordered_map<std::pair<double, double>,  boost::unordered_set<FleetButton*> > m_moving_fleet_buttons;   //!< icons representing fleets not at a system
+    boost::unordered_map<int, FleetButton*>                     m_fleet_buttons;                        //!< fleet icons, index by fleet
 
-    std::map<int, boost::signals2::connection>               m_fleet_state_change_signals;
-    std::map<int, std::vector<boost::signals2::connection> > m_system_fleet_insert_remove_signals;
+    boost::unordered_map<int, boost::signals2::connection>               m_fleet_state_change_signals;
+    boost::unordered_map<int, std::vector<boost::signals2::connection> > m_system_fleet_insert_remove_signals;
 
     std::map<int, MovementLineData>                 m_fleet_lines;                          //!< lines used for moving fleets in the main map
     std::map<int, MovementLineData>                 m_projected_fleet_lines;                //!< lines that show the projected path of the active fleet in the FleetWnd
@@ -398,16 +445,33 @@ private:
 
     std::map<boost::shared_ptr<GG::Texture>, GG::GL2DVertexBuffer>  m_star_core_quad_vertices;
     std::map<boost::shared_ptr<GG::Texture>, GG::GL2DVertexBuffer>  m_star_halo_quad_vertices;
-    std::map<boost::shared_ptr<GG::Texture>, GG::GL2DVertexBuffer>  m_galaxy_gas_quad_vertices;
-    GG::GLTexCoordBuffer                    m_star_texture_coords;
+    GG::GL2DVertexBuffer                m_galaxy_gas_quad_vertices;
+    GG::GLTexCoordBuffer                m_galaxy_gas_texture_coords;
+    GG::GLTexCoordBuffer                m_star_texture_coords;
+    GG::GL2DVertexBuffer                m_star_circle_vertices;
 
-    GG::GL2DVertexBuffer                    m_starlane_vertices;
-    GG::GLRGBAColorBuffer                   m_starlane_colors;
-    GG::GL2DVertexBuffer                    m_RC_starlane_vertices;
-    GG::GLRGBAColorBuffer                   m_RC_starlane_colors;
-    std::set<int>                       m_resourceCenters;
+    GG::GL2DVertexBuffer                m_starlane_vertices;
+    GG::GLRGBAColorBuffer               m_starlane_colors;
+    GG::GL2DVertexBuffer                m_RC_starlane_vertices;
+    GG::GLRGBAColorBuffer               m_RC_starlane_colors;
 
-    boost::shared_ptr<ShaderProgram>    m_scanline_shader;
+    std::map<boost::shared_ptr<GG::Texture>, std::pair<GG::GL2DVertexBuffer, GG::GL2DVertexBuffer> >    m_field_vertices;   //!< first buffer is visible fields, second buffer is not visible (scanlined) fields for each texture
+    GG::GL2DVertexBuffer                m_field_scanline_circles;
+    GG::GLTexCoordBuffer                m_field_texture_coords;
+
+    GG::GL2DVertexBuffer                m_visibility_radii_vertices;
+    GG::GLRGBAColorBuffer               m_visibility_radii_colors;
+    GG::GL2DVertexBuffer                m_visibility_radii_border_vertices;
+    GG::GLRGBAColorBuffer               m_visibility_radii_border_colors;
+    std::vector<std::pair<std::pair<std::size_t, std::size_t>, std::pair<std::size_t, std::size_t> > >
+                                        m_radii_radii_vertices_indices_runs;
+
+    GG::GL2DVertexBuffer                m_scale_circle_vertices;
+
+    GG::GL3DVertexBuffer                m_starfield_verts;
+    GG::GLRGBAColorBuffer               m_starfield_colours;
+
+    ScanlineRenderer                    m_scanline_shader;
 
     GG::Pt                      m_drag_offset;      //!< distance the cursor is from the upper-left corner of the window during a drag ((-1, -1) if no drag is occurring)
     bool                        m_dragged;          //!< tracks whether or not a drag occurs during a left button down sequence of events
@@ -436,6 +500,9 @@ private:
 
     std::set<int>               m_fleets_exploring;
 
+    /// indicates that refresh fleet button work should be done before rendering.
+    bool                        m_deferred_refresh_fleet_buttons;
+
     friend struct IntroMenu;
     friend struct WaitingForGameStart;
     friend struct PlayingTurn;
@@ -443,18 +510,12 @@ private:
 };
 
 
-/** Derive any window from this class to have it managed by MapWnd. For example, MapWnd will delete all open popups
-   when the end turn button is hit. */
+/** Derive any window from this class to have it managed by MapWnd. */
 class MapWndPopup : public CUIWnd {
 public:
-    MapWndPopup(const std::string& t,
-                GG::X default_x, GG::Y default_y,
-                GG::X default_w, GG::Y default_h,
-                GG::Flags<GG::WndFlag> flags,
-                const std::string& config_name = "");
-    MapWndPopup(const std::string& t,
-                GG::Flags<GG::WndFlag> flags,
-                const std::string& config_name = "");
+    MapWndPopup(const std::string& t, GG::X default_x, GG::Y default_y, GG::X default_w, GG::Y default_h,
+                GG::Flags<GG::WndFlag> flags, const std::string& config_name = "");
+    MapWndPopup(const std::string& t, GG::Flags<GG::WndFlag> flags, const std::string& config_name = "");
     virtual ~MapWndPopup();
     void    CloseClicked();
     void    Close();
