@@ -113,7 +113,6 @@ namespace {
                     // For AI players on host, have "AI" shown on droplist, with "Drop" shown as alternate selection to remove the AI
                     Insert(new TypeRow(Networking::CLIENT_TYPE_AI_PLAYER));      // "AI" display
                     Insert(new TypeRow(Networking::INVALID_CLIENT_TYPE, true));  // "Drop" option
-                    SetDropHeight(h * 3);
                     Select(0);
                 }
             } else if (client_type == Networking::CLIENT_TYPE_HUMAN_PLAYER ||
@@ -130,7 +129,7 @@ namespace {
                     Insert(new TypeRow(Networking::CLIENT_TYPE_HUMAN_OBSERVER)); // "Observer" display / option
                     Insert(new TypeRow(Networking::CLIENT_TYPE_HUMAN_MODERATOR));// "Moderator" display / option
                     Insert(new TypeRow(Networking::INVALID_CLIENT_TYPE, true));  // "Drop" option
-                    SetDropHeight(h * 4);
+
                     if (client_type == Networking::CLIENT_TYPE_HUMAN_PLAYER)
                         Select(0);
                     else if (client_type == Networking::CLIENT_TYPE_HUMAN_OBSERVER)
@@ -149,7 +148,7 @@ namespace {
                     // For empty row on host, have "None" and "Add AI" options on droplist
                     Insert(new TypeRow(Networking::INVALID_CLIENT_TYPE));        // "None" display
                     Insert(new TypeRow(Networking::CLIENT_TYPE_AI_PLAYER, true));// "Add AI" option
-                    SetDropHeight(h * 3);
+
                     Select(0);
                 }
             }
@@ -359,8 +358,8 @@ namespace {
         }
     };
 
-    const GG::X     LOBBY_WND_WIDTH(800);
-    const GG::Y     LOBBY_WND_HEIGHT(600);
+    const GG::X     LOBBY_WND_WIDTH(840);
+    const GG::Y     LOBBY_WND_HEIGHT(720);
     const int       CONTROL_MARGIN = 5; // gap to leave between controls in the window
     const GG::X     GALAXY_SETUP_PANEL_WIDTH(250);
     const GG::Y     GALAXY_SETUP_PANEL_HEIGHT(340);
@@ -374,7 +373,7 @@ namespace {
 
 MultiPlayerLobbyWnd::MultiPlayerLobbyWnd() :
     CUIWnd(UserString("MPLOBBY_WINDOW_TITLE"),
-           GG::ONTOP | GG::INTERACTIVE),
+           GG::ONTOP | GG::INTERACTIVE | GG::RESIZABLE),
     m_chat_box(0),
     m_chat_input_edit(0),
     m_new_load_game_buttons(0),
@@ -400,9 +399,9 @@ MultiPlayerLobbyWnd::MultiPlayerLobbyWnd() :
 
     m_new_load_game_buttons = new GG::RadioButtonGroup(GG::VERTICAL);
     m_new_load_game_buttons->AddButton(
-        new CUIStateButton(UserString("NEW_GAME_BN"), GG::FORMAT_LEFT, GG::SBSTYLE_3D_RADIO));
+        new CUIStateButton(UserString("NEW_GAME_BN"), GG::FORMAT_LEFT, boost::make_shared<CUIRadioRepresenter>()));
     m_new_load_game_buttons->AddButton(
-        new CUIStateButton(UserString("LOAD_GAME_BN"), GG::FORMAT_LEFT, GG::SBSTYLE_3D_RADIO));
+        new CUIStateButton(UserString("LOAD_GAME_BN"), GG::FORMAT_LEFT, boost::make_shared<CUIRadioRepresenter>()));
 
     m_browse_saves_btn = new CUIButton("...");
     m_save_file_text = new CUILabel("", GG::FORMAT_NOWRAP);
@@ -442,6 +441,7 @@ MultiPlayerLobbyWnd::MultiPlayerLobbyWnd() :
     AttachChild(m_start_conditions_text);
 
     ResetDefaultPosition();
+    SetMinSize(GG::Pt(LOBBY_WND_WIDTH, LOBBY_WND_HEIGHT));
     DoLayout();
 
     // default settings (new game)
@@ -461,10 +461,8 @@ MultiPlayerLobbyWnd::MultiPlayerLobbyWnd() :
 }
 
 void MultiPlayerLobbyWnd::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
-    const GG::Pt old_size = Size();
     CUIWnd::SizeMove(ul, lr);
-    if (old_size != Size())
-        DoLayout();
+    DoLayout();
 }
 
 GG::Rect MultiPlayerLobbyWnd::CalculatePosition() const {
@@ -578,6 +576,9 @@ void MultiPlayerLobbyWnd::Refresh() {
     m_start_game_bn->Disable(!ThisClientIsHost() || !CanStart());
 }
 
+GG::Pt MultiPlayerLobbyWnd::MinUsableSize() const
+{ return GG::Pt(LOBBY_WND_WIDTH, LOBBY_WND_HEIGHT); }
+
 void MultiPlayerLobbyWnd::DoLayout() {
     GG::X x(CONTROL_MARGIN);
 
