@@ -37,7 +37,7 @@ namespace {
     const GG::X INDENTATION(20);
     const GG::X ROW_WIDTH(PAGE_WIDTH - 4 - 14 - 5);
     const GG::X COLOR_SELECTOR_WIDTH(75);
-    const GG::X SPIN_WIDTH(65);
+    const GG::X SPIN_WIDTH(92);
     const int LAYOUT_MARGIN = 5;
 
     const std::string OPTIONS_WND_NAME = "options";
@@ -78,7 +78,11 @@ namespace {
     struct BrowseForPathButtonFunctor {
         BrowseForPathButtonFunctor(const fs::path& path, const std::vector<std::pair<std::string, std::string> >& filters,
                                    GG::Edit* edit, bool directory, bool return_relative_path) :
-            m_path(path), m_filters(filters), m_edit(edit), m_directory(directory), m_return_relative_path(return_relative_path)
+            m_path(path),
+            m_filters(filters),
+            m_edit(edit),
+            m_directory(directory),
+            m_return_relative_path(return_relative_path)
         {}
 
         void operator()() {
@@ -378,7 +382,7 @@ OptionsWnd::OptionsWnd():
     // FIXME: PAGE_WIDTH is needed to prevent triggering an assert within the TabBar class.
     // The placement of the tab register buttons assumes that the whole TabWnd is at least
     // wider than the first tab button.
-    m_tabs = new GG::TabWnd(GG::X0, GG::Y0, PAGE_WIDTH, GG::Y1, ClientUI::GetFont(), ClientUI::WndColor(), ClientUI::TextColor(), GG::TAB_BAR_DETACHED);
+    m_tabs = new GG::TabWnd(GG::X0, GG::Y0, PAGE_WIDTH, GG::Y1, ClientUI::GetFont(), ClientUI::WndColor(), ClientUI::TextColor());
 
     ResetDefaultPosition();
     SetMinSize(GG::Pt(PAGE_WIDTH + 20, PAGE_HEIGHT + 70));
@@ -399,8 +403,8 @@ OptionsWnd::OptionsWnd():
     // Audio settings tab
     current_page = CreatePage(UserString("OPTIONS_PAGE_AUDIO"));
     CreateSectionHeader(current_page, 0, UserString("OPTIONS_VOLUME_AND_MUSIC"));
-    MusicVolumeOption(current_page, 0);
-    VolumeOption(current_page, 0, "UI.sound.enabled", "UI.sound.volume", UserString("OPTIONS_UI_SOUNDS"), &OptionsWnd::UISoundsVolumeSlid, UI_sound_enabled);
+    MusicVolumeOption(current_page, 0, m_sound_feedback);
+    VolumeOption(current_page, 0, "UI.sound.enabled", "UI.sound.volume", UserString("OPTIONS_UI_SOUNDS"), UI_sound_enabled, m_sound_feedback);
     FileOption(current_page, 0, "UI.sound.bg-music", UserString("OPTIONS_BACKGROUND_MUSIC"), ClientUI::SoundDir(),
                std::make_pair(UserString("OPTIONS_MUSIC_FILE"), "*" + MUSIC_FILE_SUFFIX),
                ValidMusicFile);
@@ -485,65 +489,84 @@ OptionsWnd::OptionsWnd():
     GG::Connect(show_font_texture_button->LeftClickedSignal, &ShowFontTextureWnd);
 
     CreateSectionHeader(current_page, 0, UserString("OPTIONS_FONT_SIZES"));
-    IntOption(current_page, 0, "UI.font-size",       UserString("OPTIONS_FONT_TEXT"));
-    IntOption(current_page, 0, "UI.title-font-size", UserString("OPTIONS_FONT_TITLE"));
+    IntOption(current_page,    0, "UI.font-size",                          UserString("OPTIONS_FONT_TEXT"));
+    IntOption(current_page,    0, "UI.title-font-size",                    UserString("OPTIONS_FONT_TITLE"));
 
-    CreateSectionHeader(current_page, 0, UserString("OPTIONS_TECH_SPACING"));
-    DoubleOption(current_page, 0, "UI.tech-layout-horz-spacing", UserString("OPTIONS_HORIZONTAL"));
-    DoubleOption(current_page, 0, "UI.tech-layout-vert-spacing", UserString("OPTIONS_VERTICAL"));
+    CreateSectionHeader(current_page, 0, UserString("OPTIONS_RESEARCH_WND"));
+    DoubleOption(current_page, 0, "UI.tech-layout-horz-spacing",           UserString("OPTIONS_TECH_SPACING_HORIZONTAL"));
+    DoubleOption(current_page, 0, "UI.tech-layout-vert-spacing",           UserString("OPTIONS_TECH_SPACING_VERTICAL"));
+    DoubleOption(current_page, 0, "UI.tech-layout-zoom-scale",             UserString("OPTIONS_TECH_LAYOUT_ZOOM"));
+    DoubleOption(current_page, 0, "UI.tech-controls-graphic-size",         UserString("OPTIONS_TECH_CTRL_ICON_SIZE"));
 
     CreateSectionHeader(current_page, 0, UserString("OPTIONS_QUEUES"));
-    IntOption(current_page,  0, "UI.queue-width",                       UserString("OPTIONS_UI_QUEUE_WIDTH"));
-    BoolOption(current_page, 0, "UI.show-production-location-on-queue", UserString("OPTIONS_UI_PROD_QUEUE_LOCATION"));
+    IntOption(current_page,    0, "UI.queue-width",                        UserString("OPTIONS_UI_QUEUE_WIDTH"));
+    BoolOption(current_page,   0, "UI.show-production-location-on-queue",  UserString("OPTIONS_UI_PROD_QUEUE_LOCATION"));
 
     CreateSectionHeader(current_page, 0, UserString("OPTIONS_DESCRIPTIONS"));
-    BoolOption(current_page, 0, "UI.autogenerated-effects-descriptions", UserString("OPTIONS_AUTO_EFFECT_DESC"));
-    BoolOption(current_page, 0, "verbose-logging",                       UserString("OPTIONS_VERBOSE_LOGGING_DESC"));
-    BoolOption(current_page, 0, "verbose-sitrep",                        UserString("OPTIONS_VERBOSE_SITREP_DESC"));
+    BoolOption(current_page,   0, "UI.dump-effects-descriptions",          UserString("OPTIONS_DUMP_EFFECTS_GROUPS_DESC"));
+    BoolOption(current_page,   0, "verbose-logging",                       UserString("OPTIONS_VERBOSE_LOGGING_DESC"));
+    BoolOption(current_page,   0, "verbose-sitrep",                        UserString("OPTIONS_VERBOSE_SITREP_DESC"));
 
     m_tabs->SetCurrentWnd(0);
 
     // Galaxy Map Page
     current_page = CreatePage(UserString("OPTIONS_GALAXY_MAP"));
     CreateSectionHeader(current_page, 0, UserString("OPTIONS_SYSTEM_ICONS"));
-    IntOption(current_page,    0, "UI.system-icon-size",                UserString("OPTIONS_UI_SYSTEM_ICON_SIZE"));
-    BoolOption(current_page,   0, "UI.system-circles",                  UserString("OPTIONS_UI_SYSTEM_CIRCLES"));
-    DoubleOption(current_page, 0, "UI.system-circle-size",              UserString("OPTIONS_UI_SYSTEM_CIRCLE_SIZE"));
-    DoubleOption(current_page, 0, "UI.system-selection-indicator-size", UserString("OPTIONS_UI_SYSTEM_SELECTION_INDICATOR_SIZE"));
-    IntOption(current_page,    0, "UI.system-selection-indicator-fps",  UserString("OPTIONS_UI_SYSTEM_SELECTION_INDICATOR_FPS"));
-    IntOption(current_page,    0, "UI.system-tiny-icon-size-threshold", UserString("OPTIONS_UI_SYSTEM_TINY_ICON_SIZE_THRESHOLD"));
-    ColorOption(current_page,  0, "UI.system-name-unowned-color",       UserString("OPTIONS_UI_SYSTEM_NAME_UNOWNED_COLOR"));
-    BoolOption(current_page,   0, "UI.system-fog-of-war",               UserString("OPTIONS_UI_SYSTEM_FOG"));
-    DoubleOption(current_page, 0, "UI.system-fog-of-war-spacing",       UserString("OPTIONS_UI_SYSTEM_FOG_SPACING"));
-    BoolOption(current_page,   0, "UI.optimized-system-rendering",      UserString("OPTIONS_OPTIMIZED_SYSTEM_RENDERING"));
+    IntOption(current_page,    0, "UI.system-icon-size",                   UserString("OPTIONS_UI_SYSTEM_ICON_SIZE"));
+    BoolOption(current_page,   0, "UI.system-circles",                     UserString("OPTIONS_UI_SYSTEM_CIRCLES"));
+    DoubleOption(current_page, 0, "UI.system-circle-size",                 UserString("OPTIONS_UI_SYSTEM_CIRCLE_SIZE"));
+    DoubleOption(current_page, 0, "UI.system-selection-indicator-size",    UserString("OPTIONS_UI_SYSTEM_SELECTION_INDICATOR_SIZE"));
+    IntOption(current_page,    0, "UI.system-selection-indicator-rpm",     UserString("OPTIONS_UI_SYSTEM_SELECTION_INDICATOR_FPS"));
+    IntOption(current_page,    0, "UI.system-tiny-icon-size-threshold",    UserString("OPTIONS_UI_SYSTEM_TINY_ICON_SIZE_THRESHOLD"));
+    ColorOption(current_page,  0, "UI.system-name-unowned-color",          UserString("OPTIONS_UI_SYSTEM_NAME_UNOWNED_COLOR"));
+    BoolOption(current_page,   0, "UI.system-fog-of-war",                  UserString("OPTIONS_UI_SYSTEM_FOG"));
+    DoubleOption(current_page, 0, "UI.system-fog-of-war-spacing",          UserString("OPTIONS_UI_SYSTEM_FOG_SPACING"));
 
     CreateSectionHeader(current_page, 0, UserString("OPTIONS_FLEET_ICONS"));
-    DoubleOption(current_page, 0, "UI.tiny-fleet-button-minimum-zoom",   UserString("OPTIONS_UI_TINY_FLEET_BUTTON_MIN_ZOOM"));
-    DoubleOption(current_page, 0, "UI.small-fleet-button-minimum-zoom",  UserString("OPTIONS_UI_SMALL_FLEET_BUTTON_MIN_ZOOM"));
-    DoubleOption(current_page, 0, "UI.medium-fleet-button-minimum-zoom", UserString("OPTIONS_UI_MEDIUM_FLEET_BUTTON_MIN_ZOOM"));
-    DoubleOption(current_page, 0, "UI.fleet-selection-indicator-size",   UserString("OPTIONS_UI_FLEET_SELECTION_INDICATOR_SIZE"));
+    DoubleOption(current_page, 0, "UI.tiny-fleet-button-minimum-zoom",     UserString("OPTIONS_UI_TINY_FLEET_BUTTON_MIN_ZOOM"));
+    DoubleOption(current_page, 0, "UI.small-fleet-button-minimum-zoom",    UserString("OPTIONS_UI_SMALL_FLEET_BUTTON_MIN_ZOOM"));
+    DoubleOption(current_page, 0, "UI.medium-fleet-button-minimum-zoom",   UserString("OPTIONS_UI_MEDIUM_FLEET_BUTTON_MIN_ZOOM"));
+    DoubleOption(current_page, 0, "UI.fleet-selection-indicator-size",     UserString("OPTIONS_UI_FLEET_SELECTION_INDICATOR_SIZE"));
 
     CreateSectionHeader(current_page, 0, UserString("OPTIONS_STARLANES"));
-    DoubleOption(current_page, 0, "UI.starlane-thickness",            UserString("OPTIONS_STARLANE_THICKNESS"));
-    BoolOption(current_page,   0, "UI.resource-starlane-colouring",   UserString("OPTIONS_RESOURCE_STARLANE_COLOURING"));
-    DoubleOption(current_page, 0, "UI.starlane-core-multiplier",      UserString("OPTIONS_DB_STARLANE_CORE"));
-    BoolOption(current_page,   0, "UI.fleet-supply-lines",            UserString("OPTIONS_FLEET_SUPPLY_LINES"));
-    DoubleOption(current_page, 0, "UI.fleet-supply-line-width",       UserString("OPTIONS_FLEET_SUPPLY_LINE_WIDTH"));
-    IntOption(current_page,    0, "UI.fleet-supply-line-dot-spacing", UserString("OPTIONS_FLEET_SUPPLY_LINE_DOT_SPACING"));
-    DoubleOption(current_page, 0, "UI.fleet-supply-line-dot-rate",    UserString("OPTIONS_FLEET_SUPPLY_LINE_DOT_RATE"));
-    ColorOption(current_page,  0, "UI.unowned-starlane-colour",       UserString("OPTIONS_UNOWNED_STARLANE_COLOUR"));
+    DoubleOption(current_page, 0, "UI.starlane-thickness",                 UserString("OPTIONS_STARLANE_THICKNESS"));
+    BoolOption(current_page,   0, "UI.resource-starlane-colouring",        UserString("OPTIONS_RESOURCE_STARLANE_COLOURING"));
+    DoubleOption(current_page, 0, "UI.starlane-core-multiplier",           UserString("OPTIONS_DB_STARLANE_CORE"));
+    BoolOption(current_page,   0, "UI.fleet-supply-lines",                 UserString("OPTIONS_FLEET_SUPPLY_LINES"));
+    DoubleOption(current_page, 0, "UI.fleet-supply-line-width",            UserString("OPTIONS_FLEET_SUPPLY_LINE_WIDTH"));
+    IntOption(current_page,    0, "UI.fleet-supply-line-dot-spacing",      UserString("OPTIONS_FLEET_SUPPLY_LINE_DOT_SPACING"));
+    DoubleOption(current_page, 0, "UI.fleet-supply-line-dot-rate",         UserString("OPTIONS_FLEET_SUPPLY_LINE_DOT_RATE"));
+    ColorOption(current_page,  0, "UI.unowned-starlane-colour",            UserString("OPTIONS_UNOWNED_STARLANE_COLOUR"));
 
     CreateSectionHeader(current_page, 0, UserString("OPTIONS_GALAXY_MAP_GENERAL"));
-    BoolOption(current_page, 0, "UI.galaxy-gas-background",       UserString("OPTIONS_GALAXY_MAP_GAS"));
-    BoolOption(current_page, 0, "UI.galaxy-starfields",           UserString("OPTIONS_GALAXY_MAP_STARFIELDS"));
-    BoolOption(current_page, 0, "UI.show-galaxy-map-scale",       UserString("OPTIONS_GALAXY_MAP_SCALE_LINE"));
-    BoolOption(current_page, 0, "UI.show-galaxy-map-scale-circle",UserString("OPTIONS_GALAXY_MAP_SCALE_CIRCLE"));
-    BoolOption(current_page, 0, "UI.show-galaxy-map-zoom-slider", UserString("OPTIONS_GALAXY_MAP_ZOOM_SLIDER"));
-    BoolOption(current_page, 0, "UI.show-detection-range",        UserString("OPTIONS_GALAXY_MAP_DETECTION_RANGE"));
-    IntOption(current_page,  0, "UI.detection-range-opacity",     UserString("OPTIONS_GALAXY_MAP_DETECTION_RANGE_OPACITY"));
-    BoolOption(current_page, 0, "UI.map-right-click-popup-menu",  UserString("OPTIONS_GALAXY_MAP_POPUP"));
-    BoolOption(current_page, 0, "UI.show-unexplored_system_overlay", UserString("OPTIONS_UI_SYSTEM_UNEXPLORED_OVERLAY"));
-    BoolOption(current_page, 0, "UI.hide-map-panels",             UserString("OPTIONS_UI_HIDE_MAP_PANELS"));
+    BoolOption(current_page,   0, "UI.galaxy-gas-background",              UserString("OPTIONS_GALAXY_MAP_GAS"));
+    BoolOption(current_page,   0, "UI.galaxy-starfields",                  UserString("OPTIONS_GALAXY_MAP_STARFIELDS"));
+    BoolOption(current_page,   0, "UI.show-galaxy-map-scale",              UserString("OPTIONS_GALAXY_MAP_SCALE_LINE"));
+    BoolOption(current_page,   0, "UI.show-galaxy-map-scale-circle",       UserString("OPTIONS_GALAXY_MAP_SCALE_CIRCLE"));
+    BoolOption(current_page,   0, "UI.show-galaxy-map-zoom-slider",        UserString("OPTIONS_GALAXY_MAP_ZOOM_SLIDER"));
+    BoolOption(current_page,   0, "UI.show-detection-range",               UserString("OPTIONS_GALAXY_MAP_DETECTION_RANGE"));
+    IntOption(current_page,    0, "UI.detection-range-opacity",            UserString("OPTIONS_GALAXY_MAP_DETECTION_RANGE_OPACITY"));
+    BoolOption(current_page,   0, "UI.map-right-click-popup-menu",         UserString("OPTIONS_GALAXY_MAP_POPUP"));
+    BoolOption(current_page,   0, "UI.show-unexplored_system_overlay",     UserString("OPTIONS_UI_SYSTEM_UNEXPLORED_OVERLAY"));
+    BoolOption(current_page,   0, "UI.hide-map-panels",                    UserString("OPTIONS_UI_HIDE_MAP_PANELS"));
+
+    m_tabs->SetCurrentWnd(0);
+
+    // Objects List Page
+    current_page = CreatePage(UserString("OPTIONS_PAGE_OBJECTS_WINDOW"));
+    CreateSectionHeader(current_page, 0, UserString("OPTIONS_COLUMNS"));
+    for (unsigned int i = 0; i < 12u; ++i) {
+        std::string col_width_opt_name = "UI.objects-list-width-col-" + boost::lexical_cast<std::string>(i);
+        if (!GetOptionsDB().OptionExists(col_width_opt_name))
+            break;
+        std::string col_opt_name = "UI.objects-list-info-col-" + boost::lexical_cast<std::string>(i);
+        if (!GetOptionsDB().OptionExists(col_opt_name))
+            break;
+        std::string col_contents = GetOptionsDB().GetValueString(col_opt_name);
+        const std::string& tx_contents = (col_contents.empty() ? "" : UserString(col_contents));
+
+        IntOption(current_page, 0, col_width_opt_name,                      tx_contents);
+    }
 
     m_tabs->SetCurrentWnd(0);
 
@@ -605,14 +628,19 @@ OptionsWnd::OptionsWnd():
     // Directories tab
     current_page = CreatePage(UserString("OPTIONS_PAGE_DIRECTORIES"));
     DirectoryOption(current_page, 0, "resource-dir", UserString("OPTIONS_FOLDER_SETTINGS"), GetRootDataDir());  // GetRootDataDir() returns the default browse path when modifying this directory option.  the actual default directory (before modifying) is gotten from the specified option name "resource-dir"
-    DirectoryOption(current_page, 0, "save-dir",     UserString("OPTIONS_FOLDER_SAVE"),     GetUserDir());
+    DirectoryOption(current_page, 0, "save-dir",     UserString("OPTIONS_FOLDER_SAVE"),     GetUserDataDir());
     m_tabs->SetCurrentWnd(0);
 
     // Misc
     current_page = CreatePage(UserString("OPTIONS_PAGE_MISC"));
-    IntOption(current_page, 0, "effects-threads", UserString("OPTIONS_EFFECTS_THREADS"));
-    BoolOption(current_page, 0, "auto-add-saved-designs", UserString("OPTIONS_ADD_SAVED_DESIGNS"));
-    BoolOption(current_page, 0, "binary-serialization", UserString("OPTIONS_USE_BINARY_SERIALIZATION"));  // Consider changing to Enum to support more serialization formats
+    IntOption(current_page, 0, "effects-threads-ui",        UserString("OPTIONS_EFFECTS_THREADS_UI"));
+    IntOption(current_page, 0, "effects-threads-server",    UserString("OPTIONS_EFFECTS_THREADS_SERVER"));
+    IntOption(current_page, 0, "effects-threads-ai",        UserString("OPTIONS_EFFECTS_THREADS_AI"));
+    BoolOption(current_page, 0, "auto-add-saved-designs",   UserString("OPTIONS_ADD_SAVED_DESIGNS"));
+    BoolOption(current_page, 0, "binary-serialization",     UserString("OPTIONS_USE_BINARY_SERIALIZATION"));  // Consider changing to Enum to support more serialization formats
+    BoolOption(current_page, 0, "verbose-logging",          UserString("OPTIONS_VERBOSE_LOGGING_DESC"));
+    BoolOption(current_page, 0, "verbose-sitrep",           UserString("OPTIONS_VERBOSE_SITREP_DESC"));
+    BoolOption(current_page, 0, "effect-accounting",        UserString("OPTIONS_EFFECT_ACCOUNTING"));
     m_tabs->SetCurrentWnd(0);
 
     DoLayout();
@@ -668,7 +696,7 @@ void OptionsWnd::CreateSectionHeader(GG::ListBox* page, int indentation_level, c
 
 GG::StateButton* OptionsWnd::BoolOption(GG::ListBox* page, int indentation_level, const std::string& option_name, const std::string& text) {
     GG::ListBox::Row* row = new GG::ListBox::Row();
-    GG::StateButton* button = new CUIStateButton(text, GG::FORMAT_LEFT);
+    GG::StateButton* button = new CUIStateButton(text, GG::FORMAT_LEFT, boost::make_shared<CUICheckBoxRepresenter>());
     button->Resize(button->MinUsableSize());
     row->Resize(GG::Pt(ROW_WIDTH, button->MinUsableSize().y + 6));
     row->push_back(new RowContentsWnd(row->Width(), row->Height(), button, indentation_level));
@@ -803,9 +831,9 @@ GG::Spin<double>* OptionsWnd::DoubleOption(GG::ListBox* page, int indentation_le
     return spin;
 }
 
-void OptionsWnd::MusicVolumeOption(GG::ListBox* page, int indentation_level) {
+void OptionsWnd::MusicVolumeOption(GG::ListBox* page, int indentation_level, SoundOptionsFeedback &fb) {
     GG::ListBox::Row* row = new GG::ListBox::Row();
-    GG::StateButton* button = new CUIStateButton(UserString("OPTIONS_MUSIC"), GG::FORMAT_LEFT);
+    GG::StateButton* button = new CUIStateButton(UserString("OPTIONS_MUSIC"), GG::FORMAT_LEFT, boost::make_shared<CUICheckBoxRepresenter>());
     button->Resize(button->MinUsableSize());
     button->SetCheck(GetOptionsDB().Get<bool>("UI.sound.music-enabled"));
     boost::shared_ptr<const RangedValidator<int> > validator = boost::dynamic_pointer_cast<const RangedValidator<int> >(GetOptionsDB().GetValidator("UI.sound.music-volume"));
@@ -822,15 +850,17 @@ void OptionsWnd::MusicVolumeOption(GG::ListBox* page, int indentation_level) {
     button->SetBrowseText(UserString(GetOptionsDB().GetDescription("UI.sound.music-enabled")));
     slider->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
     slider->SetBrowseText(UserString(GetOptionsDB().GetDescription("UI.sound.music-volume")));
-    GG::Connect(button->CheckedSignal, &OptionsWnd::MusicClicked, this);
-    GG::Connect(slider->SlidSignal, &OptionsWnd::MusicVolumeSlid, this);
+    fb.SetMusicButton(button);
+    GG::Connect(button->CheckedSignal, &OptionsWnd::SoundOptionsFeedback::MusicClicked, &fb);
+    GG::Connect(slider->SlidSignal, &OptionsWnd::SoundOptionsFeedback::MusicVolumeSlid, &fb);
 }
 
-void OptionsWnd::VolumeOption(GG::ListBox* page, int indentation_level, const std::string& toggle_option_name, const std::string& volume_option_name, const std::string& text,
-                              VolumeSliderHandler volume_slider_handler, bool toggle_value)
+void OptionsWnd::VolumeOption(GG::ListBox* page, int indentation_level, const std::string& toggle_option_name,
+                              const std::string& volume_option_name, const std::string& text,
+                              bool toggle_value, SoundOptionsFeedback &fb)
 {
     GG::ListBox::Row* row = new GG::ListBox::Row();
-    GG::StateButton* button = new CUIStateButton(text, GG::FORMAT_LEFT);
+    GG::StateButton* button = new CUIStateButton(text, GG::FORMAT_LEFT, boost::make_shared<CUICheckBoxRepresenter>());
     button->Resize(button->MinUsableSize());
     button->SetCheck(toggle_value);
     boost::shared_ptr<const RangedValidator<int> > validator = boost::dynamic_pointer_cast<const RangedValidator<int> >(GetOptionsDB().GetValidator(volume_option_name));
@@ -847,8 +877,9 @@ void OptionsWnd::VolumeOption(GG::ListBox* page, int indentation_level, const st
     button->SetBrowseText(UserString(GetOptionsDB().GetDescription(toggle_option_name)));
     slider->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
     slider->SetBrowseText(UserString(GetOptionsDB().GetDescription(volume_option_name)));
-    GG::Connect(button->CheckedSignal, SetOptionFunctor<bool>(toggle_option_name));
-    GG::Connect(slider->SlidAndStoppedSignal, volume_slider_handler, this);
+    fb.SetEffectsButton(button);
+    GG::Connect(button->CheckedSignal,        &OptionsWnd::SoundOptionsFeedback::SoundEffectsEnableClicked, &fb);
+    GG::Connect(slider->SlidAndStoppedSignal, &OptionsWnd::SoundOptionsFeedback::UISoundsVolumeSlid, &fb);
 }
 
 void OptionsWnd::FileOptionImpl(GG::ListBox* page, int indentation_level, const std::string& option_name, const std::string& text, const fs::path& path,
@@ -1035,11 +1066,17 @@ void OptionsWnd::ResolutionOption(GG::ListBox* page, int indentation_level) {
     // fps
     BoolOption(page, indentation_level, "show-fps", UserString("OPTIONS_SHOW_FPS"));
 
-    GG::StateButton* limit_FPS_button = BoolOption(page, indentation_level, "limit-fps", UserString("OPTIONS_LIMIT_FPS"));
-    GG::Spin<double>* max_fps_spin = DoubleOption(page, indentation_level,  "max-fps",   UserString("OPTIONS_MAX_FPS"));
-    GG::Connect(limit_FPS_button->CheckedSignal, LimitFPSSetOptionFunctor(max_fps_spin));
-    limit_FPS_button->SetCheck(GetOptionsDB().Get<bool>("limit-fps"));
-    limit_FPS_button->CheckedSignal(limit_FPS_button->Checked());
+    //GG::StateButton* limit_FPS_button = BoolOption(page, indentation_level, "limit-fps", UserString("OPTIONS_LIMIT_FPS"));
+    //GG::Spin<double>* max_fps_spin = 
+    DoubleOption(page, indentation_level,  "max-fps",          UserString("OPTIONS_MAX_FPS"));
+    //GG::Connect(limit_FPS_button->CheckedSignal, LimitFPSSetOptionFunctor(max_fps_spin));
+    //limit_FPS_button->SetCheck(GetOptionsDB().Get<bool>("limit-fps"));
+    //limit_FPS_button->CheckedSignal(limit_FPS_button->Checked());
+
+    //GG::StateButton* limit_FPS_nofocus_button = BoolOption(page, indentation_level, "limit-fps-no-focus", UserString("OPTIONS_LIMIT_FPS_NO_FOCUS"));
+    //GG::Spin<double>* max_fps_nofocus_spin = 
+    DoubleOption(page, indentation_level,  "max-fps-no_focus", UserString("OPTIONS_MAX_FPS_NO_FOCUS"));
+    //limit_FPS_nofocus_button->SetCheck(GetOptionsDB().Get<bool>("limit-fps-no-focus"));
 
 
     // apply button, sized to fit text
@@ -1060,10 +1097,10 @@ void OptionsWnd::HotkeysPage()
     GG::ListBox* page = CreatePage(UserString("OPTIONS_PAGE_HOTKEYS"));
     std::map<std::string, std::set<std::string> > hotkeys = Hotkey::ClassifyHotkeys();
     for (std::map<std::string, std::set<std::string> >::iterator i = hotkeys.begin();
-         i != hotkeys.end(); i++)
+         i != hotkeys.end(); ++i)
     {
         CreateSectionHeader(page, 0, UserString(i->first));
-        for (std::set<std::string>::iterator j = i->second.begin(); j != i->second.end(); j++)
+        for (std::set<std::string>::iterator j = i->second.begin(); j != i->second.end(); ++j)
             HotkeyOption(page, 0, *j);
     }
     m_tabs->SetCurrentWnd(0);
@@ -1072,7 +1109,7 @@ void OptionsWnd::HotkeysPage()
 OptionsWnd::~OptionsWnd()
 {}
 
-void OptionsWnd::KeyPress (GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys)
+void OptionsWnd::KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys)
 {
     if (key == GG::GGK_ESCAPE || key == GG::GGK_RETURN || key == GG::GGK_KP_ENTER) // Same behaviour as if "done" was pressed
         DoneClicked();
@@ -1084,26 +1121,62 @@ void OptionsWnd::DoneClicked()
     m_done = true;
 }
 
-void OptionsWnd::MusicClicked(bool checked)
-{
+void OptionsWnd::SoundOptionsFeedback::SoundEffectsEnableClicked(bool checked) {
     if (checked) {
-        GetOptionsDB().Set("UI.sound.music-enabled", true);
-        Sound::GetSound().PlayMusic(GetOptionsDB().Get<std::string>("UI.sound.bg-music"), -1);
+        try {
+            Sound::GetSound().Enable();
+            GetOptionsDB().Set("UI.sound.enabled", true);
+            Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("UI.sound.button-click"), true);
+        } catch (Sound::InitializationFailureException const &e) {
+            SoundInitializationFailure(e);
+        }
     } else {
-        GetOptionsDB().Set("UI.sound.music-enabled", false);
-        Sound::GetSound().StopMusic();
+        GetOptionsDB().Set("UI.sound.enabled", false);
+        if (!GetOptionsDB().Get<bool>("UI.sound.music-enabled"))
+            Sound::GetSound().Disable();
     }
 }
 
-void OptionsWnd::MusicVolumeSlid(int pos, int low, int high)
-{
+void OptionsWnd::SoundOptionsFeedback::MusicClicked(bool checked) {
+    if (checked) {
+        try {
+            Sound::GetSound().Enable();
+            GetOptionsDB().Set("UI.sound.music-enabled", true);
+            Sound::GetSound().PlayMusic(GetOptionsDB().Get<std::string>("UI.sound.bg-music"), -1);
+        } catch (Sound::InitializationFailureException const &e) {
+            SoundInitializationFailure(e);
+        }
+    } else {
+        GetOptionsDB().Set("UI.sound.music-enabled", false);
+        Sound::GetSound().StopMusic();
+        if (!GetOptionsDB().Get<bool>("UI.sound.enabled"))
+            Sound::GetSound().Disable();
+    }
+}
+
+void OptionsWnd::SoundOptionsFeedback::MusicVolumeSlid(int pos, int low, int high) const {
     GetOptionsDB().Set("UI.sound.music-volume", pos);
     Sound::GetSound().SetMusicVolume(pos);
 }
 
-void OptionsWnd::UISoundsVolumeSlid(int pos, int low, int high)
-{
+void OptionsWnd::SoundOptionsFeedback::UISoundsVolumeSlid(int pos, int low, int high) const {
     GetOptionsDB().Set("UI.sound.volume", pos);
     Sound::GetSound().SetUISoundsVolume(pos);
     Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("UI.sound.button-click"), true);
 }
+
+void OptionsWnd::SoundOptionsFeedback::SetMusicButton(GG::StateButton* button)
+{ m_music_button = button; }
+
+void OptionsWnd::SoundOptionsFeedback::SetEffectsButton(GG::StateButton* button)
+{ m_effects_button = button; }
+
+void OptionsWnd::SoundOptionsFeedback::SoundInitializationFailure(Sound::InitializationFailureException const &e) {
+    GetOptionsDB().Set("UI.sound.enabled", false);
+    GetOptionsDB().Set("UI.sound.music-enabled", false);
+    if (m_effects_button)
+        m_effects_button->SetCheck(false);
+    if (m_music_button)
+        m_music_button->SetCheck(false);
+    ClientUI::MessageBox(UserString(e.what()), false);
+}
