@@ -20,20 +20,74 @@
 namespace {
     const int           DATA_PANEL_BORDER = 1;
 
-    boost::shared_ptr<GG::Texture> AIIcon()         { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "ai.png"); }
-    boost::shared_ptr<GG::Texture> HumanIcon()      { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "human.png"); }
-    boost::shared_ptr<GG::Texture> ObserverIcon()   { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "observer.png"); }
-    boost::shared_ptr<GG::Texture> ModeratorIcon()  { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "moderator.png"); }
-    boost::shared_ptr<GG::Texture> HostIcon()       { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "host.png"); }
-    boost::shared_ptr<GG::Texture> PlayingIcon()    { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "playing.png"); }
-    boost::shared_ptr<GG::Texture> WaitingIcon()    { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "waiting.png"); }
-    boost::shared_ptr<GG::Texture> CombatIcon()     { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "combat.png"); }
-    boost::shared_ptr<GG::Texture> WarIcon()        { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "war.png"); }
-    boost::shared_ptr<GG::Texture> PeaceIcon()      { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "peace.png"); }
-    boost::shared_ptr<GG::Texture> UnknownIcon()    { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "unknown.png"); }
-    boost::shared_ptr<GG::Texture> ShipIcon()       { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "fleet_arrived.png"); }
-    boost::shared_ptr<GG::Texture> ProductionIcon() { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "industry.png"); }
-    boost::shared_ptr<GG::Texture> ResearchIcon()   { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "research.png"); }
+    boost::shared_ptr<GG::Texture> AIIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "ai.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> HumanIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "human.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> ObserverIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "observer.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> ModeratorIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "moderator.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> HostIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "host.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> PlayingIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "playing.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> WaitingIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "waiting.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> CombatIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "combat.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> WarIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "war.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> PeaceIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "peace.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> UnknownIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "unknown.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> ShipIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "fleet_arrived.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> ProductionIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "industry.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> ResearchIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "research.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> DetectionIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "detection.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> WonIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "victory.png");
+        return retval;
+    }
+    boost::shared_ptr<GG::Texture> LostIcon() {
+        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "empire_eliminated.png");
+        return retval;
+    }
 
 
     ////////////////////////////////////////////////
@@ -52,10 +106,12 @@ namespace {
             m_empire_planet_text(0),
             m_empire_production_text(0),
             m_empire_research_text(0),
+            m_empire_detection_text(0),
             m_diplo_status(INVALID_DIPLOMATIC_STATUS),
             m_player_status(Message::WAITING),
             m_player_type(Networking::INVALID_CLIENT_TYPE),
             m_host(false),
+            m_win_status(NEITHER),
             m_selected(false)
         {
             SetChildClippingMode(ClipToClient);
@@ -66,6 +122,7 @@ namespace {
             m_empire_planet_text = new CUILabel("", GG::FORMAT_LEFT);
             m_empire_production_text = new CUILabel("", GG::FORMAT_LEFT);
             m_empire_research_text = new CUILabel("", GG::FORMAT_LEFT);
+            m_empire_detection_text = new CUILabel("", GG::FORMAT_LEFT);
 
             //AttachChild(m_player_name_text);
             AttachChild(m_empire_name_text);
@@ -73,6 +130,7 @@ namespace {
             AttachChild(m_empire_planet_text);
             AttachChild(m_empire_production_text);
             AttachChild(m_empire_research_text);
+            AttachChild(m_empire_detection_text);
 
             DoLayout();
             Update();
@@ -109,7 +167,7 @@ namespace {
 
             ProductionIcon()->OrthoBlit(UpperLeft() + m_production_icon_ul, UpperLeft() + m_production_icon_ul + ICON_SIZE);
             ResearchIcon()->OrthoBlit(UpperLeft() + m_research_icon_ul, UpperLeft() + m_research_icon_ul + ICON_SIZE);
-
+            DetectionIcon()->OrthoBlit(UpperLeft() + m_detection_icon_ul, UpperLeft() + m_detection_icon_ul + ICON_SIZE);
 
             const ClientApp* app = ClientApp::GetApp();
             if (!app) {
@@ -145,6 +203,14 @@ namespace {
  
             if (m_host)
                 HostIcon()->OrthoBlit(UpperLeft() + m_host_icon_ul, UpperLeft() + m_host_icon_ul + ICON_SIZE);
+
+
+            switch (m_win_status)
+            {
+            case WON:     WonIcon()->OrthoBlit(UpperLeft() + m_win_status_icon_ul, UpperLeft() + m_win_status_icon_ul + ICON_SIZE); break;
+            case LOST:    LostIcon()->OrthoBlit(UpperLeft() + m_win_status_icon_ul, UpperLeft() + m_win_status_icon_ul + ICON_SIZE); break;
+            case NEITHER: break;
+            }
         }
 
         void            Select(bool b)
@@ -184,6 +250,12 @@ namespace {
                     m_diplo_status = INVALID_DIPLOMATIC_STATUS;
                 else
                     m_diplo_status = Empires().GetDiplomaticStatus(player_info.empire_id, app->EmpireID());
+                if (empire->Won())
+                    m_win_status = WON; // even if you later get eliminated, you still won
+                else if (empire->Eliminated())
+                    m_win_status = LOST;
+                else
+                    m_win_status = NEITHER;
             }
 
             //m_player_name_text->SetTextColor(empire_color);
@@ -227,6 +299,7 @@ namespace {
             std::string planet_text;
             std::string production_text;
             std::string research_text;
+            std::string detection_text;
 
             if (empires_ship_count == 0.0) {
                 ship_text       = UserString("NOTHING_VALUE_SYMBOL");
@@ -252,10 +325,13 @@ namespace {
                 research_text   = DoubleToString(empires_research_points, 2, false);
             }
 
+            detection_text = DoubleToString(empire ? empire->GetMeter("METER_DETECTION_STRENGTH")->Current() : 0.0, 0, false);
+
             m_empire_ship_text->SetText(ship_text);
             m_empire_planet_text->SetText(planet_text);
             m_empire_production_text->SetText(production_text);
             m_empire_research_text->SetText(research_text);
+            m_empire_detection_text->SetText(detection_text);
 
             m_player_type = player_info.client_type;
             m_host = player_info.host;
@@ -274,6 +350,7 @@ namespace {
             const GG::X EMPIRE_PLANET_WIDTH(ClientUI::Pts()     * 16/5);
             const GG::X EMPIRE_PRODUCTION_WIDTH(ClientUI::Pts() * 16/5);
             const GG::X EMPIRE_RESEARCH_WIDTH(ClientUI::Pts()   * 16/5);
+            const GG::X EMPIRE_DETECTION_WIDTH(ClientUI::Pts()  * 16/5);
 
 
             GG::X left(DATA_PANEL_BORDER);
@@ -314,6 +391,12 @@ namespace {
             m_empire_research_text->SizeMove(GG::Pt(left, top), GG::Pt(left + EMPIRE_RESEARCH_WIDTH, bottom));
             left += EMPIRE_RESEARCH_WIDTH;
 
+            m_detection_icon_ul = GG::Pt(left, top);
+            left += GG::X(IconSize()) + PAD;
+
+            m_empire_detection_text->SizeMove(GG::Pt(left, top), GG::Pt(left + EMPIRE_DETECTION_WIDTH, bottom));
+            left += EMPIRE_DETECTION_WIDTH;
+
             m_player_status_icon_ul = GG::Pt(left, top);
             left += GG::X(IconSize()) + PAD;
 
@@ -322,6 +405,9 @@ namespace {
 
             m_host_icon_ul = GG::Pt(left, top);
             left += GG::X(IconSize()) + PAD;
+
+            m_win_status_icon_ul = GG::Pt(left, top);
+            left += GG::X(IconSize()) + PAD;
         }
 
         int                     m_player_id;
@@ -331,20 +417,28 @@ namespace {
         GG::Label*              m_empire_planet_text;
         GG::Label*              m_empire_production_text;
         GG::Label*              m_empire_research_text;
+        GG::Label*              m_empire_detection_text;
 
         GG::Pt                  m_diplo_status_icon_ul;
         GG::Pt                  m_ship_icon_ul;
         GG::Pt                  m_planet_icon_ul;
         GG::Pt                  m_production_icon_ul;
         GG::Pt                  m_research_icon_ul;
+        GG::Pt                  m_detection_icon_ul;
         GG::Pt                  m_player_status_icon_ul;
         GG::Pt                  m_player_type_icon_ul;
         GG::Pt                  m_host_icon_ul;
+        GG::Pt                  m_win_status_icon_ul;
 
         DiplomaticStatus        m_diplo_status;
         Message::PlayerStatus   m_player_status;
         Networking::ClientType  m_player_type;
         bool                    m_host;
+        enum {
+            WON,
+            LOST,
+            NEITHER
+        }                       m_win_status;
 
         bool                    m_selected;
     };
@@ -595,13 +689,13 @@ void PlayerListWnd::PlayerSelectionChanged(const GG::ListBox::SelectionSet& rows
     SelectedPlayersChangedSignal();
 }
 
-void PlayerListWnd::PlayerDoubleClicked(GG::ListBox::iterator it) {
+void PlayerListWnd::PlayerDoubleClicked(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
     int player_id = PlayerInRow(it);
     if (player_id != Networking::INVALID_PLAYER_ID)
         PlayerDoubleClickedSignal(player_id);
 }
 
-void PlayerListWnd::PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& pt) {
+void PlayerListWnd::PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
     // check that a valid player was clicked and that it wasn't this client's own player
     int clicked_player_id = PlayerInRow(it);
     if (clicked_player_id == Networking::INVALID_PLAYER_ID)
