@@ -1,4 +1,3 @@
-// -*- C++ -*-
 #ifndef _PlayerListWnd_h_
 #define _PlayerListWnd_h_
 
@@ -39,8 +38,8 @@ private:
     void            DoLayout();
 
     void            PlayerSelectionChanged(const GG::ListBox::SelectionSet& rows);
-    void            PlayerDoubleClicked(GG::ListBox::iterator it);
-    void            PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& pt);
+    void            PlayerDoubleClicked(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys);
+    void            PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys);
     int             PlayerInRow(GG::ListBox::iterator it) const;
 
     PlayerListBox*  m_player_list;
