@@ -9,6 +9,7 @@
 #include "../util/i18n.h"
 #include "../util/Order.h"
 #include "../util/OptionsDB.h"
+#include "../util/ScopedTimer.h"
 #include "../client/human/HumanClientApp.h"
 
 #include <GG/DrawUtil.h>
@@ -65,7 +66,7 @@ namespace {
             double per_turn_cost = tech ? tech->PerTurnCost(queue_element.empire_id) : 1;
             double progress = 0.0;
             if (empire && empire->TechResearched(tech_name))
-                progress = tech->ResearchCost(queue_element.empire_id);
+                progress = tech ? tech->ResearchCost(queue_element.empire_id) : 0.0;
             else if (empire)
                 progress = empire->ResearchProgress(tech_name);
 
@@ -105,8 +106,8 @@ namespace {
         const int GRAPHIC_SIZE = Value(HEIGHT - 9);    // 9 pixels accounts for border thickness so the sharp-cornered icon doesn't with the rounded panel corner
 
         const GG::X NAME_WIDTH = w - GRAPHIC_SIZE - 2*MARGIN - 3;
-        const GG::X METER_WIDTH = w - GRAPHIC_SIZE - 3*MARGIN - 3;
-        const GG::X TURNS_AND_COST_WIDTH = NAME_WIDTH/2;
+        const GG::X METER_WIDTH = w - GRAPHIC_SIZE - 4*MARGIN - 3;
+        const GG::X TURNS_AND_COST_WIDTH = NAME_WIDTH/2 - MARGIN;
 
         const Tech* tech = GetTech(m_tech_name);
         if (tech)
@@ -136,7 +137,7 @@ namespace {
         top += m_name_text->Height();
 
         m_progress_bar = new MultiTurnProgressBar(tech ? tech->ResearchTime(m_empire_id) : 1,
-                                                  turns_completed,
+                                                  turns_completed, tech ? tech->ResearchCost(m_empire_id) : 0.0, turn_spending,
                                                   GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
                                                   ClientUI::TechWndProgressBarColor(),
                                                   m_in_progress ? ClientUI::ResearchableTechFillColor() : GG::LightColor(ClientUI::ResearchableTechFillColor()) );
@@ -146,7 +147,10 @@ namespace {
 
         using boost::io::str;
 
-        std::string turns_cost_text = str(FlexibleFormat(UserString("TECH_TURN_COST_STR")) % DoubleToString(turn_spending, 3, false));
+        double max_spending_per_turn = tech->ResearchCost(m_empire_id) / m_total_turns;
+        std::string turns_cost_text = str(FlexibleFormat(UserString("TECH_TURN_COST_STR"))
+            % DoubleToString(turn_spending, 3, false)
+            % DoubleToString(max_spending_per_turn, 3, false));
         m_RPs_and_turns_text = new CUILabel(turns_cost_text, GG::FORMAT_LEFT);
         m_RPs_and_turns_text->MoveTo(GG::Pt(left, top));
         m_RPs_and_turns_text->Resize(GG::Pt(TURNS_AND_COST_WIDTH, GG::Y(FONT_PTS + MARGIN)));
@@ -191,6 +195,127 @@ namespace {
     }
 }
 
+//////////////////////////////////////////////////
+// ResearchQueueListBox                         //
+//////////////////////////////////////////////////
+class ResearchQueueListBox : public QueueListBox {
+public:
+    ResearchQueueListBox(const std::string& drop_type_str, const std::string& prompt_str) :
+        QueueListBox(drop_type_str, prompt_str)
+    {}
+
+    boost::signals2::signal<void (GG::ListBox::iterator, int)>   QueueItemRalliedToSignal;
+    boost::signals2::signal<void ()>                             ShowPediaSignal;
+
+protected:
+    void ItemRightClickedImpl(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
+        // mostly duplicated equivalent in QueueListBox, but with an extra command...
+
+        GG::MenuItem menu_contents;
+        menu_contents.next_level.push_back(GG::MenuItem(UserString("MOVE_UP_QUEUE_ITEM"),   1, false, false));
+        menu_contents.next_level.push_back(GG::MenuItem(UserString("MOVE_DOWN_QUEUE_ITEM"), 2, false, false));
+        menu_contents.next_level.push_back(GG::MenuItem(UserString("DELETE_QUEUE_ITEM"),    3, false, false));
+
+        GG::ListBox::Row* row = *it;
+        QueueRow* queue_row = row ? dynamic_cast<QueueRow*>(row) : 0;
+
+        // pedia lookup
+        std::string tech_name = queue_row->tech_name;
+        if (tech_name == "") {
+            ErrorLogger() << "Empty tech name referenced during right click";
+            return;
+        }
+
+        if (UserStringExists(tech_name))
+            tech_name = UserString(tech_name);
+
+        std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % tech_name);
+        menu_contents.next_level.push_back(GG::MenuItem(popup_label, 4, false, false));
+
+        GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
+                            ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+
+        if (popup.Run()) {
+            switch (popup.MenuID()) {
+            case 1: { // move item to top
+                if (GG::ListBox::Row* row = *it)
+                    this->QueueItemMovedSignal(row, 0);
+                break;
+            }
+            case 2: { // move item to bottom
+                if (GG::ListBox::Row* row = *it)
+                    this->QueueItemMovedSignal(row, NumRows());
+                break;
+            }
+            case 3: { // delete item
+                this->QueueItemDeletedSignal(it);
+                break;
+            }
+            case 4: { // pedia lookup
+                this->LeftClickedSignal(it, pt, modkeys);
+                ShowPediaSignal();
+                break;
+            }
+            default:
+                break;
+            }
+        }
+    }
+};
+
+//////////////////////////////////////////////////
+// ResearchQueueWnd                             //
+//////////////////////////////////////////////////
+class ResearchQueueWnd : public CUIWnd {
+public:
+    /** \name Structors */ //@{
+    ResearchQueueWnd(GG::X x, GG::Y y, GG::X w, GG::Y h) :
+        CUIWnd("", x, y, w, h, GG::INTERACTIVE | GG::RESIZABLE | GG::DRAGABLE | GG::ONTOP | PINABLE,
+               "research.ResearchQueueWnd"),
+        m_queue_lb(0)
+    {
+        Init(HumanClientApp::GetApp()->EmpireID());
+    }
+    //@}
+
+    /** \name Mutators */ //@{
+    virtual void        SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+        GG::Pt sz = Size();
+        CUIWnd::SizeMove(ul, lr);
+        if (Size() != sz)
+            DoLayout();
+    }
+
+    ResearchQueueListBox*   GetQueueListBox() { return m_queue_lb; }
+
+    void                SetEmpire(int id) {
+        if (const Empire* empire = GetEmpire(id))
+            SetName(boost::io::str(FlexibleFormat(UserString("RESEARCH_QUEUE_EMPIRE")) % empire->Name()));
+        else
+            SetName("");
+    }
+    //@}
+
+private:
+    void DoLayout() {
+        m_queue_lb->SizeMove(GG::Pt(GG::X0, GG::Y0),
+                                GG::Pt(ClientWidth(), ClientHeight() - GG::Y(CUIWnd::INNER_BORDER_ANGLE_OFFSET)));
+    }
+
+    void Init(int empire_id) {
+        m_queue_lb = new ResearchQueueListBox("RESEARCH_QUEUE_ROW", UserString("RESEARCH_QUEUE_PROMPT"));
+        m_queue_lb->SetStyle(GG::LIST_NOSORT | GG::LIST_NOSEL | GG::LIST_USERDELETE);
+        m_queue_lb->SetName("ResearchQueue ListBox");
+
+        SetEmpire(empire_id);
+
+        AttachChild(m_queue_lb);
+        DoLayout();
+    }
+
+    ResearchQueueListBox*   m_queue_lb;
+};
+
 
 //////////////////////////////////////////////////
 // ResearchWnd                                  //
@@ -198,32 +323,28 @@ namespace {
 ResearchWnd::ResearchWnd(GG::X w, GG::Y h) :
     GG::Wnd(GG::X0, GG::Y0, w, h, GG::INTERACTIVE | GG::ONTOP),
     m_research_info_panel(0),
-    m_queue_lb(0),
+    m_queue_wnd(0),
     m_tech_tree_wnd(0),
-    m_enabled(false)
+    m_enabled(false),
+    m_empire_shown_id(ALL_EMPIRES)
 {
     GG::X queue_width(GetOptionsDB().Get<int>("UI.queue-width"));
-
-    m_research_info_panel = new ProductionInfoPanel(UserString("RESEARCH_INFO_PANEL_TITLE"),
-                                                    UserString("RESEARCH_INFO_RP"),
-                                                    queue_width, GG::Y(100));
-
-    m_queue_lb = new QueueListBox("RESEARCH_QUEUE_ROW", UserString("RESEARCH_QUEUE_PROMPT"));
-    m_queue_lb->SetStyle(GG::LIST_NOSORT | GG::LIST_NOSEL | GG::LIST_USERDELETE);
-
-    GG::Connect(m_queue_lb->QueueItemMovedSignal,       &ResearchWnd::QueueItemMoved,               this);
-    GG::Connect(m_queue_lb->LeftClickedSignal,          &ResearchWnd::QueueItemClickedSlot,         this);
-    GG::Connect(m_queue_lb->DoubleClickedSignal,        &ResearchWnd::QueueItemDoubleClickedSlot,   this);
-
-
     GG::Pt tech_tree_wnd_size = ClientSize() - GG::Pt(GG::X(GetOptionsDB().Get<int>("UI.queue-width")), GG::Y0);
+
+    m_research_info_panel = new ProductionInfoPanel(UserString("RESEARCH_WND_TITLE"), UserString("RESEARCH_INFO_RP"),
+                                                    GG::X0, GG::Y0, GG::X(queue_width), GG::Y(100), "research.InfoPanel");
+    m_queue_wnd = new ResearchQueueWnd(GG::X0, GG::Y(100), queue_width, GG::Y(ClientSize().y - 100));
     m_tech_tree_wnd = new TechTreeWnd(tech_tree_wnd_size.x, tech_tree_wnd_size.y);
 
-    GG::Connect(m_tech_tree_wnd->AddTechsToQueueSignal, &ResearchWnd::AddTechsToQueueSlot,          this);
-
+    GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemMovedSignal,   &ResearchWnd::QueueItemMoved,               this);
+    GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemDeletedSignal, &ResearchWnd::DeleteQueueItem,              this);
+    GG::Connect(m_queue_wnd->GetQueueListBox()->LeftClickedSignal,      &ResearchWnd::QueueItemClickedSlot,         this);
+    GG::Connect(m_queue_wnd->GetQueueListBox()->DoubleClickedSignal,    &ResearchWnd::QueueItemDoubleClickedSlot,   this);
+    GG::Connect(m_queue_wnd->GetQueueListBox()->ShowPediaSignal,        &ResearchWnd::ShowPedia,                    this);
+    GG::Connect(m_tech_tree_wnd->AddTechsToQueueSignal,                 &ResearchWnd::AddTechsToQueueSlot,          this);
 
     AttachChild(m_research_info_panel);
-    AttachChild(m_queue_lb);
+    AttachChild(m_queue_wnd);
     AttachChild(m_tech_tree_wnd);
 
     SetChildClippingMode(ClipToClient);
@@ -248,7 +369,7 @@ void ResearchWnd::DoLayout() {
     GG::Pt queue_ul = GG::Pt(GG::X(2), m_research_info_panel->Height());
     GG::Pt queue_size = GG::Pt(m_research_info_panel->Width() - 4,
                                ClientSize().y - 4 - m_research_info_panel->Height());
-    m_queue_lb->SizeMove(queue_ul, queue_ul + queue_size);
+    m_queue_wnd->SizeMove(queue_ul, queue_ul + queue_size);
 
     GG::Pt tech_tree_wnd_size = ClientSize() - GG::Pt(m_research_info_panel->Width(), GG::Y0);
     GG::Pt tech_tree_wnd_ul = GG::Pt(m_research_info_panel->Width(), GG::Y0);
@@ -271,7 +392,7 @@ void ResearchWnd::Reset() {
     m_tech_tree_wnd->Reset();
     UpdateQueue();
     UpdateInfoPanel();
-    m_queue_lb->BringRowIntoView(m_queue_lb->begin());
+    m_queue_wnd->GetQueueListBox()->BringRowIntoView(m_queue_wnd->GetQueueListBox()->begin());
 }
 
 void ResearchWnd::Update() {
@@ -289,6 +410,19 @@ void ResearchWnd::ShowTech(const std::string& tech_name) {
     m_tech_tree_wnd->SelectTech(tech_name);
 }
 
+void ResearchWnd::ShowPedia()
+{ m_tech_tree_wnd->ShowPedia(); }
+
+void ResearchWnd::HidePedia()
+{ m_tech_tree_wnd->HidePedia(); }
+
+void ResearchWnd::TogglePedia()
+{ m_tech_tree_wnd->TogglePedia(); }
+
+bool ResearchWnd::PediaVisible()
+{ return m_tech_tree_wnd->PediaVisible(); }
+
+
 void ResearchWnd::QueueItemMoved(GG::ListBox::Row* row, std::size_t position) {
     if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(row)) {
         int empire_id = HumanClientApp::GetApp()->EmpireID();
@@ -305,6 +439,13 @@ void ResearchWnd::Sanitize()
 void ResearchWnd::Render()
 {}
 
+void ResearchWnd::SetEmpireShown(int empire_id) {
+    if (empire_id != m_empire_shown_id) {
+        m_empire_shown_id = empire_id;
+        Refresh();
+    }
+}
+
 void ResearchWnd::ResearchQueueChangedSlot() {
     UpdateQueue();
     UpdateInfoPanel();
@@ -312,32 +453,46 @@ void ResearchWnd::ResearchQueueChangedSlot() {
 }
 
 void ResearchWnd::UpdateQueue() {
+    DebugLogger() << "ResearchWnd::UpdateQueue()";
+    ScopedTimer timer("ResearchWnd::UpdateQueue");
+
+    m_queue_wnd->SetEmpire(m_empire_shown_id);
+
+    QueueListBox* queue_lb = m_queue_wnd->GetQueueListBox();
+    std::size_t first_visible_queue_row = std::distance(queue_lb->begin(), queue_lb->FirstRowShown());
+    queue_lb->Clear();
+
     const Empire* empire = GetEmpire(HumanClientApp::GetApp()->EmpireID());
     if (!empire)
         return;
 
     const ResearchQueue& queue = empire->GetResearchQueue();
-    std::size_t first_visible_queue_row = std::distance(m_queue_lb->begin(), m_queue_lb->FirstRowShown());
-    m_queue_lb->Clear();
-    const GG::X QUEUE_WIDTH = m_queue_lb->Width() - 8 - 14;
+
 
     for (ResearchQueue::const_iterator it = queue.begin(); it != queue.end(); ++it) {
         const ResearchQueue::Element& elem = *it;
-        m_queue_lb->Insert(new QueueRow(QUEUE_WIDTH, elem));
+        QueueRow* row = new QueueRow(queue_lb->RowWidth(), elem);
+        queue_lb->Insert(row);
     }
 
-    if (!m_queue_lb->Empty())
-        m_queue_lb->BringRowIntoView(--m_queue_lb->end());
-    if (first_visible_queue_row < m_queue_lb->NumRows())
-        m_queue_lb->BringRowIntoView(boost::next(m_queue_lb->begin(), first_visible_queue_row));
+    if (!queue_lb->Empty())
+        queue_lb->BringRowIntoView(--queue_lb->end());
+    if (first_visible_queue_row < queue_lb->NumRows())
+        queue_lb->BringRowIntoView(boost::next(queue_lb->begin(), first_visible_queue_row));
 }
 
 void ResearchWnd::UpdateInfoPanel() {
-    const Empire* empire = GetEmpire(HumanClientApp::GetApp()->EmpireID());
-    if (!empire)
+    const Empire* empire = GetEmpire(m_empire_shown_id);
+    if (!empire) {
+        m_research_info_panel->SetName(UserString("RESEARCH_WND_TITLE"));
+        m_research_info_panel->ClearLocalInfo();
         return;
+    } else {
+        m_research_info_panel->SetEmpireID(m_empire_shown_id);
+    }
+
     const ResearchQueue& queue = empire->GetResearchQueue();
-    float RPs = empire->ResourceProduction(RE_RESEARCH);
+    float RPs = empire->ResourceOutput(RE_RESEARCH);
     float total_queue_cost = queue.TotalRPsSpent();
     m_research_info_panel->SetTotalPointsCost(RPs, total_queue_cost);
 
@@ -399,8 +554,8 @@ void ResearchWnd::DeleteQueueItem(GG::ListBox::iterator it) {
         empire->UpdateResearchQueue();
 }
 
-void ResearchWnd::QueueItemClickedSlot(GG::ListBox::iterator it, const GG::Pt& pt) {
-    if (m_queue_lb->DisplayingValidQueueItems()) {
+void ResearchWnd::QueueItemClickedSlot(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
+    if (m_queue_wnd->GetQueueListBox()->DisplayingValidQueueItems()) {
         QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(*it);
         if (!queue_row)
             return;
@@ -408,13 +563,13 @@ void ResearchWnd::QueueItemClickedSlot(GG::ListBox::iterator it, const GG::Pt& p
     }
 }
 
-void ResearchWnd::QueueItemDoubleClickedSlot(GG::ListBox::iterator it) {
-    if (m_queue_lb->DisplayingValidQueueItems()) {
+void ResearchWnd::QueueItemDoubleClickedSlot(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
+    if (m_queue_wnd->GetQueueListBox()->DisplayingValidQueueItems()) {
         DeleteQueueItem(it);
     }
 }
 
 void ResearchWnd::EnableOrderIssuing(bool enable/* = true*/) {
     m_enabled = enable;
-    m_queue_lb->EnableOrderIssuing(m_enabled);
+    m_queue_wnd->GetQueueListBox()->EnableOrderIssuing(m_enabled);
 }
