@@ -40,8 +40,8 @@ namespace {
 
     const std::string SAVE_FILE_WND_NAME = "save-load";
 
-    const GG::X PROMT_WIDTH ( 200 );
-    const GG::Y PROMPT_HEIGHT ( 75 );
+    const GG::X PROMT_WIDTH(200);
+    const GG::Y PROMPT_HEIGHT(75);
 
     const double DEFAULT_STRETCH = 1.0;
 
@@ -314,7 +314,7 @@ public:
         SetMargin (ROW_MARGIN);
         this->m_filename = full.filename;
 
-        VarText browse_text("SAVE_DIALOG_ROW_BROWSE_TEMPLATE");
+        VarText browse_text(UserStringNop("SAVE_DIALOG_ROW_BROWSE_TEMPLATE"));
 
         for (std::vector<SaveFileColumn>::const_iterator it = visible_columns.begin();
              it != visible_columns.end(); ++it)
@@ -444,7 +444,6 @@ public:
 
     /// Loads preview data on all save files in a directory specidifed by path.
     /// @param [in] path The path of the directory
-    /// @param [out] previews The preview datas indexed by file names
     void LoadSaveGamePreviews(const fs::path& path, const std::string& extension) {
         LoadDirectories(path);
 
@@ -454,8 +453,7 @@ public:
     }
 
     /// Loads preview data on all save files in a directory specidifed by path.
-    /// @param [in] path The path of the directory
-    /// @param [out] previews The preview datas indexed by file names
+    /// @param [in] previews The preview data
     void LoadSaveGamePreviews(const std::vector<FullPreview>& previews) {
         int tooltip_delay = GetOptionsDB().Get<int>("UI.save-file-dialog.tooltip-delay");
         for (vector<FullPreview>::const_iterator it = previews.begin(); it != previews.end(); ++it)
@@ -468,8 +466,18 @@ public:
             Insert(new SaveFileRow(".."));
 
         for (fs::directory_iterator it(path); it != end_it; ++it) {
-            if (fs::is_directory(it->path()))
-                Insert(new SaveFileRow(PathString(it->path().filename())));
+            if (fs::is_directory(it->path())) {
+                fs::path last_bit_of_path = it->path().filename();
+                std::string utf8_dir_name = PathString(last_bit_of_path);
+                DebugLogger() << "SaveFileDialog::LoadDirectories name: " << utf8_dir_name << " valid UTF-8: " << IsValidUTF8(utf8_dir_name);
+                Insert(new SaveFileRow(utf8_dir_name));
+
+                //boost::filesystem::path::string_type path_native = last_bit_of_path.native();
+                //std::string path_string;
+                //utf8::utf16to8(path_native.begin(), path_native.end(), std::back_inserter(path_string));
+                //DebugLogger() << "SaveFileDialog::LoadDirectories name: " << path_string << " valid UTF-8: " << IsValidUTF8(path_string);
+                //Insert(new SaveFileRow(path_string));
+            }
         }
     }
 
@@ -477,7 +485,7 @@ public:
         for (GG::ListBox::iterator it = begin(); it != end(); ++it) {
             SaveFileRow* row = dynamic_cast<SaveFileRow*>(*it);
             if (row && row->Filename() == filename)
-                    return true;
+                return true;
         }
         return false;
     }
@@ -494,15 +502,18 @@ private:
         for (std::vector<std::string>::const_iterator it = names.begin();
              it != names.end(); ++it)
         {
+            bool found_col = false;
             for (std::vector<SaveFileColumn>::const_iterator jt = m_columns.begin();
                  jt != m_columns.end(); ++jt)
             {
                 if (jt->Name() == *it) {
                     columns.push_back(*jt);
+                    found_col = true;
                     break;
                 }
             }
-            ErrorLogger() << "SaveFileListBox::FilterColumns: Column not found: " << *it;
+            if (!found_col)
+                ErrorLogger() << "SaveFileListBox::FilterColumns: Column not found: " << *it;
         }
         DebugLogger() << "SaveFileDialog::FilterColumns: Visible columns: " << columns.size();
         return columns;
@@ -597,8 +608,16 @@ void SaveFileDialog::Init() {
     GG::Button* cancel_btn = new CUIButton(UserString("CANCEL"));
 
     m_name_edit = new CUIEdit("");
+    if (m_extension != MP_SAVE_FILE_EXTENSION && m_extension != SP_SAVE_FILE_EXTENSION) {
+        std::string savefile_ext = HumanClientApp::GetApp()->SinglePlayerGame() ? SP_SAVE_FILE_EXTENSION : MP_SAVE_FILE_EXTENSION;
+        DebugLogger() << "SaveFileDialog passed invalid extension " << m_extension << ", changing to " << savefile_ext;
+        m_extension = savefile_ext;
+    }
+
     GG::Label* filename_label = new CUILabel(UserString("SAVE_FILENAME"), GG::FORMAT_NOWRAP);
     GG::Label* directory_label = new CUILabel(UserString("SAVE_DIRECTORY"), GG::FORMAT_NOWRAP);
+    //std::cout << "pathstrnig: " << PathString(GetSaveDir()) << std::endl;
+    DebugLogger() << "pathstring: " << PathString(GetSaveDir());
     m_current_dir_edit = new CUIEdit(PathString(GetSaveDir()));
 
     m_layout->Add(directory_label, 0, 0);
@@ -648,10 +667,10 @@ void SaveFileDialog::Init() {
     GG::Connect(m_file_list->DoubleClickedSignal,               &SaveFileDialog::DoubleClickRow,    this);
     GG::Connect(m_name_edit->EditedSignal,                      &SaveFileDialog::FileNameEdited,    this);
     GG::Connect(m_current_dir_edit->EditedSignal,               &SaveFileDialog::DirectoryEdited,   this);
-    GG::Connect(GG::GUI::GetGUI()->FocusChangedSignal,          &SaveFileDialog::UpdatePreviewList, this );
+    GG::Connect(GG::GUI::GetGUI()->FocusChangedSignal,          boost::bind(&SaveFileDialog::UpdatePreviewList, this));
 
     if (!m_load_only) {
-        m_name_edit->SetText(std::string("save-") + FilenameTimestamp());
+        m_name_edit->SetText(std::string("save-") + FilenameTimestamp() + m_extension);
         m_name_edit->SelectAll();
     }
 
@@ -687,7 +706,7 @@ void SaveFileDialog::KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::F
 
     // Update list on enter if directory changed by hand
     if (key == GG::GGK_RETURN || key == GG::GGK_KP_ENTER) {
-        if (m_loaded_dir != GetDirPath() ) {
+        if (m_loaded_dir != GetDirPath()) {
             UpdatePreviewList();
         } else {
             if (GG::GUI::GetGUI()->FocusWnd() == m_name_edit) {
@@ -718,27 +737,46 @@ void SaveFileDialog::Confirm() {
 
     /// Check if we chose a directory
     std::string choice = m_name_edit->Text();
-    fs::path current_dir(GetDirPath());
-    if (!choice.empty()) {
-        fs::path chosen = current_dir / choice;
-        if (fs::is_directory(chosen)) {
-            DebugLogger() << "SaveFileDialog::Confirm: " << chosen << " is a directory. Listing content.";
-            UpdateDirectory(PathString(chosen));
-            return;
-        } else {
-            DebugLogger() << "SaveFileDialog::Confirm: File " << chosen << " chosen.";
-            // If not loading and file exists, ask to confirm override
-            // Use Result() to ensure we are using the extension.
-            if ( !m_load_only && fs::exists(fs::path(Result())) ) {
-                boost::format templ( UserString("SAVE_REALLY_OVERRIDE") );
-                std::string question = (templ % choice).str();
-                if(!Prompt(question)){
-                    return;
-                }
-            }
-        }
-    } else {
+    if (choice.empty()) {
         DebugLogger() << "SaveFileDialog::Confirm: Returning no file.";
+        CloseClicked();
+        return;
+    }
+
+    fs::path choice_path = FilenameToPath(choice);
+    DebugLogger() << "choice: " << choice << " valid utf-8: " << IsValidUTF8(choice);
+
+    fs::path current_dir = FilenameToPath(GetDirPath());
+    DebugLogger() << "current dir PathString: " << PathString(current_dir) << " valid utf-8: " << IsValidUTF8(PathString(current_dir));
+
+    fs::path chosen_full_path = current_dir / choice_path;
+    DebugLogger() << "chosen_full_path PathString: " << PathString(chosen_full_path) << " valid utf-8: " << IsValidUTF8(PathString(chosen_full_path));
+    DebugLogger() << "chosen_full_path is directory? : " << fs::is_directory(chosen_full_path);
+
+
+    if (fs::is_directory(chosen_full_path)) {
+        DebugLogger() << "SaveFileDialog::Confirm: " << PathString(chosen_full_path) << " is a directory. Listing content.";
+        UpdateDirectory(PathString(chosen_full_path));
+        return;
+
+    } else if (!m_load_only) {
+        // append appropriate extension if invalid
+        std::string chosen_ext = fs::path(chosen_full_path).extension().string();
+        if (chosen_ext != m_extension) {
+            choice += m_extension;
+            chosen_full_path += m_extension;
+            m_name_edit->SetText(m_name_edit->Text() + m_extension);
+        }
+        DebugLogger() << "SaveFileDialog::Confirm: File " << PathString(chosen_full_path) << " chosen.";
+        // If not loading and file exists(and is regular file), ask to confirm override
+        if (fs::is_regular_file(chosen_full_path)) {
+            std::string question = str((FlexibleFormat(UserString("SAVE_REALLY_OVERRIDE")) % choice));
+            if (!Prompt(question))
+                return;
+        } else if (fs::exists(chosen_full_path)) {
+            ErrorLogger() << "SaveFileDialog::Confirm: Invalid status for file: " << Result();
+            return;
+        }
     }
 
     CloseClicked();
@@ -776,14 +814,14 @@ void SaveFileDialog::AskDelete() {
     }
 }
 
-void SaveFileDialog::DoubleClickRow(GG::ListBox::iterator row) {
-    m_file_list->SelectRow ( row );
+void SaveFileDialog::DoubleClickRow(GG::ListBox::iterator row, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
+    m_file_list->SelectRow(row);
     Confirm();
 }
 
 void SaveFileDialog::Cancel() {
     DebugLogger() << "SaveFileDialog::Cancel: Dialog Canceled";
-    m_name_edit->SetText ( "" );
+    m_name_edit->SetText("");
     CloseClicked();
 }
 
@@ -799,6 +837,7 @@ void SaveFileDialog::SelectionChanged(const GG::ListBox::SelectionSet& selection
 }
 
 void SaveFileDialog::UpdateDirectory(const std::string& newdir) {
+    //std::cout << "SaveFileDialog::UpdateDirectory newdir: " << newdir << std::endl;
     SetDirPath(newdir);
     UpdatePreviewList();
 }
@@ -817,12 +856,14 @@ void SaveFileDialog::UpdatePreviewList() {
     DebugLogger() << "SaveFileDialog::UpdatePreviewList";
 
     m_file_list->Clear();
+
     // Needed because there is a bug in ListBox, where the headers
     // never resize to less wide
     m_file_list->ResetColHeaders();
+
     // If no browsing, no reloading
     if (!m_server_previews) {
-        m_file_list->LoadSaveGamePreviews(GetDirPath(), m_extension);
+        m_file_list->LoadSaveGamePreviews(FilenameToPath(GetDirPath()), m_extension);
     } else {
         PreviewInformation preview_information;
         HumanClientApp::GetApp()->RequestSavePreviews(GetDirPath(), preview_information);
@@ -834,8 +875,7 @@ void SaveFileDialog::UpdatePreviewList() {
         row->push_back(new CUILabel(SERVER_LABEL));
         m_remote_dir_dropdown->Insert(row);
 
-        for (std::vector<std::string>::const_iterator
-             it = preview_information.subdirectories.begin();
+        for (std::vector<std::string>::const_iterator it = preview_information.subdirectories.begin();
              it != preview_information.subdirectories.end(); ++it)
         {
             GG::DropDownList::Row* row = new GG::DropDownList::Row();
@@ -857,16 +897,16 @@ void SaveFileDialog::UpdatePreviewList() {
     m_file_list->BringRowIntoView(m_file_list->begin());
 
     // Remember which directory we are showing
-    m_loaded_dir = GetDirPath();
+    //m_loaded_dir = GetDirPath();
 
     CheckChoiceValidity();
 }
 
 bool SaveFileDialog::CheckChoiceValidity() {
     // Check folder validity
-    if ( !m_server_previews ) {
-        fs::path dir ( GetDirPath() );
-        if (fs::exists (dir) && fs::is_directory (dir)) {
+    if (!m_server_previews) {
+        fs::path dir(FilenameToPath(GetDirPath()));
+        if (fs::exists(dir) && fs::is_directory(dir)) {
             m_current_dir_edit->SetColor(ClientUI::TextColor());
         } else {
             m_current_dir_edit->SetColor(GG::CLR_RED);
@@ -880,17 +920,17 @@ bool SaveFileDialog::CheckChoiceValidity() {
     }
 
     // Check file name validity
-    if ( m_load_only ) {
-        if ( !m_file_list->HasFile ( m_name_edit->Text() ) ) {
+    if (m_load_only) {
+        if (!m_file_list->HasFile(m_name_edit->Text())) {
             m_confirm_btn->Disable();
             return false;
         } else {
-            m_confirm_btn->Disable ( false );
+            m_confirm_btn->Disable(false);
             return true;
         }
-    }else{
-        return true;
     }
+
+    return true;
 }
 
 void SaveFileDialog::FileNameEdited(const std::string& filename)
@@ -899,39 +939,47 @@ void SaveFileDialog::FileNameEdited(const std::string& filename)
 void SaveFileDialog::DirectoryEdited(const string& filename)
 { CheckChoiceValidity(); }
 
-std::string SaveFileDialog::GetDirPath() const{
-    std::string dir = m_current_dir_edit->Text();
+std::string SaveFileDialog::GetDirPath() const {
+    const std::string& path_edit_text = m_current_dir_edit->Text();
 
-    if (m_server_previews) {
-        // We want to indicate at all times that the saves are on the server.
-        if (dir.find(SERVER_LABEL) != 0) {
-            if (dir.find("/") != 0)
-                dir = "/" + dir;
+    //std::cout << "SaveFileDialog::GetDirPath text: " << path_edit_text << " valid UTF-8: " << utf8::is_valid(path_edit_text.begin(), path_edit_text.end()) << std::endl;
+    //DebugLogger() << "SaveFileDialog::GetDirPath text: " << path_edit_text << " valid UTF-8: " << utf8::is_valid(path_edit_text.begin(), path_edit_text.end());
+    if (!m_server_previews) {
+        return path_edit_text;
+    }
 
-            dir = SERVER_LABEL + dir;
-        }
+    std::string dir = path_edit_text;
 
-        // We should now be sure that the path is SERVER/whatever
-        if (dir.length() < SERVER_LABEL.size()) {
-            ErrorLogger() << "SaveFileDialog::GetDirPath: Error decorating directory for server: not long enough";
-            return ".";
-        } else {
-            // Translate the server label into the standard relative path marker the server understands
-            return std::string(".") + dir.substr(SERVER_LABEL.size());
-        }
+    // We want to indicate at all times that the saves are on the server.
+    if (dir.find(SERVER_LABEL) != 0) {
+        if (dir.find("/") != 0)
+            dir = "/" + dir;
+
+        dir = SERVER_LABEL + dir;
+    }
+
+    // We should now be sure that the path is SERVER/whatever
+    if (dir.length() < SERVER_LABEL.size()) {
+        ErrorLogger() << "SaveFileDialog::GetDirPath: Error decorating directory for server: not long enough";
+        return ".";
     } else {
-        return dir;
+        // Translate the server label into the standard relative path marker the server understands
+        std::string retval = "." + dir.substr(SERVER_LABEL.size());
+        //std::cout << "SaveFileDialog::GetDirPath retval: " << retval << " valid UTF-8: " << utf8::is_valid(retval.begin(), retval.end()) << std::endl;
+        DebugLogger() << "SaveFileDialog::GetDirPath retval: " << retval << " valid UTF-8: " << utf8::is_valid(retval.begin(), retval.end());
+        return retval;
     }
 }
 
 void SaveFileDialog::SetDirPath(const string& path) {
     std::string dirname = path;
     if (m_server_previews) {
-        // Change the format into the generic one
+        // convert the path string into the boost filesystem "generic" format
+        // to allow scanning for delimeters like "/"
         dirname = GenericPathString(fs::path(path));
         // Indicate that the path is on the server
         if (path.find("./") == 0) {
-            dirname = SERVER_LABEL + path.substr(1); 
+            dirname = SERVER_LABEL + path.substr(1);
         } else if (path.find(SERVER_LABEL) == 0) {
             // Already has label. No need to change
         } else {
@@ -949,18 +997,14 @@ void SaveFileDialog::SetDirPath(const string& path) {
 }
 
 std::string SaveFileDialog::Result() const {
-    std::string filename = m_name_edit->Text();
-    if ( filename.empty() ) {
+    std::string choice = m_name_edit->Text();
+    if (choice.empty()) {
         return "";
-    } else {
-        // Ensure the file has an extension
-        std::string::size_type end = filename.length();
-        std::size_t ext_length = m_extension.length();
-        if ( filename.length() < ext_length || filename.substr ( end-ext_length, ext_length ) != m_extension ) {
-            filename.append ( m_extension );
-        }
-        // Convert to a path to create a generic string representation.
-        fs::path current_dir ( GetDirPath() );
-        return ( current_dir / filename ).generic_string();
     }
+
+    fs::path choice_path = FilenameToPath(choice);
+    fs::path current_dir = FilenameToPath(GetDirPath());
+    fs::path chosen_full_path = current_dir / choice_path;
+
+    return PathString(chosen_full_path);
 }
