@@ -25,16 +25,12 @@ namespace {
         db.Add("verbose-sitrep", UserStringNop("OPTIONS_DB_VERBOSE_SITREP_DESC"),  false,  Validator<bool>());
         db.Add<std::string>("hidden-sitrep-templates", UserStringNop("OPTIONS_DB_HIDDEN_SITREP_TEMPLATES_DESC"), "");
         db.Add("UI.sitrep-icon-size", UserStringNop("OPTIONS_DB_UI_SITREP_ICONSIZE"), 24, RangedValidator<int>(12, 64));
-        db.Add("UI.sitrep-font-size", UserStringNop("OPTIONS_DB_UI_SITREP_FONTSIZE"), 12, RangedValidator<int>(10, 48));
     }
     bool temp_bool = RegisterOptions(&AddOptions);
-    
+
     GG::X GetIconSize()
     { return GG::X(GetOptionsDB().Get<int>("UI.sitrep-icon-size")); }
 
-    GG::X GetFontSize()
-    { return GG::X(GetOptionsDB().Get<int>("UI.sitrep-font-size")); }
-    
     std::map<std::string, std::string> label_display_map;
     std::map<int, std::set<std::string> > snoozed_sitreps;
     std::set<std::string> permanently_snoozed_sitreps;
@@ -43,7 +39,7 @@ namespace {
         for (int turn=start_turn; turn < start_turn + num_turns; turn++)
             snoozed_sitreps[turn].insert(sitrep_text);
     }
-    
+
     void HandleLinkClick(const std::string& link_type, const std::string& data) {
         using boost::lexical_cast;
         try {
@@ -141,7 +137,7 @@ namespace {
         // only use those ordered templates actually in the current set of sitrep templates
         std::vector<std::string> ordered_templates = OrderedSitrepTemplateStrings();
         for (std::vector<std::string>::iterator it = ordered_templates.begin(); 
-             it!=ordered_templates.end(); it++) 
+             it!=ordered_templates.end(); ++it)
         {
             if ( (template_set.find(*it) != template_set.end()) && 
                  (std::find(retval.begin(), retval.end(), *it) == retval.end()) )
@@ -149,7 +145,7 @@ namespace {
         }
 
         //now add the current templates that did not have a specified order
-        for (std::set<std::string>::iterator it = template_set.begin(); it!= template_set.end(); it++)
+        for (std::set<std::string>::iterator it = template_set.begin(); it!= template_set.end(); ++it)
             if (std::find(retval.begin(), retval.end(), *it) == retval.end())
                 retval.push_back(*it);
 
@@ -231,10 +227,10 @@ namespace {
                 int text_size = Value((m_link_text->TextLowerRight() - m_link_text->TextUpperLeft()).y);
                 int icon_size = Value(GetIconSize());
                 int max_panel_size = std::max(text_size, icon_size);
-                max_panel_size += Value(ITEM_VERTICAL_PADDING);
+                max_panel_size += Value(2*ITEM_VERTICAL_PADDING);
 
                 GG::Pt panel_size = GG::Pt(GG::X(lr.x - ul.x), GG::Y(max_panel_size));
-                GG::Control::SizeMove(ul, ul + panel_size );
+                GG::Control::SizeMove(ul, ul + panel_size);
                 DoLayout();
             }
         }
@@ -248,12 +244,12 @@ namespace {
             GG::Y ICON_HEIGHT(Value(GetIconSize()));
 
             GG::X left(GG::X0);
-            GG::Y bottom(ClientHeight());
+            GG::Y bottom(ClientHeight() - ITEM_VERTICAL_PADDING);
 
             m_icon->SizeMove(GG::Pt(left, bottom/2 - ICON_HEIGHT/2), GG::Pt(left + GetIconSize(), bottom/2 + ICON_HEIGHT/2));
             left += GetIconSize() + ICON_RIGHT_MARGIN;
 
-            m_link_text->SizeMove(GG::Pt(left, GG::Y0), GG::Pt(ClientWidth(), bottom));
+            m_link_text->SizeMove(GG::Pt(left, GG::Y0), GG::Pt(ClientWidth() - 3, bottom));
         }
 
         void            Init() {
@@ -269,7 +265,7 @@ namespace {
             AttachChild(m_icon);
 
             m_link_text = new LinkText(GG::X0, GG::Y0, GG::X1, m_sitrep_entry.GetText() + " ", 
-                                       ClientUI::GetFont(std::max(1.0*ClientUI::Pts(), 1.0*GetOptionsDB().Get<int>("UI.sitrep-font-size"))),
+                                       ClientUI::GetFont(),
                                        GG::FORMAT_LEFT | GG::FORMAT_VCENTER | GG::FORMAT_WORDBREAK, ClientUI::TextColor());
             m_link_text->SetDecorator(VarText::EMPIRE_ID_TAG, new ColorEmpire());
             AttachChild(m_link_text);
@@ -307,7 +303,7 @@ namespace {
         void SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
             const GG::Pt old_size = Size();
             GG::Pt new_size = lr - ul;
-            new_size.x -= 7; // Avoid allowing the scrollbar to hide the very rightmost pixels.
+            new_size.x -= 9; // Avoid allowing the scrollbar to hide the very rightmost pixels.
             if (!empty() && m_panel && old_size != new_size) {
                 m_panel->Resize(new_size);
                 new_size = m_panel->Size();
@@ -384,7 +380,7 @@ void SitRepPanel::DoLayout() {
 }
 
 void SitRepPanel::KeyPress(GG::Key key, boost::uint32_t key_code_point,
-                            GG::Flags<GG::ModKey> mod_keys)
+                           GG::Flags<GG::ModKey> mod_keys)
 {
     switch (key) {
     case GG::GGK_RETURN:
@@ -405,7 +401,6 @@ void SitRepPanel::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
     CUIWnd::SizeMove(ul, lr);
 
     if (old_size != GG::Wnd::Size()) {
-        GG::ListBox::iterator row_to_show = m_sitreps_lb->FirstRowShown();
         DoLayout();
         Update();
         if (!m_sitreps_lb->Empty()) {
@@ -430,7 +425,8 @@ namespace {
             for (EmpireManager::iterator it = empires.begin(); it != empires.end(); ++it)
                 sr_empires.insert(it->second);
         }
-        for (std::set<Empire*>::iterator it = sr_empires.begin(); it != sr_empires.end(); it++) {
+        for (std::set<Empire*>::iterator it = sr_empires.begin(); it != sr_empires.end(); ++it)
+        {
             for (Empire::SitRepItr sitrep_it = (*it)->SitRepBegin(); sitrep_it != (*it)->SitRepEnd(); ++sitrep_it) {
                 if (!verbose_sitrep) {
                     if (!sitrep_it->Validate())
@@ -487,19 +483,16 @@ void SitRepPanel::CloseClicked()
 
 void SitRepPanel::PrevClicked() {
     std::map<int, std::list<SitRepEntry> > turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
-    m_showing_turn = GetNextNonEmptySitrepsTurn(turns, m_showing_turn, false);
-    Update();
+    ShowSitRepsForTurn(GetNextNonEmptySitrepsTurn(turns, m_showing_turn, false));
 }
 
 void SitRepPanel::NextClicked() {
     std::map<int, std::list<SitRepEntry> > turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
-    m_showing_turn = GetNextNonEmptySitrepsTurn(turns, m_showing_turn, true);
-    Update();
+    ShowSitRepsForTurn(GetNextNonEmptySitrepsTurn(turns, m_showing_turn, true));
 }
 
 void SitRepPanel::LastClicked() {
-    m_showing_turn = CurrentTurn();
-    Update();
+    ShowSitRepsForTurn(CurrentTurn());
 }
 
 void SitRepPanel::FilterClicked() {
@@ -565,7 +558,7 @@ void SitRepPanel::FilterClicked() {
     Update();
 }
 
-void SitRepPanel::DismissSitRep(GG::ListBox::iterator it) {
+void SitRepPanel::DismissSitRep(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod) {
     SitRepRow* sitrep_row = dynamic_cast<SitRepRow*>(*it);
     if (!sitrep_row) {
         return;
@@ -578,7 +571,7 @@ void SitRepPanel::DismissSitRep(GG::ListBox::iterator it) {
     Update();
 }
 
-void SitRepPanel::DismissalMenu(GG::ListBox::iterator it, const GG::Pt& pt) {
+void SitRepPanel::DismissalMenu(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod) {
 
     GG::MenuItem menu_contents;
     std::string sitrep_text;
@@ -635,6 +628,8 @@ void SitRepPanel::DismissalMenu(GG::ListBox::iterator it, const GG::Pt& pt) {
 
 void SitRepPanel::Update() {
     DebugLogger() << "SitRepPanel::Update()";
+
+    std::size_t first_visible_row = std::distance(m_sitreps_lb->begin(), m_sitreps_lb->FirstRowShown());
     m_sitreps_lb->Clear();
 
     // Get back to sane default
@@ -667,7 +662,7 @@ void SitRepPanel::Update() {
          template_it != ordered_templates.end(); ++template_it)
     {
         for (std::list<SitRepEntry>::iterator sitrep_it = currentTurnSitreps.begin();
-             sitrep_it != currentTurnSitreps.end(); sitrep_it++)
+             sitrep_it != currentTurnSitreps.end(); ++sitrep_it)
         {
             if ((sitrep_it->GetLabelString().empty() ? sitrep_it->GetTemplateString() : sitrep_it->GetLabelString()) == *template_it) {
                 //DebugLogger() << "saving into orderedSitreps -  sitrep of template "<<*template_it<<" with full string "<< sitrep_it->GetText();
@@ -680,15 +675,21 @@ void SitRepPanel::Update() {
 
     // copy remaining unordered sitreps
     for (std::list<SitRepEntry>::iterator sitrep_it = currentTurnSitreps.begin();
-         sitrep_it != currentTurnSitreps.end(); sitrep_it++)
+         sitrep_it != currentTurnSitreps.end(); ++sitrep_it)
     { orderedSitreps.push_back(*sitrep_it); }
 
     // create UI rows for all sitrps
-    GG::X width = m_sitreps_lb->Width() - 8;
+    GG::X width = m_sitreps_lb->Width() - ClientUI::ScrollWidth();
     for (std::vector<SitRepEntry>::iterator sitrep_it = orderedSitreps.begin();
-         sitrep_it != orderedSitreps.end(); sitrep_it++)
+         sitrep_it != orderedSitreps.end(); ++sitrep_it)
     { m_sitreps_lb->Insert(new SitRepRow(width, GG::Y(ClientUI::Pts()*2), *sitrep_it)); }
 
+    if (m_sitreps_lb->NumRows() > first_visible_row) {
+        m_sitreps_lb->SetFirstRowShown(boost::next(m_sitreps_lb->begin(), first_visible_row));
+    } else if (!m_sitreps_lb->Empty()) {
+        m_sitreps_lb->BringRowIntoView(--m_sitreps_lb->end());
+    }
+
     // if at first turn with visible sitreps, disable back button
     int firstTurnWithSR = GetNextNonEmptySitrepsTurn(turns, m_showing_turn, false);
     if ((m_showing_turn < 1) || (turns.size() < 2) || (m_showing_turn == firstTurnWithSR)) {
@@ -708,8 +709,13 @@ void SitRepPanel::Update() {
     }
 }
 
-void SitRepPanel::ShowSitRepsForTurn(int turn)
-{ m_showing_turn = turn; }
+void SitRepPanel::ShowSitRepsForTurn(int turn) {
+     bool is_different_turn(m_showing_turn != turn);
+     m_showing_turn = turn;
+     Update();
+     if (is_different_turn)
+         m_sitreps_lb->SetFirstRowShown(m_sitreps_lb->begin());
+}
 
 void SitRepPanel::SetHiddenSitRepTemplates(const std::set<std::string>& templates) {
     std::set<std::string> old_hidden_sitrep_templates = m_hidden_sitrep_templates;
@@ -723,4 +729,3 @@ int SitRepPanel::NumVisibleSitrepsThisTurn() const {
     return turns[CurrentTurn()].size();
 }
 
-
