@@ -1,84 +1,100 @@
 #include "Sound.h"
 #include "../util/Logger.h"
 #include "../util/OptionsDB.h"
+#include "../util/Directories.h"
 
 #ifdef FREEORION_MACOSX
-# include <OpenAL/alc.h>
+#include <OpenAL/al.h>
+#include <OpenAL/alc.h>
 #else
-# include <AL/alc.h>
+#include <AL/alc.h>
+#include <AL/al.h>
 #endif
 
+#include <map>
+
+#include <boost/scoped_array.hpp>
+
+#include <vorbis/vorbisfile.h>
+
+class Sound::SoundImpl
+{
+public:
+    SoundImpl();  ///< ctor.
+    ~SoundImpl(); ///< dotr.
+
+    /** Enables the sound system.  Throws Sound::InitializationFailureException on failure. */
+    void Enable();
+
+    /** Disable the sound system. */
+    void Disable();
+
+    /**Initialize OpenAL*/
+    void InitOpenAL();
+
+    /**Shutdown OpenAL.*/
+    void ShutdownOpenAL();
+
+    /** Plays a music file.  The file will be played in an infinitve loop if \a loop is < 0, and it will be played \a
+        loops + 1 times otherwise. */
+    void PlayMusic(const boost::filesystem::path& path, int loops = 0);
+
+    /** Stops playing music. */
+    void StopMusic();
+
+    /** Plays a sound file. */
+    void PlaySound(const boost::filesystem::path& path, bool is_ui_sound = false);
+
+    /** Frees the cached sound data associated with the filename. */
+    void FreeSound(const boost::filesystem::path& path);
+
+    /** Frees all cached sound data. */
+    void FreeAllSounds();
+
+    /** Sets the music volume from 0 (muted) to 255 (full volume); \a vol is range-adjusted. */
+    void SetMusicVolume(int vol);
+
+    /** Sets the UI sounds volume from 0 (muted) to 255 (full volume); \a vol is range-adjusted. */
+    void SetUISoundsVolume(int vol);
+
+    /** Does the work that must be done by the sound system once per frame. */
+    void DoFrame();
+
+    bool UISoundsTemporarilyDisabled() const;
+
+    /**Increment the temporary disabled depth.*/
+    void IncDisabledCount();
+
+    /** Decrease the temporary disabled depth.*/
+    void DecDisabledCount();
+
+private:
+    static const int NUM_SOURCES = 16; // The number of sources for OpenAL to create. Should be 2 or more.
+    static const int NUM_MUSIC_BUFFERS = 2; // The number of music buffers.
+
+    ALuint                            m_sources[NUM_SOURCES]; ///< OpenAL sound sources. The first one is used for music
+    int                               m_music_loops;          ///< the number of loops of the current music to play (< 0 for loop forever)
+    std::string                       m_music_name;           ///< the name of the currently-playing music file
+    std::map<std::string, ALuint>     m_buffers;              ///< the currently-cached (and possibly playing) sounds, if any; keyed on filename
+    ALuint                            m_music_buffers[NUM_MUSIC_BUFFERS];     ///< two additional buffers for music. statically defined as they'll be changed many times.
+    OggVorbis_File                    m_ogg_file;             ///< the currently open ogg file
+    ALenum                            m_ogg_format;           ///< mono or stereo
+    ALsizei                           m_ogg_freq;             ///< sampling frequency
+    unsigned int                      m_temporary_disable_count; ///< Count of the number of times sound was disabled. Sound is enabled when this is zero.
+    /** m_initialized indicates if the sound system has been initialized.
+        The system will not be initialized if both sound effects and
+        music are disabled or if initialization failed. */
+    bool                              m_initialized;
+};
 
 namespace {
     const int BUFFER_SIZE = 409600; // The size of the buffer we read music data into.
 
-    void InitOpenAL(int num_sources, ALuint *sources, ALuint *music_buffers) {
-        ALCcontext *m_context;
-        ALCdevice *m_device;
-        ALenum error_code;
-
-        m_device = alcOpenDevice(0);    /* currently only select the default output device - usually a NULL-terminated
-                                         * string desctribing a device can be passed here (of type ALchar*) */
-        if (m_device == 0) {
-            ErrorLogger() << "Unable to initialise OpenAL device: " << alGetString(alGetError()) << "\n";
-        } else {
-            m_context = alcCreateContext(m_device, 0);   // instead of 0 we can pass a ALCint* pointing to a set of
-                                                         // attributes (ALC_FREQUENCY, ALC_REFRESH and ALC_SYNC)
-
-            if ((m_context != 0) && (alcMakeContextCurrent(m_context) == AL_TRUE)) {
-                alListenerf(AL_GAIN,1.0);
-                alGetError(); // clear possible previous errors (just to be certain)
-                alGenSources(num_sources, sources);
-                error_code = alGetError();
-                if(error_code != AL_NO_ERROR) {
-                    ErrorLogger() << "Unable to create OpenAL sources: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
-                    alcMakeContextCurrent(0);
-                    alcDestroyContext(m_context);
-                } else {
-                    alGetError();
-                    alGenBuffers(2, music_buffers);
-                    error_code = alGetError();
-                    if (error_code != AL_NO_ERROR) {
-                        ErrorLogger() << "Unable to create OpenAL buffers: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
-                        alDeleteBuffers(2, music_buffers);
-                        alcMakeContextCurrent(0);
-                        alcDestroyContext(m_context);
-                    } else {
-                        for (int i = 0; i < num_sources; ++i) {
-                            alSourcei(sources[i], AL_SOURCE_RELATIVE, AL_TRUE);
-                        }
-                        DebugLogger() << "OpenAL initialized. Version "
-                                               << alGetString(AL_VERSION)
-                                               << "Renderer "
-                                               << alGetString(AL_RENDERER)
-                                               << "Vendor "
-                                               << alGetString(AL_VENDOR) << "\n"
-                                               << "Extensions: "
-                                               << alGetString(AL_EXTENSIONS) << "\n";
-                    }
-                }
-            } else {
-                ErrorLogger() << "Unable to create OpenAL context : " << alGetString(alGetError()) << "\n";
-            }
-        }
-    }
-
-    void ShutdownOpenAL() {
-        ALCcontext* context = alcGetCurrentContext();
-        if (context != 0) {
-            ALCdevice* device = alcGetContextsDevice(context);
-            alcMakeContextCurrent(0);
-            alcDestroyContext(context);
-            if (device != 0)
-                alcCloseDevice(device);
-        }
-    }
-
 #ifdef FREEORION_WIN32
     int _fseek64_wrap(FILE *f, ogg_int64_t off, int whence) {
         if (!f)
             return -1;
-        return fseek(f,off,whence);
+        return fseek(f, off, whence);
     }
 #endif
 
@@ -87,8 +103,8 @@ namespace {
     {
         ALenum m_openal_error;
         int endian = 0; /// 0 for little-endian (x86), 1 for big-endian (ppc)
-        int bitStream,bytes,bytes_new;
-        char* array = new char[buffer_size];
+        int bitStream, bytes, bytes_new;
+        boost::scoped_array<char> array(new char[buffer_size]);
         bytes = 0;
 
         if (alcGetCurrentContext() != 0) {
@@ -100,26 +116,23 @@ namespace {
                     if (loops != 0) {   // enter here if we need to play the same file again
                         if (loops > 0)
                             loops--;
-                        ov_time_seek(ogg_file,0.0); // rewind to beginning
+                        ov_time_seek(ogg_file, 0.0); // rewind to beginning
                     }
                     else
                         break;
                 }
             } while ((buffer_size - bytes) > 4096);
             if (bytes > 0) {
-                alBufferData(bufferName, ogg_format, array, static_cast < ALsizei > (bytes), ogg_freq);
+                alBufferData(bufferName, ogg_format, array.get(), static_cast < ALsizei > (bytes), ogg_freq);
                 m_openal_error = alGetError();
                 if (m_openal_error != AL_NONE)
                     ErrorLogger() << "RefillBuffer: OpenAL ERROR: " << alGetString(m_openal_error);
             } else {
                 ov_clear(ogg_file); // the app might think we still have something to play.
-                delete [] array;
                 return 1;
             }
-            delete [] array;
             return 0;
         }
-        delete [] array;
         return 1;
     }
 }
@@ -128,10 +141,10 @@ namespace {
 // TempUISoundDisabler
 ////////////////////////////////////////////////////////////
 Sound::TempUISoundDisabler::TempUISoundDisabler()
-{ ++Sound::GetSound().m_temporary_disable_count; }
+{ Sound::GetSound().pimpl->IncDisabledCount(); }
 
 Sound::TempUISoundDisabler::~TempUISoundDisabler()
-{ --Sound::GetSound().m_temporary_disable_count; }
+{ Sound::GetSound().pimpl->DecDisabledCount(); }
 
 ////////////////////////////////////////////////////////////
 // Sound
@@ -143,40 +156,226 @@ Sound& Sound::GetSound()
 }
 
 Sound::Sound() :
+    pimpl(new SoundImpl)
+{}
+
+// Require here because SoundImpl is defined in this file.
+Sound::~Sound() {}
+
+void Sound::Enable()
+{ pimpl->Enable(); }
+
+void Sound::Disable()
+{ pimpl->Disable(); }
+
+void Sound::PlayMusic(const boost::filesystem::path& path, int loops /* = 0*/)
+{ pimpl->PlayMusic(path, loops); }
+
+void Sound::StopMusic()
+{ pimpl->StopMusic(); }
+
+void Sound::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* = false*/)
+{ pimpl->PlaySound(path, is_ui_sound); }
+
+void Sound::FreeSound(const boost::filesystem::path& path)
+{ pimpl->FreeSound(path); }
+
+void Sound::FreeAllSounds()
+{ pimpl->FreeAllSounds(); }
+
+void Sound::SetMusicVolume(int vol)
+{ pimpl->SetMusicVolume(vol); }
+
+void Sound::SetUISoundsVolume(int vol)
+{ pimpl->SetUISoundsVolume(vol); }
+
+void Sound::DoFrame()
+{ pimpl->DoFrame(); }
+
+
+
+////////////////////////////////////////////////////////////
+// SoundImpl
+////////////////////////////////////////////////////////////
+
+Sound::SoundImpl::SoundImpl() :
     m_sources(),
-    m_music_loops(),
+    m_music_loops(0),
     m_music_name(),
     m_buffers(),
     m_music_buffers(),
     m_ogg_file(),
     m_ogg_format(),
     m_ogg_freq(),
-    m_temporary_disable_count(0)
+    m_temporary_disable_count(0),
+    m_initialized(false)
 {
-    InitOpenAL(NUM_SOURCES, m_sources, m_music_buffers);
+    if (!GetOptionsDB().Get<bool>("UI.sound.enabled") && !GetOptionsDB().Get<bool>("UI.sound.music-enabled"))
+        return;
+
+    try {
+        Enable();
+    } catch (InitializationFailureException const&) {
+        // Bury the exception because the GetSound() singleton may cause
+        // the sound system to initialize at unpredictable times when
+        // the user can't be usefully informed of the failure.
+    }
 }
 
-Sound::~Sound()
-{
-    if (alcGetCurrentContext() != 0)
-    {
+Sound::SoundImpl::~SoundImpl() {
+    Disable();
+}
+
+void Sound::SoundImpl::Enable() {
+    if (m_initialized)
+        return;
+
+    //Reset playing audio
+    m_music_loops = 0;
+    m_buffers.clear();
+    m_temporary_disable_count = 0;
+
+    InitOpenAL();
+    if (!m_initialized) {
+        ErrorLogger() "Unable to initialize audio.  Sound effects and music are disabled.";
+        // TODO: Let InitOpenAL throw the OpenAL error message which
+        // might be more useful.
+        throw InitializationFailureException("ERROR_SOUND_INITIALIZATION_FAILED");
+    }
+
+    DebugLogger() << "Audio " << (m_initialized ? "enabled." : "disabled.");
+}
+
+void Sound::SoundImpl::Disable() {
+    if (!m_initialized)
+        return;
+    StopMusic();
+
+    ShutdownOpenAL();
+
+    //Reset playing audio
+    m_music_loops = 0;
+    m_buffers.clear();
+    m_temporary_disable_count = 0;
+
+    m_initialized = false;
+    DebugLogger() << "Audio " << (m_initialized ? "enabled." : "disabled.");
+}
+
+/// Initialize OpenAl and return true on success.
+// TODO rewrite with std::unique_ptr and custom deleter to remove long
+// chain of "destructor" code.
+void Sound::SoundImpl::InitOpenAL() {
+    ALCcontext *context;
+    ALCdevice *device;
+    ALenum error_code;
+
+    device = alcOpenDevice(0);    /* currently only select the default output device - usually a NULL-terminated
+                                   * string desctribing a device can be passed here (of type ALchar*) */
+    if (device == 0) {
+        ErrorLogger() << "Unable to initialise default OpenAL device.";
+        m_initialized = false;
+        return;
+    }
+
+    context = alcCreateContext(device, 0);   // instead of 0 we can pass a ALCint* pointing to a set of
+    // attributes (ALC_FREQUENCY, ALC_REFRESH and ALC_SYNC)
+
+    if (!((context != 0) && (alcMakeContextCurrent(context) == AL_TRUE) && ((error_code = alGetError()) == AL_NO_ERROR) )) {
+        ErrorLogger() << "Unable to create OpenAL context : " << alGetString(error_code) << "\n";
+        alcCloseDevice(device);
+        m_initialized = false;
+        return;
+    }
+
+    alListenerf(AL_GAIN, 1.0);
+    error_code = alGetError();
+    if (error_code != AL_NO_ERROR) {
+        ErrorLogger() << "Unable to create OpenAL listener: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
+        alcMakeContextCurrent(0);
+        alcDestroyContext(context);
+        alcCloseDevice(device);
+        m_initialized = false;
+        return;
+    }
+
+    alGenSources(NUM_SOURCES, m_sources);
+    error_code = alGetError();
+    if (error_code != AL_NO_ERROR) {
+        ErrorLogger() << "Unable to create OpenAL sources: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
+        alcMakeContextCurrent(0);
+        alcDestroyContext(context);
+        alcCloseDevice(device);
+        m_initialized = false;
+        return;
+    }
+
+    alGenBuffers(NUM_MUSIC_BUFFERS, m_music_buffers);
+    error_code = alGetError();
+    if (error_code != AL_NO_ERROR) {
+        ErrorLogger() << "Unable to create OpenAL buffers: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
+        alDeleteSources(NUM_SOURCES, m_sources);
+        alcMakeContextCurrent(0);
+        alcDestroyContext(context);
+        alcCloseDevice(device);
+        m_initialized = false;
+        return;
+    }
+
+    for (int i = 0; i < NUM_SOURCES; ++i) {
+        alSourcei(m_sources[i], AL_SOURCE_RELATIVE, AL_TRUE);
+        error_code = alGetError();
+        if (error_code != AL_NO_ERROR) {
+            ErrorLogger() << "Unable to set OpenAL source to relative: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
+            alDeleteBuffers(NUM_MUSIC_BUFFERS, m_music_buffers);
+            alDeleteSources(NUM_SOURCES, m_sources);
+            alcMakeContextCurrent(0);
+            alcDestroyContext(context);
+            alcCloseDevice(device);
+            m_initialized = false;
+            return;
+        }
+    }
+    DebugLogger() << "OpenAL initialized. Version "
+                  << alGetString(AL_VERSION)
+                  << " Renderer "
+                  << alGetString(AL_RENDERER)
+                  << " Vendor "
+                  << alGetString(AL_VENDOR) << "\n"
+                  << " Extensions: "
+                  << alGetString(AL_EXTENSIONS) << "\n";
+    m_initialized = true;
+}
+
+void Sound::SoundImpl::ShutdownOpenAL() {
+
+    if (alcGetCurrentContext() != 0) {
         alDeleteSources(NUM_SOURCES, m_sources); // Automatically stops currently playing sources
 
-        alDeleteBuffers(2, m_music_buffers);
-        for( std::map<std::string, ALuint>::iterator itr = m_buffers.begin(); itr != m_buffers.end(); ++itr )
+        alDeleteBuffers(NUM_MUSIC_BUFFERS, m_music_buffers);
+        for (std::map<std::string, ALuint>::iterator itr = m_buffers.begin(); itr != m_buffers.end(); ++itr)
             alDeleteBuffers(1, &itr->second );
     }
 
-
-    ShutdownOpenAL();
+    ALCcontext* context = alcGetCurrentContext();
+    if (context != 0) {
+        ALCdevice* device = alcGetContextsDevice(context);
+        alcMakeContextCurrent(0);
+        alcDestroyContext(context);
+        if (device != 0)
+            alcCloseDevice(device);
+    }
 }
 
-void Sound::PlayMusic(const boost::filesystem::path& path, int loops /* = 0*/)
-{
+
+void Sound::SoundImpl::PlayMusic(const boost::filesystem::path& path, int loops /* = 0*/) {
+    if (!m_initialized)
+        return;
+
     ALenum m_openal_error;
-    std::string filename = path.string();
-    FILE *m_f = 0;
-    vorbis_info *vorbis_info;
+    std::string filename = PathString(path);
+    FILE* m_f = 0;
+    vorbis_info* vorbis_info;
     m_music_loops = 0;
 
 #ifdef FREEORION_WIN32
@@ -218,10 +417,10 @@ void Sound::PlayMusic(const boost::filesystem::path& path, int loops /* = 0*/)
                 /* fill up the buffers and queue them up for the first time */
                 if (!RefillBuffer(&m_ogg_file, m_ogg_format, m_ogg_freq, m_music_buffers[0], BUFFER_SIZE, m_music_loops))
                 {
-                    alSourceQueueBuffers(m_sources[0],1,&m_music_buffers[0]); // queue up the buffer if we manage to fill it
+                    alSourceQueueBuffers(m_sources[0], 1, &m_music_buffers[0]); // queue up the buffer if we manage to fill it
                     if (!RefillBuffer(&m_ogg_file, m_ogg_format, m_ogg_freq, m_music_buffers[1], BUFFER_SIZE, m_music_loops))
                     {
-                        alSourceQueueBuffers(m_sources[0],1,&m_music_buffers[1]);
+                        alSourceQueueBuffers(m_sources[0], 1, &m_music_buffers[1]);
                         m_music_name = filename; // yup, we're playing something that takes up more than 2 buffers
                     }
                     else
@@ -250,13 +449,13 @@ void Sound::PlayMusic(const boost::filesystem::path& path, int loops /* = 0*/)
         ErrorLogger() << "PlayMusic: OpenAL ERROR: " << alGetString(m_openal_error);
 }
 
-void Sound::StopMusic()
-{
-    if (alcGetCurrentContext() != 0)
-    {
+void Sound::SoundImpl::StopMusic() {
+    if (!m_initialized)
+        return;
+
+    if (alcGetCurrentContext() != 0) {
         alSourceStop(m_sources[0]);
-        if (m_music_name.size() > 0)
-        {
+        if (m_music_name.size() > 0) {
             m_music_name.clear();  // do this to avoid music being re-started by other functions
             ov_clear(&m_ogg_file); // and unload the file for good measure. the file itself is closed now, don't re-close it again
         }
@@ -264,12 +463,11 @@ void Sound::StopMusic()
     }
 }
 
-void Sound::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* = false*/)
-{
-    if (!GetOptionsDB().Get<bool>("UI.sound.enabled") || (is_ui_sound && UISoundsTemporarilyDisabled()))
+void Sound::SoundImpl::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* = false*/) {
+    if (!m_initialized || !GetOptionsDB().Get<bool>("UI.sound.enabled") || (is_ui_sound && UISoundsTemporarilyDisabled()))
         return;
 
-    std::string filename = path.string();
+    std::string filename = PathString(path);
     ALuint current_buffer;
     ALenum source_state;
     ALsizei ogg_freq;
@@ -292,16 +490,14 @@ void Sound::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* =
     };
 #endif
 
-    if (alcGetCurrentContext() != 0)
-    {
+    if (alcGetCurrentContext() != 0) {
         /* First check if the sound data of the file we want to play is already buffered somewhere */
         std::map<std::string, ALuint>::iterator it = m_buffers.find(filename);
         if (it != m_buffers.end()) {
             current_buffer = it->second;
             found_buffer = true;
         } else {
-            if ((file = fopen(filename.c_str(), "rb")) != 0) // make sure we CAN open it
-            {
+            if ((file = fopen(filename.c_str(), "rb")) != 0) { // make sure we CAN open it
                 OggVorbis_File ogg_file;
                 vorbis_info *vorbis_info;
                 ALenum ogg_format;
@@ -348,7 +544,7 @@ void Sound::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* =
         if (found_buffer) {
             /* Now that we have the buffer, we need to find a source to send it to */
             for (m_i = 1; m_i < NUM_SOURCES; ++m_i) {   // as we're playing sounds we start at 1. 0 is reserved for music
-                alGetSourcei(m_sources[m_i],AL_SOURCE_STATE,&source_state);
+                alGetSourcei(m_sources[m_i], AL_SOURCE_STATE, &source_state);
                 if ((source_state != AL_PLAYING) && (source_state != AL_PAUSED)) {
                     found_source = true;
                     alSourcei(m_sources[m_i], AL_BUFFER, current_buffer);
@@ -367,9 +563,12 @@ void Sound::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* =
     }
 }
 
-void Sound::FreeSound(const boost::filesystem::path& path) {
+void Sound::SoundImpl::FreeSound(const boost::filesystem::path& path) {
+    if (!m_initialized)
+        return;
+
     ALenum m_openal_error;
-    std::string filename = path.string();
+    std::string filename = PathString(path);
     std::map<std::string, ALuint>::iterator it = m_buffers.find(filename);
 
     if (it != m_buffers.end()) {
@@ -383,7 +582,10 @@ void Sound::FreeSound(const boost::filesystem::path& path) {
     }
 }
 
-void Sound::FreeAllSounds() {
+void Sound::SoundImpl::FreeAllSounds() {
+    if (!m_initialized)
+        return;
+
     ALenum m_openal_error;
 
     for (std::map<std::string, ALuint>::iterator it = m_buffers.begin();
@@ -402,7 +604,10 @@ void Sound::FreeAllSounds() {
     }
 }
 
-void Sound::SetMusicVolume(int vol) {
+void Sound::SoundImpl::SetMusicVolume(int vol) {
+    if (!m_initialized)
+        return;
+
     ALenum m_openal_error;
 
     /* normalize value, then apply to all sound sources */
@@ -410,7 +615,7 @@ void Sound::SetMusicVolume(int vol) {
     GetOptionsDB().Set<int>("UI.sound.music-volume", vol);
     if (alcGetCurrentContext() != 0)
     {
-        alSourcef(m_sources[0],AL_GAIN, ((ALfloat) vol)/255.0);
+        alSourcef(m_sources[0], AL_GAIN, ((ALfloat) vol)/255.0);
         /* it is highly unlikely that we'll get an error here but better safe than sorry */
         m_openal_error = alGetError();
         if (m_openal_error != AL_NONE)
@@ -418,7 +623,10 @@ void Sound::SetMusicVolume(int vol) {
     }
 }
 
-void Sound::SetUISoundsVolume(int vol) {
+void Sound::SoundImpl::SetUISoundsVolume(int vol) {
+    if (!m_initialized)
+        return;
+
     ALenum m_openal_error;
 
     /* normalize value, then apply to all sound sources */
@@ -426,7 +634,7 @@ void Sound::SetUISoundsVolume(int vol) {
     GetOptionsDB().Set<int>("UI.sound.volume", vol);
     if (alcGetCurrentContext() != 0) {
         for (int it = 1; it < NUM_SOURCES; ++it)
-            alSourcef(m_sources[it],AL_GAIN, ((ALfloat) vol)/255.0);
+            alSourcef(m_sources[it], AL_GAIN, ((ALfloat) vol)/255.0);
         /* it is highly unlikely that we'll get an error here but better safe than sorry */
         m_openal_error = alGetError();
         if (m_openal_error != AL_NONE)
@@ -434,12 +642,15 @@ void Sound::SetUISoundsVolume(int vol) {
     }
 }
 
-void Sound::DoFrame() {
+void Sound::SoundImpl::DoFrame() {
+    if (!m_initialized)
+        return;
+
     ALint    state;
     int      num_buffers_processed;
 
     if ((alcGetCurrentContext() != 0) && (m_music_name.size() > 0)) {
-        alGetSourcei(m_sources[0],AL_BUFFERS_PROCESSED,&num_buffers_processed);
+        alGetSourcei(m_sources[0], AL_BUFFERS_PROCESSED, &num_buffers_processed);
         while (num_buffers_processed > 0) {
             ALuint buffer_name_yay;
             alSourceUnqueueBuffers (m_sources[0], 1, &buffer_name_yay);
@@ -448,7 +659,7 @@ void Sound::DoFrame() {
                 m_music_name.clear();  // m_music_name.clear() must always be called before ov_clear. Otherwise
                 break; /// this happens if RefillBuffer returns 1, meaning it encountered EOF and the file shouldn't be repeated
             }
-            alSourceQueueBuffers(m_sources[0],1,&buffer_name_yay);
+            alSourceQueueBuffers(m_sources[0], 1, &buffer_name_yay);
             num_buffers_processed--;
         }
         alGetSourcei(m_sources[0], AL_SOURCE_STATE, &state);
@@ -457,5 +668,11 @@ void Sound::DoFrame() {
     }
 } 
 
-bool Sound::UISoundsTemporarilyDisabled() const
+bool Sound::SoundImpl::UISoundsTemporarilyDisabled() const
 { return m_temporary_disable_count > 0; }
+
+void Sound::SoundImpl::IncDisabledCount()
+{ ++m_temporary_disable_count; }
+
+void Sound::SoundImpl::DecDisabledCount()
+{ --m_temporary_disable_count; }
