@@ -20,7 +20,6 @@
 #include "../Empire/Empire.h"
 
 #include <GG/DrawUtil.h>
-#include <GG/DynamicGraphic.h>
 #include <GG/StaticGraphic.h>
 #include <GG/utf8/core.h>
 #include <GG/utf8/checked.h>
@@ -32,17 +31,6 @@ namespace {
     bool PlaySounds()                   { return GetOptionsDB().Get<bool>("UI.sound.enabled"); }
     void PlaySystemIconRolloverSound()  { if (PlaySounds()) Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("UI.sound.system-icon-rollover")); }
 
-    const std::vector<boost::shared_ptr<GG::Texture> >& GetSelectionIndicatorTextures() {
-        static std::vector<boost::shared_ptr<GG::Texture> > normal_textures =
-            ClientUI::GetClientUI()->GetPrefixedTextures(ClientUI::ArtDir() / "misc" / "system_selection", "system_selection", true);
-        return normal_textures;
-    }
-    const std::vector<boost::shared_ptr<GG::Texture> >& GetSelectionIndicatorTinyTextures() {
-        static std::vector<boost::shared_ptr<GG::Texture> > tiny_textures =
-            ClientUI::GetClientUI()->GetPrefixedTextures(ClientUI::ArtDir() / "misc" / "system_selection_tiny", "system_selection_tiny", false);
-        return tiny_textures;
-    }
-    
     // Wrap content int an rgba tag with color color. Opacity ignored.
     std::string ColorTag(const std::string& content, GG::Clr color){
         boost::format templ("<rgba %d %d %d 255>%s</rgba>");
@@ -276,31 +264,26 @@ SystemIcon::SystemIcon(GG::X x, GG::Y y, GG::X w, int system_id) :
     m_tiny_graphic->Hide();
 
     // selection indicator graphic
-    const std::vector<boost::shared_ptr<GG::Texture> >& textures = GetSelectionIndicatorTextures();
-    GG::X texture_width = textures.at(0)->DefaultWidth();
-    GG::Y texture_height = textures.at(0)->DefaultHeight();
-    m_selection_indicator = new GG::DynamicGraphic(GG::X0, GG::Y0,
-                                                   texture_width, texture_height, true,
-                                                   texture_width, texture_height, 0, textures,
-                                                   GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
-    m_selection_indicator->SetFPS(ClientUI::SystemSelectionIndicatorFPS());
+    const boost::shared_ptr<GG::Texture>& texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_selection" / "system_selection2.png", true);
+
+    GG::X texture_width = texture->DefaultWidth();
+    GG::Y texture_height = texture->DefaultHeight();
+    m_selection_indicator = new RotatingGraphic(texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
+    m_selection_indicator->SetRPM(ClientUI::SystemSelectionIndicatorRPM());
     AttachChild(m_selection_indicator);
-    m_selection_indicator->Play();
+    m_selection_indicator->Resize(GG::Pt(texture_width, texture_height));
 
     // tiny selection indicator graphic
-    const std::vector<boost::shared_ptr<GG::Texture> >& tiny_textures = GetSelectionIndicatorTinyTextures();
-    texture_width = tiny_textures.at(0)->DefaultWidth();
-    texture_height = tiny_textures.at(0)->DefaultHeight();
-    m_tiny_selection_indicator = new GG::DynamicGraphic(GG::X0, GG::Y0,
-                                                        texture_width, texture_height, true,
-                                                        texture_width, texture_height, 0, tiny_textures,
-                                                        GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
-    m_tiny_selection_indicator->SetFPS(ClientUI::SystemSelectionIndicatorFPS());
+    const boost::shared_ptr<GG::Texture>& tiny_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_selection_tiny" / "system_selection_tiny2.png", true);
+    texture_width = tiny_texture->DefaultWidth();
+    texture_height = tiny_texture->DefaultHeight();
+    m_tiny_selection_indicator = new RotatingGraphic(tiny_texture, GG::GRAPHIC_NONE);
+    m_tiny_selection_indicator->SetRPM(ClientUI::SystemSelectionIndicatorRPM());
     AttachChild(m_tiny_selection_indicator);
-    m_tiny_selection_indicator->Play();
+    m_tiny_selection_indicator->Resize(GG::Pt(texture_width, texture_height));
 
     // mouseover indicator graphic
-    boost::shared_ptr<GG::Texture> mouseover_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_mouseover.png");
+    const boost::shared_ptr<GG::Texture> mouseover_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_mouseover.png");
     texture_width = mouseover_texture->DefaultWidth();
     texture_height = mouseover_texture->DefaultHeight();
     m_mouseover_indicator = new GG::StaticGraphic(mouseover_texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
@@ -457,7 +440,9 @@ void SystemIcon::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 
     // tiny graphic?
     if (m_tiny_graphic && USE_TINY_GRAPHICS) {
-        GG::Pt tiny_size = m_tiny_graphic->Size();
+        const GG::SubTexture& tiny_texture = m_tiny_graphic->GetTexture();
+        const GG::Texture* tiny_texture2 = tiny_texture.GetTexture();
+        GG::Pt tiny_size = GG::Pt(tiny_texture2->DefaultWidth(), tiny_texture2->DefaultHeight());
         GG::Pt tiny_ul(static_cast<GG::X>(middle.x - tiny_size.x / 2.0),
                        static_cast<GG::Y>(middle.y - tiny_size.y / 2.0));
         m_tiny_graphic->SizeMove(tiny_ul, tiny_ul + tiny_size);
@@ -470,7 +455,9 @@ void SystemIcon::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 
     // tiny selection indicator
     if (m_selected && m_tiny_selection_indicator && USE_TINY_GRAPHICS) {
-        GG::Pt tiny_sel_ind_size = m_tiny_selection_indicator->Size();
+        const GG::SubTexture& tiny_texture = m_tiny_selection_indicator->GetTexture();
+        const GG::Texture* tiny_texture2 = tiny_texture.GetTexture();
+        GG::Pt tiny_sel_ind_size = GG::Pt(tiny_texture2->DefaultWidth(), tiny_texture2->DefaultHeight());
         GG::Pt tiny_sel_ind_ul(static_cast<GG::X>(middle.x - tiny_sel_ind_size.x / 2.0),
                                static_cast<GG::Y>(middle.y - tiny_sel_ind_size.y / 2.0));
         m_tiny_selection_indicator->SizeMove(tiny_sel_ind_ul, tiny_sel_ind_ul + tiny_sel_ind_size);
@@ -698,8 +685,8 @@ void SystemIcon::HideName() {
 
 void SystemIcon::PositionSystemName() {
     if (m_colored_name) {
-        GG::X name_left = ( Width()  - m_colored_name->Width()          )/2;
-        GG::Y name_top =  ( Height() + GG::Y(EnclosingCircleDiameter()) )/2;
+        GG::X name_left = ( Width()  - m_colored_name->Width()           )/2;
+        GG::Y name_top =  ( Height() + GG::Y(EnclosingCircleDiameter()*2))/2;
         m_colored_name->MoveTo(GG::Pt(name_left, name_top));
     }
 }
