@@ -1,4 +1,3 @@
-// -*- C++ -*-
 #ifndef _TextBrowseWnd_h_
 #define _TextBrowseWnd_h_
 
@@ -6,10 +5,11 @@
 #include <GG/BrowseInfoWnd.h>
 
 
+
 /** A popup tooltop for display when mousing over in-game icons.  A title and some detail text.*/
 class TextBrowseWnd : public GG::BrowseInfoWnd {
 public:
-    TextBrowseWnd(const std::string& title_text, const std::string& main_text);
+    TextBrowseWnd(const std::string& title_text, const std::string& main_text, GG::X w = GG::X(200));
     virtual bool WndHasBrowseInfo(const Wnd* wnd, std::size_t mode) const;
     virtual void Render();
 
