@@ -26,6 +26,7 @@
 #include "../../util/Serialize.h"
 #include "../../util/SitRepEntry.h"
 #include "../../util/Directories.h"
+#include "../../util/Version.h"
 #include "../../universe/Planet.h"
 #include "../../universe/Species.h"
 #include "../../Empire/Empire.h"
@@ -42,6 +43,7 @@
 #include <boost/format.hpp>
 #include <boost/serialization/vector.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
+#include <boost/algorithm/string.hpp>
 
 #include <sstream>
 
@@ -97,22 +99,22 @@ namespace {
         db.Add("UI.mouse-click-repeat-delay",   UserStringNop("OPTIONS_DB_MOUSE_REPEAT_DELAY"),         360,    RangedValidator<int>(0, 1000));
         db.Add("UI.mouse-click-repeat-interval",UserStringNop("OPTIONS_DB_MOUSE_REPEAT_INTERVAL"),      15,     RangedValidator<int>(0, 1000));
 
-        db.Add("app-width",             UserStringNop("OPTIONS_DB_APP_WIDTH"),             1024,   RangedValidator<int>(800, 2560));
-        db.Add("app-height",            UserStringNop("OPTIONS_DB_APP_HEIGHT"),            768,    RangedValidator<int>(600, 1600));
-        db.Add("app-width-windowed",    UserStringNop("OPTIONS_DB_APP_WIDTH_WINDOWED"),    1024,   RangedValidator<int>(800, 2560));
-        db.Add("app-height-windowed",   UserStringNop("OPTIONS_DB_APP_HEIGHT_WINDOWED"),   768,    RangedValidator<int>(600, 1600));
-
-        const int CENTRE = static_cast<int>(SDL_WINDOWPOS_CENTERED);
-        db.Add("app-left-windowed",     UserStringNop("OPTIONS_DB_APP_LEFT_WINDOWED"),     CENTRE, OrValidator<int>( RangedValidator<int>(-10240, 10240),
-                                                                                                                     DiscreteValidator<int>(CENTRE) ));
-        db.Add("app-top-windowed",      UserStringNop("OPTIONS_DB_APP_TOP_WINDOWED"),      50,     RangedValidator<int>(-10240, 10240));
-
         Hotkey::AddHotkey("exit",       UserStringNop("HOTKEY_EXIT"),       GG::GGK_NONE,   GG::MOD_KEY_NONE);
         Hotkey::AddHotkey("quit",       UserStringNop("HOTKEY_QUIT"),       GG::GGK_NONE,   GG::MOD_KEY_NONE);
         Hotkey::AddHotkey("fullscreen", UserStringNop("HOTKEY_FULLSCREEN"), GG::GGK_RETURN, GG::MOD_KEY_ALT);
     }
     bool temp_bool = RegisterOptions(&AddOptions);
 
+    /** These options can only be validated after the graphics system (SDL) is initialized,
+        so that display size can be detected
+     */
+    const int DEFAULT_WIDTH = 1024;
+    const int DEFAULT_HEIGHT = 768;
+    const int DEFAULT_LEFT = static_cast<int>(SDL_WINDOWPOS_CENTERED);
+    const int DEFAULT_TOP = 50;
+    const int MIN_WIDTH = 800;
+    const int MIN_HEIGHT = 600;
+
     /* Sets the value of options that need language-dependent default values.*/
     void SetStringtableDependentOptionDefaults() {
         if (GetOptionsDB().Get<std::string>("GameSetup.empire-name").empty())
@@ -125,6 +127,9 @@ namespace {
             GetOptionsDB().Set("multiplayersetup.player-name", UserString("DEFAULT_PLAYER_NAME"));
     }
 
+    std::string GetGLVersionString()
+    { return boost::lexical_cast<std::string>(glGetString(GL_VERSION)); }
+
     static float stored_gl_version = -1.0f;  // to be replaced when gl version first checked
 
     float GetGLVersion() {
@@ -132,8 +137,7 @@ namespace {
             return stored_gl_version;
 
         // get OpenGL version string and parse to get version number
-        const GLubyte* gl_version = glGetString(GL_VERSION);
-        std::string gl_version_string = boost::lexical_cast<std::string>(gl_version);
+        std::string gl_version_string = GetGLVersionString();
 
         float version_number = 0.0f;
         std::istringstream iss(gl_version_string);
@@ -149,8 +153,8 @@ namespace {
         // get OpenGL version string and parse to get version number
         float version_number = GetGLVersion();
         DebugLogger() << "OpenGL Version Number: " << DoubleToString(version_number, 2, false);    // combination of floating point precision and DoubleToString preferring to round down means the +0.05 is needed to round properly
-        if (version_number < 1.5) {
-            ErrorLogger() << "OpenGL Version is less than 2.0 (official required) or 1.5 (usually works).  FreeOrion may crash when trying to start a game.";
+        if (version_number < 2.0) {
+            ErrorLogger() << "OpenGL Version is less than 2.0. FreeOrion may crash when trying to start a game.";
         }
 
         // only execute default option setting once
@@ -159,26 +163,37 @@ namespace {
         GetOptionsDB().Set<bool>("checked-gl-version", true);
 
         // if GL version is too low, set various map rendering options to
-        // disabled, so as to prevent crashes when running on systems that
-        // don't support these GL features.  these options are added to the
-        // DB in MapWnd.cpp's AddOptions and all default to true.
+        // disabled, to hopefully improve frame rate.
         if (version_number < 2.0) {
             GetOptionsDB().Set<bool>("UI.galaxy-gas-background",        false);
             GetOptionsDB().Set<bool>("UI.galaxy-starfields",            false);
-            GetOptionsDB().Set<bool>("UI.optimized-system-rendering",   false);
             GetOptionsDB().Set<bool>("UI.system-fog-of-war",            false);
         }
     }
 }
 
-HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const std::string& name, int x, int y, bool fullscreen, bool fake_mode_change) :
+void HumanClientApp::AddWindowSizeOptionsAfterMainStart(OptionsDB& db) {
+    const int max_width_plus_one = HumanClientApp::MaximumPossibleWidth() + 1;
+    const int max_height_plus_one = HumanClientApp::MaximumPossibleHeight() + 1;
+
+    db.Add("app-width",             UserStringNop("OPTIONS_DB_APP_WIDTH"),             DEFAULT_WIDTH,       RangedValidator<int>(MIN_WIDTH, max_width_plus_one));
+    db.Add("app-height",            UserStringNop("OPTIONS_DB_APP_HEIGHT"),            DEFAULT_HEIGHT,      RangedValidator<int>(MIN_HEIGHT, max_height_plus_one));
+    db.Add("app-width-windowed",    UserStringNop("OPTIONS_DB_APP_WIDTH_WINDOWED"),    DEFAULT_WIDTH,       RangedValidator<int>(MIN_WIDTH, max_width_plus_one));
+    db.Add("app-height-windowed",   UserStringNop("OPTIONS_DB_APP_HEIGHT_WINDOWED"),   DEFAULT_HEIGHT,      RangedValidator<int>(MIN_HEIGHT, max_height_plus_one));
+    db.Add("app-left-windowed",     UserStringNop("OPTIONS_DB_APP_LEFT_WINDOWED"),     DEFAULT_LEFT,        OrValidator<int>( RangedValidator<int>(-max_width_plus_one, max_width_plus_one), DiscreteValidator<int>(DEFAULT_LEFT) ));
+    db.Add("app-top-windowed",      UserStringNop("OPTIONS_DB_APP_TOP_WINDOWED"),      DEFAULT_TOP,         RangedValidator<int>(-max_height_plus_one, max_height_plus_one));
+}
+
+HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const std::string& name,
+                               int x, int y, bool fullscreen, bool fake_mode_change) :
     ClientApp(),
     SDLGUI(width, height, calculate_fps, name, x, y, fullscreen, fake_mode_change),
     m_fsm(0),
     m_single_player_game(true),
     m_game_started(false),
     m_connected(false),
-    m_auto_turns(0)
+    m_auto_turns(0),
+    m_have_window_focus(true)
 {
 #ifdef ENABLE_CRASH_BACKTRACE
     signal(SIGSEGV, SigHandler);
@@ -188,23 +203,39 @@ HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const
 #endif
     m_fsm = new HumanClientFSM(*this);
 
-    const std::string HUMAN_CLIENT_LOG_FILENAME((GetUserDir() / "freeorion.log").string());
+    const std::string HUMAN_CLIENT_LOG_FILENAME((GetUserDataDir() / "freeorion.log").string());
 
     InitLogger(HUMAN_CLIENT_LOG_FILENAME, "Client");
 
+    try {
+        DebugLogger() << "GL Version String: " << GetGLVersionString();
+    } catch (...) {
+        ErrorLogger() << "Unable to get GL Version String?";
+    }
+
+    LogDependencyVersions();
+
     boost::shared_ptr<GG::StyleFactory> style(new CUIStyle());
     SetStyleFactory(style);
 
     SetMinDragTime(0);
 
+    bool inform_user_sound_failed(false);
+    try {
+        if (GetOptionsDB().Get<bool>("UI.sound.enabled") || GetOptionsDB().Get<bool>("UI.sound.music-enabled"))
+            Sound::GetSound().Enable();
+
+        if ((GetOptionsDB().Get<bool>("UI.sound.music-enabled")))
+            Sound::GetSound().PlayMusic(GetOptionsDB().Get<std::string>("UI.sound.bg-music"), -1);
+
+        Sound::GetSound().SetMusicVolume(GetOptionsDB().Get<int>("UI.sound.music-volume"));
+        Sound::GetSound().SetUISoundsVolume(GetOptionsDB().Get<int>("UI.sound.volume"));
+    } catch (Sound::InitializationFailureException const &) {
+        inform_user_sound_failed = true;
+    }
+
     m_ui = boost::shared_ptr<ClientUI>(new ClientUI());
 
-    if ((GetOptionsDB().Get<bool>("UI.sound.music-enabled")))
-        Sound::GetSound().PlayMusic(GetOptionsDB().Get<std::string>("UI.sound.bg-music"), -1);
-
-    Sound::GetSound().SetMusicVolume(GetOptionsDB().Get<int>("UI.sound.music-volume"));
-    Sound::GetSound().SetUISoundsVolume(GetOptionsDB().Get<int>("UI.sound.volume"));
-
     EnableFPS();
     UpdateFPSLimit();
     GG::Connect(GetOptionsDB().OptionChangedSignal("show-fps"), &HumanClientApp::UpdateFPSLimit, this);
@@ -223,6 +254,7 @@ HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const
                          GetOptionsDB().Get<int>("UI.keypress-repeat-interval"));
     EnableMouseButtonDownRepeat(GetOptionsDB().Get<int>("UI.mouse-click-repeat-delay"),
                                 GetOptionsDB().Get<int>("UI.mouse-click-repeat-interval"));
+    EnableModalAcceleratorSignals(true);
 
     GG::Connect(WindowResizedSignal,    &HumanClientApp::HandleWindowResize,    this);
     GG::Connect(FocusChangedSignal,     &HumanClientApp::HandleFocusChange,     this);
@@ -238,7 +270,7 @@ HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const
     this->SetMouseLRSwapped(GetOptionsDB().Get<bool>("UI.swap-mouse-lr"));
 
     std::map<std::string, std::map<int, int> > named_key_maps;
-    parse::keymaps(GetResourceDir() / "keymaps.txt", named_key_maps);
+    parse::keymaps(named_key_maps);
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Keymaps:";
         for (std::map<std::string, std::map<int, int> >::const_iterator km_it = named_key_maps.begin();
@@ -267,6 +299,11 @@ HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const
     if (fake_mode_change && !FramebuffersAvailable()) {
         ErrorLogger() << "Requested fake mode changes, but the framebuffer opengl extension is not available. Ignoring.";
     }
+
+    // Placed after mouse initialization.
+    if (inform_user_sound_failed)
+        ClientUI::MessageBox(UserString("ERROR_SOUND_INITIALIZATION_FAILED"), false);
+
     m_fsm->initiate();
 }
 
@@ -274,9 +311,12 @@ void HumanClientApp::ConnectKeyboardAcceleratorSignals() {
     // Add global hotkeys
     HotkeyManager *hkm = HotkeyManager::GetManager();
 
-    hkm->Connect(boost::bind(&HumanClientApp::ExitGame, this), "exit");
-    hkm->Connect(boost::bind(&HumanClientApp::QuitGame, this), "quit");
-    hkm->Connect(boost::bind(&HumanClientApp::ToggleFullscreen, this), "fullscreen");
+    hkm->Connect(boost::bind(&HumanClientApp::ExitGame, this),          "exit",
+                 new NoModalWndsOpenCondition());
+    hkm->Connect(boost::bind(&HumanClientApp::QuitGame, this),          "quit",
+                 new NoModalWndsOpenCondition());
+    hkm->Connect(boost::bind(&HumanClientApp::ToggleFullscreen, this), "fullscreen",
+                 new NoModalWndsOpenCondition());
 
     hkm->RebuildShortcuts();
 }
@@ -369,6 +409,9 @@ void HumanClientApp::StartServer() {
     } else {
         DebugLogger() << "ai-config not set.";
     }
+    if (m_single_player_game) {
+        args.push_back("--singleplayer");
+    }
     m_server_process = Process(SERVER_CLIENT_EXE, args);
 }
 
@@ -389,6 +432,7 @@ void HumanClientApp::KillServer() {
 
 void HumanClientApp::NewSinglePlayerGame(bool quickstart) {
     if (!GetOptionsDB().Get<bool>("force-external-server")) {
+        m_single_player_game = true;
         try {
             StartServer();
         } catch (const std::runtime_error& err) {
@@ -497,7 +541,7 @@ void HumanClientApp::NewSinglePlayerGame(bool quickstart) {
         if (m_networking.Connected()) {
             DebugLogger() << "HumanClientApp::NewSinglePlayerGame Sending server shutdown message.";
             m_networking.SendMessage(ShutdownServerMessage(m_networking.PlayerID()));
-            boost::this_thread::sleep(boost::posix_time::seconds(1));
+            boost::this_thread::sleep_for(boost::chrono::seconds(1));
             m_networking.DisconnectFromServer();
             if (!m_networking.Connected())
                 DebugLogger() << "HumanClientApp::NewSinglePlayerGame Disconnected from server.";
@@ -524,6 +568,7 @@ void HumanClientApp::MultiPlayerGame() {
 
     if (server_name == "HOST GAME SELECTED") {
         if (!GetOptionsDB().Get<bool>("force-external-server")) {
+            m_single_player_game = false;
             try {
                 StartServer();
                 FreeServer();
@@ -565,12 +610,8 @@ void HumanClientApp::CancelMultiplayerGameFromLobby()
 
 void HumanClientApp::SaveGame(const std::string& filename) {
     Message response_msg;
-    m_networking.SendSynchronousMessage(HostSaveGameMessage(PlayerID(), filename), response_msg);
-    if (response_msg.Type() != Message::SAVE_GAME) {
-        ErrorLogger() << "HumanClientApp::SaveGame sent synchronous HostSaveGameMessage, but received back message of wrong type: " << response_msg.Type();
-        throw std::runtime_error("HumanClientApp::SaveGame synchronous message received invalid response message type");
-    }
-    HandleSaveGameDataRequest();
+    m_networking.SendMessage(HostSaveGameInitiateMessage(PlayerID(), filename));
+    DebugLogger() << "HumanClientApp::SaveGame sent save initiate message to server...";
 }
 
 void HumanClientApp::LoadSinglePlayerGame(std::string filename/* = ""*/) {
@@ -604,12 +645,13 @@ void HumanClientApp::LoadSinglePlayerGame(std::string filename/* = ""*/) {
     if (m_game_started) {
         EndGame();
         // delay to make sure old game is fully cleaned up before attempting to start a new one
-        boost::this_thread::sleep(boost::posix_time::seconds(3));
+        boost::this_thread::sleep_for(boost::chrono::seconds(3));
     } else {
         DebugLogger() << "HumanClientApp::LoadSinglePlayerGame() not already in a game, so don't need to end it";
     }
 
     if (!GetOptionsDB().Get<bool>("force-external-server")) {
+        m_single_player_game = true;
         DebugLogger() << "HumanClientApp::LoadSinglePlayerGame() Starting server";
         StartServer();
         DebugLogger() << "HumanClientApp::LoadSinglePlayerGame() Server started";
@@ -646,10 +688,15 @@ void HumanClientApp::LoadSinglePlayerGame(std::string filename/* = ""*/) {
     m_fsm->process_event(HostSPGameRequested());
 }
 
-void HumanClientApp::RequestSavePreviews(const std::string& directory, PreviewInformation& previews){
-    std::string  generic_directory = fs::path(directory).generic_string();
-    if(!m_networking.Connected()){
+void HumanClientApp::RequestSavePreviews(const std::string& directory, PreviewInformation& previews) {
+    //std::cout << "HumanClientApp::RequestSavePreviews directory: " << directory << " valid UTF-8: " << utf8::is_valid(directory.begin(), directory.end()) << std::endl;
+    DebugLogger() << "HumanClientApp::RequestSavePreviews directory: " << directory << " valid UTF-8: " << utf8::is_valid(directory.begin(), directory.end());
+
+    std::string  generic_directory = directory;//PathString(fs::path(directory));
+    if (!m_networking.Connected()) {
         DebugLogger() << "HumanClientApp::RequestSavePreviews: No game running. Start a server for savegame queries.";
+
+        m_single_player_game = true;
         StartServer();
 
         DebugLogger() << "HumanClientApp::RequestSavePreviews Connecting to server";
@@ -667,7 +714,7 @@ void HumanClientApp::RequestSavePreviews(const std::string& directory, PreviewIn
     DebugLogger() << "HumanClientApp::RequestSavePreviews Requesting previews for " << generic_directory;
     Message response;
     m_networking.SendSynchronousMessage(RequestSavePreviewsMessage(PlayerID(), generic_directory), response);
-    if(response.Type() == Message::DISPATCH_SAVE_PREVIEWS){
+    if (response.Type() == Message::DISPATCH_SAVE_PREVIEWS){
         ExtractMessageData(response, previews);
         DebugLogger() << "HumanClientApp::RequestSavePreviews Got " << previews.previews.size() << " previews.";
     }else{
@@ -675,7 +722,6 @@ void HumanClientApp::RequestSavePreviews(const std::string& directory, PreviewIn
     }
 }
 
-
 std::pair<int, int> HumanClientApp::GetWindowLeftTop() {
     int left(0), top(0);
 
@@ -727,12 +773,13 @@ void HumanClientApp::Reinitialize() {
     GG::X old_width = AppWidth();
     GG::Y old_height = AppHeight();
 
-    SetVideoMode(GG::X (size.first), GG::Y (size.second), fullscreen, fake_mode_change);
+    SetVideoMode(GG::X(size.first), GG::Y(size.second), fullscreen, fake_mode_change);
     if (fullscreen_transition) {
         FullscreenSwitchSignal(fullscreen); // after video mode is changed but before DoLayout() calls
     } else if (fullscreen &&
                (old_width != size.first || old_height != size.second) &&
-               GetOptionsDB().Get<bool>("UI.auto-reposition-windows")) {
+               GetOptionsDB().Get<bool>("UI.auto-reposition-windows"))
+    {
         // Reposition windows if in fullscreen mode... handled here instead of
         // HandleWindowResize() because the prev. fullscreen resolution is only
         // available here.
@@ -756,16 +803,16 @@ void HumanClientApp::StartTurn() {
 
     const Empire *empire = GetEmpire(EmpireID());
     if (empire) {
-        double RP = empire->ResourceProduction(RE_RESEARCH);
-        double PP = empire->ResourceProduction(RE_INDUSTRY);
+        double RP = empire->ResourceOutput(RE_RESEARCH);
+        double PP = empire->ResourceOutput(RE_INDUSTRY);
         int turn_number = CurrentTurn();
         float ratio = (RP/(PP+0.0001));
         const GG::Clr color = empire->Color();
         DebugLogger() << "Current Output (turn " << turn_number << ") RP/PP: " << ratio << " (" << RP << "/" << PP << ")";
         DebugLogger() << "EmpireColors: " << static_cast<int>(color.r)
-                                            << " " << static_cast<int>(color.g)
-                                            << " " << static_cast<int>(color.b)
-                                            << " " << static_cast<int>(color.a);
+                      << " " << static_cast<int>(color.g)
+                      << " " << static_cast<int>(color.b)
+                      << " " << static_cast<int>(color.a);
     }
 
     ClientApp::StartTurn();
@@ -773,14 +820,23 @@ void HumanClientApp::StartTurn() {
 }
 
 void HumanClientApp::HandleSystemEvents() {
-    SDLGUI::HandleSystemEvents();
+    try {
+        SDLGUI::HandleSystemEvents();
+    } catch (const utf8::invalid_utf8& e) {
+        ErrorLogger() << "UTF-8 error handling system event: " << e.what();
+    }
     if (m_connected && !m_networking.Connected()) {
         m_connected = false;
         DisconnectedFromServer();
     } else if (m_networking.MessageAvailable()) {
         Message msg;
         m_networking.GetMessage(msg);
-        HandleMessage(msg);
+        try {
+            HandleMessage(msg);
+        } catch (const std::exception& e) {
+            ErrorLogger() << "exception handing message: " << e.what();
+            ErrorLogger() << "message type: " << msg.Type() << " and text: " << msg.Text();
+        }
     }
 }
 
@@ -794,42 +850,39 @@ void HumanClientApp::HandleMessage(Message& msg) {
         std::cerr << "HumanClientApp::HandleMessage(" << msg.Type() << ")\n";
 
     switch (msg.Type()) {
-    case Message::ERROR_MSG:            m_fsm->process_event(Error(msg));                   break;
-    case Message::HOST_MP_GAME:         m_fsm->process_event(HostMPGame(msg));              break;
-    case Message::HOST_SP_GAME:         m_fsm->process_event(HostSPGame(msg));              break;
-    case Message::JOIN_GAME:            m_fsm->process_event(JoinGame(msg));                break;
-    case Message::HOST_ID:              m_fsm->process_event(HostID(msg));                  break;
-    case Message::LOBBY_UPDATE:         m_fsm->process_event(LobbyUpdate(msg));             break;
-    case Message::LOBBY_CHAT:           m_fsm->process_event(LobbyChat(msg));               break;
-    case Message::SAVE_GAME:            m_fsm->process_event(::SaveGame(msg));              break;
-    case Message::GAME_START:           m_fsm->process_event(GameStart(msg));               break;
-    case Message::TURN_UPDATE:          m_fsm->process_event(TurnUpdate(msg));              break;
-    case Message::TURN_PARTIAL_UPDATE:  m_fsm->process_event(TurnPartialUpdate(msg));       break;
-    case Message::TURN_PROGRESS:        m_fsm->process_event(TurnProgress(msg));            break;
-    case Message::PLAYER_STATUS:        m_fsm->process_event(::PlayerStatus(msg));          break;
-    case Message::PLAYER_CHAT:          m_fsm->process_event(PlayerChat(msg));              break;
-    case Message::DIPLOMACY:            m_fsm->process_event(Diplomacy(msg));               break;
-    case Message::DIPLOMATIC_STATUS:    m_fsm->process_event(DiplomaticStatusUpdate(msg));  break;
-    case Message::VICTORY_DEFEAT :      m_fsm->process_event(VictoryDefeat(msg));           break;
-    case Message::PLAYER_ELIMINATED:    m_fsm->process_event(PlayerEliminated(msg));        break;
-    case Message::END_GAME:             m_fsm->process_event(::EndGame(msg));               break;
+    case Message::ERROR_MSG:                m_fsm->process_event(Error(msg));                   break;
+    case Message::HOST_MP_GAME:             m_fsm->process_event(HostMPGame(msg));              break;
+    case Message::HOST_SP_GAME:             m_fsm->process_event(HostSPGame(msg));              break;
+    case Message::JOIN_GAME:                m_fsm->process_event(JoinGame(msg));                break;
+    case Message::HOST_ID:                  m_fsm->process_event(HostID(msg));                  break;
+    case Message::LOBBY_UPDATE:             m_fsm->process_event(LobbyUpdate(msg));             break;
+    case Message::LOBBY_CHAT:               m_fsm->process_event(LobbyChat(msg));               break;
+    case Message::SAVE_GAME_DATA_REQUEST:   m_fsm->process_event(SaveGameDataRequest(msg));     break;
+    case Message::SAVE_GAME_COMPLETE:       m_fsm->process_event(SaveGameComplete(msg));        break;
+
+    case Message::GAME_START:               m_fsm->process_event(GameStart(msg));               break;
+    case Message::TURN_UPDATE:              m_fsm->process_event(TurnUpdate(msg));              break;
+    case Message::TURN_PARTIAL_UPDATE:      m_fsm->process_event(TurnPartialUpdate(msg));       break;
+    case Message::TURN_PROGRESS:            m_fsm->process_event(TurnProgress(msg));            break;
+    case Message::PLAYER_STATUS:            m_fsm->process_event(::PlayerStatus(msg));          break;
+    case Message::PLAYER_CHAT:              m_fsm->process_event(PlayerChat(msg));              break;
+    case Message::DIPLOMACY:                m_fsm->process_event(Diplomacy(msg));               break;
+    case Message::DIPLOMATIC_STATUS:        m_fsm->process_event(DiplomaticStatusUpdate(msg));  break;
+    case Message::END_GAME:                 m_fsm->process_event(::EndGame(msg));               break;
     default:
-        ErrorLogger() << "HumanClientApp::HandleMessage : Received an unknown message type \""
-                               << msg.Type() << "\".";
+        ErrorLogger() << "HumanClientApp::HandleMessage : Received an unknown message type \"" << msg.Type() << "\".";
     }
 }
 
 void HumanClientApp::HandleSaveGameDataRequest() {
     if (INSTRUMENT_MESSAGE_HANDLING)
-        std::cerr << "HumanClientApp::HandleSaveGameDataRequest(" << Message::SAVE_GAME << ")\n";
+        std::cerr << "HumanClientApp::HandleSaveGameDataRequest(" << Message::SAVE_GAME_DATA_REQUEST << ")\n";
     SaveGameUIData ui_data;
     m_ui->GetSaveGameUIData(ui_data);
     m_networking.SendMessage(ClientSaveDataMessage(PlayerID(), Orders(), ui_data));
 }
 
 void HumanClientApp::HandleWindowMove(GG::X w, GG::Y h) {
-    //DebugLogger() << "HumanClientApp::HandleWindowMove(" << Value(w) << ", " << Value(h) << ")";
-
     if (!Fullscreen()) {
         GetOptionsDB().Set<int>("app-left-windowed", Value(w));
         GetOptionsDB().Set<int>("app-top-windowed", Value(h));
@@ -838,7 +891,6 @@ void HumanClientApp::HandleWindowMove(GG::X w, GG::Y h) {
 }
 
 void HumanClientApp::HandleWindowResize(GG::X w, GG::Y h) {
-    //DebugLogger() << "HumanClientApp::HandleWindowResize(" << Value(w) << ", " << Value(h) << ")";
     if (ClientUI* ui = ClientUI::GetClientUI()) {
         if (MapWnd* map_wnd = ui->GetMapWnd())
             map_wnd->DoLayout();
@@ -862,6 +914,8 @@ void HumanClientApp::HandleWindowResize(GG::X w, GG::Y h) {
         GetOptionsDB().Set<int>("app-height-windowed", Value(h));
     }
 
+    glViewport(0, 0, Value(w), Value(h));
+
     GetOptionsDB().Commit();
 }
 
@@ -874,10 +928,25 @@ void HumanClientApp::HandleWindowClose() {
     Exit(0);
 }
 
-void HumanClientApp::HandleFocusChange() {
-    DebugLogger() << "HumanClientApp::HandleFocusChange()";
+void HumanClientApp::HandleFocusChange(bool gained_focus) {
+    DebugLogger() << "HumanClientApp::HandleFocusChange("
+                  << (gained_focus ? "Gained Focus" : "Lost Focus")
+                  << ")";
 
-    // TODO: Does SDL need some action here?
+    m_have_window_focus = gained_focus;
+
+    // limit rendering frequency when defocused to limit CPU use
+    if (!m_have_window_focus) {
+        if (GetOptionsDB().Get<bool>("limit-fps-no-focus"))
+            this->SetMaxFPS(GetOptionsDB().Get<double>("max-fps-no_focus"));
+        else
+            this->SetMaxFPS(0.0);
+    } else {
+        if (GetOptionsDB().Get<bool>("limit-fps"))
+            this->SetMaxFPS(GetOptionsDB().Get<double>("max-fps"));
+        else
+            this->SetMaxFPS(0.0);
+    }
 
     CancelDragDrop();
     ClearEventState();
@@ -905,6 +974,9 @@ bool HumanClientApp::ToggleFullscreen() {
 void HumanClientApp::StartGame() {
     m_game_started = true;
     Orders().Reset();
+
+    if (MapWnd* map_wnd = ClientUI::GetClientUI()->GetMapWnd())
+        map_wnd->ResetEmpireShown();
 }
 
 namespace {
@@ -1055,7 +1127,7 @@ void HumanClientApp::EndGame(bool suppress_FSM_reset) {
     if (m_networking.Connected()) {
         DebugLogger() << "HumanClientApp::EndGame Sending server shutdown message.";
         m_networking.SendMessage(ShutdownServerMessage(m_networking.PlayerID()));
-        boost::this_thread::sleep(boost::posix_time::seconds(1));
+        boost::this_thread::sleep_for(boost::chrono::seconds(1));
         m_networking.DisconnectFromServer();
         if (!m_networking.Connected())
             DebugLogger() << "HumanClientApp::EndGame Disconnected from server.";
@@ -1095,9 +1167,15 @@ void HumanClientApp::DecAutoTurns(int n) {
         m_auto_turns = 0;
 }
 
-int HumanClientApp::AutoTurnsLeft()
+int HumanClientApp::AutoTurnsLeft() const
 { return m_auto_turns; }
 
+bool HumanClientApp::HaveWindowFocus() const
+{ return m_have_window_focus; }
+
+int HumanClientApp::EffectsProcessingThreads() const
+{ return GetOptionsDB().Get<int>("effects-threads-ui"); }
+
 void HumanClientApp::UpdateFPSLimit() {
     if (GetOptionsDB().Get<bool>("limit-fps")) {
         double fps = GetOptionsDB().Get<double>("max-fps");
@@ -1117,6 +1195,46 @@ void HumanClientApp::DisconnectedFromServer() {
 HumanClientApp* HumanClientApp::GetApp()
 { return dynamic_cast<HumanClientApp*>(GG::GUI::GetGUI()); }
 
-void HumanClientApp::Initialize() {
+void HumanClientApp::Initialize()
+{}
 
+void HumanClientApp::OpenURL(const std::string& url) {
+    // make sure it's a legit url
+    std::string trimmed_url = url;
+    boost::algorithm::trim(trimmed_url);
+    // shouldn't be excessively long
+    if (trimmed_url.size() > 500) { // arbitrary limit
+        ErrorLogger() << "HumanClientApp::OpenURL given bad-looking url (too long): " << trimmed_url;
+        return;
+    }
+    // should start with http:// or https://
+    if (trimmed_url.size() < 8) {
+        ErrorLogger() << "HumanClientApp::OpenURL given bad-looking url (too short): " << trimmed_url;
+        return;
+    }
+    if (trimmed_url.find_first_of("http://") != 0 &&
+        trimmed_url.find_first_of("https://") != 0)
+    {
+        ErrorLogger() << "HumanClientApp::OpenURL given url that doesn't start with http:// :" << trimmed_url;
+        return;
+    }
+    // should not have newlines...
+    if (trimmed_url.find_first_of("\n") != std::string::npos) {
+        ErrorLogger() << "HumanClientApp::OpenURL given url that contains a newline. rejecting.";
+        return;
+    }
+
+    // append url to OS-specific open command
+    std::string command;
+#ifdef _WIN32
+    command += "start ";
+#elif __APPLE__
+    command += "open ";
+#else
+    command += "xdg-open ";
+#endif
+    command += url;
+
+    // execute open command
+    system(command.c_str());
 }
