@@ -120,14 +120,18 @@ boost::statechart::result WaitingForSPHostAck::react(const Error& msg) {
     ExtractMessageData(msg.m_message, problem, fatal);
 
     ErrorLogger() << "WaitingForSPHostAck::react(const Error& msg) error: " << problem;
-    ClientUI::MessageBox(UserString(problem), true);
+
+    //Note: Any transit<> transition must occur before the MessageBox().
+    // MessageBox blocks and can allow other events to transit<> to a new state
+    // which makes this transit fatal.
+    boost::statechart::result retval = fatal ? transit<IntroMenu>() : discard_event();
 
     if (fatal) {
+        ClientUI::MessageBox(UserString(problem), true);
         Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(UserString("RETURN_TO_INTRO") + "\n");
-        return transit<IntroMenu>();
-    } else {
-        return discard_event();
     }
+
+    return retval;
 }
 
 
@@ -157,14 +161,18 @@ boost::statechart::result WaitingForMPHostAck::react(const Error& msg) {
     ExtractMessageData(msg.m_message, problem, fatal);
 
     ErrorLogger() << "WaitingForMPHostAck::react(const Error& msg) error: " << problem;
-    ClientUI::MessageBox(UserString(problem), true);
+
+    //Note: Any transit<> transition must occur before the MessageBox().
+    // MessageBox blocks and can allow other events to transit<> to a new state
+    // which makes this transit fatal.
+    boost::statechart::result retval = fatal ? transit<IntroMenu>() : discard_event();
 
     if (fatal) {
+        ClientUI::MessageBox(UserString(problem), true);
         Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(UserString("RETURN_TO_INTRO") + "\n");
-        return transit<IntroMenu>();
-    } else {
-        return discard_event();
     }
+
+    return retval;
 }
 
 
@@ -193,14 +201,18 @@ boost::statechart::result WaitingForMPJoinAck::react(const Error& msg) {
     ExtractMessageData(msg.m_message, problem, fatal);
 
     ErrorLogger() << "WaitingForMPJoinAck::react(const Error& msg) error: " << problem;
-    ClientUI::MessageBox(UserString(problem), true);
+
+    //Note: Any transit<> transition must occur before the MessageBox().
+    // MessageBox blocks and can allow other events to transit<> to a new state
+    // which makes this transit fatal.
+    boost::statechart::result retval = fatal ? transit<IntroMenu>() : discard_event();
 
     if (fatal) {
+        ClientUI::MessageBox(UserString(problem), true);
         Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(UserString("RETURN_TO_INTRO") + "\n");
-        return transit<IntroMenu>();
-    } else {
-        return discard_event();
     }
+
+    return retval;
 }
 
 
@@ -221,9 +233,13 @@ MPLobby::~MPLobby() {
 
 boost::statechart::result MPLobby::react(const Disconnection& d) {
     if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) MPLobby.Disconnection";
+    //Note: Any transit<> transition must occur before the MessageBox().
+    // MessageBox blocks and can allow other events to transit<> to a new state
+    // which makes this transit fatal.
+    boost::statechart::result retval = transit<IntroMenu>();
     ClientUI::MessageBox(UserString("SERVER_LOST"), true);
     Client().Remove(Client().GetClientUI()->GetMultiPlayerLobbyWnd());
-    return transit<IntroMenu>();
+    return retval;
 }
 
 boost::statechart::result MPLobby::react(const HostID& msg) {
@@ -297,14 +313,18 @@ boost::statechart::result MPLobby::react(const Error& msg) {
     ExtractMessageData(msg.m_message, problem, fatal);
 
     ErrorLogger() << "MPLobby::react(const Error& msg) error: " << problem;
-    ClientUI::MessageBox(UserString(problem), true);
+
+    //Note: Any transit<> transition must occur before the MessageBox().
+    // MessageBox blocks and can allow other events to transit<> to a new state
+    // which makes this transit fatal.
+    boost::statechart::result retval = fatal ? transit<IntroMenu>() : discard_event();
 
     if (fatal) {
+        ClientUI::MessageBox(UserString(problem), true);
         Client().Remove(Client().GetClientUI()->GetMultiPlayerLobbyWnd());
-        return transit<IntroMenu>();
-    } else {
-        return discard_event();
     }
+
+    return retval;
 }
 
 
@@ -318,6 +338,7 @@ PlayingGame::PlayingGame(my_context ctx) :
 
     Client().Register(Client().GetClientUI()->GetMapWnd());
     Client().GetClientUI()->GetMapWnd()->Show();
+    Client().GetClientUI()->GetMessageWnd()->Clear();
 }
 
 PlayingGame::~PlayingGame() {
@@ -359,12 +380,16 @@ boost::statechart::result PlayingGame::react(const PlayerChat& msg) {
 boost::statechart::result PlayingGame::react(const Disconnection& d) {
     if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) PlayingGame.Disconnection";
     Client().EndGame(true);
+    //Note: Any transit<> transition must occur before the MessageBox().
+    // MessageBox blocks and can allow other events to transit<> to a new state
+    // which makes this transit fatal.
+    boost::statechart::result retval = transit<IntroMenu>();
     ClientUI::MessageBox(UserString("SERVER_LOST"), true);
     Client().Remove(Client().GetClientUI()->GetMapWnd());
     Client().GetClientUI()->GetMapWnd()->RemoveWindows();
     Client().Networking().SetHostPlayerID(Networking::INVALID_PLAYER_ID);
     Client().Networking().SetPlayerID(Networking::INVALID_PLAYER_ID);
-    return transit<IntroMenu>();
+    return retval;
 }
 
 boost::statechart::result PlayingGame::react(const PlayerStatus& msg) {
@@ -401,33 +426,6 @@ boost::statechart::result PlayingGame::react(const DiplomaticStatusUpdate& u) {
     return discard_event();
 }
 
-boost::statechart::result PlayingGame::react(const VictoryDefeat& msg) {
-    if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) PlayingGame.VictoryDefeat";
-    Message::VictoryOrDefeat victory_or_defeat;
-    std::string reason_string;
-    int empire_id;
-    ExtractMessageData(msg.m_message, victory_or_defeat, reason_string, empire_id);
-
-    const Empire* empire = GetEmpire(empire_id);
-    std::string empire_name = UserString("UNKNOWN_EMPIRE");
-    if (empire)
-        empire_name = empire->Name();
-
-    //ClientUI::MessageBox(boost::io::str(FlexibleFormat(UserString(reason_string)) % empire_name));
-    return discard_event();
-}
-
-boost::statechart::result PlayingGame::react(const PlayerEliminated& msg) {
-    if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) PlayingGame.PlayerEliminated";
-    int empire_id;
-    std::string empire_name;
-    ExtractMessageData(msg.m_message, empire_id, empire_name);
-    Client().EmpireEliminatedSignal(empire_id);
-    // TODO: replace this with something better
-    //ClientUI::MessageBox(boost::io::str(FlexibleFormat(UserString("EMPIRE_DEFEATED")) % empire_name));
-    return discard_event();
-}
-
 boost::statechart::result PlayingGame::react(const EndGame& msg) {
     if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) PlayingGame.EndGame";
     Message::EndGameReason reason;
@@ -445,17 +443,17 @@ boost::statechart::result PlayingGame::react(const EndGame& msg) {
         reason_message = boost::io::str(FlexibleFormat(UserString("PLAYER_DISCONNECTED")) % reason_player_name);
         error = true;
         break;
-    case Message::YOU_ARE_ELIMINATED:
-        Client().EndGame(true);
-        reason_message = UserString("PLAYER_DEFEATED");
-        break;
     }
+    //Note: Any transit<> transition must occur before the MessageBox().
+    // MessageBox blocks and can allow other events to transit<> to a new state
+    // which makes this transit fatal.
+    boost::statechart::result retval = transit<IntroMenu>();
     ClientUI::MessageBox(reason_message, error);
     Client().Remove(Client().GetClientUI()->GetMapWnd());
     Client().GetClientUI()->GetMapWnd()->RemoveWindows();
     Client().Networking().SetHostPlayerID(Networking::INVALID_PLAYER_ID);
     Client().Networking().SetPlayerID(Networking::INVALID_PLAYER_ID);
-    return transit<IntroMenu>();
+    return retval;
 }
 
 boost::statechart::result PlayingGame::react(const ResetToIntroMenu& msg) {
@@ -475,19 +473,26 @@ boost::statechart::result PlayingGame::react(const Error& msg) {
     bool fatal;
     ExtractMessageData(msg.m_message, problem, fatal);
 
-    ErrorLogger() << "PlayingGame::react(const Error& msg) error: " << problem;
-    ClientUI::MessageBox(UserString(problem), true);
+    ErrorLogger() << "PlayingGame::react(const Error& msg) error: "
+                  << problem << "\nProblem is" <<(fatal ? "":"non-")<<" fatal";
+
+    //Note: Any transit<> transition must occur before the MessageBox().
+    // MessageBox blocks and can allow other events to transit<> to a new state
+    // which makes this transit fatal.
+    boost::statechart::result retval = fatal ? transit<IntroMenu>() : discard_event();
+    ClientUI::MessageBox(UserString(problem), false);
 
     if (fatal) {
-        Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(UserString("RETURN_TO_INTRO") + "\n");
+        ClientUI::MessageBox(UserString(problem), true);
+        Client().EndGame(true);
+        Client().GetClientUI()->GetMessageWnd()->Hide();
+        Client().GetClientUI()->GetPlayerListWnd()->Hide();
         Client().Remove(Client().GetClientUI()->GetMapWnd());
         Client().GetClientUI()->GetMapWnd()->RemoveWindows();
         Client().Networking().SetHostPlayerID(Networking::INVALID_PLAYER_ID);
         Client().Networking().SetPlayerID(Networking::INVALID_PLAYER_ID);
-        return transit<IntroMenu>();
-    } else {
-        return discard_event();
     }
+    return retval;
 }
 
 boost::statechart::result PlayingGame::react(const TurnProgress& msg) {
@@ -544,10 +549,10 @@ boost::statechart::result WaitingForGameStart::react(const GameStart& msg) {
 
     ExtractMessageData(msg.m_message,       single_player_game,             empire_id,
                        current_turn,        Empires(),                      GetUniverse(),
-                       GetSpeciesManager(), GetCombatLogManager(),          Client().Players(),
-                       orders,              loaded_game_data,               ui_data_available,
-                       ui_data,             save_state_string_available,    save_state_string,
-                       Client().GetGalaxySetupData());
+                       GetSpeciesManager(), GetCombatLogManager(),          GetSupplyManager(),
+                       Client().Players(),  orders,                         loaded_game_data,
+                       ui_data_available,   ui_data,                        save_state_string_available,
+                       save_state_string,   Client().GetGalaxySetupData());
 
     DebugLogger() << "Extracted GameStart message for turn: " << current_turn << " with empire: " << empire_id;
 
@@ -583,12 +588,29 @@ WaitingForTurnData::WaitingForTurnData(my_context ctx) :
 WaitingForTurnData::~WaitingForTurnData()
 { if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) ~WaitingForTurnData"; }
 
-boost::statechart::result WaitingForTurnData::react(const SaveGame& msg) {
-    if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) WaitingForTurnData.SaveGame";
+boost::statechart::result WaitingForTurnData::react(const SaveGameDataRequest& msg) {
+    if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) WaitingForTurnData.SaveGameDataRequest";
+    DebugLogger() << "Sending Save Game Data to Server";
+    Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(UserString("SERVER_SAVE_INITIATE_ACK") + "\n");
     Client().HandleSaveGameDataRequest();
     return discard_event();
 }
 
+boost::statechart::result WaitingForTurnData::react(const SaveGameComplete& msg) {
+    if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) WaitingForTurnData.SaveGameComplete";
+
+    std::string save_filename;
+    int bytes_written;
+    ExtractMessageData(msg.m_message, save_filename, bytes_written);
+
+
+    Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(
+        boost::io::str(FlexibleFormat(UserString("SERVER_SAVE_COMPLETE")) % save_filename % bytes_written) + "\n");
+
+    DebugLogger() << "Save Complete on Server";
+    return discard_event();
+}
+
 boost::statechart::result WaitingForTurnData::react(const TurnUpdate& msg) {
     if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) PlayingGame.TurnUpdate";
 
@@ -597,7 +619,7 @@ boost::statechart::result WaitingForTurnData::react(const TurnUpdate& msg) {
     try {
         ExtractMessageData(msg.m_message,           Client().EmpireID(),    current_turn,
                            Empires(),               GetUniverse(),          GetSpeciesManager(),
-                           GetCombatLogManager(),   Client().Players());
+                           GetCombatLogManager(),   GetSupplyManager(),     Client().Players());
     } catch (...) {
         Client().GetClientUI()->GetMessageWnd()->HandleLogMessage(UserString("ERROR_PROCESSING_SERVER_MESSAGE") + "\n");
         return discard_event();
@@ -670,12 +692,29 @@ PlayingTurn::PlayingTurn(my_context ctx) :
 PlayingTurn::~PlayingTurn()
 { if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) ~PlayingTurn"; }
 
-boost::statechart::result PlayingTurn::react(const SaveGame& msg) {
-    if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) PlayingTurn.SaveGame";
+boost::statechart::result PlayingTurn::react(const SaveGameDataRequest& msg) {
+    if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) PlayingTurn.SaveGameDataRequest";
+    DebugLogger() << "Sending Save Game Data to Server";
+    Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(UserString("SERVER_SAVE_INITIATE_ACK") + "\n");
     Client().HandleSaveGameDataRequest();
     return discard_event();
 }
 
+boost::statechart::result PlayingTurn::react(const SaveGameComplete& msg) {
+    if (TRACE_EXECUTION) DebugLogger() << "(HumanClientFSM) PlayingTurn.SaveGameComplete";
+
+    std::string save_filename;
+    int bytes_written;
+    ExtractMessageData(msg.m_message, save_filename, bytes_written);
+
+
+    Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(
+        boost::io::str(FlexibleFormat(UserString("SERVER_SAVE_COMPLETE")) % save_filename % bytes_written) + "\n");
+
+    DebugLogger() << "Save Complete on Server";
+    return discard_event();
+}
+
 boost::statechart::result PlayingTurn::react(const AdvanceTurn& d) {
     Client().StartTurn();
     return discard_event();
