@@ -5,10 +5,13 @@ import freeorion as fo
 
 from galaxy import get_systems_within_jumps
 from starsystems import star_types_real, pick_star_type
-from planets import planet_sizes_real, planet_types_real, calc_planet_size, calc_planet_type
+from planets import calc_planet_size, calc_planet_type, planet_sizes_real, planet_types_real
 from names import get_name_list, random_name
+from options import (HS_ACCEPTABLE_PLANET_TYPES, HS_MIN_PLANETS_IN_VICINITY_TOTAL,
+                     HS_MIN_PLANETS_IN_VICINITY_PER_SYSTEM, HS_MIN_DISTANCE_PRIORITY_LIMIT, HS_MAX_JUMP_DISTANCE_LIMIT,
+                     HS_VICINITY_RANGE, HS_MIN_SYSTEMS_IN_VICINITY, HS_ACCEPTABLE_PLANET_SIZES)
+
 from util import load_string_list, report_error
-from options import *
 import statistics
 
 
@@ -157,12 +160,11 @@ def find_home_systems(num_home_systems, pool_list, jump_distance, min_jump_dista
     return []
 
 
-def add_planets_to_vicinity(vicinity, num_planets):
+def add_planets_to_vicinity(vicinity, num_planets, gsd):
     """
     Adds the specified number of planets to the specified systems.
     """
     print "Adding", num_planets, "planets to the following systems:", vicinity
-    gsd = fo.get_galaxy_setup_data()  # get galaxy setup data, we will need that later
 
     # first, compile a list containing all the free orbits in the specified systems
     # begin with adding the free orbits of all systems that have a real star (that is, no neutron star, black hole,
@@ -240,7 +242,7 @@ def add_planets_to_vicinity(vicinity, num_planets):
         num_planets -= 1
 
 
-def compile_home_system_list(num_home_systems, systems):
+def compile_home_system_list(num_home_systems, systems, gsd):
     """
     Compiles a list with a requested number of home systems.
     """
@@ -359,7 +361,7 @@ def compile_home_system_list(num_home_systems, systems):
             "planets in the near vicinity, required minimum:", min_planets_in_vicinity_limit(num_systems_in_vicinity)
         if num_planets_to_add > 0:
             systems_in_vicinity.remove(home_system)  # don't add planets to the home system, so remove it from the list
-            add_planets_to_vicinity(systems_in_vicinity, num_planets_to_add)
+            add_planets_to_vicinity(systems_in_vicinity, num_planets_to_add, gsd)
 
     # as we've sorted the home system list before, lets shuffle it to ensure random order and return
     random.shuffle(home_systems)
@@ -420,15 +422,15 @@ def setup_empire(empire, empire_name, home_system, starting_species, player_name
         print "Player", player_name, ": no available foci on homeworld for starting species", starting_species
 
     # give homeworld starting buildings
-    # use the list provided in starting_buildings.txt
+    # use the list provided in scripting/starting_unlocks/buildings.inf
     print "Player", player_name, ": add starting buildings to homeworld"
-    for building in load_string_list(os.path.join(fo.get_resource_dir(), "starting_buildings.txt")):
+    for building in load_string_list(os.path.join(fo.get_resource_dir(), "scripting/starting_unlocks/buildings.inf")):
         fo.create_building(building, homeworld, empire)
 
     # unlock starting techs, buildings, hulls, ship parts, etc.
-    # use content file preunlocked_items.txt
+    # use default content file
     print "Player", player_name, ": add unlocked items"
-    for item in fo.load_item_spec_list("preunlocked_items.txt"):
+    for item in fo.load_item_spec_list():
         fo.empire_unlock_item(empire, item.type, item.name)
 
     # add premade ship designs to empire
@@ -437,9 +439,9 @@ def setup_empire(empire, empire_name, home_system, starting_species, player_name
         fo.empire_add_ship_design(empire, ship_design)
 
     # add starting fleets to empire
-    # use content file starting_fleets.txt
+    # use default content file
     print "Player", player_name, ": add starting fleets"
-    fleet_plans = fo.load_fleet_plan_list("starting_fleets.txt")
+    fleet_plans = fo.load_fleet_plan_list()
     for fleet_plan in fleet_plans:
         # first, create the fleet
         fleet = fo.create_fleet(fleet_plan.name(), home_system, empire)
