@@ -1,6 +1,8 @@
 #include "ServerNetworking.h"
 
 #include "../util/Logger.h"
+#include "../util/OptionsDB.h"
+#include "../util/Version.h"
 
 #include <GG/SignalsAndSlots.h>
 
@@ -42,9 +44,13 @@ namespace {
     }
 
     struct PlayerID {
-        PlayerID(int id) : m_id(id) {}
+        PlayerID(int id) :
+            m_id(id)
+        {}
+
         bool operator()(const PlayerConnectionPtr& player_connection)
-            { return player_connection->PlayerID() == m_id; }
+        { return player_connection->PlayerID() == m_id; }
+
     private:
         int m_id;
     };
@@ -87,19 +93,17 @@ bool PlayerConnection::IsLocalConnection() const
 void PlayerConnection::Start()
 { AsyncReadMessage(); }
 
-void PlayerConnection::SendMessage(const Message& message) {
-    /*if (TRACE_EXECUTION)
-        DebugLogger() << "ServerNetworking::SendMessage : sending message "
-                               << message;*/
-    WriteMessage(m_socket, message);
-}
+void PlayerConnection::SendMessage(const Message& message)
+{ WriteMessage(m_socket, message); }
 
-void PlayerConnection::EstablishPlayer(int id, const std::string& player_name,
-                                       Networking::ClientType client_type)
+void PlayerConnection::EstablishPlayer(int id, const std::string& player_name, Networking::ClientType client_type,
+                                       const std::string& client_version_string)
 {
     if (TRACE_EXECUTION)
         DebugLogger() << "PlayerConnection(@ " << this << ")::EstablishPlayer("
-                               << id << ", " << player_name << ", " << client_type << ")";
+                      << id << ", " << player_name << ", "
+                      << client_type << ", " << client_version_string << ")";
+
     // ensure that this connection isn't already established
     if (m_ID != INVALID_PLAYER_ID || !m_player_name.empty() || m_client_type != Networking::INVALID_CLIENT_TYPE) {
         ErrorLogger() << "PlayerConnection::EstablishPlayer attempting to re-establish an already established connection.";
@@ -123,6 +127,7 @@ void PlayerConnection::EstablishPlayer(int id, const std::string& player_name,
     m_ID = id;
     m_player_name = player_name;
     m_client_type = client_type;
+    m_client_version_string = client_version_string;
 }
 
 void PlayerConnection::SetClientType(Networking::ClientType client_type) {
@@ -131,56 +136,59 @@ void PlayerConnection::SetClientType(Networking::ClientType client_type) {
         ErrorLogger() << "PlayerConnection client type set to INVALID_CLIENT_TYPE...?";
 }
 
-PlayerConnectionPtr
-PlayerConnection::NewConnection(boost::asio::io_service& io_service,
-                                MessageAndConnectionFn nonplayer_message_callback,
-                                MessageAndConnectionFn player_message_callback,
-                                ConnectionFn disconnected_callback)
+const std::string& PlayerConnection::ClientVersionString() const
+{ return m_client_version_string; }
+
+bool PlayerConnection::ClientVersionStringMatchesThisServer() const
+{ return !m_client_version_string.empty() && m_client_version_string == FreeOrionVersionString(); }
+
+PlayerConnectionPtr PlayerConnection::NewConnection(boost::asio::io_service& io_service,
+                                                    MessageAndConnectionFn nonplayer_message_callback,
+                                                    MessageAndConnectionFn player_message_callback,
+                                                    ConnectionFn disconnected_callback)
 {
     return PlayerConnectionPtr(
-        new PlayerConnection(io_service,
-                             nonplayer_message_callback,
-                             player_message_callback,
+        new PlayerConnection(io_service, nonplayer_message_callback, player_message_callback,
                              disconnected_callback));
 }
 
 namespace {
     std::string MessageTypeName(Message::MessageType type) {
         switch(type) {
-        case Message::UNDEFINED:            return "Undefined";
-        case Message::DEBUG:                return "Debug";
-        case Message::ERROR_MSG:            return "Error";
-        case Message::HOST_SP_GAME:         return "Host SP Game";
-        case Message::HOST_MP_GAME:         return "Host MP Game";
-        case Message::JOIN_GAME:            return "Join Game";
-        case Message::HOST_ID:              return "Host ID";
-        case Message::LOBBY_UPDATE:         return "Lobby Update";
-        case Message::LOBBY_CHAT:           return "Lobby Chat";
-        case Message::LOBBY_EXIT:           return "Lobby Exit";
-        case Message::START_MP_GAME:        return "Start MP Game";
-        case Message::SAVE_GAME:            return "Save Game";
-        case Message::LOAD_GAME:            return "Load Game";
-        case Message::GAME_START:           return "Game Start";
-        case Message::TURN_UPDATE:          return "Turn Update";
-        case Message::TURN_PARTIAL_UPDATE:  return "Turn Partial Update";
-        case Message::TURN_ORDERS:          return "Turn Orders";
-        case Message::TURN_PROGRESS:        return "Turn Progress";
-        case Message::PLAYER_STATUS:        return "Player Status";
-        case Message::CLIENT_SAVE_DATA:     return "Client Save Data";
-        case Message::PLAYER_CHAT:          return "Player Chat";
-        case Message::DIPLOMACY:            return "Diplomacy";
-        case Message::DIPLOMATIC_STATUS:    return "Diplomatic Status";
+        case Message::UNDEFINED:                return "Undefined";
+        case Message::DEBUG:                    return "Debug";
+        case Message::ERROR_MSG:                return "Error";
+        case Message::HOST_SP_GAME:             return "Host SP Game";
+        case Message::HOST_MP_GAME:             return "Host MP Game";
+        case Message::JOIN_GAME:                return "Join Game";
+        case Message::HOST_ID:                  return "Host ID";
+        case Message::LOBBY_UPDATE:             return "Lobby Update";
+        case Message::LOBBY_CHAT:               return "Lobby Chat";
+        case Message::LOBBY_EXIT:               return "Lobby Exit";
+        case Message::START_MP_GAME:            return "Start MP Game";
+        case Message::SAVE_GAME_INITIATE:       return "Save Game";
+        case Message::SAVE_GAME_DATA_REQUEST:   return "Save Game";
+        case Message::SAVE_GAME_COMPLETE:       return "Save Game";
+        case Message::LOAD_GAME:                return "Load Game";
+        case Message::GAME_START:               return "Game Start";
+        case Message::TURN_UPDATE:              return "Turn Update";
+        case Message::TURN_PARTIAL_UPDATE:      return "Turn Partial Update";
+        case Message::TURN_ORDERS:              return "Turn Orders";
+        case Message::TURN_PROGRESS:            return "Turn Progress";
+        case Message::PLAYER_STATUS:            return "Player Status";
+        case Message::CLIENT_SAVE_DATA:         return "Client Save Data";
+        case Message::PLAYER_CHAT:              return "Player Chat";
+        case Message::DIPLOMACY:                return "Diplomacy";
+        case Message::DIPLOMATIC_STATUS:        return "Diplomatic Status";
         case Message::REQUEST_NEW_OBJECT_ID:    return "Request New Object ID";
         case Message::DISPATCH_NEW_OBJECT_ID:   return "Dispatch New Object ID";
         case Message::REQUEST_NEW_DESIGN_ID:    return "Request New Design ID";
         case Message::DISPATCH_NEW_DESIGN_ID:   return "Dispatch New Design ID";
-        case Message::VICTORY_DEFEAT:       return "Victory/Defeat";
-        case Message::PLAYER_ELIMINATED:    return "Player Elimination";
-        case Message::END_GAME:             return "End Game";
-        case Message::MODERATOR_ACTION:     return "Moderator Action";
-        case Message::SHUT_DOWN_SERVER:     return "Shut Down Server";
-        case Message::REQUEST_SAVE_PREVIEWS:return "Request save previews";
-        default:                            return "Unknown Type";
+        case Message::END_GAME:                 return "End Game";
+        case Message::MODERATOR_ACTION:         return "Moderator Action";
+        case Message::SHUT_DOWN_SERVER:         return "Shut Down Server";
+        case Message::REQUEST_SAVE_PREVIEWS:    return "Request save previews";
+        default:                                return "Unknown Type";
         };
     }
 }
@@ -193,18 +201,15 @@ void PlayerConnection::HandleMessageBodyRead(boost::system::error_code error,
             error == boost::asio::error::connection_reset) {
             EventSignal(boost::bind(m_disconnected_callback, shared_from_this()));
         } else {
-            ErrorLogger() << "PlayerConnection::HandleMessageBodyRead(): error \""
-                                   << error << "\"";
+            ErrorLogger() << "PlayerConnection::HandleMessageBodyRead(): error \"" << error << "\"";
         }
     } else {
         assert(static_cast<int>(bytes_transferred) <= m_incoming_header_buffer[4]);
         if (static_cast<int>(bytes_transferred) == m_incoming_header_buffer[4]) {
             if (TRACE_EXECUTION && m_incoming_message.Type() != Message::REQUEST_NEW_DESIGN_ID) {   // new design id messages ignored due to log spam
-                DebugLogger() << "Server received message from player id: "
-                                       << m_incoming_message.SendingPlayer()
-                                       << " of type "
-                                       << MessageTypeName(m_incoming_message.Type())
-                                       << " and size "<< m_incoming_message.Size();
+                DebugLogger() << "Server received message from player id: " << m_incoming_message.SendingPlayer()
+                              << " of type " << MessageTypeName(m_incoming_message.Type())
+                              << " and size " << m_incoming_message.Size();
                 //DebugLogger() << "     Full message: " << m_incoming_message;
             }
             if (EstablishedPlayer()) {
@@ -236,14 +241,14 @@ void PlayerConnection::HandleMessageHeaderRead(boost::system::error_code error,
         if (m_new_connection) {
             // wait half a second if the first data read is an error; we
             // probably just need more setup time
-            boost::this_thread::sleep(boost::posix_time::milliseconds(500));
+            boost::this_thread::sleep_for(boost::chrono::milliseconds(500));
         } else {
             if (error == boost::asio::error::eof ||
                 error == boost::asio::error::connection_reset) {
                 EventSignal(boost::bind(m_disconnected_callback, shared_from_this()));
             } else {
                 ErrorLogger() << "PlayerConnection::HandleMessageHeaderRead(): "
-                                       << "error \"" << error << "\"";
+                              << "error \"" << error << "\"";
             }
         }
     } else {
@@ -273,7 +278,7 @@ void PlayerConnection::AsyncReadMessage() {
 // DiscoveryServer
 ////////////////////////////////////////////////////////////////////////////////
 DiscoveryServer::DiscoveryServer(boost::asio::io_service& io_service) :
-    m_socket(io_service, udp::endpoint(udp::v4(), DISCOVERY_PORT))
+    m_socket(io_service, udp::endpoint(udp::v4(), Networking::DiscoveryPort()))
 { Listen(); }
 
 void DiscoveryServer::Listen() {
@@ -305,12 +310,19 @@ ServerNetworking::ServerNetworking(boost::asio::io_service& io_service,
                                    MessageAndConnectionFn player_message_callback,
                                    ConnectionFn disconnected_callback) :
     m_host_player_id(Networking::INVALID_PLAYER_ID),
-    m_discovery_server(new DiscoveryServer(io_service)),
+    m_discovery_server(0),
     m_player_connection_acceptor(io_service),
     m_nonplayer_message_callback(nonplayer_message_callback),
     m_player_message_callback(player_message_callback),
     m_disconnected_callback(disconnected_callback)
-{ Init(); }
+{
+    if (!GetOptionsDB().Get<bool>("singleplayer")) {
+        // only start discovery service for multiplayer servers.
+        m_discovery_server = new DiscoveryServer(io_service);
+    }
+
+    Init();
+}
 
 ServerNetworking::~ServerNetworking()
 { delete m_discovery_server; }
@@ -470,7 +482,17 @@ void ServerNetworking::SetHostPlayerID(int host_player_id)
 { m_host_player_id = host_player_id; }
 
 void ServerNetworking::Init() {
-    tcp::endpoint endpoint(tcp::v4(), MESSAGE_PORT);
+    tcp::endpoint endpoint(tcp::v4(), Networking::MessagePort());
+
+    if (GetOptionsDB().Get<bool>("singleplayer")) {
+        // when hosting a single player game only accept connections from
+        // the localhost via the loopback interface instead of the any
+        // interface.
+        // This should prevent unnecessary triggering of Desktop Firewalls as
+        // reported by various users when running single player games.
+        endpoint.address(boost::asio::ip::address_v4::loopback());
+    }
+
     m_player_connection_acceptor.open(endpoint.protocol());
     m_player_connection_acceptor.set_option(
         boost::asio::socket_base::reuse_address(true));
@@ -502,8 +524,7 @@ void ServerNetworking::AcceptConnection(PlayerConnectionPtr player_connection,
 {
     if (!error) {
         if (TRACE_EXECUTION)
-            DebugLogger() << "ServerNetworking::AcceptConnection : connected to "
-                                   << "new player";
+            DebugLogger() << "ServerNetworking::AcceptConnection : connected to new player";
         m_player_connections.insert(player_connection);
         player_connection->Start();
         AcceptNextConnection();
@@ -515,7 +536,7 @@ void ServerNetworking::AcceptConnection(PlayerConnectionPtr player_connection,
 void ServerNetworking::DisconnectImpl(PlayerConnectionPtr player_connection) {
     if (TRACE_EXECUTION)
         DebugLogger() << "ServerNetworking::DisconnectImpl : disconnecting player "
-                               << player_connection->PlayerID();
+                      << player_connection->PlayerID();
     m_player_connections.erase(player_connection);
     m_disconnected_callback(player_connection);
 }
