@@ -1,4 +1,3 @@
-// -*- C++ -*-
 #ifndef _Tokens_h_
 #define _Tokens_h_
 
@@ -16,6 +15,7 @@
     (Adequate)                                  \
     (Affiliation)                               \
     (Age)                                       \
+    (Aggressive)                                \
     (Alignment)                                 \
     (AlignmentEffects)                          \
     (All)                                       \
@@ -24,11 +24,13 @@
     (AnyEmpire)                                 \
     (Armed)                                     \
     (Armour)                                    \
+    (ArrivedOnTurn)                             \
     (Article)                                   \
     (Application)                               \
     (Asteroids)                                 \
     (Attack)                                    \
     (Barren)                                    \
+    (Basic)                                     \
     (BlackHole)                                 \
     (Blue)                                      \
     (Bombard)                                   \
@@ -57,6 +59,7 @@
     (Colour)                                    \
     (Condition)                                 \
     (Construction)                              \
+    (Consumption)                               \
     (ContainedBy)                               \
     (Contains)                                  \
     (Core)                                      \
@@ -106,13 +109,17 @@
     (EnqueueLocation)                           \
     (Environment)                               \
     (Environments)                              \
+    (ETA)                                       \
     (ExploredByEmpire)                          \
+    (Exclusions)                                \
     (External)
 
 #define TOKEN_SEQ_4                             \
     (Field)                                     \
     (FieldType)                                 \
-    (Fighters)                                  \
+    (FighterBay)                                \
+    (FighterHangar)                             \
+    (FighterWeapon)                             \
     (FinalDestinationID)                        \
     (Fleet)                                     \
     (FleetID)                                   \
@@ -121,6 +128,7 @@
     (Focus)                                     \
     (FocusType)                                 \
     (Fuel)                                      \
+    (Full)                                      \
     (GalaxyAge)                                 \
     (GalaxyMaxAIAggression)                     \
     (GalaxyMonsterFrequency)                    \
@@ -157,6 +165,7 @@
     (HullSpeed)                                 \
     (HullStealth)                               \
     (HullStructure)                             \
+    (Human)                                     \
     (Icon)                                      \
     (ID)                                        \
     (If)                                        \
@@ -165,6 +174,7 @@
     (InSystem)                                  \
     (Interceptor)                               \
     (Internal)                                  \
+    (Invisible)                                 \
     (Item)                                      \
     (Jumps)                                     \
     (JumpsBetween)                              \
@@ -185,6 +195,8 @@
     (LowestCostResearchableTech)                \
     (LowestCostTransferrableTech)               \
     (Max)                                       \
+    (MaxCapacity)                               \
+    (MaxDamage)                                 \
     (MaxDefense)                                \
     (MaxFuel)                                   \
     (MaximumNumberOf)                           \
@@ -227,6 +239,7 @@
     (NextSystemID)                              \
     (NextTurnPopGrowth)                         \
     (NextYoungerStarType)                       \
+    (NoDefaultCapacityEffect)                   \
     (None)                                      \
     (NoStar)                                    \
     (NoStringtableLookup)                       \
@@ -234,6 +247,7 @@
     (Number)                                    \
     (NumberOf)                                  \
     (NumShips)                                  \
+    (NumStarlanes)                              \
 
 #define TOKEN_SEQ_8                             \
     (Object)                                    \
@@ -256,12 +270,14 @@
     (PartCapacity)                              \
     (PartDamage)                                \
     (PartClass)                                 \
+    (Partial)                                   \
     (PartName)
 
 #define TOKEN_SEQ_9                             \
     (Parts)                                     \
     (PartClassInShipDesign)                     \
     (PartsInShipDesign)                         \
+    (Passive)                                   \
     (Planet)                                    \
     (Planetbound)                               \
     (PlanetEnvironment)                         \
@@ -285,8 +301,9 @@
     (Product)                                   \
     (ProductionCenter)                          \
     (ProductionLocation)                        \
-    (Progress)                                  \
-    (Property)
+    (PropagatedSupplyRange)                     \
+    (Property)                                  \
+    (Progress)
 
 #define TOKEN_SEQ_10                            \
     (Radiated)                                  \
@@ -332,8 +349,11 @@
     (SetFuel)                                   \
     (SetHappiness)                              \
     (SetIndustry)                               \
+    (SetMaxCapacity)                            \
+    (SetMaxDamage)                              \
     (SetMaxDefense)                             \
     (SetMaxFuel)                                \
+    (SetMaxSecondaryStat)                       \
     (SetMaxShield)                              \
     (SetMaxStructure)                           \
     (SetMaxSupply)                              \
@@ -349,6 +369,7 @@
     (SetRange)                                  \
     (SetRebelTroops)                            \
     (SetResearch)                               \
+    (SetSecondaryStat)                          \
     (SetShield)                                 \
     (SetSize)                                   \
     (SetSpecialCapacity)                        \
@@ -368,6 +389,7 @@
     (SetTexture)                                \
     (SetTrade)                                  \
     (SetTroops)                                 \
+    (SetVisibility)                             \
     (Shield)                                    \
     (ShipDesign)                                \
     (ShipDesignsDestroyed)                      \
@@ -383,6 +405,7 @@
     (Short_Description)                         \
     (ShortestPath)                              \
     (ShortRange)                                \
+    (Shots)                                     \
     (Sin)                                       \
     (Size)                                      \
     (SizeAsDouble)
@@ -424,6 +447,7 @@
     (Structure)                                 \
     (Sum)                                       \
     (Supply)                                    \
+    (SupplyingEmpire)                           \
     (Swamp)                                     \
     (System)                                    \
     (SystemID)
@@ -439,7 +463,6 @@
     (TargetResearch)                            \
     (TargetTrade)                               \
     (Tech)                                      \
-    (TechType)                                  \
     (Terran)                                    \
     (TestValue)                                 \
     (TheEmpire)                                 \
@@ -471,11 +494,12 @@
     (Unowned)                                   \
     (Unproducible)                              \
     (Unresearchable)                            \
+    (UpgradeVisibility)                         \
     (UserString)                                \
     (Value)                                     \
-    (ValueTest)                                 \
     (Victory)                                   \
     (VisibleToEmpire)                           \
+    (Visibility)                                \
     (White)                                     \
     (WithinDistance)                            \
     (WithinStarlaneJumps)                       \
