@@ -84,15 +84,14 @@ namespace {
                 ErrorLogger() << "GetHostNameFromSinglePlayerSetupData got single player setup data to load a game, but also player setup data for a new game.  Ignoring player setup data";
 
 
-            std::vector<PlayerSaveGameData> player_save_game_data;
-            LoadPlayerSaveGameData(single_player_setup_data.m_filename,
-                                   player_save_game_data);
+            std::vector<PlayerSaveHeaderData> player_save_header_data;
+            LoadPlayerSaveHeaderData(single_player_setup_data.m_filename, player_save_header_data);
 
             // find which player was the human (and thus the host) in the saved game
-            for (std::vector<PlayerSaveGameData>::const_iterator save_data_it = player_save_game_data.begin();
-                 save_data_it != player_save_game_data.end(); ++save_data_it)
+            for (std::vector<PlayerSaveHeaderData>::const_iterator save_data_it = player_save_header_data.begin();
+                 save_data_it != player_save_header_data.end(); ++save_data_it)
             {
-                const PlayerSaveGameData& psgd = *save_data_it;
+                const PlayerSaveHeaderData& psgd = *save_data_it;
                 if (psgd.m_client_type == Networking::CLIENT_TYPE_HUMAN_PLAYER)
                     return psgd.m_name;
             }
@@ -124,7 +123,8 @@ namespace {
         if (empire_names.empty())
             UserStringList("EMPIRE_NAMES", empire_names);
         std::set<std::string> validNames(empire_names.begin(), empire_names.end());
-        for (std::list<std::pair<int, PlayerSetupData> >::iterator player_setup_it = players.begin(); player_setup_it != players.end(); player_setup_it++) {
+        for (std::list<std::pair<int, PlayerSetupData> >::iterator player_setup_it = players.begin(); player_setup_it != players.end(); ++player_setup_it)
+        {
             std::set<std::string>::iterator name_it = validNames.find(player_setup_it->second.m_empire_name);
             if (name_it != validNames.end())
                 validNames.erase(name_it);
@@ -140,10 +140,35 @@ namespace {
             return UserString("EMPIRE");
         }
     }
+
+
+    /** Note:  This Exits on fatal errors and does not return.*/
+    void HandleErrorMessage(const Error& msg, ServerApp &server) {
+        std::string problem;
+        bool fatal;
+        ExtractMessageData(msg.m_message, problem, fatal);
+
+        std::stringstream ss;
+
+        ss << "Server received from player "
+           << msg.m_player_connection->PlayerName() << "("
+           << msg.m_player_connection->PlayerID() << ")"
+           << (fatal?" a fatal":" an")
+           << " error message: " << problem;
+
+        if (fatal) {
+            FatalLogger() << ss.str();
+            SendMessageToAllPlayers(msg.m_message);
+            boost::this_thread::sleep_for(boost::chrono::seconds(2));
+            server.Exit(1);
+        }
+
+        ErrorLogger() << ss.str();
+    }
 }
 
 ////////////////////////////////////////////////////////////
-// MessageEventBase
+// Disconnection
 ////////////////////////////////////////////////////////////
 Disconnection::Disconnection(PlayerConnectionPtr& player_connection) :
     m_player_connection(player_connection)
@@ -194,16 +219,19 @@ void ServerFSM::HandleNonLobbyDisconnection(const Disconnection& d) {
         if (m_server.m_networking.PlayerIsHost(player_connection->PlayerID()))
             m_server.SelectNewHost();
 
-    } else if ((player_connection->GetClientType() == Networking::CLIENT_TYPE_HUMAN_PLAYER ||
-                player_connection->GetClientType() == Networking::CLIENT_TYPE_AI_PLAYER) &&
-                m_server.m_eliminated_players.find(id) == m_server.m_eliminated_players.end())
+    } else if (player_connection->GetClientType() == Networking::CLIENT_TYPE_HUMAN_PLAYER ||
+               player_connection->GetClientType() == Networking::CLIENT_TYPE_AI_PLAYER)
     {
-        // player abnormally disconnected during a regular game
-        DebugLogger() << "ServerFSM::HandleNonLobbyDisconnection : Lost connection to player #" << boost::lexical_cast<std::string>(id)
-                               << ", named \"" << player_connection->PlayerName() << "\"; server terminating.";
-        std::string message = player_connection->PlayerName();
-        for (ServerNetworking::const_established_iterator it = m_server.m_networking.established_begin(); it != m_server.m_networking.established_end(); ++it) {
-            if ((*it)->PlayerID() != id) {
+        const Empire* empire = GetEmpire(m_server.PlayerEmpireID(id));
+        // eliminated and non-empire players can leave safely
+        if (empire && !empire->Eliminated()) {
+            // player abnormally disconnected during a regular game
+            DebugLogger() << "ServerFSM::HandleNonLobbyDisconnection : Lost connection to player #" << id
+                          << ", named \"" << player_connection->PlayerName() << "\"; server terminating.";
+            std::string message = player_connection->PlayerName();
+            for (ServerNetworking::const_established_iterator it = m_server.m_networking.established_begin(); it != m_server.m_networking.established_end(); ++it) {
+                if ((*it)->PlayerID() == id)
+                    continue;
                 // in the future we may find a way to recover from this, but for now we will immediately send a game ending message as well
                 (*it)->SendMessage(EndGameMessage((*it)->PlayerID(), Message::PLAYER_DISCONNECT, player_connection->PlayerName()));
             }
@@ -214,7 +242,7 @@ void ServerFSM::HandleNonLobbyDisconnection(const Disconnection& d) {
     if (m_server.m_networking.empty() || m_server.m_ai_client_processes.size() == m_server.m_networking.NumEstablishedPlayers()) {
         DebugLogger() << "ServerFSM::HandleNonLobbyDisconnection : All human players disconnected; server terminating.";
         // HACK! Pause for a bit to let the player disconnected and end game messages propogate.
-        boost::this_thread::sleep(boost::posix_time::seconds(2));
+        boost::this_thread::sleep_for(boost::chrono::seconds(2));
         m_server.Exit(1);
     }
 }
@@ -236,7 +264,10 @@ sc::result Idle::react(const HostMPGame& msg) {
     const Message& message = msg.m_message;
     const PlayerConnectionPtr& player_connection = msg.m_player_connection;
 
-    std::string host_player_name = message.Text();
+    std::string host_player_name;
+    std::string client_version_string;
+    ExtractMessageData(message, host_player_name, client_version_string);
+
     // validate host name (was found and wasn't empty)
     if (host_player_name.empty()) {
         ErrorLogger() << "Idle::react(const HostMPGame& msg) got an empty host player name";
@@ -246,7 +277,7 @@ sc::result Idle::react(const HostMPGame& msg) {
     DebugLogger() << "Idle::react(HostMPGame) about to establish host";
 
     int host_player_id = server.m_networking.NewPlayerID();
-    player_connection->EstablishPlayer(host_player_id, host_player_name, Networking::CLIENT_TYPE_HUMAN_PLAYER);
+    player_connection->EstablishPlayer(host_player_id, host_player_name, Networking::CLIENT_TYPE_HUMAN_PLAYER, client_version_string);
     server.m_networking.SetHostPlayerID(host_player_id);
 
     DebugLogger() << "Idle::react(HostMPGame) about to send acknowledgement to host";
@@ -266,7 +297,8 @@ sc::result Idle::react(const HostSPGame& msg) {
     const PlayerConnectionPtr& player_connection = msg.m_player_connection;
 
     boost::shared_ptr<SinglePlayerSetupData> single_player_setup_data(new SinglePlayerSetupData);
-    ExtractMessageData(message, *single_player_setup_data);
+    std::string client_version_string;
+    ExtractMessageData(message, *single_player_setup_data, client_version_string);
 
 
     // get host player's name from setup data or saved file
@@ -286,7 +318,7 @@ sc::result Idle::react(const HostSPGame& msg) {
 
 
     int host_player_id = server.m_networking.NewPlayerID();
-    player_connection->EstablishPlayer(host_player_id, host_player_name, Networking::CLIENT_TYPE_HUMAN_PLAYER);
+    player_connection->EstablishPlayer(host_player_id, host_player_name, Networking::CLIENT_TYPE_HUMAN_PLAYER, client_version_string);
     server.m_networking.SetHostPlayerID(host_player_id);
     player_connection->SendMessage(HostSPAckMessage(host_player_id));
 
@@ -297,6 +329,10 @@ sc::result Idle::react(const HostSPGame& msg) {
     return transit<WaitingForSPGameJoiners>();
 }
 
+sc::result Idle::react(const Error& msg) {
+    HandleErrorMessage(msg, Server());
+    return discard_event();
+}
 
 ////////////////////////////////////////////////////////////
 // MPLobby
@@ -424,14 +460,15 @@ sc::result MPLobby::react(const JoinGame& msg) {
 
     std::string player_name;
     Networking::ClientType client_type;
-    ExtractMessageData(message, player_name, client_type);
+    std::string client_version_string;
+    ExtractMessageData(message, player_name, client_type, client_version_string);
     // TODO: check if player name is unique.  If not, modify it slightly to be unique.
 
     // assign unique player ID to newly connected player
     int player_id = server.m_networking.NewPlayerID();
 
     // establish player with requested client type and acknowldge via connection
-    player_connection->EstablishPlayer(player_id, player_name, client_type);
+    player_connection->EstablishPlayer(player_id, player_name, client_type, client_version_string);
     player_connection->SendMessage(JoinAckMessage(player_id));
 
     // inform player of host
@@ -648,7 +685,7 @@ sc::result MPLobby::react(const LobbyUpdate& msg) {
         }
     }
 
-    // propegate lobby changes to players, so everyone has the latest updated
+    // propagate lobby changes to players, so everyone has the latest updated
     // version of the lobby data
     for (ServerNetworking::const_established_iterator player_connection_it = server.m_networking.established_begin();
          player_connection_it != server.m_networking.established_end(); ++player_connection_it)
@@ -768,6 +805,10 @@ sc::result MPLobby::react(const HostSPGame& msg) {
     return discard_event();
 }
 
+sc::result MPLobby::react(const Error& msg) {
+    HandleErrorMessage(msg, Server());
+    return discard_event();
+}
 
 ////////////////////////////////////////////////////////////
 // WaitingForSPGameJoiners
@@ -817,10 +858,9 @@ WaitingForSPGameJoiners::WaitingForSPGameJoiners(my_context c) :
             players.clear();
         }
 
-        std::vector<PlayerSaveGameData> player_save_game_data;
+        std::vector<PlayerSaveHeaderData> player_save_header_data;
         try {
-            LoadPlayerSaveGameData(m_single_player_setup_data->m_filename,
-                                   player_save_game_data);
+            LoadPlayerSaveHeaderData(m_single_player_setup_data->m_filename, player_save_header_data);
         } catch (const std::exception& e) {
             SendMessageToHost(ErrorMessage(UserStringNop("UNABLE_TO_READ_SAVE_FILE"), true));
             post_event(LoadSaveFileFailed());
@@ -828,10 +868,10 @@ WaitingForSPGameJoiners::WaitingForSPGameJoiners(my_context c) :
         }
 
         // add player setup data for each player in saved gamed
-        for (std::vector<PlayerSaveGameData>::const_iterator save_data_it = player_save_game_data.begin();
-             save_data_it != player_save_game_data.end(); ++save_data_it)
+        for (std::vector<PlayerSaveHeaderData>::const_iterator save_data_it = player_save_header_data.begin();
+             save_data_it != player_save_header_data.end(); ++save_data_it)
         {
-            const PlayerSaveGameData& psgd = *save_data_it;
+            const PlayerSaveHeaderData& psgd = *save_data_it;
             if (psgd.m_client_type == Networking::CLIENT_TYPE_HUMAN_PLAYER ||
                 psgd.m_client_type == Networking::CLIENT_TYPE_AI_PLAYER)
             {
@@ -872,7 +912,8 @@ sc::result WaitingForSPGameJoiners::react(const JoinGame& msg) {
 
     std::string player_name("Default_Player_Name_in_WaitingForSPGameJoiners::react(const JoinGame& msg)");
     Networking::ClientType client_type(Networking::INVALID_CLIENT_TYPE);
-    ExtractMessageData(message, player_name, client_type);
+    std::string client_version_string;
+    ExtractMessageData(message, player_name, client_type, client_version_string);
 
     int player_id = server.m_networking.NewPlayerID();
 
@@ -886,7 +927,7 @@ sc::result WaitingForSPGameJoiners::react(const JoinGame& msg) {
         } else {
             // expected player
             // let the networking system know what socket this player is on
-            player_connection->EstablishPlayer(player_id, player_name, client_type);
+            player_connection->EstablishPlayer(player_id, player_name, client_type, client_version_string);
             player_connection->SendMessage(JoinAckMessage(player_id));
 
             // remove name from expected names list, so as to only allow one connection per AI
@@ -903,7 +944,7 @@ sc::result WaitingForSPGameJoiners::react(const JoinGame& msg) {
             server.m_networking.Disconnect(player_connection);
         } else {
             // expected human player
-            player_connection->EstablishPlayer(player_id, player_name, client_type);
+            player_connection->EstablishPlayer(player_id, player_name, client_type, client_version_string);
             player_connection->SendMessage(JoinAckMessage(player_id));
         }
     } else {
@@ -921,9 +962,12 @@ sc::result WaitingForSPGameJoiners::react(const CheckStartConditions& u) {
 
     // if all expected players have connected, proceed to start new or load game
     if (static_cast<int>(server.m_networking.NumEstablishedPlayers()) == m_num_expected_players) {
+        DebugLogger() << "WaitingForSPGameJoiners::react(const CheckStartConditions& u) : have all " << m_num_expected_players << " expected players connected.";
         if (m_single_player_setup_data->m_new_game) {
+            DebugLogger() << "Initializing new SP game...";
             server.NewSPGameInit(*m_single_player_setup_data);
         } else {
+            DebugLogger() << "Loading SP game save file: " << m_single_player_setup_data->m_filename;
             try {
                 LoadGame(m_single_player_setup_data->m_filename,            *m_server_save_game_data,
                          m_player_save_game_data,   GetUniverse(),          Empires(),
@@ -934,8 +978,7 @@ sc::result WaitingForSPGameJoiners::react(const CheckStartConditions& u) {
                 return transit<Idle>();
             }
 
-            server.LoadSPGameInit(m_player_save_game_data,
-                                  m_server_save_game_data);
+            server.LoadSPGameInit(m_player_save_game_data, m_server_save_game_data);
         }
         return transit<PlayingGame>();
     }
@@ -948,6 +991,10 @@ sc::result WaitingForSPGameJoiners::react(const LoadSaveFileFailed& u) {
     return transit<Idle>();
 }
 
+sc::result WaitingForSPGameJoiners::react(const Error& msg) {
+    HandleErrorMessage(msg, Server());
+    return discard_event();
+}
 
 ////////////////////////////////////////////////////////////
 // WaitingForMPGameJoiners
@@ -997,7 +1044,8 @@ sc::result WaitingForMPGameJoiners::react(const JoinGame& msg) {
 
     std::string player_name("Default_Player_Name_in_WaitingForMPGameJoiners::react(const JoinGame& msg)");
     Networking::ClientType client_type(Networking::INVALID_CLIENT_TYPE);
-    ExtractMessageData(message, player_name, client_type);
+    std::string client_version_string;
+    ExtractMessageData(message, player_name, client_type, client_version_string);
 
     int player_id = server.m_networking.NewPlayerID();
 
@@ -1011,7 +1059,7 @@ sc::result WaitingForMPGameJoiners::react(const JoinGame& msg) {
         } else {
             // expected player
             // let the networking system know what socket this player is on
-            player_connection->EstablishPlayer(player_id, player_name, client_type);
+            player_connection->EstablishPlayer(player_id, player_name, client_type, client_version_string);
             player_connection->SendMessage(JoinAckMessage(player_id));
 
             // remove name from expected names list, so as to only allow one connection per AI
@@ -1020,15 +1068,14 @@ sc::result WaitingForMPGameJoiners::react(const JoinGame& msg) {
 
     } else if (client_type == Networking::CLIENT_TYPE_HUMAN_PLAYER) {
         // verify that there is room left for this player
-        int already_connected_players = m_expected_ai_player_names.size() +
-                                        server.m_networking.NumEstablishedPlayers();
+        int already_connected_players = m_expected_ai_player_names.size() + server.m_networking.NumEstablishedPlayers();
         if (already_connected_players >= m_num_expected_players) {
             // too many human players
             ErrorLogger() << "WaitingForSPGameJoiners.JoinGame : A human player attempted to join the game but there was not enough room.  Terminating connection.";
             server.m_networking.Disconnect(player_connection);
         } else {
             // expected human player
-            player_connection->EstablishPlayer(player_id, player_name, client_type);
+            player_connection->EstablishPlayer(player_id, player_name, client_type, client_version_string);
             player_connection->SendMessage(JoinAckMessage(player_id));
         }
     } else {
@@ -1049,16 +1096,24 @@ sc::result WaitingForMPGameJoiners::react(const CheckStartConditions& u) {
     ServerApp& server = Server();
 
     if (static_cast<int>(server.m_networking.NumEstablishedPlayers()) == m_num_expected_players) {
-        if (m_player_save_game_data.empty())
+        if (m_player_save_game_data.empty()) {
+            DebugLogger() << "Initializing new MP game...";
             server.NewMPGameInit(*m_lobby_data);
-        else
+        } else {
+            DebugLogger() << "Initializing loaded MP game";
             server.LoadMPGameInit(*m_lobby_data, m_player_save_game_data, m_server_save_game_data);
+        }
         return transit<PlayingGame>();
     }
 
     return discard_event();
 }
 
+sc::result WaitingForMPGameJoiners::react(const Error& msg) {
+    HandleErrorMessage(msg, Server());
+    return discard_event();
+}
+
 
 ////////////////////////////////////////////////////////////
 // PlayingGame
@@ -1104,8 +1159,13 @@ sc::result PlayingGame::react(const ModeratorAct& msg) {
     int player_id = message.SendingPlayer();
     ServerApp& server = Server();
 
-    // TODO: Check that sender is a moderator
-    //if (client_type == Networking::CLIENT_TYPE_HUMAN_OBSERVER) {
+    const PlayerConnectionPtr& player_connection = msg.m_player_connection;
+    Networking::ClientType client_type = player_connection->GetClientType();
+
+    if (client_type != Networking::CLIENT_TYPE_HUMAN_MODERATOR) {
+        ErrorLogger() << "PlayingGame::react(ModeratorAct): Non-moderator player sent moderator action, ignorning";
+        return discard_event();
+    }
 
     Moderator::ModeratorAction* action = 0;
     ExtractMessageData(message, action);
@@ -1117,10 +1177,10 @@ sc::result PlayingGame::react(const ModeratorAct& msg) {
         action->Execute();
 
         // update player(s) of changed gamestate as result of action
+        bool use_binary_serialization = player_connection->ClientVersionStringMatchesThisServer();
         server.m_networking.SendMessage(TurnProgressMessage(Message::DOWNLOADING, player_id));
-        server.m_networking.SendMessage(TurnPartialUpdateMessage(player_id,
-                                                                 server.PlayerEmpireID(player_id),
-                                                                 GetUniverse()));
+        server.m_networking.SendMessage(TurnPartialUpdateMessage(player_id, server.PlayerEmpireID(player_id),
+                                                                 GetUniverse(), use_binary_serialization));
     }
 
     delete action;
@@ -1128,6 +1188,11 @@ sc::result PlayingGame::react(const ModeratorAct& msg) {
     return discard_event();
 }
 
+sc::result PlayingGame::react(const Error& msg) {
+    HandleErrorMessage(msg, Server());
+    return discard_event();
+}
+
 
 ////////////////////////////////////////////////////////////
 // WaitingForTurnEnd
@@ -1295,9 +1360,8 @@ sc::result WaitingForTurnEndIdle::react(const SaveGameRequest& msg) {
     const PlayerConnectionPtr& player_connection = msg.m_player_connection;
 
     if (!server.m_networking.PlayerIsHost(player_connection->PlayerID())) {
-        return discard_event();
         ErrorLogger() << "WaitingForTurnEndIdle.SaveGameRequest : Player #" << message.SendingPlayer()
-                               << " attempted to initiate a game save, but is not the host.  Ignoring request connection.";
+                      << " attempted to initiate a game save, but is not the host.  Ignoring request connection.";
         player_connection->SendMessage(ErrorMessage(UserStringNop("NON_HOST_SAVE_REQUEST_IGNORED"), false));
         return discard_event();
     }
@@ -1322,8 +1386,7 @@ WaitingForSaveData::WaitingForSaveData(my_context c) :
     {
         PlayerConnectionPtr player = *player_it;
         int player_id = player->PlayerID();
-        bool host = server.m_networking.PlayerIsHost(player_id);
-        player->SendMessage(ServerSaveGameMessage(player_id, host));
+        player->SendMessage(ServerSaveGameDataRequestMessage(player_id, false));
         m_needed_reponses.insert(player_id);
     }
 }
@@ -1389,30 +1452,33 @@ sc::result WaitingForSaveData::react(const ClientSaveData& msg) {
     // if all players have responded, proceed with save and continue game
     m_players_responded.insert(message.SendingPlayer());
     if (m_players_responded == m_needed_reponses) {
-        ServerSaveGameData server_data(server.m_current_turn, server.m_victors);
+        ServerSaveGameData server_data(server.m_current_turn);
 
         // retreive requested save name from Base state, which should have been
         // set in WaitingForTurnEndIdle::react(const SaveGameRequest& msg)
         const std::string& save_filename = context<WaitingForTurnEnd>().m_save_filename;
+        int bytes_written = 0;
 
         // save game...
         try {
-            SaveGame(save_filename,     server_data,    m_player_save_game_data,
-                     GetUniverse(),     Empires(),      GetSpeciesManager(),
-                     GetCombatLogManager(),             server.m_galaxy_setup_data,
-                     !server.m_single_player_game
-                    );
+            bytes_written = SaveGame(save_filename,     server_data,    m_player_save_game_data,
+                                     GetUniverse(),     Empires(),      GetSpeciesManager(),
+                                     GetCombatLogManager(),             server.m_galaxy_setup_data,
+                                     !server.m_single_player_game);
 
         } catch (const std::exception&) {
             DebugLogger() << "Catch std::exception&";
             SendMessageToAllPlayers(ErrorMessage(UserStringNop("UNABLE_TO_WRITE_SAVE_FILE"), false));
         }
-        
+
+        // inform players that save is complete
+        SendMessageToAllPlayers(ServerSaveGameCompleteMessage(save_filename, bytes_written));
+
         DebugLogger() << "Finished ClientSaveData from within if.";
         context<WaitingForTurnEnd>().m_save_filename = "";
         return transit<WaitingForTurnEndIdle>();
     }
-    
+
     DebugLogger() << "Finished ClientSaveData from outside of if.";
     return discard_event();
 }
@@ -1470,4 +1536,3 @@ sc::result ProcessingTurn::react(const CheckTurnEndConditions& c) {
     if (TRACE_EXECUTION) DebugLogger() << "(ServerFSM) ProcessingTurn.CheckTurnEndConditions";
     return discard_event();
 }
-
