@@ -3,7 +3,6 @@
 #include "../util/i18n.h"
 #include "../util/Logger.h"
 #include "../util/OptionsDB.h"
-#include "../util/Directories.h"
 #include "../parse/Parse.h"
 
 const Encyclopedia& GetEncyclopedia() {
@@ -14,7 +13,13 @@ const Encyclopedia& GetEncyclopedia() {
 Encyclopedia::Encyclopedia() :
     articles()
 {
-    parse::encyclopedia_articles(GetResourceDir() / "encyclopedia.txt", *this);
+    try {
+        parse::encyclopedia_articles(*this);
+    } catch (const std::exception& e) {
+        ErrorLogger() << "Failed parsing encyclopedia articles: error: " << e.what();
+        throw e;
+    }
+
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "(Category) Encyclopedia Articles:";
         for (std::map<std::string, std::vector<EncyclopediaArticle> >::const_iterator
