@@ -10,6 +10,7 @@
 #include "../util/ScopedTimer.h"
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
+#include "../Empire/Supply.h"
 
 #include <boost/bind.hpp>
 #include <boost/lexical_cast.hpp>
@@ -18,12 +19,13 @@
 #include <cmath>
 
 namespace {
+    const std::set<int> EMPTY_SET;
     const double MAX_SHIP_SPEED = 500.0;        // max allowed speed of ship movement
     const double FLEET_MOVEMENT_EPSILON = 0.1;  // how close a fleet needs to be to a system to have arrived in the system
 
     bool SystemHasNoVisibleStarlanes(int system_id, int empire_id)
     { return !GetUniverse().SystemHasVisibleStarlanes(system_id, empire_id); }
-    
+
     template<typename IT>
     double PathLength (const IT& begin, const IT& end) {
         if (begin == end) {
@@ -33,22 +35,23 @@ namespace {
             for (std::list<int>::const_iterator it = begin; it != end; ++it) {
                 std::list<int>::const_iterator next_it = it;    ++next_it;
                 //DebugLogger() << "Fleet::SetRoute() new route has system id " << *it;
-                
+
                 if (next_it == end)
                     break;  // current system is the last on the route, so don't need to add any additional distance.
-                    
-                    TemporaryPtr<const System> cur_sys = GetSystem(*it);
+
+                TemporaryPtr<const System> cur_sys = GetSystem(*it);
+
                 if (!cur_sys) {
                     ErrorLogger() << "Fleet::SetRoute() couldn't get system with id " << *it;
                     return distance;
                 }
-                
+
                 TemporaryPtr<const System> next_sys = GetSystem(*next_it);
                 if (!next_sys) {
                     ErrorLogger() << "Fleet::SetRoute() couldn't get system with id " << *next_it;
                     return distance;
                 }
-                
+
                 double dist_x = next_sys->X() - cur_sys->X();
                 double dist_y = next_sys->Y() - cur_sys->Y();
                 distance += std::sqrt(dist_x*dist_x + dist_y*dist_y);
@@ -67,7 +70,7 @@ namespace {
             ship->MoveTo(x, y);
         }
     }
-    
+
     void InsertFleetWithShips(TemporaryPtr<Fleet>& fleet, TemporaryPtr<System>& system){
         system->Insert(fleet);
         std::vector<TemporaryPtr<Ship> > ships = Objects().FindObjects<Ship>(fleet->ShipIDs());
@@ -78,7 +81,6 @@ namespace {
             system->Insert(ship);
         }
     }
-    
 }
 
 // static(s)
@@ -219,9 +221,8 @@ const std::string& Fleet::PublicName(int empire_id) const {
         return UserString("OBJ_FLEET");
 }
 
-const std::list<int>& Fleet::TravelRoute() const {
-    return m_travel_route;
-}
+const std::list<int>& Fleet::TravelRoute() const
+{ return m_travel_route; }
 
 std::list<MovePathNode> Fleet::MovePath(bool flag_blockades /*= false*/) const
 { return MovePath(TravelRoute(), flag_blockades); }
@@ -246,14 +247,9 @@ std::list<MovePathNode> Fleet::MovePath(const std::list<int>& route, bool flag_b
     //DebugLogger() << "Fleet " << this->Name() << " movePath fuel: " << fuel << " sys id: " << this->SystemID();
 
     // determine all systems where fleet(s) can be resupplied if fuel runs out
-    int owner = this->Owner();
-    const Empire* empire = GetEmpire(owner);
-    std::set<int> fleet_supplied_systems;
-    std::set<int> unobstructed_systems;
-    if (empire) {
-        fleet_supplied_systems = empire->FleetSupplyableSystemIDs();
-        unobstructed_systems = empire->SupplyUnobstructedSystems();
-    }
+    const Empire* empire = GetEmpire(this->Owner());
+    const std::set<int>& fleet_supplied_systems = GetSupplyManager().FleetSupplyableSystemIDs(this->Owner());
+    const std::set<int>& unobstructed_systems = empire ? empire->SupplyUnobstructedSystems() : EMPTY_SET;
 
     // determine if, given fuel available and supplyable systems, fleet will ever be able to move
     if (fuel < 1.0f &&
@@ -456,7 +452,7 @@ std::list<MovePathNode> Fleet::MovePath(const std::list<int>& route, bool flag_b
             ++route_it;
             if (route_it != route.end()) {
                 // update next system on route and distance to it from current position
-                next_system = GetEmpireKnownSystem(*route_it, owner);
+                next_system = GetEmpireKnownSystem(*route_it, this->Owner());
                 if (next_system) {
                     //DebugLogger() << "Fleet::MovePath checking unrestriced lane travel";
                     clear_exit = clear_exit || (next_system && next_system->ID() == m_arrival_starlane) ||
@@ -631,7 +627,7 @@ float Fleet::MaxFuel() const {
 }
 
 int Fleet::FinalDestinationID() const {
-    if(m_travel_route.empty()){
+    if (m_travel_route.empty()) {
         return INVALID_OBJECT_ID;
     } else {
         return m_travel_route.back();
@@ -840,14 +836,12 @@ void Fleet::MovementPhase() {
 
     // if owner of fleet can resupply ships at the location of this fleet, then
     // resupply all ships in this fleet
-    if (empire && empire->SystemHasFleetSupply(fleet->SystemID())) {
+    if (GetSupplyManager().SystemHasFleetSupply(fleet->SystemID(), fleet->Owner())) {
         for (std::vector<TemporaryPtr<Ship> >::iterator ship_it = ships.begin();
              ship_it != ships.end(); ++ship_it)
         {
             TemporaryPtr<Ship> ship = *ship_it;
             ship->Resupply();
-            if (Meter* fuel_meter = ship->UniverseObject::GetMeter(METER_FUEL))
-                fuel_meter->BackPropegate();
         }
     }
 
@@ -911,7 +905,7 @@ void Fleet::MovementPhase() {
                     TemporaryPtr<Ship> ship = *ship_it;
                     if (Meter* fuel_meter = ship->UniverseObject::GetMeter(METER_FUEL)) {
                         fuel_meter->AddToCurrent(0.1001f);  // .0001 to prevent rounding down
-                        fuel_meter->BackPropegate();
+                        fuel_meter->BackPropagate();
                     }
                 }
             }
@@ -972,7 +966,7 @@ void Fleet::MovementPhase() {
                 // TODO: Notify the suer with a sitrep?
             }
 
-            bool resupply_here = empire ? empire->SystemHasFleetSupply(system->ID()) : false;
+            bool resupply_here = GetSupplyManager().SystemHasFleetSupply(system->ID(), this->Owner());
 
             // if this system can provide supplies, reset consumed fuel and refuel ships
             if (resupply_here) {
@@ -1044,7 +1038,7 @@ void Fleet::MovementPhase() {
             TemporaryPtr<Ship> ship = *ship_it;
             if (Meter* meter = ship->UniverseObject::GetMeter(METER_FUEL)) {
                 meter->AddToCurrent(-fuel_consumed);
-                meter->BackPropegate();
+                meter->BackPropagate();
             }
         }
     }
@@ -1055,7 +1049,7 @@ void Fleet::ResetTargetMaxUnpairedMeters() {
 
     // give fleets base stealth very high, so that they can (almost?) never be
     // seen by empires that don't own them, unless their ships are seen and
-    // that visibility is propegated to the fleet that contains the ships
+    // that visibility is propagated to the fleet that contains the ships
     if (Meter* stealth = GetMeter(METER_STEALTH)) {
         stealth->ResetCurrent();
         stealth->AddToCurrent(2000.0f);
@@ -1074,7 +1068,7 @@ void Fleet::CalculateRouteTo(int target_system_id) {
     if (m_prev_system != INVALID_OBJECT_ID && SystemID() == m_prev_system) {
         // if we haven't actually left yet, we have to move from whichever system we are at now
 
-        if (!GetSystem(target_system_id)){
+        if (!GetSystem(target_system_id)) {
             // destination system doesn't exist or doesn't exist in known universe, so can't move to it.  leave route empty.
             SetRoute(route);
             return;
@@ -1085,7 +1079,7 @@ void Fleet::CalculateRouteTo(int target_system_id) {
             path = GetUniverse().ShortestPath(m_prev_system, target_system_id, this->Owner());
         } catch (...) {
             DebugLogger() << "Fleet::CalculateRoute couldn't find route to system(s):"
-            << " fleet's previous: " << m_prev_system << " or moving to: " << target_system_id;
+                          << " fleet's previous: " << m_prev_system << " or moving to: " << target_system_id;
         }
         SetRoute(path.first);
         return;
@@ -1098,7 +1092,7 @@ void Fleet::CalculateRouteTo(int target_system_id) {
     // have visibility of the destination system, since this was preventing the
     // human client's attempts to find routes for enemy fleets, for which the
     // player's client doesn't know which systems are visible, and since
-    // visibility of a system on the current turn isn't necessary to plot 
+    // visibility of a system on the current turn isn't necessary to plot
     // route to it now that empire's can remember systems they've seen on
     // previous turns.
     //if (universe.GetObjectVisibilityByEmpire(dest_system_id, this->Owner()) <= VIS_NO_VISIBILITY) {
@@ -1120,13 +1114,12 @@ void Fleet::CalculateRouteTo(int target_system_id) {
 
     // if we're between systems, the shortest route may be through either one
     if (this->CanChangeDirectionEnRoute()) {
-
         std::pair<std::list<int>, double> path1;
         try {
             path1 = GetUniverse().ShortestPath(m_next_system, dest_system_id, this->Owner());
         } catch (...) {
             DebugLogger() << "Fleet::CalculateRoute couldn't find route to system(s):"
-                                   << " fleet's next: " << m_next_system << " or destination: " << dest_system_id;
+                          << " fleet's next: " << m_next_system << " or destination: " << dest_system_id;
         }
         const std::list<int>& sys_list1 = path1.first;
         if (sys_list1.empty()) {
@@ -1147,7 +1140,7 @@ void Fleet::CalculateRouteTo(int target_system_id) {
             path2 = GetUniverse().ShortestPath(m_prev_system, dest_system_id, this->Owner());
         } catch (...) {
             DebugLogger() << "Fleet::CalculateRoute couldn't find route to system(s):"
-                                   << " fleet's previous: " << m_prev_system << " or destination: " << dest_system_id;
+                          << " fleet's previous: " << m_prev_system << " or destination: " << dest_system_id;
         }
         const std::list<int>& sys_list2 = path2.first;
         if (sys_list2.empty()) {
@@ -1177,7 +1170,7 @@ void Fleet::CalculateRouteTo(int target_system_id) {
             path = GetUniverse().ShortestPath(m_next_system, dest_system_id, this->Owner());
         } catch (...) {
             DebugLogger() << "Fleet::CalculateRoute couldn't find route to system(s):"
-                                   << " fleet's next: " << m_next_system << " or destination: " << dest_system_id;
+                          << " fleet's next: " << m_next_system << " or destination: " << dest_system_id;
         }
         SetRoute(path.first);
     }
@@ -1213,7 +1206,7 @@ bool Fleet::BlockadedAtSystem(int start_system_id, int dest_system_id) const {
     }
 
     const Empire* empire = GetEmpire(this->Owner());
-    if (empire) {  
+    if (empire) {
         std::set<int> unobstructed_systems = empire->SupplyUnobstructedSystems();
         if (unobstructed_systems.find(start_system_id) != unobstructed_systems.end())
             return false;
@@ -1224,17 +1217,17 @@ bool Fleet::BlockadedAtSystem(int start_system_id, int dest_system_id) const {
         }
     }
 
-    float lowestShipStealth = 99999.9f; // arbitrary large number. actual stealth of ships should be less than this...
+    float lowest_ship_stealth = 99999.9f; // arbitrary large number. actual stealth of ships should be less than this...
     std::vector<TemporaryPtr<const Ship> > ships = Objects().FindObjects<const Ship>(this->ShipIDs());
     for (std::vector<TemporaryPtr<const Ship> >::iterator it = ships.begin();
          it != ships.end(); ++it)
     {
         TemporaryPtr<const Ship> ship = *it;
-        if (lowestShipStealth > ship->CurrentMeterValue(METER_STEALTH))
-            lowestShipStealth = ship->CurrentMeterValue(METER_STEALTH);
+        if (lowest_ship_stealth > ship->CurrentMeterValue(METER_STEALTH))
+            lowest_ship_stealth = ship->CurrentMeterValue(METER_STEALTH);
     }
 
-    float monsterDetection = 0.0f;
+    float monster_detection = 0.0f;
     std::vector<TemporaryPtr<const Fleet> > fleets =
         Objects().FindObjects<const Fleet>(current_system->FleetIDs());
     for (std::vector<TemporaryPtr<const Fleet> >::iterator fleet_it = fleets.begin();
@@ -1249,13 +1242,13 @@ bool Fleet::BlockadedAtSystem(int start_system_id, int dest_system_id) const {
              ship_it != ships.end(); ++ship_it)
         {
             TemporaryPtr<const Ship> ship = *ship_it;
-            float curDetect = ship->CurrentMeterValue(METER_DETECTION);
-            if (curDetect >= monsterDetection)
-                monsterDetection = curDetect;
+            float cur_detection = ship->CurrentMeterValue(METER_DETECTION);
+            if (cur_detection >= monster_detection)
+                monster_detection = cur_detection;
         }
     }
 
-    bool canBeBlockaded = false;
+    bool can_be_blockaded = false;
     for (std::vector<TemporaryPtr<const Fleet> >::iterator fleet_it = fleets.begin();
          fleet_it != fleets.end(); ++fleet_it)
     {
@@ -1270,18 +1263,18 @@ bool Fleet::BlockadedAtSystem(int start_system_id, int dest_system_id) const {
         }
         bool can_see;
         if (!fleet->Unowned()) {
-            can_see = (GetEmpire(fleet->Owner())->GetMeter("METER_DETECTION_STRENGTH")->Current() >= lowestShipStealth);
+            can_see = (GetEmpire(fleet->Owner())->GetMeter("METER_DETECTION_STRENGTH")->Current() >= lowest_ship_stealth);
         } else {
-            can_see = (monsterDetection >= lowestShipStealth);
+            can_see = (monster_detection >= lowest_ship_stealth);
         }
         bool at_war = Unowned() || fleet->Unowned() ||
                       Empires().GetDiplomaticStatus(this->Owner(), fleet->Owner()) == DIPLO_WAR;
         bool aggressive = (fleet->Aggressive() || fleet->Unowned());
 
         if (aggressive && fleet->HasArmedShips() && at_war && can_see && (unrestricted || not_yet_in_system))
-            canBeBlockaded = true; // don't exit early here, because blockade may yet be thwarted by ownership & presence check above
+            can_be_blockaded = true; // don't exit early here, because blockade may yet be thwarted by ownership & presence check above
     }
-    if (canBeBlockaded) {
+    if (can_be_blockaded) {
         return true;
     }
     return false;
@@ -1399,13 +1392,101 @@ void Fleet::ShortenRouteToEndAtSystem(std::list<int>& travel_route, int last_sys
         travel_route.push_back(*m_travel_route.begin());
 }
 
-std::string Fleet::GenerateFleetName(const std::vector<int>& ship_ids, int new_fleet_id) {
+std::string Fleet::GenerateFleetName() {
     // TODO: Change returned name based on passed ship designs.  eg. return "colony fleet" if
     // ships are colony ships, or "battle fleet" if ships are armed.
-    if (new_fleet_id == INVALID_OBJECT_ID)
+    if (ID() == INVALID_OBJECT_ID)
         return UserString("NEW_FLEET_NAME_NO_NUMBER");
 
-    return boost::io::str(FlexibleFormat(UserString("NEW_FLEET_NAME")) % boost::lexical_cast<std::string>(new_fleet_id));
+    std::vector<TemporaryPtr<const Ship> > ships;
+    for (std::set<int>::iterator it = m_ships.begin(); it != m_ships.end(); ++it) {
+        if (TemporaryPtr<const Ship> ship = GetShip(*it)) {
+            ships.push_back(ship);
+        }
+    }
+
+    std::vector<TemporaryPtr<const Ship> >::iterator it = ships.begin();
+
+    // TODO C++11 replace all of these loops with std::all_of
+    bool all_monster(true);
+    while (it != ships.end()) {
+        if (!(*it)->IsMonster()) {
+            all_monster = false;
+            break;
+        }
+        ++it;
+    }
+
+    if (all_monster)
+        return boost::io::str(FlexibleFormat(UserString("NEW_MONSTER_FLEET_NAME")) % boost::lexical_cast<std::string>(ID()));
+
+    bool all_colony(true);
+    it = ships.begin();
+    while (it != ships.end()) {
+        if (!(*it)->CanColonize()) {
+            all_colony = false;
+            break;
+        }
+        ++it;
+    }
+
+    if (all_colony)
+        return boost::io::str(FlexibleFormat(UserString("NEW_COLONY_FLEET_NAME")) % boost::lexical_cast<std::string>(ID()));
+
+    bool all_non_coms(true);
+    it = ships.begin();
+    while (it != ships.end()) {
+        if ((*it)->IsArmed() || (*it)->CanHaveTroops() || (*it)->CanBombard()) {
+            all_non_coms = false;
+            break;
+        }
+        ++it;
+    }
+
+    if (all_non_coms)
+        return boost::io::str(FlexibleFormat(UserString("NEW_RECON_FLEET_NAME")) % boost::lexical_cast<std::string>(ID()));
+
+    bool all_troop(true);
+    it = ships.begin();
+    while (it != ships.end()) {
+        if (!(*it)->CanHaveTroops()) {
+            all_troop = false;
+            break;
+        }
+        ++it;
+    }
+
+    if (all_troop)
+        return boost::io::str(FlexibleFormat(UserString("NEW_TROOP_FLEET_NAME")) % boost::lexical_cast<std::string>(ID()));
+
+    bool all_bombard(true);
+    it = ships.begin();
+    while (it != ships.end()) {
+        if (!(*it)->CanBombard()) {
+            all_bombard = false;
+            break;
+        }
+        ++it;
+    }
+
+    if (all_bombard)
+        return boost::io::str(FlexibleFormat(UserString("NEW_BOMBARD_FLEET_NAME")) % boost::lexical_cast<std::string>(ID()));
+
+    bool mixed_combat(true);
+    it = ships.begin();
+    while (it != ships.end()) {
+        if (!((*it)->IsArmed() || (*it)->CanHaveTroops() || (*it)->CanBombard())) {
+            mixed_combat = false;
+            break;
+        }
+        ++it;
+    }
+
+    if (mixed_combat)
+        return boost::io::str(FlexibleFormat(UserString("NEW_BATTLE_FLEET_NAME")) % boost::lexical_cast<std::string>(ID()));
+
+
+    return boost::io::str(FlexibleFormat(UserString("NEW_FLEET_NAME")) % boost::lexical_cast<std::string>(ID()));
 }
 
 void Fleet::SetGiveToEmpire(int empire_id) {
