@@ -20,8 +20,8 @@
 #include <boost/assign/list_of.hpp>
 
 namespace {
-    // high tilt is arbitrarily taken to mean 35 degrees or more
-    const float HIGH_TILT_THERSHOLD = 35.0f;
+    // high tilt is arbitrarily taken to mean 45 degrees or more
+    const float HIGH_TILT_THERESHOLD = 45.0f;
 
     float SizeRotationFactor(PlanetSize size) {
         switch (size) {
@@ -85,7 +85,7 @@ Planet::Planet(PlanetType type, PlanetSize size) :
     m_orbital_period(1.0f),
     m_initial_orbital_position(RandZeroToOne() * 2 * 3.14159f),
     m_rotational_period(1.0f),
-    m_axial_tilt(RandZeroToOne() * HIGH_TILT_THERSHOLD),
+    m_axial_tilt(RandZeroToOne() * HIGH_TILT_THERESHOLD),
     m_just_conquered(false),
     m_is_about_to_be_colonized(false),
     m_is_about_to_be_invaded(false),
@@ -467,13 +467,13 @@ float Planet::NextTurnCurrentMeterValue(MeterType type) const {
 
     const Meter* meter = GetMeter(type);
     if (!meter) {
-        throw std::invalid_argument("Planet::NextTurnCurrentMeterValue passed meter type that the Planet does not have, but should.");
+        throw std::invalid_argument("Planet::NextTurnCurrentMeterValue passed meter type that the Planet does not have, but should: " + boost::lexical_cast<std::string>(type));
     }
     float current_meter_value = meter->Current();
 
     const Meter* max_meter = GetMeter(max_meter_type);
     if (!max_meter) {
-        throw std::runtime_error("Planet::NextTurnCurrentMeterValue dealing with invalid meter type");
+        throw std::runtime_error("Planet::NextTurnCurrentMeterValue dealing with invalid meter type: " + boost::lexical_cast<std::string>(type));
     }
     float max_meter_value = max_meter->Current();
 
@@ -485,6 +485,61 @@ float Planet::NextTurnCurrentMeterValue(MeterType type) const {
     return std::min(current_meter_value + 1.0f, max_meter_value);
 }
 
+std::string Planet::CardinalSuffix() const {
+    std::string retval = "";
+    // Planets are grouped into asteroids, and non-asteroids
+    // Asteroids receive a localized prefix
+    if (Type() == PT_ASTEROIDS)
+        retval.append(UserString("NEW_ASTEROIDS_SUFFIX") + " ");
+
+    TemporaryPtr<System> cur_system = GetSystem(SystemID());
+    if (cur_system) {
+        if (cur_system->OrbitOfPlanet(ID()) < 0) {
+            ErrorLogger() << "Planet " << Name() << "(" << ID() << ") "
+                          << "has no current orbit";
+            retval.append(RomanNumber(1));
+            return retval;
+        }
+        int num_planets_lteq = 0;  // number of planets at this orbit or smaller
+        int num_planets_total = 0;
+        bool prior_current_planet = true;
+        const std::vector<int>& sys_orbits = cur_system->PlanetIDsByOrbit();
+        for (std::vector<int>::const_iterator it = sys_orbits.begin();
+             it != sys_orbits.end(); ++it)
+        {
+            if (*it == INVALID_OBJECT_ID)
+                continue;
+
+            PlanetType other_planet_type = GetPlanet(*it)->Type();
+            if (other_planet_type == INVALID_PLANET_TYPE)
+                continue;
+
+            if (Type() != PT_ASTEROIDS) {
+                if (other_planet_type != PT_ASTEROIDS) {
+                    ++num_planets_total;
+                    if (prior_current_planet)
+                        ++num_planets_lteq;
+                }
+            } else {
+                if (other_planet_type == PT_ASTEROIDS) {
+                    ++num_planets_total;
+                    if (prior_current_planet)
+                        ++num_planets_lteq;
+                }
+            }
+
+            if (*it == ID())
+                prior_current_planet =false;
+        }
+        // For asteroids: If no other asteroids in this system, suffix does not receive a number
+        if (Type() != PT_ASTEROIDS || (Type() == PT_ASTEROIDS && num_planets_total > 1))
+            retval.append(RomanNumber(num_planets_lteq));
+    } else {
+        ErrorLogger() << "Planet " << Name() << "(" << ID() << ") not assigned to a system";
+    }
+    return retval;
+}
+
 int Planet::ContainerObjectID() const
 { return this->SystemID(); }
 
@@ -538,6 +593,15 @@ void Planet::SetType(PlanetType type) {
     StateChangedSignal();
 }
 
+void Planet::SetOriginalType(PlanetType type) {
+    if (type <= INVALID_PLANET_TYPE)
+        type = PT_SWAMP;
+    if (NUM_PLANET_TYPES <= type)
+        type = PT_GASGIANT;
+    m_original_type = type;
+    StateChangedSignal();
+}
+
 void Planet::SetSize(PlanetSize size) {
     if (size <= SZ_NOWORLD)
         size = SZ_TINY;
@@ -547,24 +611,12 @@ void Planet::SetSize(PlanetSize size) {
     StateChangedSignal();
 }
 
-void Planet::SetOrbitalPeriod(unsigned int orbit) {
-    assert(orbit < 10);
-    const double THIRD_ORBIT_PERIOD = 4;
-    const double THIRD_ORBIT_RADIUS = OrbitalRadius(2);
-    const double ORBIT_RADIUS = OrbitalRadius(orbit);
-    // Kepler's third law.
-    m_orbital_period =
-        std::sqrt(std::pow(THIRD_ORBIT_PERIOD, 2.0) /
-                  std::pow(THIRD_ORBIT_RADIUS, 3.0) *
-                  std::pow(ORBIT_RADIUS, 3.0));
-}
-
 void Planet::SetRotationalPeriod(Day days)
 { m_rotational_period = days; }
 
 void Planet::SetHighAxialTilt() {
     const double MAX_TILT = 90.0;
-    m_axial_tilt = HIGH_TILT_THERSHOLD + RandZeroToOne() * (MAX_TILT - HIGH_TILT_THERSHOLD);
+    m_axial_tilt = HIGH_TILT_THERESHOLD + RandZeroToOne() * (MAX_TILT - HIGH_TILT_THERESHOLD);
 }
 
 void Planet::AddBuilding(int building_id) {
@@ -657,21 +709,21 @@ void Planet::Conquer(int conquerer) {
     SetOwner(conquerer);
 
     GetMeter(METER_SUPPLY)->SetCurrent(0.0f);
-    GetMeter(METER_SUPPLY)->BackPropegate();
+    GetMeter(METER_SUPPLY)->BackPropagate();
     GetMeter(METER_INDUSTRY)->SetCurrent(0.0f);
-    GetMeter(METER_INDUSTRY)->BackPropegate();
+    GetMeter(METER_INDUSTRY)->BackPropagate();
     GetMeter(METER_RESEARCH)->SetCurrent(0.0f);
-    GetMeter(METER_RESEARCH)->BackPropegate();
+    GetMeter(METER_RESEARCH)->BackPropagate();
     GetMeter(METER_TRADE)->SetCurrent(0.0f);
-    GetMeter(METER_TRADE)->BackPropegate();
+    GetMeter(METER_TRADE)->BackPropagate();
     GetMeter(METER_CONSTRUCTION)->SetCurrent(0.0f);
-    GetMeter(METER_CONSTRUCTION)->BackPropegate();
+    GetMeter(METER_CONSTRUCTION)->BackPropagate();
     GetMeter(METER_DEFENSE)->SetCurrent(0.0f);
-    GetMeter(METER_DEFENSE)->BackPropegate();
+    GetMeter(METER_DEFENSE)->BackPropagate();
     GetMeter(METER_SHIELD)->SetCurrent(0.0f);
-    GetMeter(METER_SHIELD)->BackPropegate();
+    GetMeter(METER_SHIELD)->BackPropagate();
     GetMeter(METER_HAPPINESS)->SetCurrent(0.0f);
-    GetMeter(METER_HAPPINESS)->BackPropegate();
+    GetMeter(METER_HAPPINESS)->BackPropagate();
 }
 
 bool Planet::Colonize(int empire_id, const std::string& species_name, double population) {
@@ -736,7 +788,7 @@ bool Planet::Colonize(int empire_id, const std::string& species_name, double pop
     // set colony population
     GetMeter(METER_POPULATION)->SetCurrent(population);
     GetMeter(METER_TARGET_POPULATION)->SetCurrent(population);
-    BackPropegateMeters();
+    BackPropagateMeters();
 
 
     // set specified empire as owner
@@ -810,12 +862,10 @@ void Planet::PopGrowthProductionResearchPhase() {
 
     PopCenterPopGrowthProductionResearchPhase();
 
-    // check for planets with zero population.  If they have a species set, then
-    // they probably just starved
+    // check for colonies without positive population, and change to outposts
     if (!SpeciesName().empty() && GetMeter(METER_POPULATION)->Current() <= 0.0f) {
         if (Empire* empire = GetEmpire(this->Owner())) {
-            // generate starvation sitrep for empire that owns this depopulated planet
-            empire->AddSitRepEntry(CreatePlanetStarvedToDeathSitRep(this->ID()));
+            empire->AddSitRepEntry(CreatePlanetDepopulatedSitRep(this->ID()));
 
             // record depopulation of planet with species while owned by this empire
             std::map<std::string, int>::iterator species_it = empire->SpeciesPlanetsDepoped().find(SpeciesName());
@@ -876,26 +926,3 @@ void Planet::ClampMeters() {
     UniverseObject::GetMeter(METER_REBEL_TROOPS)->ClampCurrentToRange();
     UniverseObject::GetMeter(METER_DETECTION)->ClampCurrentToRange();
 }
-
-// free functions
-
-float PlanetRadius(PlanetSize size) {
-    float retval = 0.0f;
-    switch (size) {
-    case INVALID_PLANET_SIZE: retval = 0.0f; break;
-    case SZ_NOWORLD:          retval = 0.0f; break;
-    case SZ_TINY:             retval = 2.0f; break;
-    case SZ_SMALL:            retval = 3.5f; break;
-    default:
-    case SZ_MEDIUM:           retval = 5.0f; break;
-    case SZ_LARGE:            retval = 7.0f; break;
-    case SZ_HUGE:             retval = 9.0f; break;
-    case SZ_ASTEROIDS:        retval = 0.0f; break;
-    case SZ_GASGIANT:         retval = 11.0f; break; // this one goes to eleven
-    case NUM_PLANET_SIZES:    retval = 0.0f; break;
-    };
-    return retval;
-}
-
-float AsteroidBeltRadius()
-{ return 12.5f; }
