@@ -12,10 +12,13 @@
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
 
+#include <boost/lexical_cast.hpp>
+
 class Species;
 const Species* GetSpecies(const std::string& name);
 
 Ship::Ship() :
+    UniverseObject(),
     m_design_id(ShipDesign::INVALID_DESIGN_ID),
     m_fleet_id(INVALID_OBJECT_ID),
     m_ordered_scrapped(false),
@@ -23,11 +26,13 @@ Ship::Ship() :
     m_ordered_invade_planet_id(INVALID_OBJECT_ID),
     m_ordered_bombard_planet_id(INVALID_OBJECT_ID),
     m_last_turn_active_in_combat(INVALID_GAME_TURN),
-    m_produced_by_empire_id(ALL_EMPIRES)
+    m_produced_by_empire_id(ALL_EMPIRES),
+    m_arrived_on_turn(INVALID_GAME_TURN)
 {}
 
 Ship::Ship(int empire_id, int design_id, const std::string& species_name,
            int produced_by_empire_id/* = ALL_EMPIRES*/) :
+    UniverseObject(),
     m_design_id(design_id),
     m_fleet_id(INVALID_OBJECT_ID),
     m_ordered_scrapped(false),
@@ -36,7 +41,8 @@ Ship::Ship(int empire_id, int design_id, const std::string& species_name,
     m_ordered_bombard_planet_id(INVALID_OBJECT_ID),
     m_last_turn_active_in_combat(INVALID_GAME_TURN),
     m_species_name(species_name),
-    m_produced_by_empire_id(produced_by_empire_id)
+    m_produced_by_empire_id(produced_by_empire_id),
+    m_arrived_on_turn(CurrentTurn())
 {
     if (!GetShipDesign(design_id))
         throw std::invalid_argument("Attempted to construct a Ship with an invalid design id");
@@ -73,18 +79,22 @@ Ship::Ship(int empire_id, int design_id, const std::string& species_name,
             }
 
             switch (part->Class()) {
-            case PC_SHORT_RANGE:
-            case PC_MISSILES:
-            case PC_FIGHTERS:
-            case PC_POINT_DEFENSE: {
-                m_part_meters[std::make_pair(METER_DAMAGE,  part->Name())];
-                break;
-            }
             case PC_COLONY:
             case PC_TROOPS: {
                 m_part_meters[std::make_pair(METER_CAPACITY, part->Name())];
                 break;
             }
+            case PC_DIRECT_WEAPON:      // capacity is damage, secondary stat is shots per attack
+            case PC_FIGHTER_HANGAR: {   // capacity is how many fighters contained, secondary stat is damage per fighter attack
+                m_part_meters[std::make_pair(METER_SECONDARY_STAT,      part->Name())];
+                m_part_meters[std::make_pair(METER_MAX_SECONDARY_STAT,  part->Name())];
+                // intentionally no break here
+            }
+            case PC_FIGHTER_BAY: {      // capacity is how many fighters launched per combat round
+                m_part_meters[std::make_pair(METER_CAPACITY,            part->Name())];
+                m_part_meters[std::make_pair(METER_MAX_CAPACITY,        part->Name())];
+                break;
+            }
             default:
                 break;
             }
@@ -128,22 +138,23 @@ void Ship::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id)
 
         if (vis >= VIS_PARTIAL_VISIBILITY) {
             if (this->Unowned())
-                this->m_name =              copied_ship->m_name;
+                this->m_name =                  copied_ship->m_name;
 
-            this->m_design_id =             copied_ship->m_design_id;
+            this->m_design_id =                 copied_ship->m_design_id;
             for (PartMeterMap::const_iterator it = copied_ship->m_part_meters.begin();
                  it != copied_ship->m_part_meters.end(); ++it)
-            { this->m_part_meters[it->first]; }
-            this->m_species_name =          copied_ship->m_species_name;
+            { this->m_part_meters = copied_ship->m_part_meters; }
+            this->m_species_name =              copied_ship->m_species_name;
+
+            this->m_last_turn_active_in_combat= copied_ship->m_last_turn_active_in_combat;
+            this->m_produced_by_empire_id =     copied_ship->m_produced_by_empire_id;
+            this->m_arrived_on_turn =           copied_ship->m_arrived_on_turn;
 
             if (vis >= VIS_FULL_VISIBILITY) {
                 this->m_ordered_scrapped =          copied_ship->m_ordered_scrapped;
                 this->m_ordered_colonize_planet_id= copied_ship->m_ordered_colonize_planet_id;
                 this->m_ordered_invade_planet_id  = copied_ship->m_ordered_invade_planet_id;
                 this->m_ordered_bombard_planet_id = copied_ship->m_ordered_bombard_planet_id;
-                this->m_last_turn_active_in_combat= copied_ship->m_last_turn_active_in_combat;
-                this->m_part_meters =               copied_ship->m_part_meters;
-                this->m_produced_by_empire_id =     copied_ship->m_produced_by_empire_id;
             }
         }
     }
@@ -214,10 +225,10 @@ std::string Ship::Dump() const {
        << " fleet id: " << m_fleet_id
        << " species name: " << m_species_name
        << " produced by empire id: " << m_produced_by_empire_id
-       << " fighters: ";
-    //typedef std::map<std::pair<MeterType, std::string>, Meter> PartMeters;
-    os << " part meters: ";
-    for (PartMeterMap::const_iterator it = m_part_meters.begin(); it != m_part_meters.end();) {
+       << " arrived on turn: " << m_arrived_on_turn;
+    if (!m_part_meters.empty()) {
+        os << " part meters: ";
+        for (PartMeterMap::const_iterator it = m_part_meters.begin(); it != m_part_meters.end();) {
         const std::string part_name = it->first.second;
         MeterType meter_type = it->first.first;
         const Meter& meter = it->second;
@@ -226,6 +237,7 @@ std::string Ship::Dump() const {
            << UserString(EnumToString(meter_type))
            << ": " << meter.Current() << "  ";
     }
+    }
     return os.str();
 }
 
@@ -325,6 +337,11 @@ float Ship::TroopCapacity() const {
     return retval;
 }
 
+bool Ship::CanHaveTroops() const {
+    const ShipDesign* design = Design();
+    return design ? design->HasTroops() : false;
+}
+
 const std::string& Ship::PublicName(int empire_id) const {
     // Disclose real ship name only to fleet owners. Rationale: a player who
     // doesn't know the design for a particular ship can easily guess it if the
@@ -351,7 +368,7 @@ TemporaryPtr<UniverseObject> Ship::Accept(const UniverseObjectVisitor& visitor)
 float Ship::NextTurnCurrentMeterValue(MeterType type) const {
     const Meter* meter = UniverseObject::GetMeter(type);
     if (!meter)
-        throw std::invalid_argument("Ship::NextTurnCurrentMeterValue passed meter type that the Ship does not have.");
+        throw std::invalid_argument("Ship::NextTurnCurrentMeterValue passed meter type that the Ship does not have: " + boost::lexical_cast<std::string>(type));
     float current_meter_value = meter->Current();
 
     //if (type == METER_FUEL) {
@@ -387,7 +404,7 @@ float Ship::NextTurnCurrentMeterValue(MeterType type) const {
 
     const Meter* target_meter = UniverseObject::GetMeter(target_meter_type);
     if (!target_meter)
-        throw std::runtime_error("Ship::NextTurnCurrentMeterValue dealing with invalid meter type");
+        throw std::runtime_error("Ship::NextTurnCurrentMeterValue dealing with invalid meter type: " + boost::lexical_cast<std::string>(type));
     float target_meter_value = target_meter->Current();
 
     // meter growth or decay towards target is one per turn.
@@ -422,43 +439,98 @@ float Ship::InitialPartMeterValue(MeterType type, const std::string& part_name)
     return 0.0f;
 }
 
-float Ship::TotalWeaponsDamage(float shield_DR /* = 0.0 */) const {
+float Ship::TotalWeaponsDamage(float shield_DR, bool include_fighters) const {
     // sum up all individual weapons' attack strengths
-    float total_attack = 0.0;
-    std::vector<float> all_weapons_damage = AllWeaponsDamage(shield_DR);
+    float total_attack = 0.0f;
+    std::vector<float> all_weapons_damage = AllWeaponsDamage(shield_DR, include_fighters);
     for (std::vector<float>::iterator it = all_weapons_damage.begin(); it != all_weapons_damage.end(); ++it)
         total_attack += *it;
     return total_attack;
 }
 
-std::vector<float> Ship::AllWeaponsDamage(float shield_DR /* = 0.0 */) const {
+namespace {
+    std::vector<float> WeaponDamageImpl(const Ship* ship, const ShipDesign* design,
+                                        float DR, bool max, bool include_fighters)
+    {
+        std::vector<float> retval;
+
+        if (!ship || !design)
+            return retval;
+        const std::vector<std::string>& parts = design->Parts();
+        if (parts.empty())
+            return retval;
+
+        MeterType METER = max ? METER_MAX_CAPACITY : METER_CAPACITY;
+        MeterType SECONDARY_METER = max ? METER_MAX_SECONDARY_STAT : METER_SECONDARY_STAT;
+
+        float fighter_damage = 0.0f;
+        int fighter_launch_capacity = 0;
+        int available_fighters = 0;
+
+        // for each weapon part, get its damage meter value
+        for (std::vector<std::string>::const_iterator part_it = parts.begin();
+             part_it != parts.end(); ++part_it)
+        {
+            const std::string& part_name = *part_it;
+            const PartType* part = GetPartType(part_name);
+            if (!part)
+                continue;
+            ShipPartClass part_class = part->Class();
+
+            // get the attack power for each weapon part
+            if (part_class == PC_DIRECT_WEAPON) {
+                float part_attack = ship->CurrentPartMeterValue(METER, part_name);
+                float part_shots = ship->CurrentPartMeterValue(SECONDARY_METER, part_name);
+                if (part_attack > DR)
+                    retval.push_back((part_attack - DR)*part_shots);
+
+            } else if (part_class == PC_FIGHTER_BAY && include_fighters) {
+                // launch capacity determined by capacity of bay
+                fighter_launch_capacity += static_cast<int>(ship->CurrentPartMeterValue(METER, part_name));
+
+            } else if (part_class == PC_FIGHTER_HANGAR && include_fighters) {
+                // attack strength of a ship's fighters determined by the hangar...
+                fighter_damage = ship->CurrentPartMeterValue(SECONDARY_METER, part_name) - DR;  // assuming all hangars have the same damage...
+                available_fighters += std::max(0, static_cast<int>(ship->CurrentPartMeterValue(METER, part_name)));
+            }
+        }
+
+        if (!include_fighters || fighter_damage <= 0.0f || available_fighters <= 0 || fighter_launch_capacity <= 0)
+            return retval;
+
+        // assuming 3 combat "bouts"
+        int fighter_shots = std::min(available_fighters, fighter_launch_capacity);  // how many fighters launched in bout 1
+        available_fighters -= fighter_shots;
+        fighter_shots *= 2;                                                         // first-bout-launched fighters attack in bouts 2 and 3
+        fighter_shots += std::min(available_fighters, fighter_launch_capacity);     // second-bout-launched fighters attack only in bout 3
+
+        // how much damage does a fighter shot do?
+        fighter_damage = std::max(0.0f, fighter_damage);
+
+        retval.push_back(fighter_damage * fighter_shots / 3.0f);    // divide by 3 because fighter calculation is for a full combat, but direct firefor one attack
+
+        return retval;
+    }
+}
+
+std::vector<float> Ship::AllWeaponsDamage(float shield_DR, bool include_fighters) const {
     std::vector<float> retval;
 
     const ShipDesign* design = GetShipDesign(m_design_id);
     if (!design)
         return retval;
-    const std::vector<std::string>& parts = design->Parts();
 
-    // for each weapon part, get its damage meter value
-    for (std::vector<std::string>::const_iterator part_it = parts.begin();
-         part_it != parts.end(); ++part_it)
-    {
-        const std::string& part_name = *part_it;
-        const PartType* part = GetPartType(part_name);
-        if (!part)
-            continue;
-        ShipPartClass part_class = part->Class();
+    return WeaponDamageImpl(this, design, shield_DR, false, include_fighters);
+}
 
-        // get the attack power for each weapon part
-        float part_attack = 0.0;
+std::vector<float> Ship::AllWeaponsMaxDamage(float shield_DR , bool include_fighters) const {
+    std::vector<float> retval;
 
-        if (part_class == PC_SHORT_RANGE || part_class == PC_POINT_DEFENSE || part_class == PC_MISSILES || part_class == PC_FIGHTERS)
-            part_attack = this->CurrentPartMeterValue(METER_DAMAGE, part_name);
+    const ShipDesign* design = GetShipDesign(m_design_id);
+    if (!design)
+        return retval;
 
-        if (part_attack > shield_DR)
-            retval.push_back(part_attack-shield_DR);
-    }
-    return retval;
+    return WeaponDamageImpl(this, design, shield_DR, true, include_fighters);
 }
 
 void Ship::SetFleetID(int fleet_id) {
@@ -468,15 +540,49 @@ void Ship::SetFleetID(int fleet_id) {
     }
 }
 
+void Ship::SetArrivedOnTurn(int turn) {
+    if (m_arrived_on_turn != turn) {
+        m_arrived_on_turn = turn;
+        StateChangedSignal();
+    }
+}
+
+void Ship::BackPropagateMeters() {
+    UniverseObject::BackPropagateMeters();
+
+    // ship part meter back propagation, since base class function doesn't do this...
+    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it)
+        it->second.BackPropagate();
+}
+
 void Ship::Resupply() {
     Meter* fuel_meter = UniverseObject::GetMeter(METER_FUEL);
     const Meter* max_fuel_meter = UniverseObject::GetMeter(METER_MAX_FUEL);
     if (!fuel_meter || !max_fuel_meter) {
         ErrorLogger() << "Ship::Resupply couldn't get fuel meters!";
-        return;
+    } else {
+        fuel_meter->SetCurrent(max_fuel_meter->Current());
+        fuel_meter->BackPropagate();
     }
 
-    fuel_meter->SetCurrent(max_fuel_meter->Current());
+    // set all part capacities equal to any associated max capacity
+    // this "upgrades" any direct-fire weapon parts to their latest-allowed
+    // strengths, and replaces any lost fighters
+    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+        PartMeterMap::iterator max_it = m_part_meters.end();
+        if (it->first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
+        } else if (it->first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        }
+        if (max_it == m_part_meters.end())
+            continue;
+
+        const Meter& max_meter = max_it->second;
+
+        it->second.SetCurrent(max_meter.Current());
+        it->second.BackPropagate();
+    }
 }
 
 void Ship::SetSpecies(const std::string& species_name) {
@@ -530,10 +636,57 @@ void Ship::ResetTargetMaxUnpairedMeters() {
 
     UniverseObject::GetMeter(METER_DETECTION)->ResetCurrent();
     UniverseObject::GetMeter(METER_SPEED)->ResetCurrent();
-    UniverseObject::GetMeter(METER_SPEED)->ResetCurrent();
+    //UniverseObject::GetMeter(METER_STEALTH)->ResetCurrent(); redundant with base class function
 
-    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it)
-    { it->second.ResetCurrent(); }
+    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+        PartMeterMap::iterator max_it = m_part_meters.end();
+        if (it->first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
+        } else if (it->first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        }
+        if (max_it != m_part_meters.end())
+            continue;
+        it->second.ResetCurrent();
+    }
+}
+
+void Ship::ResetPairedActiveMeters() {
+    UniverseObject::ResetPairedActiveMeters();
+
+    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+        PartMeterMap::iterator max_it = m_part_meters.end();
+        if (it->first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
+        } else if (it->first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        }
+        if (max_it != m_part_meters.end())
+            it->second.SetCurrent(it->second.Initial());
+    }
+}
+
+void Ship::SetShipMetersToMax() {
+    UniverseObject::GetMeter(METER_MAX_FUEL)->SetCurrent(Meter::LARGE_VALUE);
+    UniverseObject::GetMeter(METER_MAX_SHIELD)->SetCurrent(Meter::LARGE_VALUE);
+    UniverseObject::GetMeter(METER_MAX_STRUCTURE)->SetCurrent(Meter::LARGE_VALUE);
+    UniverseObject::GetMeter(METER_FUEL)->SetCurrent(Meter::LARGE_VALUE);
+    UniverseObject::GetMeter(METER_SHIELD)->SetCurrent(Meter::LARGE_VALUE);
+    UniverseObject::GetMeter(METER_STRUCTURE)->SetCurrent(Meter::LARGE_VALUE);
+
+    // some part capacity meters may have an associated max capacity...
+    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+        PartMeterMap::iterator max_it = m_part_meters.end();
+        if (it->first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
+        } else if (it->first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        }
+        if (max_it == m_part_meters.end())
+            continue;
+        max_it->second.SetCurrent(Meter::LARGE_VALUE);
+        it->second.SetCurrent(Meter::LARGE_VALUE);
+    }
 }
 
 void Ship::PopGrowthProductionResearchPhase() {
@@ -544,6 +697,8 @@ void Ship::PopGrowthProductionResearchPhase() {
     UniverseObject::GetMeter(METER_RESEARCH)->SetCurrent(Ship::NextTurnCurrentMeterValue(METER_RESEARCH));
     UniverseObject::GetMeter(METER_TRADE)->SetCurrent(Ship::NextTurnCurrentMeterValue(METER_TRADE));
 
+    // part capacity meters set to max only by being in supply
+
     StateChangedSignal();
 }
 
@@ -565,10 +720,32 @@ void Ship::ClampMeters() {
 
     UniverseObject::GetMeter(METER_DETECTION)->ClampCurrentToRange();
     UniverseObject::GetMeter(METER_SPEED)->ClampCurrentToRange();
-    UniverseObject::GetMeter(METER_SPEED)->ClampCurrentToRange();
 
-    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it)
+    // clamp most part meters to basic range limits
+    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+        if (it->first.first == METER_CAPACITY || it->first.first == METER_SECONDARY_STAT)
+            continue;
         it->second.ClampCurrentToRange();
+    }
+
+    // special case extra clamping for paired CAPACTY meters dependent on their associated MAX_CAPACITY meter...
+    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+        PartMeterMap::iterator max_it = m_part_meters.end();
+        if (it->first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
+        } else if (it->first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        }
+
+        if (max_it == m_part_meters.end()) {
+            // no found max meter, revert to normal clamping for this meter
+            it->second.ClampCurrentToRange();
+            continue;
+        }
+
+        const Meter& max_meter = max_it->second;
+        it->second.ClampCurrentToRange(Meter::DEFAULT_VALUE, max_meter.Current());
+    }
 }
 
 ////////////////////
