@@ -1,7 +1,6 @@
 #include "ShipDesign.h"
 
 #include "../util/OptionsDB.h"
-#include "../util/Directories.h"
 #include "../util/Logger.h"
 #include "../util/AppInterface.h"
 #include "../parse/Parse.h"
@@ -117,7 +116,12 @@ PartTypeManager::PartTypeManager() {
 
     s_instance = this;
 
-    parse::ship_parts(GetResourceDir() / "ship_parts.txt", m_parts);
+    try {
+        parse::ship_parts(m_parts);
+    } catch (const std::exception& e) {
+        ErrorLogger() << "Failed parsing ship_parts.txt: error: " << e.what();
+        throw;
+    }
 
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Part Types:";
@@ -182,18 +186,23 @@ namespace {
 // PartType
 ////////////////////////////////////////////////
 void PartType::Init(const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects) {
-    if (m_capacity != 0) {
+    if ((m_capacity != 0 || m_secondary_stat != 0) && m_add_standard_capacity_effect) {
         switch (m_class) {
-        case PC_SHORT_RANGE:
-        case PC_POINT_DEFENSE:
-        case PC_MISSILES:
-        case PC_FIGHTERS:
-            m_effects.push_back(IncreaseMeter(METER_DAMAGE,         m_name, m_capacity, false));
-            break;
         case PC_COLONY:
         case PC_TROOPS:
             m_effects.push_back(IncreaseMeter(METER_CAPACITY,       m_name, m_capacity, false));
             break;
+        case PC_FIGHTER_HANGAR: {   // capacity indicates how many fighters are stored in this type of part (combined for all copies of the part)
+            m_effects.push_back(IncreaseMeter(METER_MAX_CAPACITY,       m_name, m_capacity, true));         // stacking capacities allowed for this part, so each part contributes to the total capacity
+            m_effects.push_back(IncreaseMeter(METER_MAX_SECONDARY_STAT, m_name, m_secondary_stat, false));  // stacking damage not allowed, as damage per shot should be the same regardless of number of shots
+            break;
+        }
+        case PC_FIGHTER_BAY:        // capacity indicates how many fighters each instance of the part can launch per combat bout...
+        case PC_DIRECT_WEAPON: {    // capacity indicates weapon damage per shot
+            m_effects.push_back(IncreaseMeter(METER_MAX_CAPACITY,       m_name, m_capacity, false));
+            m_effects.push_back(IncreaseMeter(METER_MAX_SECONDARY_STAT, m_name, m_secondary_stat, false));
+            break;
+        }
         case PC_SHIELD:
             m_effects.push_back(IncreaseMeter(METER_MAX_SHIELD,     m_capacity));
             break;
@@ -240,27 +249,38 @@ PartType::~PartType()
 float PartType::Capacity() const
 { return m_capacity; }
 
-const std::string PartType::CapacityDescription() const {
+float PartType::SecondaryStat() const
+{ return m_secondary_stat; }
+
+std::string PartType::CapacityDescription() const {
     std::string desc_string;
-    float d = Capacity();
+    float main_stat = Capacity();
+    float sdry_stat = SecondaryStat();
 
     switch (m_class) {
     case PC_FUEL:
     case PC_TROOPS:
     case PC_COLONY:
-        desc_string += UserString("PART_DESC_CAPACITY");
+    case PC_FIGHTER_BAY:
+        desc_string += str(FlexibleFormat(UserString("PART_DESC_CAPACITY")) % main_stat);
+        break;
+    case PC_DIRECT_WEAPON:
+        desc_string += str(FlexibleFormat(UserString("PART_DESC_DIRECT_FIRE_STATS")) % main_stat % sdry_stat);
+        break;
+    case PC_FIGHTER_HANGAR:
+        desc_string += str(FlexibleFormat(UserString("PART_DESC_HANGAR_STATS")) % main_stat % sdry_stat);
         break;
     case PC_SHIELD:
-        desc_string = UserString("PART_DESC_SHIELD_STRENGTH");
+        desc_string = str(FlexibleFormat(UserString("PART_DESC_SHIELD_STRENGTH")) % main_stat);
         break;
     case PC_DETECTION:
-        desc_string = UserString("PART_DESC_DETECTION");
+        desc_string = str(FlexibleFormat(UserString("PART_DESC_DETECTION")) % main_stat);
         break;
     default:
-        desc_string = UserString("PART_DESC_STRENGTH");
+        desc_string = str(FlexibleFormat(UserString("PART_DESC_STRENGTH")) % main_stat);
         break;
     }
-    return str(FlexibleFormat(desc_string) % d);
+    return desc_string;
 }
 
 bool PartType::CanMountInSlotType(ShipSlotType slot_type) const {
@@ -426,7 +446,12 @@ HullTypeManager::HullTypeManager() {
 
     s_instance = this;
 
-    parse::ship_hulls(GetResourceDir() / "ship_hulls.txt", m_hulls);
+    try {
+        parse::ship_hulls(m_hulls);
+    } catch (const std::exception& e) {
+        ErrorLogger() << "Failed parsing ship_hulls.txt: error: " << e.what();
+        throw;
+    }
 
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Hull Types:";
@@ -470,7 +495,9 @@ const int       ShipDesign::MAX_ID            = 2000000000;
 ShipDesign::ShipDesign() :
     m_id(INVALID_OBJECT_ID),
     m_name(),
+    m_description(),
     m_designed_on_turn(UniverseObject::INVALID_OBJECT_AGE),
+    m_designed_by_empire(ALL_EMPIRES),
     m_hull(),
     m_parts(),
     m_is_monster(false),
@@ -490,7 +517,8 @@ ShipDesign::ShipDesign() :
     m_research_generation(0.0),
     m_industry_generation(0.0),
     m_trade_generation(0.0),
-    m_is_production_location(false)
+    m_is_production_location(false),
+    m_producible(false)
 {}
 
 ShipDesign::ShipDesign(const std::string& name, const std::string& description,
@@ -522,7 +550,8 @@ ShipDesign::ShipDesign(const std::string& name, const std::string& description,
     m_research_generation(0.0),
     m_industry_generation(0.0),
     m_trade_generation(0.0),
-    m_is_production_location(false)
+    m_is_production_location(false),
+    m_producible(false)
 {
     // expand parts list to have empty values if fewer parts are given than hull has slots
     if (const HullType* hull_type = GetHullType(m_hull)) {
@@ -625,7 +654,6 @@ bool ShipDesign::CanColonize() const {
     return false;
 }
 
-//// TEMPORARY
 float ShipDesign::Defense() const {
     // accumulate defense from defensive parts in design.
     float total_defense = 0.0f;
@@ -645,34 +673,59 @@ float ShipDesign::Attack() const {
 }
 
 float ShipDesign::AdjustedAttack(float shield) const {
-    // total damage against a target with the given shield.
-    // accumulate attack stat from all weapon parts in design
-    const PartTypeManager& manager = GetPartTypeManager();
+    // total damage against a target with the given shield (damage reduction)
+    // assuming full load of fighters that are not destroyed during the battle
+    int available_fighters = 0;
+    int fighter_launch_capacity = 0;
+    float fighter_damage = 0.0f;
+    float direct_attack = 0.0f;
 
-    float total_attack = 0.0f;
-    std::vector<std::string> all_parts = Parts();
-    for (std::vector<std::string>::const_iterator it = all_parts.begin(); it != all_parts.end(); ++it) {
-        const PartType* part = manager.GetPartType(*it);
-        if (part && (part->Class() == PC_SHORT_RANGE || part->Class() == PC_POINT_DEFENSE ||
-                     part->Class() == PC_MISSILES    || part->Class() == PC_FIGHTERS))
-        { total_attack += std::max(0.0f, static_cast<float>(part->Capacity()) - shield); }
+    for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it) {
+        const PartType* part = GetPartType(*it);
+        if (!part)
+            continue;
+        ShipPartClass part_class = part->Class();
+
+        // direct weapon and fighter-related parts all handled differently...
+        if (part_class == PC_DIRECT_WEAPON) {
+            float part_attack = part->Capacity();
+            if (part_attack > shield)
+                direct_attack += (part_attack - shield)*part->SecondaryStat();  // here, secondary stat is number of shots per round
+        } else if (part_class == PC_FIGHTER_HANGAR) {
+            available_fighters += part->Capacity();
+        } else if (part_class == PC_FIGHTER_BAY) {
+            fighter_launch_capacity += part->Capacity();
+            fighter_damage = part->SecondaryStat();                             // here, secondary stat is fighter damage per shot
+        }
     }
-    return total_attack;
+
+    // assuming 3 combat "bouts"
+    int fighter_shots = std::min(available_fighters, fighter_launch_capacity);  // how many fighters launched in bout 1
+    available_fighters -= fighter_shots;
+    fighter_shots *= 2;                                                         // first-bout-launched fighters attack in bouts 2 and 3
+    fighter_shots += std::min(available_fighters, fighter_launch_capacity);     // second-bout-launched fighters attack only in bout 3
+
+    // how much damage does a fighter shot do?
+    fighter_damage = std::max(0.0f, fighter_damage - shield);
+
+    return direct_attack + fighter_shots*fighter_damage/3.0f;   // divide by 3 because fighter calculation is for a full combat, but direct firefor one attack
 }
-//// END TEMPORARY
 
 std::vector<std::string> ShipDesign::Parts(ShipSlotType slot_type) const {
     std::vector<std::string> retval;
 
     const HullType* hull = GetHull();
-    assert(hull);
+    if (!hull) {
+        ErrorLogger() << "Design hull not found: " << m_hull;
+        return retval;
+    }
     const std::vector<HullType::Slot>& slots = hull->Slots();
 
-    unsigned int size = m_parts.size();
-    assert(size == hull->NumSlots());
+    if (m_parts.empty())
+        return retval;
 
-    // add to output vector each part that is in a slot of the indicated ShipSlotType 
-    for (unsigned int i = 0; i < size; ++i)
+    // add to output vector each part that is in a slot of the indicated ShipSlotType
+    for (unsigned int i = 0; i < m_parts.size(); ++i)
         if (slots[i].type == slot_type)
             retval.push_back(m_parts[i]);
 
@@ -688,8 +741,7 @@ std::vector<std::string> ShipDesign::Weapons() const {
         if (!part)
             continue;
         ShipPartClass part_class = part->Class();
-        if (part_class == PC_SHORT_RANGE    || part_class == PC_MISSILES ||
-            part_class == PC_FIGHTERS       || part_class == PC_POINT_DEFENSE)
+        if (part_class == PC_DIRECT_WEAPON || part_class == PC_FIGHTER_BAY)
         { retval.push_back(part_name); }
     }
     return retval;
@@ -778,14 +830,38 @@ bool ShipDesign::ValidDesign(const std::string& hull, const std::vector<std::str
 
     const std::vector<HullType::Slot>& slots = hull_type->Slots();
 
+    // check hull exclusions against all parts...
+    const std::set<std::string>& hull_exclusions = hull_type->Exclusions();
+    for (std::vector<std::string>::const_iterator part_it = parts.begin(); part_it != parts.end(); ++part_it) {
+        if (part_it->empty())
+            continue;
+        if (hull_exclusions.find(*part_it) != hull_exclusions.end())
+            return false;
+    }
+
+    // check part exclusions against other parts and hull
+    std::set<std::string> already_seen_component_names;
+    already_seen_component_names.insert(hull);
+    for (std::vector<std::string>::const_iterator part_it = parts.begin(); part_it != parts.end(); ++part_it) {
+        const PartType* part_type = GetPartType(*part_it);
+        if (!part_type)
+            continue;
+        const std::set<std::string>& part_exclusions = part_type->Exclusions();
+        for (std::set<std::string>::const_iterator ex_it = part_exclusions.begin(); ex_it != part_exclusions.end(); ++ex_it) {
+            if (already_seen_component_names.find(*ex_it) != already_seen_component_names.end())
+                return false;
+        }
+        already_seen_component_names.insert(*part_it);
+    }
+
+
     // ensure all passed parts can be mounted in slots of type they were passed for
-    const PartTypeManager& part_manager = GetPartTypeManager();
     for (unsigned int i = 0; i < size; ++i) {
         const std::string& part_name = parts[i];
         if (part_name.empty())
             continue;   // if part slot is empty, ignore - doesn't invalidate design
 
-        const PartType* part = part_manager.GetPartType(part_name);
+        const PartType* part = GetPartType(part_name);
         if (!part) {
             DebugLogger() << "ShipDesign::ValidDesign: part not found: " << part_name;
             return false;
@@ -833,10 +909,8 @@ void ShipDesign::BuildStatCaches() {
             m_producible = false;
 
         switch (part->Class()) {
-        case PC_SHORT_RANGE:
-        case PC_MISSILES:
-        case PC_FIGHTERS:
-        case PC_POINT_DEFENSE: {
+        case PC_DIRECT_WEAPON:
+        case PC_FIGHTER_BAY: {
             m_is_armed = true;
             break;
         }
@@ -948,9 +1022,19 @@ PredefinedShipDesignManager::PredefinedShipDesignManager() {
 
     DebugLogger() << "Initializing PredefinedShipDesignManager";
 
-    parse::ship_designs(GetResourceDir() / "premade_ship_designs.txt", m_ship_designs);
+    try {
+        parse::ship_designs(m_ship_designs);
+    } catch (const std::exception& e) {
+        ErrorLogger() << "Failed parsing ship designs: error: " << e.what();
+        throw;
+    }
 
-    parse::ship_designs(GetResourceDir() / "space_monsters.txt", m_monster_designs);
+    try {
+        parse::monster_designs(m_monster_designs);
+    } catch (const std::exception& e) {
+        ErrorLogger() << "Failed parsing monster designs: error: " << e.what();
+        throw;
+    }
 
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Predefined Ship Designs:";
@@ -1109,6 +1193,7 @@ int PredefinedShipDesignManager::GetDesignID(const std::string& name) const {
     return it->second;
 }
 
+
 ///////////////////////////////////////////////////////////
 // Free Functions                                        //
 ///////////////////////////////////////////////////////////
