@@ -1,4 +1,3 @@
-// -*- C++ -*-
 #ifndef _ShipDesign_h_
 #define _ShipDesign_h_
 
@@ -29,32 +28,38 @@ class Empire;
 
 /** Common parameters for PartType and HullType constructors.  Used as temporary
   * storage for parsing to reduce number of sub-items parsed per item. */
-struct PartHullCommonParams {
-    PartHullCommonParams() :
+struct CommonParams {
+    CommonParams() :
         production_cost(0),
         production_time(0),
         producible(false),
         tags(),
+        production_meter_consumption(),
+        production_special_consumption(),
         location(0),
-        effects(),
-        icon()
+        enqueue_location(0),
+        effects()
     {}
-    PartHullCommonParams(ValueRef::ValueRefBase<double>* production_cost_,
-                         ValueRef::ValueRefBase<int>* production_time_,
-                         bool producible_,
-                         const std::set<std::string>& tags_,
-                         Condition::ConditionBase* location_,
-                         const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects_,
-                         const std::string& icon_) :
+    CommonParams(ValueRef::ValueRefBase<double>* production_cost_,
+                 ValueRef::ValueRefBase<int>* production_time_,
+                 bool producible_,
+                 const std::set<std::string>& tags_,
+                 Condition::ConditionBase* location_,
+                 const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects_,
+                 std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> > production_meter_consumption_,
+                 std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> > production_special_consumption_,
+                 Condition::ConditionBase* enqueue_location_) :
         production_cost(production_cost_),
         production_time(production_time_),
         producible(producible_),
         tags(),
+        production_meter_consumption(production_meter_consumption_),
+        production_special_consumption(production_special_consumption_),
         location(location_),
-        effects(effects_),
-        icon(icon_)
+        enqueue_location(enqueue_location_),
+        effects(effects_)
     {
-        for ( std::set< std::string >::iterator tag_it = tags_.begin(); tag_it != tags_.end(); tag_it++)
+        for (std::set< std::string >::iterator tag_it = tags_.begin(); tag_it != tags_.end(); ++tag_it)
             tags.insert(boost::to_upper_copy<std::string>(*tag_it));
     }
 
@@ -62,9 +67,30 @@ struct PartHullCommonParams {
     ValueRef::ValueRefBase<int>*                            production_time;
     bool                                                    producible;
     std::set<std::string>                                   tags;
+    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+                                                            production_meter_consumption;
+    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+                                                            production_special_consumption;
     Condition::ConditionBase*                               location;
+    Condition::ConditionBase*                               enqueue_location;
     std::vector<boost::shared_ptr<Effect::EffectsGroup> >   effects;
-    std::string                                             icon;
+};
+
+struct MoreCommonParams {
+    MoreCommonParams() :
+        name(),
+        description(),
+        exclusions()
+    {}
+    MoreCommonParams(const std::string& name_, const std::string& description_,
+                     const std::set<std::string>& exclusions_) :
+        name(name_),
+        description(description_),
+        exclusions(exclusions_)
+    {}
+    std::string             name;
+    std::string             description;
+    std::set<std::string>   exclusions;
 };
 
 /** A type of ship part */
@@ -76,33 +102,48 @@ public:
         m_description("indescribable"),
         m_class(INVALID_SHIP_PART_CLASS),
         m_capacity(0.0f),
+        m_secondary_stat(1.0f),
         m_production_cost(0),
         m_production_time(0),
+        m_producible(false),
         m_mountable_slot_types(),
         m_tags(),
+        m_production_meter_consumption(),
+        m_production_special_consumption(),
         m_location(0),
+        m_exclusions(),
         m_effects(),
-        m_icon()
+        m_icon(),
+        m_add_standard_capacity_effect(false)
     {}
-    PartType(const std::string& name, const std::string& description,
-             ShipPartClass part_class, double capacity,
-             const PartHullCommonParams& common_params,
-             std::vector<ShipSlotType> mountable_slot_types) :
-        m_name(name),
-        m_description(description),
+
+    PartType(ShipPartClass part_class, double capacity, double stat2,
+             const CommonParams& common_params,
+             const MoreCommonParams& more_common_params,
+             std::vector<ShipSlotType> mountable_slot_types,
+             const std::string& icon,
+             bool add_standard_capacity_effect = true) :
+        m_name(more_common_params.name),
+        m_description(more_common_params.description),
         m_class(part_class),
         m_capacity(capacity),
+        m_secondary_stat(stat2),
         m_production_cost(common_params.production_cost),
         m_production_time(common_params.production_time),
         m_producible(common_params.producible),
         m_mountable_slot_types(mountable_slot_types),
         m_tags(),
+        m_production_meter_consumption(common_params.production_meter_consumption),
+        m_production_special_consumption(common_params.production_special_consumption),
         m_location(common_params.location),
+        m_exclusions(more_common_params.exclusions),
         m_effects(),
-        m_icon(common_params.icon)
+        m_icon(icon),
+        m_add_standard_capacity_effect(add_standard_capacity_effect)
     {
+        //std::cout << "part type: " << m_name << " producible: " << m_producible << std::endl;
         Init(common_params.effects);
-        for ( std::set< std::string >::iterator tag_it = common_params.tags.begin(); tag_it != common_params.tags.end(); tag_it++)
+        for (std::set<std::string>::iterator tag_it = common_params.tags.begin(); tag_it != common_params.tags.end(); ++tag_it)
             m_tags.insert(boost::to_upper_copy<std::string>(*tag_it));
     }
 
@@ -114,7 +155,9 @@ public:
     const std::string&      Description() const     { return m_description; }       ///< returns description string, generally a UserString key.
     ShipPartClass           Class() const           { return m_class; }             ///< returns that class of part that this is.
     float                   Capacity() const;
-    const std::string       CapacityDescription() const;                            ///< returns a translated description of the part capacity, with numeric value
+    std::string             CapacityDescription() const;                            ///< returns a translated description of the part capacity, with numeric value
+    float                   SecondaryStat() const;
+
     bool                    CanMountInSlotType(ShipSlotType slot_type) const;       ///< returns true if this part can be placed in a slot of the indicated type
     const std::vector<ShipSlotType>&
                             MountableSlotTypes() const  { return m_mountable_slot_types; }
@@ -124,8 +167,14 @@ public:
     int                     ProductionTime(int empire_id, int location_id) const;   ///< returns the number of turns required to produce this part
     bool                    Producible() const      { return m_producible; }        ///< returns whether this part type is producible by players and appears on the design screen
 
+    const std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+                            ProductionMeterConsumption() const  { return m_production_meter_consumption; }
+    const std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+                            ProductionSpecialConsumption() const{ return m_production_special_consumption; }
+
     const std::set<std::string>& Tags() const       { return m_tags; }
     const Condition::ConditionBase* Location() const{ return m_location; }          ///< returns the condition that determines the locations where ShipDesign containing part can be produced
+    const std::set<std::string>& Exclusions() const { return m_exclusions; }        ///< returns the names of other content that cannot be used in the same ship design as this part
     const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& Effects() const
                                                     { return m_effects; }           ///< returns the EffectsGroups that encapsulate the effects this part has
     const std::string&      Icon() const            { return m_icon; }              ///< returns icon graphic that represents part in UI
@@ -138,14 +187,21 @@ private:
     std::string                                             m_description;
     ShipPartClass                                           m_class;
     float                                                   m_capacity;
+    float                                                   m_secondary_stat;   // damage for a hangar bay, shots per turn for a weapon, etc.
     ValueRef::ValueRefBase<double>*                         m_production_cost;
     ValueRef::ValueRefBase<int>*                            m_production_time;
     bool                                                    m_producible;
     std::vector<ShipSlotType>                               m_mountable_slot_types;
     std::set<std::string>                                   m_tags;
+    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+                                                            m_production_meter_consumption;
+    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+                                                            m_production_special_consumption;
     Condition::ConditionBase*                               m_location;
+    std::set<std::string>                                   m_exclusions;
     std::vector<boost::shared_ptr<Effect::EffectsGroup> >   m_effects;
     std::string                                             m_icon;
+    bool                                                    m_add_standard_capacity_effect;
 
     friend class boost::serialization::access;
     template <class Archive>
@@ -250,46 +306,20 @@ public:
         m_producible(false),
         m_slots(),
         m_tags(),
+        m_production_meter_consumption(),
+        m_production_special_consumption(),
         m_location(0),
         m_effects(),
         m_graphic(),
         m_icon()
     {}
 
-    HullType(const std::string& name, const std::string& description,
-             float fuel, float battle_speed, float speed,
-             float stealth, float structure,
-             const PartHullCommonParams& common_params,
+    HullType(const HullTypeStats& stats, const CommonParams& common_params,
+             const MoreCommonParams& more_common_params,
              const std::vector<Slot>& slots,
-             const std::string& graphic) :
-        m_name(name),
-        m_description(description),
-        m_speed(speed),
-        m_fuel(fuel),
-        m_stealth(stealth),
-        m_structure(structure),
-        m_production_cost(common_params.production_cost),
-        m_production_time(common_params.production_time),
-        m_producible(common_params.producible),
-        m_slots(slots),
-        m_tags(),
-        m_location(common_params.location),
-        m_effects(),
-        m_graphic(graphic),
-        m_icon(common_params.icon)
-    {
-        Init(common_params.effects);
-        for ( std::set< std::string >::iterator tag_it = common_params.tags.begin(); tag_it != common_params.tags.end(); tag_it++)
-            m_tags.insert(boost::to_upper_copy<std::string>(*tag_it));
-    }
-
-    HullType(const std::string& name, const std::string& description,
-             const HullTypeStats& stats,
-             const PartHullCommonParams& common_params,
-             const std::vector<Slot>& slots,
-             const std::string& graphic) :
-        m_name(name),
-        m_description(description),
+             const std::string& icon, const std::string& graphic) :
+        m_name(more_common_params.name),
+        m_description(more_common_params.description),
         m_speed(stats.speed),
         m_fuel(stats.fuel),
         m_stealth(stats.stealth),
@@ -299,13 +329,17 @@ public:
         m_producible(common_params.producible),
         m_slots(slots),
         m_tags(),
+        m_production_meter_consumption(common_params.production_meter_consumption),
+        m_production_special_consumption(common_params.production_special_consumption),
         m_location(common_params.location),
+        m_exclusions(more_common_params.exclusions),
         m_effects(),
         m_graphic(graphic),
-        m_icon(common_params.icon)
+        m_icon(icon)
     {
+        //std::cout << "hull type: " << m_name << " producible: " << m_producible << std::endl;
         Init(common_params.effects);
-        for ( std::set< std::string >::iterator tag_it = common_params.tags.begin(); tag_it != common_params.tags.end(); tag_it++)
+        for (std::set< std::string >::iterator tag_it = common_params.tags.begin(); tag_it != common_params.tags.end(); ++tag_it)
             m_tags.insert(boost::to_upper_copy<std::string>(*tag_it));
     }
 
@@ -330,14 +364,22 @@ public:
     int                 ProductionTime(int empire_id, int location_id) const;   ///< returns the number of turns required to produce this hull
     bool                Producible() const      { return m_producible; }        ///< returns whether this hull type is producible by players and appears on the design screen
 
+    const std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+                            ProductionMeterConsumption() const  { return m_production_meter_consumption; }
+    const std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+                            ProductionSpecialConsumption() const{ return m_production_special_consumption; }
+
     unsigned int        NumSlots() const        { return m_slots.size(); }      ///< returns total number of of slots in hull
     unsigned int        NumSlots(ShipSlotType slot_type) const;                 ///< returns number of of slots of indicated type in hull
     const std::vector<Slot>&    Slots() const   { return m_slots; }             ///< returns vector of slots in hull
 
     const std::set<std::string>& Tags() const   { return m_tags; }
 
+    bool HasTag(const std::string& tag) const   { return m_tags.count(tag) != 0; }
+
     const Condition::ConditionBase* Location() const
     { return m_location; }                                                      ///< returns the condition that determines the locations where ShipDesign containing hull can be produced
+    const std::set<std::string>& Exclusions() const { return m_exclusions; }    ///< returns the names of other content that cannot be used in the same ship design as this part
     const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& Effects() const
     { return m_effects; }                                                       ///< returns the EffectsGroups that encapsulate the effects this part hull has
 
@@ -359,7 +401,12 @@ private:
     bool                                                    m_producible;
     std::vector<Slot>                                       m_slots;
     std::set<std::string>                                   m_tags;
+    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+                                                            m_production_meter_consumption;
+    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+                                                            m_production_special_consumption;
     Condition::ConditionBase*                               m_location;
+    std::set<std::string>                                   m_exclusions;
     std::vector<boost::shared_ptr<Effect::EffectsGroup> >   m_effects;
     std::string                                             m_graphic;
     std::string                                             m_icon;
@@ -460,12 +507,9 @@ public:
     bool                            IsArmed() const         { return m_is_armed; }
     bool                            IsMonster() const       { return m_is_monster; }
 
-    /////// TEMPORARY ///////
-    float  Defense() const;
-    float  Attack() const;
-    float  AdjustedAttack(float shield) const;
-    /////// TEMPORARY ///////
-
+    float                           Attack() const;
+    float                           AdjustedAttack(float shield) const;
+    float                           Defense() const;
 
     const std::string&              Hull() const            { return m_hull; }      ///< returns name of hull on which design is based
     const HullType*                 GetHull() const
@@ -628,13 +672,19 @@ void PartType::serialize(Archive& ar, const unsigned int version)
         & BOOST_SERIALIZATION_NVP(m_description)
         & BOOST_SERIALIZATION_NVP(m_class)
         & BOOST_SERIALIZATION_NVP(m_capacity)
+        & BOOST_SERIALIZATION_NVP(m_secondary_stat)
         & BOOST_SERIALIZATION_NVP(m_production_cost)
         & BOOST_SERIALIZATION_NVP(m_production_time)
+        & BOOST_SERIALIZATION_NVP(m_producible)
         & BOOST_SERIALIZATION_NVP(m_mountable_slot_types)
         & BOOST_SERIALIZATION_NVP(m_tags)
+        & BOOST_SERIALIZATION_NVP(m_production_meter_consumption)
+        & BOOST_SERIALIZATION_NVP(m_production_special_consumption)
         & BOOST_SERIALIZATION_NVP(m_location)
+        & BOOST_SERIALIZATION_NVP(m_exclusions)
         & BOOST_SERIALIZATION_NVP(m_effects)
-        & BOOST_SERIALIZATION_NVP(m_icon);
+        & BOOST_SERIALIZATION_NVP(m_icon)
+        & BOOST_SERIALIZATION_NVP(m_add_standard_capacity_effect);
 }
 
 template <class Archive>
@@ -648,9 +698,13 @@ void HullType::serialize(Archive& ar, const unsigned int version)
         & BOOST_SERIALIZATION_NVP(m_structure)
         & BOOST_SERIALIZATION_NVP(m_production_cost)
         & BOOST_SERIALIZATION_NVP(m_production_time)
+        & BOOST_SERIALIZATION_NVP(m_producible)
         & BOOST_SERIALIZATION_NVP(m_slots)
         & BOOST_SERIALIZATION_NVP(m_tags)
+        & BOOST_SERIALIZATION_NVP(m_production_meter_consumption)
+        & BOOST_SERIALIZATION_NVP(m_production_special_consumption)
         & BOOST_SERIALIZATION_NVP(m_location)
+        & BOOST_SERIALIZATION_NVP(m_exclusions)
         & BOOST_SERIALIZATION_NVP(m_effects)
         & BOOST_SERIALIZATION_NVP(m_graphic)
         & BOOST_SERIALIZATION_NVP(m_icon);
