@@ -8,7 +8,6 @@
 #include "ValueRef.h"
 #include "../parse/Parse.h"
 #include "../util/OptionsDB.h"
-#include "../util/Directories.h"
 #include "../util/Logger.h"
 #include "../util/Random.h"
 #include "../util/AppInterface.h"
@@ -146,8 +145,7 @@ std::string Species::Dump() const {
     return retval;
 }
 
-std::string Species::GameplayDescription() const
-{
+std::string Species::GameplayDescription() const {
     std::stringstream result;
 
     result << UserString(m_gameplay_description);
@@ -158,17 +156,15 @@ std::string Species::GameplayDescription() const
          m_effects.end() != it; ++it)
     {
         const std::string& description = (*it)->GetDescription();
+        if (description.empty())
+            continue;
 
-        if (!description.empty())
-        {
-            if (requires_separator)
-            {
-                result << "\n";
-                requires_separator = false;
-            }
-
-            result << UserString(description) << "\n";
+        if (requires_separator) {
+            result << "\n";
+            requires_separator = false;
         }
+
+        result << UserString(description) << "\n";
     }
 
     return result.str();
@@ -326,7 +322,14 @@ SpeciesManager::SpeciesManager() {
     if (s_instance)
         throw std::runtime_error("Attempted to create more than one SpeciesManager.");
     s_instance = this;
-    parse::species(GetResourceDir() / "species.txt", m_species);
+
+    try {
+        parse::species(m_species);
+    } catch (const std::exception& e) {
+        ErrorLogger() << "Failed parsing species: error: " << e.what();
+        throw e;
+    }
+
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Species:";
         for (iterator it = begin(); it != end(); ++it) {
