@@ -5,7 +5,6 @@
 #include "ObjectMap.h"
 #include "../parse/Parse.h"
 #include "../util/OptionsDB.h"
-#include "../util/Directories.h"
 #include "../util/Logger.h"
 #include "../util/AppInterface.h"
 #include "../Empire/Empire.h"
@@ -94,14 +93,6 @@ std::string Tech::Dump() const {
     retval += DumpIndent() + "name = \"" + m_name + "\"\n";
     retval += DumpIndent() + "description = \"" + m_description + "\"\n";
     retval += DumpIndent() + "shortdescription = \"" + m_short_description + "\"\n";
-    retval += DumpIndent() + "techtype = ";
-    switch (m_type) {
-    case TT_THEORY:      retval += "Theory"; break;
-    case TT_APPLICATION: retval += "Application"; break;
-    case TT_REFINEMENT:  retval += "Refinement"; break;
-    default: retval += "?"; break;
-    }
-    retval += "\n";
     retval += DumpIndent() + "category = \"" + m_category + "\"\n";
     retval += DumpIndent() + "researchcost = " + m_research_cost->Dump() + "\n";
     retval += DumpIndent() + "researchturns = " + m_research_turns->Dump() + "\n";
@@ -328,7 +319,12 @@ TechManager::TechManager() {
 
     std::set<std::string> categories_seen_in_techs;
 
-    parse::techs(GetResourceDir() / "techs.txt", m_techs, m_categories, categories_seen_in_techs);
+    try {
+        parse::techs(m_techs, m_categories, categories_seen_in_techs);
+    } catch (const std::exception& e) {
+        ErrorLogger() << "Failed parsing techs: error: " << e.what();
+        throw e;
+    }
 
     std::set<std::string> empty_defined_categories;
     for (std::map<std::string, TechCategory*>::iterator map_it = m_categories.begin(); map_it != m_categories.end(); ++map_it) {
@@ -423,11 +419,6 @@ std::string TechManager::FindIllegalDependencies() {
                 stream << "ERROR: Tech \"" << tech->Name() << "\" requires a missing or malformed tech \"" << *prereq_it << "\" as its prerequisite.";
                 return stream.str();
             }
-            //TechType prereq_type = prereq_tech->Type();
-            //if (tech_type == TT_THEORY && prereq_type != TT_THEORY)
-            //    retval += "ERROR: Theory tech \"" + tech->Name() + "\" requires non-Theory tech \"" + prereq_tech->Name() + "\"; Theory techs can only require other Theory techs.\n";
-            //if (prereq_type == TT_REFINEMENT && tech_type != TT_REFINEMENT)
-            //    retval += "ERROR: Non-Refinement Tech \"" + tech->Name() + "\" requires Refinement tech \"" + prereq_tech->Name() + "\"; Refinement techs cannot be requirements for anything but other Refinement techs.\n";
         }
     }
     return retval;
@@ -498,7 +489,7 @@ std::string TechManager::FindRedundantDependency() {
         const Tech* tech = *it;
         if (!tech) {
             std::stringstream stream;
-            stream << "ERROR: Missing tech referenced in techs.txt for unknown reasons...";
+            stream << "ERROR: Missing referenced tech for unknown reasons...";
             return stream.str();
         }
         std::set<std::string> prereqs = tech->Prerequisites();
