@@ -1,4 +1,3 @@
-// -*- C++ -*-
 #ifndef _Universe_h_
 #define _Universe_h_
 
@@ -232,10 +231,6 @@ public:
       * were applied to the meter. */
     const Effect::AccountingMap&            GetEffectAccountingMap() const {return m_effect_accounting_map;}
 
-    /** Returns set of objects that have been marked by the Victory effect
-      * to grant their owners victory. */
-    const std::multimap<int, std::string>&  GetMarkedForVictory() const {return m_marked_for_victory;}
-
     const std::map<std::string, std::map<int, std::map<int, double> > >&
                                             GetStatRecords() const { return m_stat_records; }
 
@@ -264,19 +259,19 @@ public:
       * executes all effects on all objects.  Then clamps meter values so
       * target and max meters are within a reasonable range and any current
       * meters with associated max meters are limited by their max. */
-    void            ApplyAllEffectsAndUpdateMeters();
+    void            ApplyAllEffectsAndUpdateMeters(bool do_accounting = true);
 
     /** Determines all effectsgroups' target sets, then resets meters and
       * executes only SetMeter effects on all objects whose ids are listed in
       * \a object_ids.  Then clamps meter values so target and max meters are
       * within a reasonable range and any current meters with associated max
       * meters are limited by their max. */
-    void            ApplyMeterEffectsAndUpdateMeters(const std::vector<int>& object_ids);
+    void            ApplyMeterEffectsAndUpdateMeters(const std::vector<int>& object_ids, bool do_accounting = true);
 
     /** Calls above ApplyMeterEffectsAndUpdateMeters() function on all objects.*/
-    void            ApplyMeterEffectsAndUpdateMeters();
+    void            ApplyMeterEffectsAndUpdateMeters(bool do_accounting = true);
     /** Like ApplyMeterEffectsAndUpdateMeters(), but only target, max and unpaired meters are updated.*/
-    void            ApplyMeterEffectsAndUpdateTargetMaxUnpairedMeters();
+    void            ApplyMeterEffectsAndUpdateTargetMaxUnpairedMeters(bool do_accounting = true);
 
     /** Executes effects that modify objects' appearance in the human client. */
     void            ApplyAppearanceEffects(const std::vector<int>& object_ids);
@@ -307,15 +302,26 @@ public:
     void            UpdateMeterEstimates();
 
     /** Sets all objects' meters' initial values to their current values. */
-    void            BackPropegateObjectMeters();
+    void            BackPropagateObjectMeters();
 
     /** Sets indicated objects' meters' initial values to their current values. */
-    void            BackPropegateObjectMeters(const std::vector<int>& object_ids);
+    void            BackPropagateObjectMeters(const std::vector<int>& object_ids);
 
     /** Determines which empires can see which objects at what visibility
       * level, based on  */
     void            UpdateEmpireObjectVisibilities();
 
+    /** Sets a special record of visibility that overrides the standard
+      * empire-object visibility after the latter is processed. */
+    void            SetEffectDerivedVisibility(int empire_id, int object_id, Visibility vis);
+
+    /** Applies empire-object visibilities set by effects. */
+    void            ApplyEffectDerivedVisibilities();
+
+    /** If an \p empire_id can't currently see \p object_id, then remove
+     * \p object_id' object from the object map and the set of known objects. */
+    void            ForgetKnownObject(int empire_id, int object_id);
+
     /** Sets visibility for indicated \a empire_id of object with \a object_id
       * a vis */
     void            SetEmpireObjectVisibility(int empire_id, int object_id, Visibility vis);
@@ -389,12 +395,6 @@ public:
       * information about this design is retained in the Universe. */
     bool            DeleteShipDesign(int design_id);
 
-    /** Cleans up internal storage of now-invalidated empire ID. */
-    void            HandleEmpireElimination(int empire_id);
-
-    /** Used by the Victory effect to mark an object to give it owner victory. */
-    void            EffectVictory(int object_id, const std::string& reason_string);
-
     /** Sets whether to inhibit UniverseObjectSignals.  Inhibits if \a inhibit
       * is true, and (re)enables UniverseObjectSignals if \a inhibit is false. */
     void            InhibitUniverseObjectSignals(bool inhibit = true);
@@ -528,6 +528,8 @@ private:
     EmpireObjectVisibilityMap       m_empire_object_visibility;         ///< map from empire id to (map from object id to visibility of that object for that empire)
     EmpireObjectVisibilityTurnMap   m_empire_object_visibility_turns;   ///< map from empire id to (map from object id to (map from Visibility rating to turn number on which the empire last saw the object at the indicated Visibility rating or higher)
 
+    EmpireObjectVisibilityMap       m_effect_specified_empire_object_visibilities;
+
     EmpireObjectSpecialsMap         m_empire_object_visible_specials;   ///< map from empire id to (map from object id to (set of names of specials that empire can see are on that object) )
 
     ObjectKnowledgeMap              m_empire_known_destroyed_object_ids;///< map from empire id to (set of object ids that the empire knows have been destroyed)
@@ -536,10 +538,9 @@ private:
     ShipDesignMap                   m_ship_designs;                     ///< ship designs in the universe
     std::map<int, std::set<int> >   m_empire_known_ship_design_ids;     ///< ship designs known to each empire
 
-    mutable distance_matrix_storage<short>
-                                    m_system_jumps;                     ///< indexed by system graph index (not system id), caches the smallest number of jumps to travel between all the systems
-    boost::shared_ptr<GraphImpl>    m_graph_impl;                       ///< a graph in which the systems are vertices and the starlanes are edges
-    boost::unordered_map<int, size_t>  m_system_id_to_graph_index;
+    mutable distance_matrix_storage<short>  m_system_jumps;             ///< indexed by system graph index (not system id), caches the smallest number of jumps to travel between all the systems
+    boost::shared_ptr<GraphImpl>            m_graph_impl;               ///< a graph in which the systems are vertices and the starlanes are edges
+    boost::unordered_map<int, size_t>       m_system_id_to_graph_index;
 
     Effect::AccountingMap           m_effect_accounting_map;            ///< map from target object id, to map from target meter, to orderered list of structs with details of an effect and what it does to the meter
     Effect::DiscrepancyMap          m_effect_discrepancy_map;           ///< map from target object id, to map from target meter, to discrepancy between meter's actual initial value, and the initial value that this meter should have as far as the client can tell: the unknown factor affecting the meter
@@ -548,7 +549,6 @@ private:
     int                             m_last_allocated_design_id;
 
     std::map<int, std::set<int> >   m_marked_destroyed;                 ///< used while applying effects to cache objects that have been destroyed.  this allows to-be-destroyed objects to remain undestroyed until all effects have been processed, which ensures that to-be-destroyed objects still exist when other effects need to access them as a source object. key is destroyed object, and value set are the ids of objects that caused the destruction (may be multiples destroying a single target on a given turn)
-    std::multimap<int, std::string> m_marked_for_victory;               ///< used while applying effects to cache objects whose owner should be victorious.  Victory testing is done separately from effects execution, so this needs to be stored temporarily...
 
     double                          m_universe_width;
     bool                            m_inhibit_universe_object_signals;
