@@ -13,349 +13,10 @@
 #include "Species.h"
 #include "ValueRef.h"
 
-
-DataTableMap& UniverseDataTables() {
-    static DataTableMap map;
-    if (map.empty())
-        LoadDataTables((GetResourceDir() / "universe_tables.txt").string(), map);
-    return map;
-}
-
 //////////////////////////////////////////
 //  Universe Setup Functions            //
 //////////////////////////////////////////
 
-namespace {
-    const double        PI                          = 3.141592653589793;
-    const int           MAX_ATTEMPTS_PLACE_SYSTEM   = 100;
-
-    double CalcNewPosNearestNeighbour(double x, double y, const std::vector<SystemPosition>& positions)
-    {
-        if (positions.size() == 0) // means that we are placing the first star, return a max val to not reject placement
-            return 1e6;
-
-        unsigned int j;
-        double lowest_dist=  (positions[0].x  - x ) * (positions[0].x  - x ) 
-            + (positions[0].y - y) * (positions[0].y - y),distance=0.0;
-
-        for (j = 1; j < positions.size(); ++j) {
-            distance =  (positions[j].x  - x ) * (positions[j].x  - x ) 
-                + (positions[j].y - y) * (positions[j].y - y);
-            if(lowest_dist>distance)
-                lowest_dist = distance;
-        }
-        return lowest_dist;
-    }
-}
-
-double CalcTypicalUniverseWidth(int size)
-{ return (1000.0 / std::sqrt(150.0)) * std::sqrt(static_cast<double>(size)); }
-
-void SpiralGalaxyCalcPositions(std::vector<SystemPosition>& positions,
-                               unsigned int arms, unsigned int stars, double width, double height)
-{
-    double arm_offset     = RandDouble(0.0,2.0*PI);
-    double arm_angle      = 2.0*PI / arms;
-    double arm_spread     = 0.3 * PI / arms;
-    double arm_length     = 1.5 * PI;
-    double center         = 0.25;
-    double x,y;
-    
-    int i, attempts;
-    
-    GaussianDistType  random_gaussian = GaussianDist(0.0,arm_spread);
-    SmallIntDistType  random_arm      = SmallIntDist(0  ,arms);
-    DoubleDistType    random_angle    = DoubleDist  (0.0,2.0*PI);
-    DoubleDistType    random_radius   = DoubleDist  (0.0,  1.0);
-    
-    for (i = 0, attempts = 0; i < static_cast<int>(stars) && attempts < MAX_ATTEMPTS_PLACE_SYSTEM; ++i, ++attempts) {
-        double radius = random_radius();
-        
-        if (radius < center) {
-            double angle = random_angle();
-            x = radius * cos( arm_offset + angle );
-            y = radius * sin( arm_offset + angle );
-        } else {
-            double arm    = static_cast<double>(random_arm()) * arm_angle;
-            double angle  = random_gaussian();
-            
-            x = radius * cos( arm_offset + arm + angle + radius * arm_length );
-            y = radius * sin( arm_offset + arm + angle + radius * arm_length );
-        }
-        
-        x = (x + 1) * width / 2.0;
-        y = (y + 1) * height / 2.0;
-        
-        if (x < 0 || width <= x || y < 0 || height <= y)
-            continue;
-        
-        // See if new star is too close to any existing star.
-        double lowest_dist = CalcNewPosNearestNeighbour(x, y, positions);
-        
-        // If so, we try again or give up.
-        if (lowest_dist < MIN_SYSTEM_SEPARATION * MIN_SYSTEM_SEPARATION) {
-            if (attempts < MAX_ATTEMPTS_PLACE_SYSTEM - 1) {
-                --i; //try again for same star
-            } else {
-                attempts=0; //give up on this star, move on to next
-                DebugLogger() << "Giving up on placing star "<< i <<" with lowest dist squared " << lowest_dist;
-            }
-            continue;
-        }
-        
-        // Add the new star location.
-        positions.push_back(SystemPosition(x, y));
-        
-        // Note that attempts is reset for every star.
-        attempts = 0;
-    }
-}
-
-void EllipticalGalaxyCalcPositions(std::vector<SystemPosition>& positions,
-                                   unsigned int stars, double width, double height)
-{
-    const double ellipse_width_vs_height = RandDouble(0.4, 0.6);
-    const double rotation = RandDouble(0.0, PI),
-    rotation_sin = std::sin(rotation),
-    rotation_cos = std::cos(rotation);
-    const double gap_constant = .95;
-    const double gap_size = 1.0 - gap_constant * gap_constant * gap_constant;
-    
-    // Random number generators.
-    DoubleDistType radius_dist = DoubleDist(0.0, gap_constant);
-    DoubleDistType random_angle  = DoubleDist(0.0, 2.0 * PI);
-    
-    // Used to give up when failing to place a star too often.
-    int attempts = 0;
-    
-    // For each attempt to place a star...
-    for (unsigned int i = 0; i < stars && attempts < MAX_ATTEMPTS_PLACE_SYSTEM; ++i, ++attempts){
-        double radius = radius_dist();
-        // Adjust for bigger density near center and create gap.
-        radius = radius * radius * radius + gap_size;
-        double angle  = random_angle();
-        
-        // Rotate for individual angle and apply elliptical shape.
-        double x1 = radius * std::cos(angle);
-        double y1 = radius * std::sin(angle) * ellipse_width_vs_height;
-        
-        // Rotate for ellipse angle.
-        double x = x1 * rotation_cos - y1 * rotation_sin;
-        double y = x1 * rotation_sin + y1 * rotation_cos;
-        
-        // Move from [-1.0, 1.0] universe coordinates.
-        x = (x + 1.0) * width / 2.0;
-        y = (y + 1.0) * height / 2.0;
-        
-        // Discard stars that are outside boundaries (due to possible rounding errors).
-        if (x < 0 || x >= width || y < 0 || y >= height)
-            continue;
-        
-        // See if new star is too close to any existing star.
-        double lowest_dist = CalcNewPosNearestNeighbour(x, y, positions);
-        
-        // If so, we try again or give up.
-        if (lowest_dist < MIN_SYSTEM_SEPARATION * MIN_SYSTEM_SEPARATION) {
-            if (attempts < MAX_ATTEMPTS_PLACE_SYSTEM - 1) {
-                --i; //try again for same star
-            } else {
-                attempts=0; //give up on this star, move on to next
-                DebugLogger() << "Giving up on placing star "<< i <<" with lowest dist squared " << lowest_dist;
-            }
-            continue;
-        }
-        
-        // Add the new star location.
-        positions.push_back(SystemPosition(x, y));
-        
-        // Note that attempts is reset for every star.
-        attempts = 0;
-    }
-}
-
-void ClusterGalaxyCalcPositions(std::vector<SystemPosition>& positions, unsigned int clusters,
-                                unsigned int stars, double width, double height)
-{
-    if (stars < 1) {
-        ErrorLogger() << "ClusterGalaxyCalcPositions requested for 0 stars";
-        return;
-    }
-    if (clusters < 1) {
-        ErrorLogger() << "ClusterGalaxyCalcPositions requested for 0 clusters. defaulting to 1";
-        clusters = 1;
-    }
-    
-    
-    // probability of systems which don't belong to a cluster
-    const double system_noise = 0.15;
-    double ellipse_width_vs_height = RandDouble(0.2,0.5);
-    // first innermost pair hold cluster position, second innermost pair stores help values for cluster rotation (sin,cos)
-    std::vector<std::pair<std::pair<double, double>, std::pair<double, double> > > clusters_position;
-    unsigned int i,j,attempts;
-    
-    DoubleDistType random_zero_to_one = DoubleDist(0.0, 1.0);
-    DoubleDistType random_angle = DoubleDist(0.0, 2.0*PI);
-    
-    for (i = 0, attempts = 0;
-         i < clusters && static_cast<int>(attempts) < MAX_ATTEMPTS_PLACE_SYSTEM;
-         i++, attempts++)
-    {
-        // prevent cluster position near borders (and on border)
-        double x = ((random_zero_to_one()*2.0 - 1.0) / (clusters + 1.0))*clusters;
-        double y = ((random_zero_to_one()*2.0 - 1.0) / (clusters + 1.0))*clusters;
-        
-        
-        // ensure all clusters have a min separation to each other (search isn't opimized, not worth the effort)
-        for (j = 0; j < clusters_position.size(); j++) {
-            if ((clusters_position[j].first.first - x)*(clusters_position[j].first.first - x)+ (clusters_position[j].first.second - y)*(clusters_position[j].first.second - y)
-                < (2.0/clusters))
-                break;
-        }
-        if (j < clusters_position.size()) {
-            i--;
-            continue;
-        }
-        
-        attempts = 0;
-        double rotation = RandDouble(0.0,PI);
-        clusters_position.push_back(std::pair<std::pair<double,double>,std::pair<double,double> >(std::pair<double,double>(x,y),std::pair<double,double>(sin(rotation),cos(rotation))));
-    }
-    
-    for (i = 0, attempts = 0; i < stars && attempts<100; i++, attempts++) {
-        double x,y;
-        if (random_zero_to_one() < system_noise) {
-            x = random_zero_to_one() * 2.0 - 1.0;
-            y = random_zero_to_one() * 2.0 - 1.0;
-        } else {
-            short  cluster = i % clusters_position.size();
-            double radius  = random_zero_to_one();
-            double angle   = random_angle();
-            double x1,y1;
-            
-            x1 = radius * cos(angle);
-            y1 = radius * sin(angle)*ellipse_width_vs_height;
-            
-            x = x1*clusters_position[cluster].second.second + y1*clusters_position[cluster].second.first;
-            y =-x1*clusters_position[cluster].second.first  + y1*clusters_position[cluster].second.second;
-            
-            x = x/sqrt((double)clusters) + clusters_position[cluster].first.first;
-            y = y/sqrt((double)clusters) + clusters_position[cluster].first.second;
-        }
-        x = (x+1)*width /2.0;
-        y = (y+1)*height/2.0;
-        
-        if (x<0 || width<=x || y<0 || height<=y)
-            continue;
-        
-        // See if new star is too close to any existing star.
-        double lowest_dist = CalcNewPosNearestNeighbour(x, y, positions);
-        
-        // If so, we try again or give up.
-        if (lowest_dist < MIN_SYSTEM_SEPARATION * MIN_SYSTEM_SEPARATION) {
-            if (attempts < MAX_ATTEMPTS_PLACE_SYSTEM - 1) {
-                --i; //try again for same star
-            } else {
-                attempts=0; //give up on this star, move on to next
-                DebugLogger() << "Giving up on placing star "<< i <<" with lowest dist squared " << lowest_dist;
-            }
-            continue;
-        }
-        
-        // Add the new star location.
-        positions.push_back(SystemPosition(x, y));
-        
-        // Note that attempts is reset for every star.
-        attempts = 0;
-    }
-}
-
-void RingGalaxyCalcPositions(std::vector<SystemPosition>& positions, unsigned int stars,
-                             double width, double height)
-{
-    double RING_WIDTH = width / 4.0;
-    double RING_RADIUS = (width - RING_WIDTH) / 2.0;
-    
-    DoubleDistType   theta_dist = DoubleDist(0.0, 2.0 * PI);
-    GaussianDistType radius_dist = GaussianDist(RING_RADIUS, RING_WIDTH / 3.0);
-    
-    for (unsigned int i = 0, attempts = 0; i < stars && static_cast<int>(attempts) < MAX_ATTEMPTS_PLACE_SYSTEM; ++i, ++attempts) {
-        double theta = theta_dist();
-        double radius = radius_dist();
-        
-        double x = width / 2.0 + radius * std::cos(theta);
-        double y = height / 2.0 + radius * std::sin(theta);
-        
-        if (x < 0 || width <= x || y < 0 || height <= y)
-            continue;
-        
-        // See if new star is too close to any existing star.
-        double lowest_dist=CalcNewPosNearestNeighbour(x, y,positions);
-        
-        // If so, we try again or give up.
-        if (lowest_dist < MIN_SYSTEM_SEPARATION * MIN_SYSTEM_SEPARATION) {
-            if (attempts < MAX_ATTEMPTS_PLACE_SYSTEM - 1) {
-                --i; //try again for same star
-            } else {
-                attempts=0; //give up on this star, move on to next
-                DebugLogger() << "Giving up on placing star "<< i <<" with lowest dist squared " << lowest_dist;
-            }
-            continue;
-        }
-        
-        // Add the new star location.
-        positions.push_back(SystemPosition(x, y));
-        
-        // Note that attempts is reset for every star.
-        attempts = 0;
-    }
-}
-
-void IrregularGalaxyPositions(std::vector<SystemPosition>& positions, unsigned int stars,
-                              double width, double height)
-{
-    DebugLogger() << "IrregularGalaxyPositions";
-
-    unsigned int positions_placed = 0;
-    for (unsigned int i = 0, attempts = 0; i < stars && static_cast<int>(attempts) < MAX_ATTEMPTS_PLACE_SYSTEM; ++i, ++attempts) {
-
-        double x = width * RandZeroToOne();
-        double y = height * RandZeroToOne();
-
-        DebugLogger() << "... potential position: (" << x << ", " << y << ")";
-
-        // reject positions outside of galaxy: minimum 0, maximum height or width.  shouldn't be a problem,
-        // but I'm copying this from one of the other generation functions and figure it might as well remain
-        if (x < 0 || width <= x || y < 0 || height <= y)
-            continue;
-
-        // See if new star is too close to any existing star.
-        double lowest_dist = CalcNewPosNearestNeighbour(x, y, positions);
-
-        // If so, we try again or give up.
-        if (lowest_dist < MIN_SYSTEM_SEPARATION * MIN_SYSTEM_SEPARATION) {
-            if (attempts < MAX_ATTEMPTS_PLACE_SYSTEM - 1) {
-                --i; //try again for same star
-            } else {
-                attempts=0; //give up on this star, move on to next
-                DebugLogger() << "Giving up on placing star "<< i <<" with lowest dist squared " << lowest_dist;
-            }
-            continue;
-        }
-
-        // Add the new star location.
-        positions.push_back(SystemPosition(x, y));
-
-        DebugLogger() << "... added system at (" << x << ", " << y << ") after " << attempts << " attempts";
-
-        positions_placed++;
-
-        // Note that attempts is reset for every star.
-        attempts = 0;
-    }
-    DebugLogger() << "... placed " << positions_placed << " systems";
-}
-
-
 namespace Delauney {
     /** simple 2D point.  would have used array of systems, but System
       * class has limits on the range of positions that would interfere
@@ -483,7 +144,7 @@ namespace Delauney {
         // vector of x and y positions of stars
         std::vector<Delauney::DTPoint> points;
         // pointer to main list of triangles algorithm works with.
-        std::list<Delauney::DTTriangle> *triList;
+        std::list<Delauney::DTTriangle> *triangle_list;
         // list of indices in vector of points extracted from removed triangles that need to be retriangulated
         std::list<Delauney::SortValInt> pointNumList;
         double vx, vy, mag;  // vector components, magnitude
@@ -505,13 +166,13 @@ namespace Delauney {
         points.push_back(Delauney::DTPoint(2.0 * (GetUniverse().UniverseWidth() + 1.0), -1.0));
         points.push_back(Delauney::DTPoint(-1.0, 2.0 * (GetUniverse().UniverseWidth() + 1.0)));
 
-        // initialize triList.  algorithm adds and removes triangles from this list, and the resulting
+        // initialize triangle_list.  algorithm adds and removes triangles from this list, and the resulting
         // list is returned (so should be deleted externally)
-        triList = new std::list<Delauney::DTTriangle>;
+        triangle_list = new std::list<Delauney::DTTriangle>;
 
         // add last three points into the first triangle, the "covering triangle"
         theSize = static_cast<int>(points.size());
-        triList->push_front(Delauney::DTTriangle(theSize-1, theSize-2, theSize-3, points));
+        triangle_list->push_front(Delauney::DTTriangle(theSize-1, theSize-2, theSize-3, points));
 
         // loop through "real" points (from systems, not the last three added to make the covering triangle)
         for (n = 0; n < theSize - 3; n++) {
@@ -519,8 +180,8 @@ namespace Delauney {
 
             // check each triangle in list, to see if the new point lies in its circumcircle.  if so, delete
             // the triangle and add its vertices to a list 
-            itCur = triList->begin();
-            itEnd = triList->end();
+            itCur = triangle_list->begin();
+            itEnd = triangle_list->end();
             while (itCur != itEnd) {
                 // get current triangle
                 Delauney::DTTriangle& tri = *itCur;
@@ -560,7 +221,7 @@ namespace Delauney {
                                     pointNumList.insert(itCur2, Delauney::SortValInt(num, mag));
                                     break;
                                 }
-                                itCur2++;
+                                ++itCur2;
                             }
                             if (itCur2 == itEnd2) {
                                 // point wasn't added, so should go at end
@@ -570,7 +231,7 @@ namespace Delauney {
                     } // end for c
 
                     // remove current triangle from list of triangles
-                    itCur = triList->erase(itCur);
+                    itCur = triangle_list->erase(itCur);
                 } else {
                     // point not in circumcircle for this triangle
                     // to go next triangle in list
@@ -584,7 +245,7 @@ namespace Delauney {
             assert(itCur2 != itEnd2);
 
             // add triangle for last and first points and n
-            triList->push_front(Delauney::DTTriangle(n, (pointNumList.front()).num, (pointNumList.back()).num, points));
+            triangle_list->push_front(Delauney::DTTriangle(n, (pointNumList.front()).num, (pointNumList.back()).num, points));
 
             num = itCur2->num;
             ++itCur2;
@@ -592,13 +253,13 @@ namespace Delauney {
                 num2 = num;
                 num = itCur2->num;
 
-                triList->push_front(Delauney::DTTriangle(n, num2, num, points));
+                triangle_list->push_front(Delauney::DTTriangle(n, num2, num, points));
 
                 ++itCur2;
             } // end while
 
         } // end for
-        return triList;
+        return triangle_list;
     } // end function
 }
 
@@ -646,7 +307,7 @@ namespace {
 
         DebugLogger() << "Initializing FleetPlanManager";
 
-        parse::fleet_plans(GetResourceDir() / "starting_fleets.txt", m_plans);
+        parse::fleet_plans(m_plans);
 
 #ifdef OUTPUT_PLANS_LIST
         DebugLogger() << "Starting Fleet Plans:";
@@ -697,12 +358,13 @@ namespace {
 
         DebugLogger() << "Initializing MonsterFleetPlanManager";
 
-        parse::monster_fleet_plans(GetResourceDir() / "space_monster_spawn_fleets.txt", m_plans);
+        parse::monster_fleet_plans(m_plans);
 
 //#ifdef OUTPUT_PLANS_LIST
         DebugLogger() << "Starting Monster Fleet Plans:";
         for (iterator it = begin(); it != end(); ++it)
-            DebugLogger() << " ... " << (*it)->Name() << " spawn rate: " << (*it)->SpawnRate() << " spawn limit: " << (*it)->SpawnLimit();
+            DebugLogger() << " ... " << (*it)->Name() << " spawn rate: " << (*it)->SpawnRate()
+                          << " spawn limit: " << (*it)->SpawnLimit();
 //#endif
     }
 
@@ -790,10 +452,10 @@ namespace {
                     accessibleSystemsMap.insert(std::pair<int, int>(curLaneDest, curDepth + 1));
                    }
 
-                curSysLanesSetIter++;
+                ++curSysLanesSetIter;
             }
 
-            sysListIter++;
+            ++sysListIter;
         }
         return false; // default
     }
@@ -830,19 +492,19 @@ namespace {
 
         int curNumLanes;
 
-        int numSys = systems.size();
+        int num_systems = systems.size();
         // make sure data is consistent
-        if (static_cast<int>(laneSetArray.size()) != numSys) {
+        if (static_cast<int>(laneSetArray.size()) != num_systems) {
             ErrorLogger() << "CullAngularlyTooCloseLanes got different size vectors of lane sets and systems.  Doing nothing.";
             return;
         }
 
-        if (numSys < 3) return;  // nothing worth doing for less than three systems
+        if (num_systems < 3) return;  // nothing worth doing for less than three systems
 
         //DebugLogger() << "Culling Too Close Angularly Lanes";
 
         // loop through systems
-        for (curSys = 0; curSys < numSys; curSys++) {
+        for (curSys = 0; curSys < num_systems; curSys++) {
             // get position of current system (for use in calculated vectors)
             startX = systems[curSys]->X();
             startY = systems[curSys]->Y();
@@ -875,12 +537,12 @@ namespace {
                     tempVectAndMag = VectAndMagTypeQQ(tempVect, mag1);
                     laneVectsMap.insert( MapInsertableTypeQQ(dest1, tempVectAndMag) );
 
-                    laneSetIter1++;
+                    ++laneSetIter1;
                 }
 
                 // iterate through lanes of curSys
                 laneSetIter1 = laneSetArray[curSys].begin();
-                laneSetIter1++;  // start at second, since iterators are used in pairs, and starting both at the first wouldn't be a valid pair
+                ++laneSetIter1;  // start at second, since iterators are used in pairs, and starting both at the first wouldn't be a valid pair
                 while (laneSetIter1 != laneSetEnd) {
                     // get destination of current starlane
                     dest1 = *laneSetIter1;
@@ -941,11 +603,11 @@ namespace {
                                 }
                             }
 
-                            laneSetIter2++;
+                            ++laneSetIter2;
                         }
                     }
 
-                    laneSetIter1++;
+                    ++laneSetIter1;
                 }
             }
         }
@@ -960,13 +622,13 @@ namespace {
             laneSetArray[lane1.second].erase(lane1.first);
 
             // check that removing lane hasn't disconnected systems
-            if (!ConnectedWithin(lane1.first, lane1.second, numSys, laneSetArray)) {
+            if (!ConnectedWithin(lane1.first, lane1.second, num_systems, laneSetArray)) {
                 // they aren't connected... reconnect them
                 laneSetArray[lane1.first].insert(lane1.second);
                 laneSetArray[lane1.second].insert(lane1.first);
             }
 
-            lanesToRemoveIter++;
+            ++lanesToRemoveIter;
         }
     }
 
@@ -991,19 +653,19 @@ namespace {
         std::pair<int, int> lane;
         typedef std::pair<double, std::pair<int, int> > MapInsertableTypeQQ;
 
-        int numSys = systems.size();
+        int num_systems = systems.size();
         // make sure data is consistent
-        if (static_cast<int>(laneSetArray.size()) != numSys) {
+        if (static_cast<int>(laneSetArray.size()) != num_systems) {
             return;
         }
 
-        if (numSys < 2) return;  // nothing worth doing for less than two systems (no lanes!)
+        if (num_systems < 2) return;  // nothing worth doing for less than two systems (no lanes!)
 
         // get squared max lane lenth, so as to eliminate the need to take square roots of lane lenths...
         double maxLaneLength2 = maxLaneLength*maxLaneLength;
 
         // loop through systems
-        for (curSys = 0; curSys < numSys; curSys++) {
+        for (curSys = 0; curSys < num_systems; curSys++) {
             // get position of current system (for use in calculating vector)
             startX = systems[curSys]->X();
             startY = systems[curSys]->Y();
@@ -1031,7 +693,7 @@ namespace {
                     lanesToRemoveMap.insert( MapInsertableTypeQQ(laneLength2, lane) );
                 } 
 
-                laneSetIter++;
+                ++laneSetIter;
             }
         }
 
@@ -1052,21 +714,18 @@ namespace {
                 laneSetArray[lane.second].erase(lane.first);
 
                 // if removing lane has disconnected systems, reconnect them
-                if (!ConnectedWithin(lane.first, lane.second, numSys, laneSetArray)) {
+                if (!ConnectedWithin(lane.first, lane.second, num_systems, laneSetArray)) {
                     laneSetArray[lane.first].insert(lane.second);
                     laneSetArray[lane.second].insert(lane.first);
                 }
             }
-            lanesToRemoveIter++;
+            ++lanesToRemoveIter;
         }
     }
 }
 
-void GenerateStarlanes(GalaxySetupOption freq) {
-    if (freq == GALAXY_SETUP_NONE)
-        return;
-
-    int numSys, s1, s2, s3; // numbers of systems, indices in vec_sys
+void GenerateStarlanes(int max_jumps_between_systems, int max_starlane_length) {
+    int num_systems, s1, s2, s3; // numbers of systems, indices in vec_sys
     int n; // loop counter
 
     std::vector<int> triVerts;  // indices of stars that form vertices of a triangle
@@ -1075,42 +734,36 @@ void GenerateStarlanes(GalaxySetupOption freq) {
     std::vector<std::set<int> > laneSetArray;
 
     // array of set to store possible starlanes for each star, as extracted form triangulation
-    std::vector<std::set<int> > potentialLaneSetArray;
+    std::vector<std::set<int> > potential_lane_set_array;
 
     // iterators for traversing lists of starlanes
     std::set<int>::iterator laneSetIter, laneSetEnd, laneSetIter2, laneSetEnd2;
 
     // get systems
     std::vector<TemporaryPtr<System> > sys_vec = Objects().FindObjects<System>();
-    numSys = sys_vec.size();  // (actually = number of systems + 1)
+    num_systems = sys_vec.size();  // (actually = number of systems + 1)
 
     // pass systems to Delauney Triangulation routine, getting array of triangles back
-    std::list<Delauney::DTTriangle>* triList = Delauney::DelauneyTriangulate(sys_vec);
-    if (!triList ||triList->empty()) {
+    std::list<Delauney::DTTriangle>* triangle_list = Delauney::DelauneyTriangulate(sys_vec);
+    if (!triangle_list ||triangle_list->empty()) {
         ErrorLogger() << "Got no list or blank list of triangles from Triangulation.";
         return;
     }
 
     Delauney::DTTriangle tri;
-
-    // convert passed StarlaneFrequency freq into maximum number of starlane jumps between systems that are
-    // "adjacent" in the delauney triangulation.  (separated by a single potential starlane).
-    // these numbers can be tweaked
-    int maxJumpsBetweenSystems = UniverseDataTables()["MaxJumpsBetweenSystems"][0][freq];
-
     // initialize arrays...
-    potentialLaneSetArray.resize(numSys);
-    for (n = 0; n < numSys; n++) {
-        potentialLaneSetArray[n].clear();
+    potential_lane_set_array.resize(num_systems);
+    for (n = 0; n < num_systems; n++) {
+        potential_lane_set_array[n].clear();
     }
-    laneSetArray.resize(numSys);
-    for (n = 0; n < numSys; n++) {
+    laneSetArray.resize(num_systems);
+    for (n = 0; n < num_systems; n++) {
         laneSetArray[n].clear();
     }
 
     // extract triangles from list, add edges to sets of potential starlanes for each star (in array)
-    while (!triList->empty()) {
-        tri = triList->front();
+    while (!triangle_list->empty()) {
+        tri = triangle_list->front();
 
         triVerts = tri.Verts();
         s1 = triVerts[0];
@@ -1121,66 +774,65 @@ void GenerateStarlanes(GalaxySetupOption freq) {
         // only stars that actually exist.  triangle generation uses three extra points which don't
         // represent actual systems and which need to be weeded out here.
         if ((s1 >= 0) && (s2 >= 0) && (s3 >= 0)) {
-            if ((s1 < numSys) && (s2 < numSys)) {
-                potentialLaneSetArray[s1].insert(s2);
-                potentialLaneSetArray[s2].insert(s1);
+            if ((s1 < num_systems) && (s2 < num_systems)) {
+                potential_lane_set_array[s1].insert(s2);
+                potential_lane_set_array[s2].insert(s1);
             }
-            if ((s1 < numSys) && (s3 < numSys)) {
-                potentialLaneSetArray[s1].insert(s3);
-                potentialLaneSetArray[s3].insert(s1);
+            if ((s1 < num_systems) && (s3 < num_systems)) {
+                potential_lane_set_array[s1].insert(s3);
+                potential_lane_set_array[s3].insert(s1);
             }
-            if ((s2 < numSys) && (s3 < numSys)) {
-                potentialLaneSetArray[s2].insert(s3);
-                potentialLaneSetArray[s3].insert(s2);
+            if ((s2 < num_systems) && (s3 < num_systems)) {
+                potential_lane_set_array[s2].insert(s3);
+                potential_lane_set_array[s3].insert(s2);
             }
         }
 
-        triList->pop_front();
+        triangle_list->pop_front();
     }
 
     // cleanup
-    delete triList;
+    delete triangle_list;
 
     //DebugLogger() << "Extracted Potential Starlanes from Triangulation";
 
-    double maxStarlaneLength = UniverseDataTables()["MaxStarlaneLength"][0][0];
-    CullTooLongLanes(maxStarlaneLength, potentialLaneSetArray, sys_vec);
+    CullTooLongLanes(max_starlane_length, potential_lane_set_array, sys_vec);
 
-    CullAngularlyTooCloseLanes(0.98, potentialLaneSetArray, sys_vec);
+    CullAngularlyTooCloseLanes(0.98, potential_lane_set_array, sys_vec);
 
     //DebugLogger() << "Culled Agularly Too Close Lanes";
 
-    laneSetArray = potentialLaneSetArray;
+    laneSetArray = potential_lane_set_array;
 
     // attempt removing lanes, but don't do so if it would make the systems
     // the lane connects too far apart
-    for (n = 0; n < numSys; ++n) {
-        laneSetIter = potentialLaneSetArray[n].begin();
+    for (n = 0; n < num_systems; ++n) {
+        laneSetIter = potential_lane_set_array[n].begin();
 
-        while (laneSetIter != potentialLaneSetArray[n].end()) {
+        while (laneSetIter != potential_lane_set_array[n].end()) {
             s1 = *laneSetIter;
 
             // try removing lane
             laneSetArray[n].erase(s1);
             laneSetArray[s1].erase(n);
 
-            if (!ConnectedWithin(n, s1, maxJumpsBetweenSystems, laneSetArray)) {
+            if (!ConnectedWithin(n, s1, max_jumps_between_systems, laneSetArray)) {
                 // lane removal was a bad idea.  restore it
                 laneSetArray[n].insert(s1);
                 laneSetArray[s1].insert(n);
             }
 
-            laneSetIter++;
+            ++laneSetIter;
         } // end while
     }
 
     // add the starlane to the stars
-    for (n = 0; n < numSys; ++n) {
+    for (n = 0; n < num_systems; ++n) {
         const std::set<int>& lanes = laneSetArray[n];
         for (std::set<int>::const_iterator it = lanes.begin(); it != lanes.end(); ++it)
             sys_vec[n]->AddStarlane(sys_vec[*it]->ID()); // System::AddStarlane() expects a system ID
     }
-    
+
     DebugLogger() << "Initializing System Graph";
     GetUniverse().InitializeSystemGraph();
 }
@@ -1217,7 +869,7 @@ void SetNativePopulationValues(ObjectMap& object_map) {
         // only applies to unowned planets
         if (meter && targetmax_meter && it->Unowned()) {
             double r = RandZeroToOne();
-            double factor = (0.1<r)?r:0.1;
+            double factor = (0.1 < r) ? r : 0.1;
             meter->SetCurrent(targetmax_meter->Current() * factor);
         }
     }
@@ -1230,19 +882,18 @@ bool SetEmpireHomeworld(Empire* empire, int planet_id, std::string species_name)
     if (home_planet)
         home_system = GetSystem(home_planet->SystemID());
     if (!home_planet || !home_system) {
-        ErrorLogger() << "UniverseGenerator::SetEmpireHomeworld: couldn't get homeworld or system for empire" << empire->EmpireID();
+        ErrorLogger() << "SetEmpireHomeworld: couldn't get homeworld or system for empire" << empire->EmpireID();
         return false;
     }
 
-    DebugLogger() << "UniverseGenerator::SetEmpireHomeworld: setting system " << home_system->ID()
-                           << " (planet " <<  home_planet->ID()
-                           << ") to be home system for empire " << empire->EmpireID();
+    DebugLogger() << "SetEmpireHomeworld: setting system " << home_system->ID()
+                  << " (planet " <<  home_planet->ID() << ") to be home system for empire " << empire->EmpireID();
 
     // get species, check if it exists
     Species* species = GetSpeciesManager().GetSpecies(species_name);
     if (!species) {
-        ErrorLogger() << "UniverseGenerator::SetEmpireHomeworld: couldn't get species \""
-                               << species_name << "\" to set with homeworld id " << home_planet->ID();
+        ErrorLogger() << "SetEmpireHomeworld: couldn't get species \""
+                      << species_name << "\" to set with homeworld id " << home_planet->ID();
         return false;
     }
 
@@ -1257,7 +908,10 @@ bool SetEmpireHomeworld(Empire* empire, int planet_id, std::string species_name)
         // assuming enum values are ordered in increasing goodness...
         PlanetType preferred_planet_type = sept.rbegin()->second;
 
+        // both the current as well as the original type need to be set to the preferred type
         home_planet->SetType(preferred_planet_type);
+        home_planet->SetOriginalType(preferred_planet_type);
+        // set planet size according to planet type
         if (preferred_planet_type == PT_ASTEROIDS)
             home_planet->SetSize(SZ_ASTEROIDS);
         else if (preferred_planet_type == PT_GASGIANT)
@@ -1288,7 +942,8 @@ void InitEmpires(const std::map<int, PlayerSetupData>& player_setup_data)
     {
         int         player_id =     setup_data_it->first;
         if (player_id == Networking::INVALID_PLAYER_ID)
-            ErrorLogger() << "Universe::InitEmpires player id (" << player_id << ") is invalid";
+            ErrorLogger() << "InitEmpires player id (" << player_id << ") is invalid";
+
         // use player ID for empire ID so that the calling code can get the
         // correct empire for each player ID  in player_setup_data
         int         empire_id =     player_id;
@@ -1318,7 +973,7 @@ void InitEmpires(const std::map<int, PlayerSetupData>& player_setup_data)
         std::string empire_name = UserString("EMPIRE") + boost::lexical_cast<std::string>(empire_id);
 
         DebugLogger() << "Universe::InitEmpires creating new empire" << " with ID: " << empire_id
-                               << " for player: " << player_name << " (with player id: " << player_id << ")";
+                      << " for player: " << player_name << " (with player id: " << player_id << ")";
 
         // create new Empire object through empire manager
         Empires().CreateEmpire(empire_id, empire_name, player_name, empire_colour);
