@@ -1,4 +1,3 @@
-// -*- C++ -*-
 #ifndef _OptionsDB_h_
 #define _OptionsDB_h_
 
@@ -114,7 +113,8 @@ public:
     bool        OptionExists(const std::string& name) const
     { return m_options.find(name) != m_options.end() && m_options.at(name).recognized; }
 
-    /** write back the optionDB's state to the XML config file */
+    /** write back the optionDB's state to the XML config file
+        if it has changed since it was last saved. */
     void        Commit();
 
     /** validates a value for an option. throws std::runtime_error if no option
@@ -146,6 +146,13 @@ public:
         return boost::any_cast<T>(it->second.default_value);
     }
 
+    bool IsDefaultValue(const std::string& name) const {
+        std::map<std::string, Option>::const_iterator it = m_options.find(name);
+        if (!OptionExists(it))
+            throw std::runtime_error("OptionsDB::IsDefaultValue<>() : Attempted to get nonexistent option \"" + name + "\".");
+        return IsDefaultValue(it);
+    }
+
     /** returns the string representation of the value of the option \a name.*/
     std::string GetValueString(const std::string& option_name) const;
 
@@ -200,6 +207,7 @@ public:
             }
         }
         m_options[name] = Option(static_cast<char>(0), name, value, default_value, description, validator.Clone(), storable, false, true);
+        m_dirty = true;
         OptionAddedSignal(name);
     }
 
@@ -227,6 +235,7 @@ public:
             }
         }
         m_options[name] = Option(short_name, name, value, default_value, description, validator.Clone(), storable, false, true);
+        m_dirty = true;
         OptionAddedSignal(name);
     }
 
@@ -248,6 +257,7 @@ public:
         }
         m_options[name] = Option(static_cast<char>(0), name, value, boost::lexical_cast<std::string>(false),
                                  description, 0, storable, true, true);
+        m_dirty = true;
         OptionAddedSignal(name);
     }
 
@@ -269,6 +279,7 @@ public:
         }
         m_options[name] = Option(short_name, name, value, boost::lexical_cast<std::string>(false),
                                  description, 0, storable, true, true);
+        m_dirty = true;
         OptionAddedSignal(name);
     }
 
@@ -286,10 +297,7 @@ public:
         std::map<std::string, Option>::iterator it = m_options.find(name);
         if (!OptionExists(it))
             throw std::runtime_error("OptionsDB::Set<>() : Attempted to set nonexistent option \"" + name + "\".");
-        if (it->second.value.type() != typeid(T))
-            throw boost::bad_any_cast();
-        it->second.value = value;
-        (*it->second.option_changed_sig_ptr)();
+        m_dirty |= it->second.SetFromValue(value);
     }
 
     /** fills some or all of the options of the DB from values passed in from
@@ -307,7 +315,11 @@ private:
                const boost::any& default_value_, const std::string& description_,
                const ValidatorBase *validator_, bool storable_, bool flag_, bool recognized_);
 
-        void            SetFromString(const std::string& str);
+        // SetFromValue returns true if this->value is successfully changed
+        template <typename T>
+        bool            SetFromValue(const T& value_);
+        // SetFromString returns true if this->value is successfully changed
+        bool            SetFromString(const std::string& str);
         std::string     ValueToString() const;
         std::string     DefaultValueToString() const;
 
@@ -327,20 +339,49 @@ private:
         static std::map<char, std::string> short_names;   ///< the master list of abbreviated option names, and their corresponding long-form names
     };
 
-    /** indicates whether the option referenced by \it has been added to this
+    /** indicates whether the option referenced by \a it has been added to this
         OptionsDB.  Overloaded for convenient use within other OptionsDB
         functions */
     bool        OptionExists(std::map<std::string, Option>::const_iterator it) const
     { return it != m_options.end() && it->second.recognized; }
 
+    /** indicates whether the current value of the option references by \a is
+        the default value for that option */
+    bool        IsDefaultValue(std::map<std::string, Option>::const_iterator it) const
+    { return it != m_options.end() && it->second.ValueToString() == it->second.DefaultValueToString(); }
+
     OptionsDB();
 
     void        SetFromXMLRecursive(const XMLElement& elem, const std::string& section_name);
 
     std::map<std::string, Option>   m_options;
     static OptionsDB*               s_options_db;
+    bool                            m_dirty; //< has OptionsDB changed since last Commit()
 
     friend OptionsDB& GetOptionsDB();
 };
 
+template <typename T>
+bool OptionsDB::Option::SetFromValue(const T& value_) {
+    if (value.type() != typeid(T))
+        throw boost::bad_any_cast();
+
+    bool changed = false;
+
+    if (!flag) {
+        changed =  validator->String(value) != validator->String(value_);
+    } else {
+        changed = (boost::lexical_cast<std::string>(boost::any_cast<bool>(value))
+                   != boost::lexical_cast<std::string>(boost::any_cast<bool>(value_)));
+    }
+
+    if (changed) {
+        value = value_;
+        (*option_changed_sig_ptr)();
+    }
+    return changed;
+}
+
+
+
 #endif // _OptionsDB_h_
