@@ -104,7 +104,9 @@ void RenameOrder::ExecuteImpl() const {
 // CreateFleetOrder
 ////////////////////////////////////////////////
 NewFleetOrder::NewFleetOrder() :
-    Order()
+    Order(),
+    m_fleet_names(),
+    m_system_id(INVALID_OBJECT_ID)
 {}
 
 NewFleetOrder::NewFleetOrder(int empire, const std::string& fleet_name, int fleet_id,
@@ -161,11 +163,11 @@ void NewFleetOrder::ExecuteImpl() const {
 
     GetUniverse().InhibitUniverseObjectSignals(true);
     std::vector<TemporaryPtr<Fleet> > created_fleets;
-    created_fleets.reserve(m_fleet_names.size());
+    created_fleets.reserve(m_ship_id_groups.size());
 
 
     // create fleet for each group of ships
-    for (int i = 0; i < static_cast<int>(m_fleet_names.size()); ++i) {
+    for (int i = 0; i < static_cast<int>(m_ship_id_groups.size()); ++i) {
         const std::string&      fleet_name =    m_fleet_names[i];
         int                     fleet_id =      m_fleet_ids[i];
         const std::vector<int>& ship_ids =      m_ship_id_groups[i];
@@ -226,6 +228,9 @@ void NewFleetOrder::ExecuteImpl() const {
         }
         fleet->AddShips(validated_ships_ids);
 
+        if (fleet_name.empty())
+            fleet->Rename(fleet->GenerateFleetName());
+
         created_fleets.push_back(fleet);
     }
 
@@ -316,16 +321,18 @@ void FleetMoveOrder::ExecuteImpl() const {
         if (fleet_sys_id != INVALID_OBJECT_ID) {
             // fleet is in a system.  Its move path should also start from that system.
             if (fleet_sys_id != m_start_system) {
-                ErrorLogger() << "Empire with id " << EmpireID() << " ordered a fleet to move from a system with id " << m_start_system <<
-                                        " that it is not at.  Fleet is located at system with id " << fleet_sys_id;
+                ErrorLogger() << "Empire with id " << EmpireID()
+                              << " ordered a fleet to move from a system with id " << m_start_system
+                              << " that it is not at.  Fleet is located at system with id " << fleet_sys_id;
                 return;
             }
         } else {
             // fleet is not in a system.  Its move path should start from the next system it is moving to.
             int next_system = fleet->NextSystemID();
             if (next_system != m_start_system) {
-                ErrorLogger() << "Empire with id " << EmpireID() << " ordered a fleet to move starting from a system with id " << m_start_system <<
-                                        ", but the fleet's next destination is system with id " << next_system;
+                ErrorLogger() << "Empire with id " << EmpireID()
+                              << " ordered a fleet to move starting from a system with id " << m_start_system
+                              << ", but the fleet's next destination is system with id " << next_system;
                 return;
             }
         }
@@ -333,8 +340,9 @@ void FleetMoveOrder::ExecuteImpl() const {
         // We should append and there is something to append to
         int last_system = fleet->TravelRoute().back();
         if (last_system != m_start_system) {
-            ErrorLogger() << "Empire with id " << EmpireID() << " ordered a fleet to continue from system with id " << m_start_system <<
-            ", but the fleet's current route won't lead there, it leads to system " << last_system;
+            ErrorLogger() << "Empire with id " << EmpireID()
+                          << " ordered a fleet to continue from system with id " << m_start_system
+                          << ", but the fleet's current route won't lead there, it leads to system " << last_system;
             return;
         }
     }
@@ -342,14 +350,14 @@ void FleetMoveOrder::ExecuteImpl() const {
 
     // convert list of ids to list of System
     std::list<int> route_list;
-    
+
     if(m_append && !fleet->TravelRoute().empty()){
         route_list = fleet->TravelRoute();
         route_list.erase(--route_list.end());// Remove the last one since it is the first one of the other
     }
-    
+
     std::copy(m_route.begin(), m_route.end(), std::back_inserter(route_list));
-    
+
 
     // validate route.  Only allow travel between systems connected in series by starlanes known to this fleet's owner.
 
@@ -359,7 +367,7 @@ void FleetMoveOrder::ExecuteImpl() const {
         DebugLogger() << "FleetMoveOrder::ExecuteImpl rejected out of range move order";
         return;
     }
-    
+
     std::string waypoints;
     for (std::list<int>::iterator it = route_list.begin(); it != route_list.end(); ++it) {
         waypoints += std::string(" ") + boost::lexical_cast<std::string>(*it);
@@ -785,7 +793,7 @@ DeleteFleetOrder::DeleteFleetOrder() :
     m_fleet(-1)
 {}
 
-DeleteFleetOrder::DeleteFleetOrder(int empire, int fleet) : 
+DeleteFleetOrder::DeleteFleetOrder(int empire, int fleet) :
     Order(empire),
     m_fleet(fleet)
 {}
@@ -824,7 +832,7 @@ ChangeFocusOrder::ChangeFocusOrder() :
     m_focus()
 {}
 
-ChangeFocusOrder::ChangeFocusOrder(int empire, int planet, const std::string& focus) : 
+ChangeFocusOrder::ChangeFocusOrder(int empire, int planet, const std::string& focus) :
     Order(empire),
     m_planet(planet),
     m_focus(focus)
@@ -889,109 +897,86 @@ void ResearchQueueOrder::ExecuteImpl() const {
 ////////////////////////////////////////////////
 ProductionQueueOrder::ProductionQueueOrder() :
     Order(),
-    m_build_type(INVALID_BUILD_TYPE),
-    m_item_name(""),
-    m_design_id(INVALID_OBJECT_ID),
+    m_item(),
     m_number(0),
     m_location(INVALID_OBJECT_ID),
     m_index(INVALID_INDEX),
     m_new_quantity(INVALID_QUANTITY),
     m_new_blocksize(INVALID_QUANTITY),
-    m_new_index(INVALID_INDEX)
+    m_new_index(INVALID_INDEX),
+    m_rally_point_id(INVALID_OBJECT_ID)
 {}
 
-ProductionQueueOrder::ProductionQueueOrder(int empire, BuildType build_type, const std::string& item, int number, int location) :
+ProductionQueueOrder::ProductionQueueOrder(int empire, const ProductionQueue::ProductionItem& item, int number, int location, int pos) :
     Order(empire),
-    m_build_type(build_type),
-    m_item_name(item),
-    m_design_id(INVALID_OBJECT_ID),
+    m_item(item),
     m_number(number),
     m_location(location),
     m_index(INVALID_INDEX),
     m_new_quantity(INVALID_QUANTITY),
     m_new_blocksize(INVALID_QUANTITY),
-    m_new_index(INVALID_INDEX)
-{
-    if (m_build_type == BT_SHIP) {
-        ErrorLogger() << "Attempted to construct a ProductionQueueOrder for a BT_SHIP with a name, not a design id";
-    }
-}
-
-ProductionQueueOrder::ProductionQueueOrder(int empire, BuildType build_type, int design_id, int number, int location) :
-    Order(empire),
-    m_build_type(build_type),
-    m_item_name(""),
-    m_design_id(design_id),
-    m_number(number),
-    m_location(location),
-    m_index(INVALID_INDEX),
-    m_new_quantity(INVALID_QUANTITY),
-    m_new_blocksize(INVALID_QUANTITY),
-    m_new_index(INVALID_INDEX)
-{
-    if (m_build_type == BT_BUILDING) {
-        ErrorLogger() << "Attempted to construct a ProductionQueueOrder for a BT_BUILDING with a design id, not a name";
-    }
-}
+    m_new_index(pos),
+    m_rally_point_id(INVALID_OBJECT_ID)
+{}
 
 ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_quantity, int new_blocksize) :
     Order(empire),
-    m_build_type(INVALID_BUILD_TYPE),
-    m_item_name(""),
-    m_design_id(INVALID_OBJECT_ID),
+    m_item(),
     m_number(0),
     m_location(INVALID_OBJECT_ID),
     m_index(index),
     m_new_quantity(new_quantity),
     m_new_blocksize(new_blocksize),
-    m_new_index(INVALID_INDEX)
-{
-    if (m_build_type == BT_BUILDING) {
-        ErrorLogger() << "Attempted to construct a ProductionQueueOrder for changing quantity &/or blocksize of a BT_BUILDING";
-    }
-}
+    m_new_index(INVALID_INDEX),
+    m_rally_point_id(INVALID_OBJECT_ID)
+{}
 
 ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_quantity, bool dummy) :
     Order(empire),
-    m_build_type(INVALID_BUILD_TYPE),
-    m_item_name(""),
-    m_design_id(INVALID_OBJECT_ID),
+    m_item(),
     m_number(0),
     m_location(INVALID_OBJECT_ID),
     m_index(index),
     m_new_quantity(new_quantity),
     m_new_blocksize(INVALID_QUANTITY),
-    m_new_index(INVALID_INDEX)
-{
-    if (m_build_type == BT_BUILDING) {
-        ErrorLogger() << "Attempted to construct a ProductionQueueOrder for changing quantity of a BT_BUILDING";
-    }
-}
+    m_new_index(INVALID_INDEX),
+    m_rally_point_id(INVALID_OBJECT_ID)
+{}
 
-ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_index) :
+ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int rally_point_id, bool dummy1, bool dummy2) :
     Order(empire),
-    m_build_type(INVALID_BUILD_TYPE),
-    m_item_name(""),
-    m_design_id(INVALID_OBJECT_ID),
+    m_item(),
     m_number(0),
     m_location(INVALID_OBJECT_ID),
     m_index(index),
     m_new_quantity(INVALID_QUANTITY),
     m_new_blocksize(INVALID_QUANTITY),
-    m_new_index(new_index)
+    m_new_index(INVALID_INDEX),
+    m_rally_point_id(rally_point_id)
+{}
+
+ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_index) :
+    Order(empire),
+    m_item(),
+    m_number(0),
+    m_location(INVALID_OBJECT_ID),
+    m_index(index),
+    m_new_quantity(INVALID_QUANTITY),
+    m_new_blocksize(INVALID_QUANTITY),
+    m_new_index(new_index),
+    m_rally_point_id(INVALID_OBJECT_ID)
 {}
 
 ProductionQueueOrder::ProductionQueueOrder(int empire, int index) :
     Order(empire),
-    m_build_type(INVALID_BUILD_TYPE),
-    m_item_name(""),
-    m_design_id(INVALID_OBJECT_ID),
+    m_item(),
     m_number(0),
     m_location(INVALID_OBJECT_ID),
     m_index(index),
     m_new_quantity(INVALID_QUANTITY),
     m_new_blocksize(INVALID_QUANTITY),
-    m_new_index(INVALID_INDEX)
+    m_new_index(INVALID_INDEX),
+    m_rally_point_id(INVALID_OBJECT_ID)
 {}
 
 void ProductionQueueOrder::ExecuteImpl() const {
@@ -999,24 +984,30 @@ void ProductionQueueOrder::ExecuteImpl() const {
 
     Empire* empire = GetEmpire(EmpireID());
     try {
-        if (m_build_type == BT_BUILDING)
-            empire->PlaceBuildInQueue(BT_BUILDING, m_item_name, m_number, m_location);
-        else if (m_build_type == BT_SHIP)
-            empire->PlaceBuildInQueue(BT_SHIP, m_design_id, m_number, m_location);
-        else if (m_new_blocksize != INVALID_QUANTITY) {
+        if (m_item.build_type == BT_BUILDING || m_item.build_type == BT_SHIP) {
+            empire->PlaceBuildInQueue(m_item, m_number, m_location, m_new_index);
+
+        } else if (m_new_blocksize != INVALID_QUANTITY) {
             DebugLogger() << "ProductionQueueOrder quantity " << m_new_quantity << " Blocksize " << m_new_blocksize;
             empire->SetBuildQuantityAndBlocksize(m_index, m_new_quantity, m_new_blocksize);
-        }
-        else if (m_new_quantity != INVALID_QUANTITY)
+
+        } else if (m_new_quantity != INVALID_QUANTITY) {
             empire->SetBuildQuantity(m_index, m_new_quantity);
-        else if (m_new_index != INVALID_INDEX)
+
+        } else if (m_new_index != INVALID_INDEX) {
             empire->MoveBuildWithinQueue(m_index, m_new_index);
-        else if (m_index != INVALID_INDEX) {
+
+        } else if (m_rally_point_id != INVALID_OBJECT_ID) {
+            DebugLogger() << "ProductionQueueOrder setting rally point to id: " << m_rally_point_id;
+            empire->SetBuildRallyPoint(m_index, m_rally_point_id);
+
+        } else if (m_index != INVALID_INDEX) {
             DebugLogger() << "ProductionQueueOrder removing build from index " << m_index;
             empire->RemoveBuildFromQueue(m_index);
-        }
-        else
+
+        } else {
             ErrorLogger() << "Malformed ProductionQueueOrder.";
+        }
     } catch (const std::exception& e) {
         ErrorLogger() << "Build order execution threw exception: " << e.what();
     }
@@ -1028,11 +1019,14 @@ void ProductionQueueOrder::ExecuteImpl() const {
 ShipDesignOrder::ShipDesignOrder() :
     Order(),
     m_design_id(INVALID_OBJECT_ID),
+    m_update_name_or_description(false),
     m_delete_design_from_empire(false),
     m_create_new_design(false),
+    m_move_design(false),
     m_designed_on_turn(0),
     m_is_monster(false),
-    m_name_desc_in_stringtable(false)
+    m_name_desc_in_stringtable(false),
+    m_design_id_after(INVALID_OBJECT_ID)
 {}
 
 ShipDesignOrder::ShipDesignOrder(int empire, int existing_design_id_to_remember) :
@@ -1041,9 +1035,11 @@ ShipDesignOrder::ShipDesignOrder(int empire, int existing_design_id_to_remember)
     m_update_name_or_description(false),
     m_delete_design_from_empire(false),
     m_create_new_design(false),
+    m_move_design(false),
     m_designed_on_turn(0),
     m_is_monster(false),
-    m_name_desc_in_stringtable(false)
+    m_name_desc_in_stringtable(false),
+    m_design_id_after(INVALID_OBJECT_ID)
 {}
 
 ShipDesignOrder::ShipDesignOrder(int empire, int design_id_to_erase, bool dummy) :
@@ -1052,9 +1048,11 @@ ShipDesignOrder::ShipDesignOrder(int empire, int design_id_to_erase, bool dummy)
     m_update_name_or_description(false),
     m_delete_design_from_empire(true),
     m_create_new_design(false),
+    m_move_design(false),
     m_designed_on_turn(0),
     m_is_monster(false),
-    m_name_desc_in_stringtable(false)
+    m_name_desc_in_stringtable(false),
+    m_design_id_after(INVALID_OBJECT_ID)
 {}
 
 ShipDesignOrder::ShipDesignOrder(int empire, int new_design_id, const ShipDesign& ship_design) :
@@ -1063,6 +1061,7 @@ ShipDesignOrder::ShipDesignOrder(int empire, int new_design_id, const ShipDesign
     m_update_name_or_description(false),
     m_delete_design_from_empire(false),
     m_create_new_design(true),
+    m_move_design(false),
     m_name(ship_design.Name()),
     m_description(ship_design.Description()),
     m_designed_on_turn(ship_design.DesignedOnTurn()),
@@ -1071,7 +1070,8 @@ ShipDesignOrder::ShipDesignOrder(int empire, int new_design_id, const ShipDesign
     m_is_monster(ship_design.IsMonster()),
     m_icon(ship_design.Icon()),
     m_3D_model(ship_design.Model()),
-    m_name_desc_in_stringtable(ship_design.LookupInStringtable())
+    m_name_desc_in_stringtable(ship_design.LookupInStringtable()),
+    m_design_id_after(INVALID_OBJECT_ID)
 {}
 
 ShipDesignOrder::ShipDesignOrder(int empire, int existing_design_id, const std::string& new_name/* = ""*/, const std::string& new_description/* = ""*/) :
@@ -1080,11 +1080,26 @@ ShipDesignOrder::ShipDesignOrder(int empire, int existing_design_id, const std::
     m_update_name_or_description(true),
     m_delete_design_from_empire(false),
     m_create_new_design(false),
+    m_move_design(false),
     m_name(new_name),
     m_description(new_description),
     m_designed_on_turn(0),
     m_is_monster(false),
-    m_name_desc_in_stringtable(false)
+    m_name_desc_in_stringtable(false),
+    m_design_id_after(INVALID_OBJECT_ID)
+{}
+
+ShipDesignOrder::ShipDesignOrder(int empire, int design_id, int design_id_after) :
+    Order(empire),
+    m_design_id(design_id),
+    m_update_name_or_description(false),
+    m_delete_design_from_empire(false),
+    m_create_new_design(false),
+    m_move_design(true),
+    m_designed_on_turn(0),
+    m_is_monster(false),
+    m_name_desc_in_stringtable(false),
+    m_design_id_after(design_id_after)
 {}
 
 void ShipDesignOrder::ExecuteImpl() const {
@@ -1135,6 +1150,18 @@ void ShipDesignOrder::ExecuteImpl() const {
         }
         GetUniverse().RenameShipDesign(m_design_id, m_name, m_description);
 
+    } else if (m_move_design) {
+        //Move an existing design from its current location to just before the after_design
+        if (!empire->ShipDesignKept(m_design_id)) {
+            ErrorLogger() << "Tried to move a ShipDesign that the empire wasn't remembering";
+            return;
+        }
+        if (m_design_id == m_design_id_after)
+            return;
+
+        empire->RemoveShipDesign(m_design_id);
+        empire->AddShipDesign(m_design_id, m_design_id_after);
+        DebugLogger() << "Move Ship Design " << m_design_id << " to before " << m_design_id_after;
     } else {
         // player is ordering empire to retain a particular design, so that is can
         // be used to construct ships by that empire.
@@ -1281,3 +1308,25 @@ bool GiveObjectToEmpireOrder::UndoImpl() const {
     return false;
 }
 
+////////////////////////////////////////////////
+// ForgetOrder
+////////////////////////////////////////////////
+ForgetOrder::ForgetOrder() :
+    Order(),
+    m_object_id(INVALID_OBJECT_ID)
+{}
+
+ForgetOrder::ForgetOrder(int empire, int object_id) :
+    Order(empire),
+    m_object_id(object_id)
+{}
+
+void ForgetOrder::ExecuteImpl() const {
+    ValidateEmpireID();
+    int empire_id = EmpireID();
+
+    DebugLogger() << "ForgetOrder::ExecuteImpl empire: " << empire_id
+                  << " for object: " << m_object_id;
+
+    GetUniverse().ForgetKnownObject(empire_id, m_object_id);
+}
