@@ -1,9 +1,9 @@
-// -*- C++ -*-
 #ifndef _Order_h_
 #define _Order_h_
 
 #include "../universe/Enums.h"
 #include "Export.h"
+#include "../Empire/Empire.h"
 
 #include <boost/serialization/access.hpp>
 #include <boost/serialization/nvp.hpp>
@@ -13,7 +13,6 @@
 
 class ShipDesign;
 
-
 /////////////////////////////////////////////////////
 // Order
 /////////////////////////////////////////////////////
@@ -30,7 +29,7 @@ class FO_COMMON_API Order {
 public:
     /** \name Structors */ //@{
     Order(); ///< default ctor
-    Order(int empire) : m_empire(empire) {}     ///< ctor taking the ID of the Empire issuing the order
+    Order(int empire) : m_empire(empire), m_executed(false) {}     ///< ctor taking the ID of the Empire issuing the order
     virtual ~Order() {}
     //@}
 
@@ -88,7 +87,7 @@ public:
 
 private:
     /**
-     * Preconditions of execute: 
+     * Preconditions of execute:
      *    - the designated planet must exist, be owned by the issuing empire
      *
      *  Postconditions:
@@ -109,7 +108,7 @@ private:
 /////////////////////////////////////////////////////
 // NewFleetOrder
 /////////////////////////////////////////////////////
-/** the Order subclass that represents forming a new fleet. 
+/** the Order subclass that represents forming a new fleet.
     Only one of system or position will be used to place the new fleet.*/
 class FO_COMMON_API NewFleetOrder : public Order {
 public:
@@ -179,7 +178,7 @@ private:
      *    - m_fleet is a valid id of a fleet owned by the order-giving empire
      *    - if the fleet is located in a system, m_start_system is the id of that system
      *    - if the fleet is not located in a system, m_start_system is the id of the system the fleet is moving to
-     *    - 
+     *    -
      *
      *  Postconditions:
      *    - TODO: WRITE THIS
@@ -409,7 +408,7 @@ public:
 
 private:
     /**
-     * Preconditions of execute: 
+     * Preconditions of execute:
      *    - the designated planet must exist, be owned by the issuing empire
      *
      *  Postconditions:
@@ -457,7 +456,7 @@ private:
 // ProductionQueueOrder
 /////////////////////////////////////////////////////
 /** The Order subclass that represents changing an empire's production queue.
-  * The 5-arg ctor adds the build to the end of \a empire's queue, the 3-arg
+  * The ProductionItem ctor adds the build to the beginning or end of \a empire's queue, the 3-arg
   * ctor moves an existing build from its current location at \a index to a new
   * one at \a new_index, and the 2-arg ctor removes the build at \a index from
   * \a empire's queue. */
@@ -465,9 +464,9 @@ class FO_COMMON_API ProductionQueueOrder : public Order {
 public:
     /** \name Structors */ //@{
     ProductionQueueOrder();
-    ProductionQueueOrder(int empire, BuildType build_type, const std::string& item, int number, int location);
-    ProductionQueueOrder(int empire, BuildType build_type, int design_id, int number, int location);
+    ProductionQueueOrder(int empire, const ProductionQueue::ProductionItem& item, int number, int location, int pos = -1);
     ProductionQueueOrder(int empire, int index, int new_quantity, bool dummy);
+    ProductionQueueOrder(int empire, int index, int rally_point_id, bool dummy1, bool dummy2);
     ProductionQueueOrder(int empire, int index, int new_quantity, int new_blocksize);
     ProductionQueueOrder(int empire, int index, int new_index);
     ProductionQueueOrder(int empire, int index);
@@ -476,15 +475,14 @@ public:
 private:
     virtual void ExecuteImpl() const;
 
-    BuildType   m_build_type;
-    std::string m_item_name;
-    int         m_design_id;
+    ProductionQueue::ProductionItem m_item;
     int         m_number;
     int         m_location;
     int         m_index;
     int         m_new_quantity;
     int         m_new_blocksize;
     int         m_new_index;
+    int         m_rally_point_id;
 
     static const int INVALID_INDEX = -500;
     static const int INVALID_QUANTITY = -1000;
@@ -507,7 +505,9 @@ private:
   * The 3-arg ctor taking a ShipDesign argument creates a new shipdesign in the
   * universe's catalog of shipdesigns with the passed new design id, and adds
   * this design to the \a empire's set of remembered designs.  The new design
-  * must be marked as designed by this \a empire.*/
+  * must be marked as designed by this \a empire.
+  * The 3-arg ctor (int,int,int) moves a design_id_to_move to before design_id_after
+  */
 class FO_COMMON_API ShipDesignOrder : public Order {
 public:
     /** \name Structors */ //@{
@@ -516,6 +516,7 @@ public:
     ShipDesignOrder(int empire, int design_id_to_erase, bool dummy);
     ShipDesignOrder(int empire, int new_design_id, const ShipDesign& ship_design);
     ShipDesignOrder(int empire, int existing_design_id, const std::string& new_name = "", const std::string& new_description = "");
+    ShipDesignOrder(int empire, int design_id_to_move, int design_id_after);
     //@}
 
 private:
@@ -543,6 +544,7 @@ private:
     bool                        m_update_name_or_description;
     bool                        m_delete_design_from_empire;
     bool                        m_create_new_design;
+    bool                        m_move_design;
 
     // details of design to create
     std::string                 m_name;
@@ -554,6 +556,7 @@ private:
     std::string                 m_icon;
     std::string                 m_3D_model;
     bool                        m_name_desc_in_stringtable;
+    int                         m_design_id_after;             //<location after the inserted design
     // end details of design to create
 
     friend class boost::serialization::access;
@@ -672,6 +675,39 @@ private:
     void serialize(Archive& ar, const unsigned int version);
 };
 
+/////////////////////////////////////////////////////
+// ForgetOrder
+/////////////////////////////////////////////////////
+/** ForgetOrder removes the object from the empire's known objects. */
+class FO_COMMON_API ForgetOrder : public Order {
+public:
+    /** \name Structors */ //@{
+    ForgetOrder();
+    ForgetOrder(int empire, int object_id);
+    //@}
+
+    /** \name Accessors */ //@{
+    int             ObjectID() const { return m_object_id; }///< returns ID of object selected in this order
+    //@}
+
+private:
+    /**
+     *  Preconditions:
+     *     - m_object_id must be the ID of an object not owned by issuing empire
+     *
+     *  Postconditions:
+     *     - the object is removed from the table of known objects.
+     */
+    virtual void    ExecuteImpl() const;
+
+    int m_object_id;
+
+    friend class boost::serialization::access;
+    template <class Archive>
+    void serialize(Archive& ar, const unsigned int version);
+};
+
+
 // Note: *::serialize() implemented in SerializeOrderSet.cpp.
 
 #endif // _Order_h_
